@@ -1,7 +1,14 @@
-// alloc.cpp -- Memory Allocation Subsystem.
-//
-// $Id: alloc.cpp,v 1.9 2006/01/08 04:26:01 sdennis Exp $
-//
+/*! \file alloc.cpp
+ *  Memory Allocation Subsystem.
+ *
+ * $Id: alloc.cpp 492 2006-11-30 17:12:25Z brazilofmux $
+ *
+ * The functions here manage pools of often-used buffers of fixed-size.  It
+ * adds value by greatly reducing the number and strength of calls to the
+ * underlying platform's memory management. It also adds headers and footer to
+ * detect misuse of buffers by its callers.
+ */
+
 #include "copyright.h"
 #include "autoconf.h"
 #include "config.h"
@@ -42,7 +49,8 @@ typedef struct pool_footer
 
 typedef struct pooldata
 {
-    size_t pool_size;               // Size in bytes of a buffer
+    size_t pool_client_size;        // Size in bytes of a buffer as seen by client.
+    size_t pool_alloc_size;         // Size as allocated from system.
     unsigned int poolmagic;         // Magic number specific to this pool
     POOLHDR *free_head;             // Buffer freelist head
     POOLHDR *chain_head;            // Buffer chain head
@@ -55,12 +63,14 @@ typedef struct pooldata
 static POOL pools[NUM_POOLS];
 static const char *poolnames[] =
 {
-    "Lbufs", "Sbufs", "Mbufs", "Bools", "Descs", "Qentries", "Pcaches"
+    "Lbufs", "Sbufs", "Mbufs", "Bools", "Descs", "Qentries", "Pcaches", "Lbufrefs", "Regrefs"
 };
 
 void pool_init(int poolnum, int poolsize)
 {
-    pools[poolnum].pool_size = poolsize;
+    pools[poolnum].pool_client_size = poolsize;
+    pools[poolnum].pool_alloc_size  = poolsize + sizeof(POOLHDR) + sizeof(POOLFTR);
+    mux_assert(pools[poolnum].pool_client_size < pools[poolnum].pool_alloc_size);
     pools[poolnum].poolmagic = CRC32_ProcessInteger2(poolnum, poolsize);
     pools[poolnum].free_head = NULL;
     pools[poolnum].chain_head = NULL;
@@ -87,14 +97,14 @@ static void pool_err
     {
         STARTLOG(logflag, logsys, "ALLOC");
         Log.tinyprintf("%s[%d] (tag %s) %s in %s line %d at %lx. (%s)", action,
-            pools[poolnum].pool_size, tag, reason, file, line, (long)ph,
+            pools[poolnum].pool_client_size, tag, reason, file, line, (long)ph,
             mudstate.debug_cmd);
         ENDLOG;
     }
     else if (logflag != LOG_ALLOCATE)
     {
         Log.tinyprintf(ENDLINE "***< %s[%d] (tag %s) %s in %s line %d at %lx. >***",
-            action, pools[poolnum].pool_size, tag, reason, file, line, (long)ph);
+            action, pools[poolnum].pool_client_size, tag, reason, file, line, (long)ph);
     }
 }
 
@@ -105,12 +115,12 @@ static void pool_vfy(int poolnum, const char *tag, const char *file, const int l
     char *h;
 
     lastph = NULL;
-    size_t psize = pools[poolnum].pool_size;
+    size_t psize = pools[poolnum].pool_client_size;
     for (ph = pools[poolnum].chain_head; ph; lastph = ph, ph = ph->next)
     {
         h = (char *)ph;
         h += sizeof(POOLHDR);
-        h += pools[poolnum].pool_size;
+        h += pools[poolnum].pool_client_size;
         pf = (POOLFTR *) h;
 
         if (ph->magicnum != pools[poolnum].poolmagic)
@@ -173,7 +183,7 @@ char *pool_alloc(int poolnum, const char *tag, const char *file, const int line)
        && ph->magicnum == pools[poolnum].poolmagic)
     {
         p = (char *)(ph + 1);
-        pf = (POOLFTR *)(p + pools[poolnum].pool_size);
+        pf = (POOLFTR *)(p + pools[poolnum].pool_client_size);
         pools[poolnum].free_head = ph->nxtfree;
 
         // Check for corrupted footer, just report and fix it.
@@ -202,18 +212,31 @@ char *pool_alloc(int poolnum, const char *tag, const char *file, const int line)
             pools[poolnum].tot_alloc = pools[poolnum].num_alloc;
         }
 
-        ph = (POOLHDR *)MEMALLOC(pools[poolnum].pool_size + sizeof(POOLHDR)
-           + sizeof(POOLFTR));
-        ISOUTOFMEMORY(ph);
+        ph = NULL;
+        try
+        {
+            ph = reinterpret_cast<POOLHDR *>(new char[pools[poolnum].pool_alloc_size]);
+        }
+        catch (...)
+        {
+            ; // Nothing.
+        }
+
+        if (NULL == ph)
+        {
+            ISOUTOFMEMORY(ph);
+            return NULL;
+        }
+
         p = (char *)(ph + 1);
-        pf = (POOLFTR *)(p + pools[poolnum].pool_size);
+        pf = (POOLFTR *)(p + pools[poolnum].pool_client_size);
 
         // Initialize.
         //
         ph->next = pools[poolnum].chain_head;
         ph->nxtfree = NULL;
         ph->magicnum = pools[poolnum].poolmagic;
-        ph->pool_size = pools[poolnum].pool_size;
+        ph->pool_size = pools[poolnum].pool_client_size;
         pf->magicnum = pools[poolnum].poolmagic;
         *((unsigned int *)p) = pools[poolnum].poolmagic;
         pools[poolnum].chain_head = ph;
@@ -229,7 +252,7 @@ char *pool_alloc(int poolnum, const char *tag, const char *file, const int line)
        && start_log("DBG", "ALLOC"))
     {
         Log.tinyprintf("Alloc[%d] (tag %s) in %s line %d buffer at %lx. (%s)",
-            pools[poolnum].pool_size, tag, file, line, (long)ph, mudstate.debug_cmd);
+            pools[poolnum].pool_client_size, tag, file, line, (long)ph, mudstate.debug_cmd);
         end_log();
     }
 
@@ -288,9 +311,22 @@ char *pool_alloc_lbuf(const char *tag, const char *file, const int line)
             pools[POOL_LBUF].tot_alloc = pools[POOL_LBUF].num_alloc;
         }
 
-        ph = (POOLHDR *)MEMALLOC(LBUF_SIZE + sizeof(POOLHDR)
-           + sizeof(POOLFTR));
-        ISOUTOFMEMORY(ph);
+        ph = NULL;
+        try
+        {
+            ph = reinterpret_cast<POOLHDR *>(new char[LBUF_SIZE + sizeof(POOLHDR) + sizeof(POOLFTR)]);
+        }
+        catch (...)
+        {
+            ; // Nothing.
+        }
+
+        if (NULL == ph)
+        {
+            ISOUTOFMEMORY(ph);
+            return NULL;
+        }
+
         p = (char *)(ph + 1);
         pf = (POOLFTR *)(p + LBUF_SIZE);
 
@@ -341,7 +377,7 @@ void pool_free(int poolnum, char *buf, const char *file, const int line)
         return;
     }
     POOLHDR *ph = ((POOLHDR *)(buf)) - 1;
-    POOLFTR *pf = (POOLFTR *)(buf + pools[poolnum].pool_size);
+    POOLFTR *pf = (POOLFTR *)(buf + pools[poolnum].pool_client_size);
     unsigned int *pui = (unsigned int *)buf;
 
     if (mudconf.paranoid_alloc)
@@ -373,7 +409,7 @@ void pool_free(int poolnum, char *buf, const char *file, const int line)
 
     // Verify that we are not trying to free someone else's buffer.
     //
-    if (ph->pool_size != pools[poolnum].pool_size)
+    if (ph->pool_size != pools[poolnum].pool_client_size)
     {
         pool_err("BUG", LOG_ALWAYS, poolnum, ph->buf_tag, ph, "Free",
                  "Attempt to free into a different pool.", file, line);
@@ -385,7 +421,7 @@ void pool_free(int poolnum, char *buf, const char *file, const int line)
        && start_log("DBG", "ALLOC"))
     {
         Log.tinyprintf("Free[%d] (tag %s) in %s line %d buffer at %lx. (%s)",
-            pools[poolnum].pool_size, ph->buf_tag, file, line, (long)ph,
+            pools[poolnum].pool_client_size, ph->buf_tag, file, line, (long)ph,
             mudstate.debug_cmd);
         end_log();
     }
@@ -517,26 +553,18 @@ static void pool_trace(dbref player, int poolnum, const char *text)
 
 void list_bufstats(dbref player)
 {
-    char buff[MBUF_SIZE];
-
-    notify(player, "Buffer Stats  Size     InUse     Total        Allocs   Lost");
-
-    int i;
-    for (i = 0; i < NUM_POOLS; i++)
+    notify(player, "Buffer Stats  Size      InUse      Total           Allocs   Lost");
+    for (int i = 0; i < NUM_POOLS; i++)
     {
-        char szNumAlloc[22];
-        char szMaxAlloc[22];
-        char szTotAlloc[22];
-        char szNumLost[22];
+        char buff[MBUF_SIZE];
+        char *p = buff;
 
-        mux_i64toa(pools[i].num_alloc, szNumAlloc);
-        mux_i64toa(pools[i].max_alloc, szMaxAlloc);
-        mux_i64toa(pools[i].tot_alloc, szTotAlloc);
-        mux_i64toa(pools[i].num_lost,  szNumLost);
-
-        sprintf(buff, "%-12s %5d%10s%10s%14s%7s",
-            poolnames[i], (int)pools[i].pool_size,
-            szNumAlloc, szMaxAlloc, szTotAlloc, szNumLost);
+        p += LeftJustifyString(p,  12, poolnames[i]);                   *p++ = ' ';
+        p += RightJustifyNumber(p,  5, pools[i].pool_client_size, ' '); *p++ = ' ';
+        p += RightJustifyNumber(p, 10, pools[i].num_alloc,        ' '); *p++ = ' ';
+        p += RightJustifyNumber(p, 10, pools[i].max_alloc,        ' '); *p++ = ' ';
+        p += RightJustifyNumber(p, 16, pools[i].tot_alloc,        ' '); *p++ = ' ';
+        p += RightJustifyNumber(p,  6, pools[i].num_lost,         ' '); *p++ = '\0';
         notify(player, buff);
     }
 }
@@ -566,7 +594,8 @@ void pool_reset(void)
             unsigned int *ibuf = (unsigned int *)h;
             if (*ibuf == pools[i].poolmagic)
             {
-                MEMFREE(ph);
+                char *p = reinterpret_cast<char *>(ph);
+                delete [] p;
                 ph = NULL;
             }
             else
