@@ -1,16 +1,24 @@
-// svdocache.cpp -- Attribute caching module.
-//
-// $Id: attrcache.cpp,v 1.21 2006/01/07 06:26:07 sdennis Exp $
-//
-// MUX 2.4
-// Copyright (C) 1998 through 2004 Solid Vertical Domains, Ltd. All
-// rights not explicitly given are reserved.
-//
+/*! \file attrcache.cpp
+ *  Attribute caching module.
+ *
+ * $Id: attrcache.cpp 3049 2007-12-30 06:35:41Z brazilofmux $
+ *
+ * The functions here manage the upper-level attribute value cache for
+ * disk-based mode. It's not used in memory-based builds. The lower-level
+ * cache is managed in svdhash.cpp
+ *
+ * The upper-level cache is organized by a CHashTable and a linked list. The
+ * former allows random access while the linked list helps find the
+ * least-recently-used attribute.
+ */
+
 #include "copyright.h"
 #include "autoconf.h"
 #include "config.h"
 #include "externs.h"
 
+#if !defined(MEMORY_BASED)
+
 static CHashFile hfAttributeFile;
 static bool cache_initted = false;
 
@@ -35,12 +43,12 @@ typedef struct tagCacheEntryHeader
     struct tagCacheEntryHeader *pPrevEntry;
     struct tagCacheEntryHeader *pNextEntry;
     Aname attrKey;
-    unsigned int nSize;
+    size_t nSize;
 } CENT_HDR, *PCENT_HDR;
 
 static PCENT_HDR pCacheHead = 0;
 static PCENT_HDR pCacheTail = 0;
-static unsigned int CacheSize = 0;
+static size_t CacheSize = 0;
 
 int cache_init(const char *game_dir_file, const char *game_pag_file,
     int nCachePages)
@@ -66,8 +74,8 @@ void cache_redirect(void)
     for (int i = 0; i < N_TEMP_FILES; i++)
     {
         char TempFileName[20];
-        sprintf(TempFileName, "$convtemp.%d", i);
-        TempFiles[i] = fopen(TempFileName, "wb+");
+        mux_sprintf(TempFileName, sizeof(TempFileName), "$convtemp.%d", i);
+        mux_assert(mux_fopen(&TempFiles[i], TempFileName, "wb+"));
         mux_assert(TempFiles[i]);
         setvbuf(TempFiles[i], NULL, _IOFBF, 16384);
     }
@@ -106,12 +114,23 @@ void cache_pass2(void)
         }
         fclose(TempFiles[i]);
         char TempFileName[20];
-        sprintf(TempFileName, "$convtemp.%d", i);
+        mux_sprintf(TempFileName, sizeof(TempFileName), "$convtemp.%d", i);
         RemoveFile(TempFileName);
         fprintf(stderr, ENDLINE);
     }
 }
 
+void cache_cleanup(void)
+{
+    for (int i = 0; i < N_TEMP_FILES; i++)
+    {
+        fclose(TempFiles[i]);
+        char TempFileName[20];
+        mux_sprintf(TempFileName, sizeof(TempFileName), "$convtemp.%d", i);
+        RemoveFile(TempFileName);
+    }
+}
+
 void cache_close(void)
 {
     hfAttributeFile.CloseAll();
@@ -218,7 +237,7 @@ static void TrimCache(void)
     }
 }
 
-const char *cache_get(Aname *nam, int *pLen)
+const char *cache_get(Aname *nam, size_t *pLen)
 {
     if (  nam == (Aname *) 0
        || !cache_initted)
@@ -255,9 +274,8 @@ const char *cache_get(Aname *nam, int *pLen)
     }
 
     UINT32 nHash = CRC32_ProcessInteger2(nam->object, nam->attrnum);
+    UINT32 iDir = hfAttributeFile.FindFirstKey(nHash);
 
-    HP_DIRINDEX iDir;
-    iDir = hfAttributeFile.FindFirstKey(nHash);
     while (iDir != HF_FIND_END)
     {
         HP_HEAPLENGTH nRecord;
@@ -358,7 +376,7 @@ bool cache_put(Aname *nam, const char *value, size_t len)
         return true;
     }
 
-    HP_DIRINDEX iDir = hfAttributeFile.FindFirstKey(nHash);
+    UINT32 iDir = hfAttributeFile.FindFirstKey(nHash);
     while (iDir != HF_FIND_END)
     {
         HP_HEAPLENGTH nRecord;
@@ -447,8 +465,8 @@ void cache_del(Aname *nam)
 #endif
 
     UINT32 nHash = CRC32_ProcessInteger2(nam->object, nam->attrnum);
+    UINT32 iDir = hfAttributeFile.FindFirstKey(nHash);
 
-    HP_DIRINDEX iDir = hfAttributeFile.FindFirstKey(nHash);
     while (iDir != HF_FIND_END)
     {
         HP_HEAPLENGTH nRecord;
@@ -480,3 +498,5 @@ void cache_del(Aname *nam)
         }
     }
 }
+
+#endif // MEMORY_BASED
