@@ -1,7 +1,14 @@
-// boolexp.cpp
-//
-// $Id: boolexp.cpp,v 1.18 2006/01/08 04:28:44 sdennis Exp $
-//
+/*! \file boolexp.cpp
+ *  Boolean Expressions for @locks
+ *
+ * $Id: boolexp.cpp 3182 2008-01-18 08:01:21Z brazilofmux $
+ *
+ * The functions here evaluate and organize boolean expressions used in
+ * locks.  Lock evaluation contains one of server's the three parsers.  The
+ * other two parsers are for functions (see eval.cpp) and commands (see
+ * command.cpp).
+ */
+
 #include "copyright.h"
 #include "autoconf.h"
 #include "config.h"
@@ -110,7 +117,7 @@ bool eval_boolexp(dbref player, dbref thing, dbref from, BOOLEXP *b)
                 STARTLOG(LOG_BUGS, "BUG", "LOCK");
                 log_name_and_loc(player);
                 buff = alloc_mbuf("eval_boolexp.LOG.indir");
-                sprintf(buff, ": Lock had bad indirection (%c, type %d)",
+                mux_sprintf(buff, MBUF_SIZE, ": Lock had bad indirection (%c, type %d)",
                     INDIR_TOKEN, b->sub1->type);
                 log_text(buff);
                 free_mbuf(buff);
@@ -183,24 +190,22 @@ bool eval_boolexp(dbref player, dbref thing, dbref from, BOOLEXP *b)
         {
             bCheck = true;
         }
+
         if (bCheck)
         {
-            char **preserve = NULL;
-            int *preserve_len = NULL;
-            preserve = PushPointers(MAX_GLOBAL_REGS);
-            preserve_len = PushIntegers(MAX_GLOBAL_REGS);
-            save_global_regs("eval_boolexp_save", preserve, preserve_len);
+            reg_ref **preserve = NULL;
+            preserve = PushRegisters(MAX_GLOBAL_REGS);
+            save_global_regs(preserve);
 
             buff2 = bp = alloc_lbuf("eval_boolexp");
             str = buff;
             mux_exec(buff2, &bp, source, player, player,
-                EV_FIGNORE | EV_EVAL | EV_FCHECK | EV_TOP, &str,
-                (char **)NULL, 0);
+                AttrTrace(aflags, EV_FIGNORE|EV_EVAL|EV_FCHECK|EV_TOP), &str,
+                NULL, 0);
             *bp = '\0';
 
-            restore_global_regs("eval_boolexp_save", preserve, preserve_len);
-            PopIntegers(preserve_len, MAX_GLOBAL_REGS);
-            PopPointers(preserve, MAX_GLOBAL_REGS);
+            restore_global_regs(preserve);
+            PopRegisters(preserve, MAX_GLOBAL_REGS);
 
             bCheck = !string_compare(buff2, (char *)b->sub1);
             free_lbuf(buff2);
@@ -307,7 +312,7 @@ static BOOLEXP *test_atr(char *s)
     boolexp_type locktype;
 
     char *buff = alloc_lbuf("test_atr");
-    strcpy(buff, s);
+    mux_strncpy(buff, s, LBUF_SIZE-1);
     for (s = buff; *s && (*s != ':') && (*s != '/'); s++)
     {
         ; // Nothing.
@@ -691,6 +696,12 @@ static BOOLEXP *parse_boolexp_E(void)
 
 BOOLEXP *parse_boolexp(dbref player, const char *buf, bool internal)
 {
+    if (  NULL == buf
+       || '\0' == buf[0])
+    {
+        return TRUE_BOOLEXP;
+    }
+
     size_t n = strlen(buf);
     if (n > sizeof(parsestore)-1)
     {
@@ -699,11 +710,6 @@ BOOLEXP *parse_boolexp(dbref player, const char *buf, bool internal)
     memcpy(parsestore, buf, n+1);
     parsebuf = parsestore;
     parse_player = player;
-    if (  buf == NULL
-       || *buf == '\0')
-    {
-        return TRUE_BOOLEXP;
-    }
     if (!mudstate.bStandAlone)
     {
         parsing_internal = internal;
