@@ -1,18 +1,18 @@
 // command.h -- declarations used by the command processor.
 //
-// $Id: command.h,v 1.17 2006/01/07 23:46:32 sdennis Exp $
+// $Id: command.h 2734 2007-10-28 23:02:55Z brazilofmux $
 //
 
 #ifndef __COMMAND_H
 #define __COMMAND_H
 
 #define CMD_NO_ARG(name)              extern void name(dbref executor, dbref caller, dbref enactor, int)
-#define CMD_ONE_ARG(name)             extern void name(dbref executor, dbref caller, dbref enactor, int, char *)
-#define CMD_ONE_ARG_CMDARG(name)      extern void name(dbref executor, dbref caller, dbref enactor, int, char *, char *[], int)
+#define CMD_ONE_ARG(name)             extern void name(dbref executor, dbref caller, dbref enactor, int eval, int, char *)
+#define CMD_ONE_ARG_CMDARG(name)      extern void name(dbref executor, dbref caller, dbref enactor, int eval, int, char *, char *[], int)
 #define CMD_TWO_ARG(name)             extern void name(dbref executor, dbref caller, dbref enactor, int, int, char *, char *)
-#define CMD_TWO_ARG_CMDARG(name)      extern void name(dbref executor, dbref caller, dbref enactor, int, char *, char *, char*[], int)
-#define CMD_TWO_ARG_ARGV(name)        extern void name(dbref executor, dbref caller, dbref enactor, int, char *, char *[], int)
-#define CMD_TWO_ARG_ARGV_CMDARG(name) extern void name(dbref executor, dbref caller, dbref enactor, int, char *, char *[], int, char*[], int)
+#define CMD_TWO_ARG_CMDARG(name)      extern void name(dbref executor, dbref caller, dbref enactor, int eval, int, char *, char *, char*[], int)
+#define CMD_TWO_ARG_ARGV(name)        extern void name(dbref executor, dbref caller, dbref enactor, int eval, int, char *, char *[], int)
+#define CMD_TWO_ARG_ARGV_CMDARG(name) extern void name(dbref executor, dbref caller, dbref enactor, int eval, int, char *, char *[], int, char*[], int)
 
 /* Command function handlers */
 /* from comsys.c */
@@ -153,10 +153,11 @@ CMD_ONE_ARG(do_hook);           // Set additional operations for a command
 CMD_TWO_ARG(do_flag);           // Rename a flag or remove flag aliases
 CMD_ONE_ARG(do_train);          // Display code to room, then execute
 CMD_TWO_ARG(do_moniker);        // Set accented, colorized name of object
+CMD_TWO_ARG(do_plusemail);
 
 typedef struct
 {
-    char    *cmdname;
+    const char *cmdname;
     NAMETAB *switches;
     int     perms;
     int     extra;
@@ -167,29 +168,29 @@ typedef struct
 
 typedef struct
 {
-    char    *cmdname;
+    const char *cmdname;
     NAMETAB *switches;
     int     perms;
     int     extra;
     int     callseq;
     int     hookmask;
-    void    (*handler)(dbref executor, dbref caller, dbref enactor, int, char *);
+    void    (*handler)(dbref executor, dbref caller, dbref enactor, int eval, int, char *);
 } CMDENT_ONE_ARG;
 
 typedef struct
 {
-    char    *cmdname;
+    const char *cmdname;
     NAMETAB *switches;
     int     perms;
     int     extra;
     int     callseq;
     int     hookmask;
-    void    (*handler)(dbref executor, dbref caller, dbref enactor, int, char *, char *[], int);
+    void    (*handler)(dbref executor, dbref caller, dbref enactor, int eval, int, char *, char *[], int);
 } CMDENT_ONE_ARG_CMDARG;
 
 typedef struct
 {
-    char    *cmdname;
+    const char *cmdname;
     NAMETAB *switches;
     int     perms;
     int     extra;
@@ -200,35 +201,35 @@ typedef struct
 
 typedef struct
 {
-    char    *cmdname;
+    const char *cmdname;
     NAMETAB *switches;
     int     perms;
     int     extra;
     int     callseq;
     int     hookmask;
-    void    (*handler)(dbref executor, dbref caller, dbref enactor, int, char *, char *, char*[], int);
+    void    (*handler)(dbref executor, dbref caller, dbref enactor, int, int, char *, char *, char*[], int);
 } CMDENT_TWO_ARG_CMDARG;
 
 typedef struct
 {
-    char    *cmdname;
+    const char *cmdname;
     NAMETAB *switches;
     int     perms;
     int     extra;
     int     callseq;
     int     hookmask;
-    void    (*handler)(dbref executor, dbref caller, dbref enactor, int, char *, char *[], int);
+    void    (*handler)(dbref executor, dbref caller, dbref enactor, int eval, int, char *, char *[], int);
 } CMDENT_TWO_ARG_ARGV;
 
 typedef struct
 {
-    char    *cmdname;
+    const char *cmdname;
     NAMETAB *switches;
     int     perms;
     int     extra;
     int     callseq;
     int     hookmask;
-    void    (*handler)(dbref executor, dbref caller, dbref enactor, int,
+    void    (*handler)(dbref executor, dbref caller, dbref enactor, int eval, int,
                        char *, char *[], int, char*[], int);
 } CMDENT_TWO_ARG_ARGV_CMDARG;
 
@@ -313,7 +314,13 @@ extern NAMETAB list_names[];
 #define CA_NO_SUSPECT 0x00008000  /* Not by SUSPECT players */
 #define CA_NO_GUEST   0x00010000  /* Not by GUEST players */
 #define CA_NO_UNINS   0x00020000  /* Not by UNINSPECTED players */
+#if defined(FIRANMUX)
+#define CA_NO_IMMOBILE   0x00040000    /* Not by IMMOBILE players */
+#define CA_NO_RESTRICTED 0x00080000    /* Not by RESTRICTED players */
+#define CA_CANTBE_MASK (CA_NO_HAVEN|CA_NO_ROBOT|CA_NO_SLAVE|CA_NO_SUSPECT|CA_NO_GUEST|CA_NO_UNINS|CA_NO_IMMOBILE|CA_NO_RESTRICTED)
+#else
 #define CA_CANTBE_MASK (CA_NO_HAVEN|CA_NO_ROBOT|CA_NO_SLAVE|CA_NO_SUSPECT|CA_NO_GUEST|CA_NO_UNINS)
+#endif // FIRANMUX
 
 #define CA_MARKER0    0x00002000
 #define CA_MARKER1    0x00004000
