@@ -1,6 +1,6 @@
 // comsys.h
 //
-// $Id: comsys.h,v 1.5 2006/01/07 08:10:56 sdennis Exp $
+// $Id: comsys.h 3052 2007-12-30 06:36:27Z brazilofmux $
 //
 
 #ifndef __COMSYS_H__
@@ -15,11 +15,13 @@ struct chanentry
 
 #define NUM_COMSYS 500
 
+#define MAX_USERS_PER_CHANNEL 1000000
 #define MAX_CHANNEL_LEN 50
 #define MAX_HEADER_LEN  100
 #define MAX_TITLE_LEN   200
 #define MAX_ALIAS_LEN   5
 #define ALIAS_SIZE      (MAX_ALIAS_LEN+1)
+#define MAX_COST        32767
 
 struct comuser
 {
@@ -69,12 +71,14 @@ bool test_transmit_access(dbref player, struct channel *chan);
 bool test_receive_access(dbref player, struct channel *chan);
 void do_joinchannel(dbref player, struct channel *ch);
 void do_comdisconnectchannel(dbref player, char *channel);
-void load_channels(FILE *fp);
+void load_channels_V0123(FILE *fp);
+void load_channels_V4(FILE *fp);
 void purge_comsystem(void);
 void save_channels(FILE *fp);
 void destroy_comsys(comsys_t *c);
 void sort_com_aliases(comsys_t *c);
-void load_comsystem(FILE *fp);
+void load_comsystem_V0123(FILE *fp);
+void load_comsystem_V4(FILE *fp);
 void save_comsystem(FILE *fp);
 void SendChannelMessage
 (
@@ -113,9 +117,14 @@ struct channel *select_channel(char *channel);
 struct comuser *select_user(struct channel *ch, dbref player);
 
 char  *get_channel_from_alias();
+char *MakeCanonicalComAlias
+(
+    const char *pAlias,
+    size_t *nValidAlias,
+    bool *bValidAlias
+);
 
 bool  do_comsystem(dbref who, char *cmd);
-void  do_chanlist(dbref executor, dbref caller, dbref enactor, int key, char *pattern);
 
 #define CHANNEL_PLAYER_JOIN     (0x00000001UL)
 #define CHANNEL_PLAYER_TRANSMIT (0x00000002UL)
