@@ -1,7 +1,7 @@
 /*! \file conf.cpp
  *  Set up configuration information and static data.
  *
- * $Id: conf.cpp 2969 2007-12-22 23:32:11Z brazilofmux $
+ * $Id: conf.cpp 3420 2008-03-26 02:49:27Z brazilofmux $
  *
  * This parses the configuration files and controls configuration options used
  * to control the server and its behavior.
@@ -212,7 +212,6 @@ void cf_init(void)
     mudconf.terse_look      = true;
     mudconf.terse_movemsg   = true;
     mudconf.trace_topdown   = true;
-    mudconf.use_http        = false;
 
     // -- ??? Running SC on a non-SC DB may cause problems.
     //
@@ -2045,7 +2044,6 @@ static CONF conftable[] =
     {"trust_site",                cf_site,        CA_GOD,    CA_DISABLED, (int *)&mudstate.suspect_list,   NULL,               0},
     {"uncompress_program",        cf_string_dyn,  CA_STATIC, CA_GOD,      (int *)&mudconf.uncompress,      NULL, SIZEOF_PATHNAME},
     {"unowned_safe",              cf_bool,        CA_GOD,    CA_PUBLIC,   (int *)&mudconf.safe_unowned,    NULL,               0},
-    {"use_http",                  cf_bool,        CA_STATIC, CA_PUBLIC,   (int *)&mudconf.use_http,        NULL,               0},
     {"user_attr_access",          cf_modify_bits, CA_GOD,    CA_DISABLED, &mudconf.vattr_flags,            attraccess_nametab, 0},
     {"user_attr_per_hour",        cf_int,         CA_GOD,    CA_PUBLIC,   &mudconf.vattr_per_hour,         NULL,               0},
     {"wait_cost",                 cf_int,         CA_GOD,    CA_PUBLIC,   &mudconf.waitcost,               NULL,               0},
