@@ -1,11 +1,12 @@
-// cque.cpp -- commands and functions for manipulating the command queue.
-//
-// $Id: cque.cpp,v 1.35 2006/01/07 23:46:32 sdennis Exp $
-//
-// MUX 2.4
-// Copyright (C) 1998 through 2004 Solid Vertical Domains, Ltd. All
-// rights not explicitly given are reserved.
-//
+/*! \file cque.cpp
+ *  Commands and functions for manipulating the command queue.
+ *
+ * $Id: cque.cpp 2803 2007-11-27 00:30:39Z brazilofmux $
+ *
+ * This forms the upper-level command list queue, and includes timed commands
+ * and semaphores.  The lower-level task implementation is found in timer.cpp.
+ */
+
 #include "copyright.h"
 #include "autoconf.h"
 #include "config.h"
@@ -24,7 +25,7 @@ static CLinearTimeDelta GetProcessorUsage(void)
 {
     CLinearTimeDelta ltd;
 #ifdef WIN32
-    if (platform == VER_PLATFORM_WIN32_NT)
+    if (fpGetProcessTimes)
     {
         FILETIME ftCreate;
         FILETIME ftExit;
@@ -70,7 +71,7 @@ static int add_to(dbref executor, int am, int attrnum)
     num += am;
 
     char buff[20];
-    int nlen = 0;
+    size_t nlen = 0;
     *buff = '\0';
     if (num)
     {
@@ -104,17 +105,13 @@ static void Task_RunQueueEntry(void *pEntry, int iUnused)
             //
             for (int i = 0; i < MAX_GLOBAL_REGS; i++)
             {
-                if (point->scr[i])
+                if (mudstate.global_regs[i])
                 {
-                    int n = strlen(point->scr[i]);
-                    memcpy(mudstate.global_regs[i], point->scr[i], n+1);
-                    mudstate.glob_reg_len[i] = n;
-                }
-                else
-                {
-                    mudstate.global_regs[i][0] = '\0';
-                    mudstate.glob_reg_len[i] = 0;
+                    RegRelease(mudstate.global_regs[i]);
+                    mudstate.global_regs[i] = NULL;
                 }
+                mudstate.global_regs[i] = point->scr[i];
+                point->scr[i] = NULL;
             }
 
             char *command = point->comm;
@@ -162,7 +159,8 @@ static void Task_RunQueueEntry(void *pEntry, int iUnused)
                     CLinearTimeDelta ltdUsageBegin = GetProcessorUsage();
 
                     char *log_cmdbuf = process_command(executor, point->caller,
-                        point->enactor, false, cp, point->env, point->nargs);
+                        point->enactor, point->eval, false, cp, point->env,
+                        point->nargs);
 
                     CLinearTimeAbsolute ltaEnd;
                     ltaEnd.GetUTC();
@@ -186,7 +184,7 @@ static void Task_RunQueueEntry(void *pEntry, int iUnused)
                         STARTLOG(LOG_PROBLEMS, "CMD", "CPU");
                         log_name_and_loc(executor);
                         char *logbuf = alloc_lbuf("do_top.LOG.cpu");
-                        sprintf(logbuf, " queued command taking %s secs (enactor #%d): ",
+                        mux_sprintf(logbuf, LBUF_SIZE, " queued command taking %s secs (enactor #%d): ",
                             ltd.ReturnSecondsString(4), point->enactor);
                         log_text(logbuf);
                         free_lbuf(logbuf);
@@ -222,16 +220,26 @@ static void Task_RunQueueEntry(void *pEntry, int iUnused)
             mudstate.inpipe = false;
             mudstate.poutobj = NOTHING;
         }
-        MEMFREE(point->text);
-        point->text = NULL;
-        free_qentry(point);
     }
 
     for (int i = 0; i < MAX_GLOBAL_REGS; i++)
     {
-        mudstate.global_regs[i][0] = '\0';
-        mudstate.glob_reg_len[i] = 0;
+        if (point->scr[i])
+        {
+            RegRelease(point->scr[i]);
+            point->scr[i] = NULL;
+        }
+
+        if (mudstate.global_regs[i])
+        {
+            RegRelease(mudstate.global_regs[i]);
+            mudstate.global_regs[i] = NULL;
+        }
     }
+
+    MEMFREE(point->text);
+    point->text = NULL;
+    free_qentry(point);
 }
 
 // ---------------------------------------------------------------------------
@@ -312,6 +320,16 @@ static int CallBack_HaltQueue(PTASK_RECORD p)
             {
                 add_to(point->sem, -1, point->attr);
             }
+
+            for (int i = 0; i < MAX_GLOBAL_REGS; i++)
+            {
+                if (point->scr[i])
+                {
+                    RegRelease(point->scr[i]);
+                    point->scr[i] = NULL;
+                }
+            }
+
             MEMFREE(point->text);
             point->text = NULL;
             free_qentry(point);
@@ -355,10 +373,11 @@ int halt_que(dbref executor, dbref object)
 // ---------------------------------------------------------------------------
 // do_halt: Command interface to halt_que.
 //
-void do_halt(dbref executor, dbref caller, dbref enactor, int key, char *target)
+void do_halt(dbref executor, dbref caller, dbref enactor, int eval, int key, char *target)
 {
     UNUSED_PARAMETER(caller);
     UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(eval);
 
     dbref executor_targ, obj_targ;
 
@@ -450,9 +469,20 @@ static int CallBack_NotifySemaphoreDrainOrAll(PTASK_RECORD p)
                 //
                 giveto(point->executor, mudconf.waitcost);
                 a_Queue(Owner(point->executor), -1);
+
+                for (int i = 0; i < MAX_GLOBAL_REGS; i++)
+                {
+                    if (point->scr[i])
+                    {
+                        RegRelease(point->scr[i]);
+                        point->scr[i] = NULL;
+                    }
+                }
+
                 MEMFREE(point->text);
                 point->text = NULL;
                 free_qentry(point);
+
                 return IU_REMOVE_TASK;
             }
             else
@@ -483,7 +513,8 @@ static int CallBack_NotifySemaphoreFirstOrQuiet(PTASK_RECORD p)
 {
     // If we've notified enough, exit.
     //
-    if (  Notify_Key == NFY_NFY
+    if (  (  NFY_NFY == Notify_Key
+          || NFY_QUIET == Notify_Key)
        && Notify_Num_Done >= Notify_Num_Max)
     {
         return IU_DONE;
@@ -554,7 +585,8 @@ int nfy_que(dbref sem, int attr, int key, int count)
 
     // Update the sem waiters count.
     //
-    if (key == NFY_NFY)
+    if (  NFY_NFY == key
+       || NFY_QUIET == key)
     {
         add_to(sem, -count, attr);
     }
@@ -653,20 +685,28 @@ void do_notify
 // ---------------------------------------------------------------------------
 // setup_que: Set up a queue entry.
 //
-static BQUE *setup_que(dbref executor, dbref caller, dbref enactor,
-                       char *command, char *args[], int nargs, char *sargs[])
+static BQUE *setup_que
+(
+    dbref    executor,
+    dbref    caller,
+    dbref    enactor,
+    int      eval,
+    char    *command,
+    int      nargs,
+    char    *args[],
+    reg_ref *sargs[]
+)
 {
-    int a;
-    BQUE *tmp;
-
     // Can we run commands at all?
     //
     if (Halted(executor))
+    {
         return NULL;
+    }
 
     // Make sure executor can afford to do it.
     //
-    a = mudconf.waitcost;
+    int a = mudconf.waitcost;
     if (mudconf.machinecost && RandomINT32(0, mudconf.machinecost-1) == 0)
     {
         a++;
@@ -700,20 +740,21 @@ static BQUE *setup_que(dbref executor, dbref caller, dbref enactor,
 
     // Calculate the length of the save string.
     //
-    unsigned int tlen = 0;
-    static unsigned int nCommand;
-    static unsigned int nLenEnv[NUM_ENV_VARS];
-    static unsigned int nLenRegs[MAX_GLOBAL_REGS];
+    size_t tlen = 0;
+    static size_t nCommand;
+    static size_t nLenEnv[NUM_ENV_VARS];
 
     if (command)
     {
         nCommand = strlen(command) + 1;
         tlen = nCommand;
     }
+
     if (nargs > NUM_ENV_VARS)
     {
         nargs = NUM_ENV_VARS;
     }
+
     for (a = 0; a < nargs; a++)
     {
         if (args[a])
@@ -722,21 +763,10 @@ static BQUE *setup_que(dbref executor, dbref caller, dbref enactor,
             tlen += nLenEnv[a];
         }
     }
-    if (sargs)
-    {
-        for (a = 0; a < MAX_GLOBAL_REGS; a++)
-        {
-            if (sargs[a])
-            {
-                nLenRegs[a] = strlen(sargs[a]) + 1;
-                tlen += nLenRegs[a];
-            }
-        }
-    }
 
     // Create the qeue entry and load the save string.
     //
-    tmp = alloc_qentry("setup_que.qblock");
+    BQUE *tmp = alloc_qentry("setup_que.qblock");
     tmp->comm = NULL;
 
     char *tptr = tmp->text = (char *)MEMALLOC(tlen);
@@ -748,6 +778,7 @@ static BQUE *setup_que(dbref executor, dbref caller, dbref enactor,
         tmp->comm = tptr;
         tptr += nCommand;
     }
+
     for (a = 0; a < nargs; a++)
     {
         if (args[a])
@@ -761,26 +792,30 @@ static BQUE *setup_que(dbref executor, dbref caller, dbref enactor,
             tmp->env[a] = NULL;
         }
     }
+
     for ( ; a < NUM_ENV_VARS; a++)
     {
         tmp->env[a] = NULL;
     }
-    for (a = 0; a < MAX_GLOBAL_REGS; a++)
-    {
-        tmp->scr[a] = NULL;
-    }
+
     if (sargs)
     {
         for (a = 0; a < MAX_GLOBAL_REGS; a++)
         {
+            tmp->scr[a] = sargs[a];
             if (sargs[a])
             {
-                memcpy(tptr, sargs[a], nLenRegs[a]);
-                tmp->scr[a] = tptr;
-                tptr += nLenRegs[a];
+                RegAddRef(sargs[a]);
             }
         }
     }
+    else
+    {
+        for (a = 0; a < MAX_GLOBAL_REGS; a++)
+        {
+            tmp->scr[a] = NULL;
+        }
+    }
 
     // Load the rest of the queue block.
     //
@@ -790,6 +825,7 @@ static BQUE *setup_que(dbref executor, dbref caller, dbref enactor,
     tmp->attr = 0;
     tmp->enactor = enactor;
     tmp->caller = caller;
+    tmp->eval = eval;
     tmp->nargs = nargs;
     return tmp;
 }
@@ -799,17 +835,18 @@ static BQUE *setup_que(dbref executor, dbref caller, dbref enactor,
 //
 void wait_que
 (
-    dbref executor,
-    dbref caller,
-    dbref enactor,
-    bool bTimed,
+    dbref    executor,
+    dbref    caller,
+    dbref    enactor,
+    int      eval,
+    bool     bTimed,
     CLinearTimeAbsolute &ltaWhen,
-    dbref sem,
-    int   attr,
-    char *command,
-    char *args[],
-    int   nargs,
-    char *sargs[]
+    dbref    sem,
+    int      attr,
+    char    *command,
+    int      nargs,
+    char    *args[],
+    reg_ref *sargs[]
 )
 {
     if (!(mudconf.control_flags & CF_INTERP))
@@ -817,7 +854,11 @@ void wait_que
         return;
     }
 
-    BQUE *tmp = setup_que(executor, caller, enactor, command, args, nargs, sargs);
+    BQUE *tmp = setup_que(executor, caller, enactor, eval,
+        command,
+        nargs, args,
+        sargs);
+
     if (!tmp)
     {
         return;
@@ -872,17 +913,18 @@ void wait_que
 //
 void sql_que
 (
-    dbref executor,
-    dbref caller,
-    dbref enactor,
-    bool bTimed,
+    dbref    executor,
+    dbref    caller,
+    dbref    enactor,
+    int      eval,
+    bool     bTimed,
     CLinearTimeAbsolute &ltaWhen,
-    dbref thing,
-    int   attr,
-    char *command,
-    char *args[],
-    int   nargs,
-    char *sargs[]
+    dbref    thing,
+    int      attr,
+    char    *command,
+    int      nargs,
+    char    *args[],
+    reg_ref *sargs[]
 )
 {
     if (!(mudconf.control_flags & CF_INTERP))
@@ -890,7 +932,11 @@ void sql_que
         return;
     }
 
-    BQUE *tmp = setup_que(executor, caller, enactor, command, args, nargs, sargs);
+    BQUE *tmp = setup_que(executor, caller, enactor, eval,
+        command,
+        nargs, args,
+        sargs);
+
     if (!tmp)
     {
         return;
@@ -926,6 +972,7 @@ void do_wait
     dbref executor,
     dbref caller,
     dbref enactor,
+    int   eval,
     int key,
     char *event,
     char *cmd,
@@ -950,8 +997,10 @@ void do_wait
             ltd.SetSecondsString(event);
             ltaWhen += ltd;
         }
-        wait_que(executor, caller, enactor, true, ltaWhen, NOTHING, 0, cmd,
-            cargs, ncargs, mudstate.global_regs);
+        wait_que(executor, caller, enactor, eval, true, ltaWhen, NOTHING, 0,
+            cmd,
+            ncargs, cargs,
+            mudstate.global_regs);
         return;
     }
 
@@ -1025,8 +1074,10 @@ void do_wait
             thing = NOTHING;
             bTimed = false;
         }
-        wait_que(executor, caller, enactor, bTimed, ltaWhen, thing, atr,
-            cmd, cargs, ncargs, mudstate.global_regs);
+        wait_que(executor, caller, enactor, eval, bTimed, ltaWhen, thing, atr,
+            cmd,
+            ncargs, cargs,
+            mudstate.global_regs);
     }
 }
 
@@ -1039,6 +1090,7 @@ void do_query
     dbref executor,
     dbref caller,
     dbref enactor,
+    int   eval,
     int   key,
     char *dbref_attr,
     char *dbname_query,
@@ -1276,10 +1328,11 @@ int CallBack_ShowSQLQueries(PTASK_RECORD p)
 // ---------------------------------------------------------------------------
 // do_ps: tell executor what commands they have pending in the queue
 //
-void do_ps(dbref executor, dbref caller, dbref enactor, int key, char *target)
+void do_ps(dbref executor, dbref caller, dbref enactor, int eval, int key, char *target)
 {
     UNUSED_PARAMETER(caller);
     UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(eval);
 
     char *bufp;
     dbref executor_targ, obj_targ;
@@ -1368,19 +1421,19 @@ void do_ps(dbref executor, dbref caller, dbref enactor, int key, char *target)
     //
     bufp = alloc_mbuf("do_ps");
 #ifdef QUERY_SLAVE
-    sprintf(bufp, "Totals: Wait Queue...%d/%d  Semaphores...%d/%d  SQL %d/%d",
+    mux_sprintf(bufp, MBUF_SIZE, "Totals: Wait Queue...%d/%d  Semaphores...%d/%d  SQL %d/%d",
         Shown_RunQueueEntry, Total_RunQueueEntry,
         Shown_SemaphoreTimeout, Total_SemaphoreTimeout,
         Shown_SQLTimeout, Total_SQLTimeout);
 #else
-    sprintf(bufp, "Totals: Wait Queue...%d/%d  Semaphores...%d/%d",
+    mux_sprintf(bufp, MBUF_SIZE, "Totals: Wait Queue...%d/%d  Semaphores...%d/%d",
         Shown_RunQueueEntry, Total_RunQueueEntry,
         Shown_SemaphoreTimeout, Total_SemaphoreTimeout);
 #endif // QUERY_SLAVE
     notify(executor, bufp);
     if (Wizard(executor))
     {
-        sprintf(bufp, "        System Tasks.....%d", Total_SystemTasks);
+        mux_sprintf(bufp, MBUF_SIZE, "        System Tasks.....%d", Total_SystemTasks);
         notify(executor, bufp);
     }
     free_mbuf(bufp);
@@ -1399,6 +1452,7 @@ static int CallBack_Warp(PTASK_RECORD p)
         if (point->IsTimed)
         {
             point->waittime -= ltdWarp;
+            p->ltaWhen -= ltdWarp;
             return IU_UPDATE_TASK;
         }
     }
@@ -1408,10 +1462,11 @@ static int CallBack_Warp(PTASK_RECORD p)
 // ---------------------------------------------------------------------------
 // do_queue: Queue management
 //
-void do_queue(dbref executor, dbref caller, dbref enactor, int key, char *arg)
+void do_queue(dbref executor, dbref caller, dbref enactor, int eval, int key, char *arg)
 {
     UNUSED_PARAMETER(caller);
     UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(eval);
 
     if (key == QUEUE_KICK)
     {
@@ -1436,7 +1491,6 @@ void do_queue(dbref executor, dbref caller, dbref enactor, int key, char *arg)
     else if (key == QUEUE_WARP)
     {
         int iWarp = mux_atol(arg);
-        CLinearTimeDelta ltdWarp;
         ltdWarp.SetSeconds(iWarp);
         if (scheduler.GetMinPriority() <= PRIORITY_CF_DEQUEUE_DISABLED)
         {
