@@ -1,7 +1,11 @@
-// create.cpp -- Commands that create new objects.
-//
-// $Id: create.cpp,v 1.20 2006/01/31 00:17:26 sdennis Exp $
-//
+/*! \file create.cpp
+ *  Commands that create new objects.
+ *
+ * $Id: create.cpp 571 2006-12-27 17:56:12Z brazilofmux $
+ *
+ * This affects creation and destruction of all object types as well as their
+ * initial and final relationships with other objects.
+ */
 
 #include "copyright.h"
 #include "autoconf.h"
@@ -120,9 +124,10 @@ static void open_exit(dbref player, dbref loc, char *direction, char *linkto)
     }
 }
 
-void do_open(dbref executor, dbref caller, dbref enactor, int key,
+void do_open(dbref executor, dbref caller, dbref enactor, int eval, int key,
              char *direction, char *links[], int nlinks)
 {
+    UNUSED_PARAMETER(eval);
     UNUSED_PARAMETER(caller);
     UNUSED_PARAMETER(enactor);
 
@@ -252,7 +257,7 @@ void do_link
     //
     if (!where || !*where)
     {
-        do_unlink(executor, caller, enactor, key, what);
+        do_unlink(executor, caller, enactor, 0, key, what);
         return;
     }
 
@@ -388,7 +393,7 @@ void do_link
 
         STARTLOG(LOG_BUGS, "BUG", "OTYPE");
         buff = alloc_mbuf("do_link.LOG.badtype");
-        sprintf(buff, "Strange object type: object #%d = %d",
+        mux_sprintf(buff, MBUF_SIZE, "Strange object type: object #%d = %d",
             thing, Typeof(thing));
         log_text(buff);
         free_mbuf(buff);
@@ -486,9 +491,10 @@ void do_parent
 // ---------------------------------------------------------------------------
 // do_dig: Create a new room.
 //
-void do_dig(dbref executor, dbref caller, dbref enactor, int key, char *name,
-            char *args[], int nargs)
+void do_dig(dbref executor, dbref caller, dbref enactor, int eval, int key,
+            char *name, char *args[], int nargs)
 {
+    UNUSED_PARAMETER(eval);
     UNUSED_PARAMETER(caller);
 
     // we don't need to know player's location!  hooray!
@@ -595,21 +601,28 @@ void do_clone
     UNUSED_PARAMETER(nargs);
 
     dbref clone, thing, new_owner, loc;
-    FLAG rmv_flags;
     int cost;
 
-    if ((key & CLONE_INVENTORY) || !Has_location(executor))
+    if (  (key & CLONE_INVENTORY)
+       || !Has_location(executor))
+    {
         loc = executor;
+    }
     else
+    {
         loc = Location(executor);
+    }
 
     if (!Good_obj(loc))
+    {
         return;
+    }
 
     init_match(executor, name, NOTYPE);
     match_everything(0);
     thing = noisy_match_result();
-    if ((thing == NOTHING) || (thing == AMBIGUOUS))
+    if (  NOTHING == thing
+       || AMBIGUOUS == thing)
     {
         return;
     }
@@ -679,7 +692,7 @@ void do_clone
     // Go make the clone object.
     //
     bool bValid;
-    int nValidName;
+    size_t nValidName;
     char *pValidName = MakeCanonicalObjectName(arg2, &nValidName, &bValid);
     const char *clone_name;
     if (bValid)
@@ -716,14 +729,9 @@ void do_clone
     //
     s_Pennies(clone, OBJECT_ENDOWMENT(cost));
 
-    // Clear out problem flags from the original
-    //
-    rmv_flags = WIZARD;
-    if (!(key & CLONE_INHERIT) || !Inherits(executor))
-    {
-        rmv_flags |= INHERIT | IMMORTAL;
-    }
-    s_Flags(clone, FLAG_WORD1, Flags(thing) & ~rmv_flags);
+    FLAGSET clearflags;
+    TranslateFlags_Clone(clearflags.word, executor, key);
+    SetClearFlags(clone, clearflags.word, NULL);
 
     // Tell creator about it
     //
@@ -783,8 +791,7 @@ void do_clone
         {
             s_Parent(clone, Parent(thing));
         }
-        did_it(executor, clone, 0, NULL, 0, NULL, A_ACLONE,
-               (char **)NULL, 0);
+        did_it(executor, clone, 0, NULL, 0, NULL, A_ACLONE, 0, NULL, 0);
     }
     else
     {
@@ -886,7 +893,7 @@ static bool can_destroy_player(dbref player, dbref victim)
         notify_quiet(player, "Sorry, no suicide allowed.");
         return false;
     }
-    if (Wizard(victim))
+    if (RealWizard(victim))
     {
         notify_quiet(player, "Even you can't do that!");
         return false;
@@ -894,10 +901,11 @@ static bool can_destroy_player(dbref player, dbref victim)
     return true;
 }
 
-void do_destroy(dbref executor, dbref caller, dbref enactor, int key, char *what)
+void do_destroy(dbref executor, dbref caller, dbref enactor, int eval, int key, char *what)
 {
     UNUSED_PARAMETER(caller);
     UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(eval);
 
     // You can destroy anything you control.
     //
@@ -961,7 +969,7 @@ void do_destroy(dbref executor, dbref caller, dbref enactor, int key, char *what
     }
 
     char *NameOfType = alloc_sbuf("do_destroy.NameOfType");
-    strcpy(NameOfType, object_types[Typeof(thing)].name);
+    mux_strncpy(NameOfType, object_types[Typeof(thing)].name, SBUF_SIZE-1);
     mux_strlwr(NameOfType);
     if (Going(thing))
     {
@@ -1049,7 +1057,7 @@ void do_destroy(dbref executor, dbref caller, dbref enactor, int key, char *what
             else
             {
                 char *tname = alloc_lbuf("destroy_obj");
-                strcpy(tname, Moniker(ThingOwner));
+                mux_strncpy(tname, Moniker(ThingOwner), LBUF_SIZE-1);
                 notify(executor, tprintf("Destroyed %s's %s(#%d).",
                     tname, Moniker(thing), thing));
                 free_lbuf(tname);
