@@ -1,6 +1,6 @@
 // db_rw.cpp
 //
-// $Id: db_rw.cpp,v 1.23 2006/09/05 23:42:17 sdennis Exp $
+// $Id: db_rw.cpp 3050 2007-12-30 06:36:06Z brazilofmux $
 //
 
 #include "copyright.h"
@@ -154,7 +154,17 @@ static BOOLEXP *getboolexp1(FILE *f)
     case '"':
         ungetc(c, f);
         buff = alloc_lbuf("getboolexp_quoted");
-        strcpy(buff, getstring_noalloc(f, 1));
+        {
+            size_t nBuffer;
+            char *pBuffer = getstring_noalloc(f, true, &nBuffer);
+            if (LBUF_SIZE - 1 < nBuffer)
+            {
+                nBuffer = LBUF_SIZE - 1;
+                pBuffer[nBuffer] = '\0';
+            }
+            memcpy(buff, pBuffer, nBuffer+1);
+        }
+
         c = fgetc(f);
         if (c == EOF)
         {
@@ -187,8 +197,17 @@ static BOOLEXP *getboolexp1(FILE *f)
             {
                 b->type = BOOLEXP_ATR;
             }
+
             buff = alloc_lbuf("getboolexp1.attr_lock");
-            strcpy(buff, getstring_noalloc(f, 1));
+            size_t nBuffer;
+            char *pBuffer = getstring_noalloc(f, true, &nBuffer);
+            if (LBUF_SIZE - 1 < nBuffer)
+            {
+                nBuffer = LBUF_SIZE - 1;
+                pBuffer[nBuffer] = '\0';
+            }
+            memcpy(buff, pBuffer, nBuffer+1);
+
             b->sub1 = (BOOLEXP *)StringClone(buff);
             free_lbuf(buff);
         }
@@ -315,16 +334,16 @@ static BOOLEXP *getboolexp(FILE *f)
     int c = getc(f);
     mux_assert(c == '\n');
 
-    if (g_format == F_MUX)
+    if ((c = getc(f)) != '\n')
     {
-        if ((c = getc(f)) != '\n')
-        {
-            ungetc(c, f);
-        }
+        ungetc(c, f);
     }
     return b;
 }
 
+int g_max_nam_atr = INT_MIN;
+int g_max_obj_atr = INT_MIN;
+
 /* ---------------------------------------------------------------------------
  * get_list: Read attribute list from flat file.
  */
@@ -342,15 +361,54 @@ static bool get_list(FILE *f, dbref i)
             atr = getref(f);
             if (atr > 0)
             {
+#if defined(FIRANMUX_CONVERT)
+                switch (atr)
+                {
+                case A_COLOR_OLD:
+                    atr = A_COLOR;
+                    break;
+
+                case A_ALEAD_OLD:
+                    atr = A_ALEAD;
+                    break;
+
+                case A_EXITFORMAT_OLD:
+                    atr = A_EXITFORMAT;
+                    break;
+                }
+#endif // FIRANMUX_CONVERT
+
+                // Maximum attribute number across all objects.
+                //
+                if (g_max_obj_atr < atr)
+                {
+                    g_max_obj_atr = atr;
+                }
+
                 // Store the attr
                 //
-                atr_add_raw(i, atr, getstring_noalloc(f, true));
+                size_t nBuffer;
+                const char *pBuffer;
+                if (3 == g_version)
+                {
+                    size_t nBufferUnicode;
+                    UTF8 *pBufferUnicode = (UTF8 *)getstring_noalloc(f, true, &nBufferUnicode);
+                    pBufferUnicode = convert_color(pBufferUnicode);
+                    pBuffer = ConvertToLatin(pBufferUnicode);
+                    nBuffer = strlen(pBuffer);
+                }
+                else
+                {
+                    pBuffer = getstring_noalloc(f, true, &nBuffer);
+                }
+                atr_add_raw_LEN(i, atr, pBuffer, nBuffer);
             }
             else
             {
                 // Silently discard
                 //
-                getstring_noalloc(f, true);
+                size_t nBuffer;
+                (void)getstring_noalloc(f, true, &nBuffer);
             }
             break;
 
@@ -375,7 +433,10 @@ static bool get_list(FILE *f, dbref i)
 
             // We've found a bad spot.  I hope things aren't too bad.
             //
-            getstring_noalloc(f, true);
+            {
+                size_t nBuffer;
+                (void)getstring_noalloc(f, true, &nBuffer);
+            }
         }
     }
 }
@@ -506,22 +567,27 @@ dbref db_read(FILE *f, int *db_format, int *db_version, int *db_flags)
     int aflags;
     BOOLEXP *tempbool;
     char *buff;
-    int nVisualWidth;
+    size_t nVisualWidth;
+    size_t nBuffer;
 
     g_format = F_UNKNOWN;
     g_version = 0;
     g_flags = 0;
+    g_max_nam_atr = INT_MIN;
+    g_max_obj_atr = INT_MIN;
+
 
     bool header_gotten = false;
     bool size_gotten = false;
     bool nextattr_gotten = false;
 
+    bool convert_values = false;
     bool read_attribs = true;
     bool read_name = true;
     bool read_key = true;
     bool read_money = true;
 
-    int nName;
+    size_t nName;
     bool bValid;
     char *pName;
 
@@ -572,7 +638,7 @@ dbref db_read(FILE *f, int *db_format, int *db_version, int *db_flags)
                 // USER-NAMED ATTRIBUTE
                 //
                 anum = getref(f);
-                tstr = getstring_noalloc(f, true);
+                tstr = getstring_noalloc(f, true, &nBuffer);
                 if (mux_isdigit(*tstr))
                 {
                     aflags = 0;
@@ -586,10 +652,52 @@ dbref db_read(FILE *f, int *db_format, int *db_version, int *db_flags)
                 {
                     aflags = mudconf.vattr_flags;
                 }
+
+                if (3 == g_version)
+                {
+                    tstr = ConvertToLatin((UTF8 *)tstr);
+                }
+
                 pName = MakeCanonicalAttributeName(tstr, &nName, &bValid);
                 if (bValid)
                 {
-                    vattr_define_LEN(pName, nName, anum, aflags);
+#if defined(FIRANMUX_CONVERT)
+                    {
+                        // Does this user attribute conflict with any built-in attribute names?
+                        //
+                        ATTR *a = (ATTR *)hashfindLEN(pName, nName, &mudstate.attr_name_htab);
+                        if (a)
+                        {
+                            Log.tinyprintf("Renaming conflicting user attribute, %s, to FIRAN_%s." ENDLINE, pName, pName);
+                            char *p = alloc_lbuf("db_read");
+                            char *q = p;
+                            safe_str("FIRAN_", p, &q);
+                            safe_str(pName, p, &q);
+                            *q = '\0';
+
+                            pName = MakeCanonicalAttributeName(p, &nName, &bValid);
+                            free_lbuf(p);
+
+                            a = vattr_find_LEN(pName, nName);
+                            if (a)
+                            {
+                                Log.tinyprintf("ERROR: Renamed user attribute, %s, already exists." ENDLINE, pName);
+                            }
+                        }
+                    }
+
+                    if (bValid)
+#endif // FIRANMUX_CONVERT
+                    {
+                        // Maximum attribute number across all names.
+                        //
+                        if (g_max_nam_atr < anum)
+                        {
+                            g_max_nam_atr = anum;
+                        }
+
+                        vattr_define_LEN(pName, nName, anum, aflags);
+                    }
                 }
             }
             else if (ch == 'X')
@@ -599,33 +707,45 @@ dbref db_read(FILE *f, int *db_format, int *db_version, int *db_flags)
                 if (header_gotten)
                 {
                     Log.tinyprintf(ENDLINE "Duplicate MUX version header entry at object %d, ignored." ENDLINE, i);
-                    tstr = getstring_noalloc(f, 0);
+                    tstr = getstring_noalloc(f, false, &nBuffer);
                 }
                 else
                 {
                     header_gotten = true;
                     g_format = F_MUX;
                     g_version = getref(f);
-                    mux_assert((g_version & MANDFLAGS) == MANDFLAGS);
-
-                    // Otherwise extract feature flags
-                    //
-                    if (g_version & V_DATABASE)
-                    {
-                        read_attribs = false;
-                        read_name = !(g_version & V_ATRNAME);
-                    }
-                    read_key = !(g_version & V_ATRKEY);
-                    read_money = !(g_version & V_ATRMONEY);
                     g_flags = g_version & ~V_MASK;
-
                     g_version &= V_MASK;
+
                     if (  g_version < MIN_SUPPORTED_VERSION
                        || MAX_SUPPORTED_VERSION < g_version)
                     {
                         Log.tinyprintf(ENDLINE "Unsupported flatfile version: %d." ENDLINE, g_version);
                         return -1;
                     }
+
+                    mux_assert(  (  (  1 == g_version
+                                    || 2 == g_version)
+                                 && (g_flags & MANDFLAGS_V2) == MANDFLAGS_V2)
+                              || (  3 == g_version
+                                 && (g_flags & MANDFLAGS_V3) == MANDFLAGS_V3));
+
+                    // Otherwise extract feature flags
+                    //
+                    if (g_flags & V_DATABASE)
+                    {
+                        if (3 == g_version)
+                        {
+                            // We'll convert the external database from UTF-8
+                            // to Latin-1 at the end.
+                            //
+                            convert_values = true;
+                        }
+                        read_attribs = false;
+                        read_name = !(g_flags & V_ATRNAME);
+                    }
+                    read_key = !(g_flags & V_ATRKEY);
+                    read_money = !(g_flags & V_ATRMONEY);
                 }
             }
             else if (ch == 'S')
@@ -635,7 +755,7 @@ dbref db_read(FILE *f, int *db_format, int *db_version, int *db_flags)
                 if (size_gotten)
                 {
                     Log.tinyprintf(ENDLINE "Duplicate size entry at object %d, ignored." ENDLINE, i);
-                    tstr = getstring_noalloc(f, 0);
+                    tstr = getstring_noalloc(f, false, &nBuffer);
                 }
                 else
                 {
@@ -650,7 +770,7 @@ dbref db_read(FILE *f, int *db_format, int *db_version, int *db_flags)
                 if (nextattr_gotten)
                 {
                     Log.tinyprintf(ENDLINE "Duplicate next free vattr entry at object %d, ignored." ENDLINE, i);
-                    tstr = getstring_noalloc(f, 0);
+                    tstr = getstring_noalloc(f, false, &nBuffer);
                 }
                 else
                 {
@@ -661,7 +781,7 @@ dbref db_read(FILE *f, int *db_format, int *db_version, int *db_flags)
             else
             {
                 Log.tinyprintf(ENDLINE "Unexpected character '%c' in MUX header near object #%d, ignored." ENDLINE, ch, i);
-                tstr = getstring_noalloc(f, 0);
+                tstr = getstring_noalloc(f, false, &nBuffer);
             }
             break;
 
@@ -671,7 +791,12 @@ dbref db_read(FILE *f, int *db_format, int *db_version, int *db_flags)
 
             if (read_name)
             {
-                tstr = getstring_noalloc(f, true);
+                tstr = getstring_noalloc(f, true, &nBuffer);
+                if (3 == g_version)
+                {
+                    tstr = (char *)convert_color((UTF8 *)tstr);
+                    tstr = ConvertToLatin((UTF8 *)tstr);
+                }
                 buff = alloc_lbuf("dbread.s_Name");
                 (void)ANSI_TruncateToField(tstr, MBUF_SIZE, buff, MBUF_SIZE,
                     &nVisualWidth, ANSI_ENDGOAL_NORMAL);
@@ -730,7 +855,7 @@ dbref db_read(FILE *f, int *db_format, int *db_version, int *db_flags)
             //
             if (read_money)
             {
-                s_Pennies(i, getref(f));
+                s_PenniesDirect(i, getref(f));
             }
 
             // FLAGS
@@ -764,7 +889,7 @@ dbref db_read(FILE *f, int *db_format, int *db_version, int *db_flags)
             break;
 
         case '*':   // EOF marker
-            tstr = getstring_noalloc(f, 0);
+            tstr = getstring_noalloc(f, false, &nBuffer);
             if (strncmp(tstr, "**END OF DUMP***", 16))
             {
                 Log.tinyprintf(ENDLINE "Bad EOF marker at object #%d" ENDLINE, i);
@@ -772,6 +897,87 @@ dbref db_read(FILE *f, int *db_format, int *db_version, int *db_flags)
             }
             else
             {
+                // Attribute number warnings.
+                //
+                if (g_max_nam_atr < g_max_obj_atr)
+                {
+                    Log.tinyprintf(ENDLINE "Warning: One or more attribute values are unnamed. Did you use ./Backup on a running game?");
+                }
+
+                if (!nextattr_gotten)
+                {
+                    Log.tinyprintf(ENDLINE "Warning: Missing +N<next free>. Adjusting.");
+                }
+
+                if (mudstate.attr_next <= g_max_nam_atr)
+                {
+                    if (nextattr_gotten)
+                    {
+                        Log.tinyprintf(ENDLINE "Warning: +N<next free attr> conflicts with existing attribute names. Adjusting.");
+                    }
+                    mudstate.attr_next = g_max_nam_atr + 1;
+                }
+
+                if (mudstate.attr_next <= g_max_obj_atr)
+                {
+                    if (nextattr_gotten)
+                    {
+                        Log.tinyprintf(ENDLINE "Warning: +N<next free attr> conflicts object attribute numbers. Adjusting.");
+                    }
+                    mudstate.attr_next = g_max_nam_atr + 1;
+                }
+
+                int max_atr = A_USER_START;
+                if (max_atr < g_max_nam_atr)
+                {
+                    max_atr = g_max_nam_atr;
+                }
+
+                if (max_atr < g_max_obj_atr)
+                {
+                    max_atr = g_max_obj_atr;
+                }
+
+                if (max_atr + 1 < mudstate.attr_next)
+                {
+                    if (nextattr_gotten)
+                    {
+                        Log.tinyprintf(ENDLINE "Info: +N<next free attr> can be safely adjusted down.");
+                    }
+                    mudstate.attr_next = max_atr + 1;
+                }
+
+                if (convert_values)
+                {
+                    Log.WriteString("Converting external database to Latin-1 " ENDLINE);
+                    Log.Flush();
+
+                    // Convert every attribute on every object in the external database.
+                    //
+                    dbref iObject;
+                    atr_push();
+                    DO_WHOLE_DB(iObject)
+                    {
+                        char *as;
+                        for (int iAttr = atr_head(iObject, &as); iAttr; iAttr = atr_next(&as))
+                        {
+                            if (  0 < iAttr
+                               && iAttr <= anum_alc_top)
+                            {
+                                const UTF8 *pUnicode = (const UTF8 *)atr_get_raw(iObject, iAttr);
+                                if (NULL != pUnicode)
+                                {
+                                    pUnicode = convert_color(pUnicode);
+                                    const char *pLatin = ConvertToLatin(pUnicode);
+                                    size_t nLatin = strlen(pLatin);
+                                    atr_add_raw_LEN(iObject, iAttr, pLatin, nLatin);
+                                }
+                            }
+                        }
+                    }
+                    atr_pop();
+                }
+
                 *db_version = g_version;
                 *db_format = g_format;
                 *db_flags = g_flags;
@@ -869,6 +1075,7 @@ static bool db_write_object(FILE *f, dbref i, int db_format, int flags)
                 }
                 j = a->number;
             }
+
             if (j < A_USER_START)
             {
                 switch (j)
@@ -892,10 +1099,11 @@ static bool db_write_object(FILE *f, dbref i, int db_format, int flags)
                     continue;
                 }
             }
+
             // Format is: ">%d\n", j
             //
             const char *p = atr_get_raw(i, j);
-            int n = mux_ltoa(j, buf+1) + 1;
+            size_t n = mux_ltoa(j, buf+1) + 1;
             buf[n++] = '\n';
             fwrite(buf, sizeof(char), n, f);
             putstring(f, p);
@@ -950,7 +1158,7 @@ dbref db_write(FILE *f, int format, int version)
             *pBuffer++ = '"';
             pBuffer += mux_ltoa(vp->flags, pBuffer);
             *pBuffer++ = ':';
-            int nNameLength = strlen(vp->name);
+            size_t nNameLength = strlen(vp->name);
             memcpy(pBuffer, vp->name, nNameLength);
             pBuffer += nNameLength;
             *pBuffer++ = '"';
@@ -979,7 +1187,7 @@ dbref db_write(FILE *f, int format, int version)
         {
             // Format is: "!%d\n", i
             //
-            int n = mux_ltoa(i, buf+1) + 1;
+            size_t n = mux_ltoa(i, buf+1) + 1;
             buf[n++] = '\n';
             fwrite(buf, sizeof(char), n, f);
             db_write_object(f, i, format, flags);
