@@ -1,10 +1,6 @@
 // eval.cpp -- Command evaluation and cracking.
 //
-// $Id: eval.cpp,v 1.34 2006/09/11 23:57:47 sdennis Exp $
-//
-// MUX 2.4
-// Copyright (C) 1998 through 2004 Solid Vertical Domains, Ltd. All
-// rights not explicitly given are reserved.
+// $Id: eval.cpp 2720 2007-10-28 01:15:01Z brazilofmux $
 //
 #include "copyright.h"
 #include "autoconf.h"
@@ -93,7 +89,7 @@ static char *parse_to_cleanup( int eval, int first, char *cstr, char *rstr,
 // A code 4 or above means that the client-specified delim cannot override it.
 // A code 8 is temporary.
 //
-static char isSpecial_L3[256] =
+static int isSpecial_L3[256] =
 {
     7, 0, 0, 0, 0, 0, 0, 0,  0, 0, 0, 0, 0, 0, 0, 0, // 0x00-0x0F
     0, 0, 0, 0, 0, 0, 0, 0,  0, 0, 0, 4, 0, 0, 0, 0, // 0x10-0x1F
@@ -466,7 +462,7 @@ TryAgain:
 // the characters to another buffer anyway and is more than able to perform the
 // escapes and trimming.
 //
-static char *parse_to_lite(char **dstr, char delim1, char delim2, int *nLen, int *iWhichDelim)
+static char *parse_to_lite(char **dstr, char delim1, char delim2, size_t *nLen, int *iWhichDelim)
 {
 #define stacklim 32
     char stack[stacklim];
@@ -724,7 +720,7 @@ TryAgain:
 // modified.
 //
 char *parse_arglist( dbref executor, dbref caller, dbref enactor, char *dstr,
-                     char delim, dbref eval, char *fargs[], dbref nfargs,
+                     char delim, int eval, char *fargs[], dbref nfargs,
                      char *cargs[], dbref ncargs, int *nArgsParsed )
 {
     char *rstr, *tstr, *bp, *str;
@@ -736,7 +732,7 @@ char *parse_arglist( dbref executor, dbref caller, dbref enactor, char *dstr,
         return NULL;
     }
 
-    int nLen;
+    size_t nLen;
     int iWhichDelim;
     rstr = parse_to_lite(&dstr, delim, '\0', &nLen, &iWhichDelim);
     arg = 0;
@@ -765,7 +761,7 @@ char *parse_arglist( dbref executor, dbref caller, dbref enactor, char *dstr,
         }
         else
         {
-            strcpy(fargs[arg], tstr);
+            mux_strncpy(fargs[arg], tstr, LBUF_SIZE-1);
         }
         arg++;
     }
@@ -788,7 +784,7 @@ static char *parse_arglist_lite( dbref executor, dbref caller, dbref enactor,
         return NULL;
     }
 
-    int nLen;
+    size_t nLen;
     int peval = eval;
     if (eval & EV_EVAL)
     {
@@ -899,7 +895,7 @@ static void tcache_add(dbref player, char *orig, char *result)
             TCENT *xp = (TCENT *) alloc_sbuf("tcache_add.sbuf");
             char *tp = alloc_lbuf("tcache_add.lbuf");
 
-            int nvw;
+            size_t nvw;
             ANSI_TruncateToField(result, LBUF_SIZE, tp, LBUF_SIZE,
                 &nvw, ANSI_ENDGOAL_NORMAL);
             xp->result = tp;
@@ -993,7 +989,7 @@ static const unsigned char isSpecial_L2[256] =
      18,  0,  0,  0,  0,  0,  0,  0,   0,  0,  0,  0,  0,  0,  0,  0, // 0x00-0x0F
       0,  0,  0,  0,  0,  0,  0,  0,   0,  0,  0,  0,  0,  0,  0,  0, // 0x10-0x1F
       0,  4,  0,  3,  0, 11,  0,  0,   0,  0,  0,  0,  0,  0,  0,  0, // 0x20-0x2F
-      1,  1,  1,  1,  1,  1,  1,  1,   1,  1,  0,  0,  0,  0,  0,  0, // 0x30-0x3F
+      1,  1,  1,  1,  1,  1,  1,  1,   1,  1,  0,  0,  0, 21,  0,  0, // 0x30-0x3F
      20,145,  7,  6,  0,  0,  0,  0,   0,  0,  0,  0,  9,147,140,144, // 0x40-0x4F
     143,130,  5,142,  8,  0,138,  0,   6,  0,  0,  0,  0,  0,  0,  0, // 0x50-0x5F
      20, 17,  7,  6,  0,  0,  0,  0,   0,  0,  0,  0,  9, 19, 12, 16, // 0x60-0x6F
@@ -1046,42 +1042,42 @@ void PopPointers(char **p, int nNeeded)
     pPtrsFrame->nptrs += nNeeded;
 }
 
-#define INTS_PER_FRAME ((LBUF_SIZE - sizeof(char *) - sizeof(int))/sizeof(int))
-typedef struct tag_intsframe
+#define REFS_PER_FRAME ((LBUF_SIZE - sizeof(void *) - sizeof(int))/sizeof(void *))
+typedef struct tag_refsframe
 {
-    int   nints;
-    int   ints[INTS_PER_FRAME];
-    struct tag_intsframe *next;
-} IntsFrame;
+    int      nrefs;
+    reg_ref *refs[REFS_PER_FRAME];
+    struct tag_refsframe *next;
+} RefsFrame;
 
-static IntsFrame *pIntsFrame = NULL;
+static RefsFrame *pRefsFrame = NULL;
 
-int *PushIntegers(int nNeeded)
+reg_ref **PushRegisters(int nNeeded)
 {
-    if (  !pIntsFrame
-       || pIntsFrame->nints < nNeeded)
+    if (  !pRefsFrame
+       || pRefsFrame->nrefs < nNeeded)
     {
-        IntsFrame *p = (IntsFrame *)alloc_lbuf("PushIntegers");
-        p->next = pIntsFrame;
-        p->nints = INTS_PER_FRAME;
-        pIntsFrame = p;
+        RefsFrame *p = (RefsFrame *)alloc_lbuf("PushRegisters");
+        p->next = pRefsFrame;
+        p->nrefs = REFS_PER_FRAME;
+        pRefsFrame = p;
     }
-    pIntsFrame->nints -= nNeeded;
-    return pIntsFrame->ints + pIntsFrame->nints;
+    pRefsFrame->nrefs -= nNeeded;
+    return pRefsFrame->refs + pRefsFrame->nrefs;
 }
 
-void PopIntegers(int *pi, int nNeeded)
+void PopRegisters(reg_ref **p, int nNeeded)
 {
-    UNUSED_PARAMETER(pi);
+    UNUSED_PARAMETER(p);
 
-    if (pIntsFrame->nints == INTS_PER_FRAME)
+    if (pRefsFrame->nrefs == REFS_PER_FRAME)
     {
-        IntsFrame *p = pIntsFrame->next;
-        free_lbuf((char *)pIntsFrame);
-        pIntsFrame = p;
+        RefsFrame *q = pRefsFrame->next;
+        free_lbuf((char *)pRefsFrame);
+        pRefsFrame = q;
     }
-    //mux_assert(pi == pIntsFrame->ints + pIntsFrame->nints);
-    pIntsFrame->nints += nNeeded;
+    //mux_assert(p == pRefsFrame->refs + pRefsFrame->nrefs);
+    pRefsFrame->nrefs += nNeeded;
 }
 
 void mux_exec( char *buff, char **bufc, dbref executor, dbref caller,
@@ -1105,10 +1101,11 @@ void mux_exec( char *buff, char **bufc, dbref executor, dbref caller,
     char *TempPtr;
     char *tstr, *tbuf, *start, *oldp, *savestr;
     const char *constbuf;
-    int ch;
+    char ch;
     char *realbuff = NULL, *realbp = NULL;
     dbref aowner;
     int nfargs, aflags, feval, i;
+    size_t n;
     bool ansi = false;
     FUN *fp;
     UFUN *ufp;
@@ -1129,7 +1126,7 @@ void mux_exec( char *buff, char **bufc, dbref executor, dbref caller,
     int at_space = 1;
     int gender = -1;
 
-    bool is_trace = Trace(executor) && !(eval & EV_NOTRACE);
+    bool is_trace = (Trace(executor) || (eval & EV_TRACE)) && !(eval & EV_NOTRACE);
     bool is_top = false;
 
     // Extend the buffer if we need to.
@@ -1152,7 +1149,7 @@ void mux_exec( char *buff, char **bufc, dbref executor, dbref caller,
     {
         is_top = tcache_empty();
         savestr = alloc_lbuf("exec.save");
-        strcpy(savestr, pdstr);
+        mux_strncpy(savestr, pdstr, LBUF_SIZE-1);
     }
 
     // Save Parser Mode.
@@ -1181,7 +1178,7 @@ void mux_exec( char *buff, char **bufc, dbref executor, dbref caller,
             {
                 ; // Nothing.
             }
-            size_t n = p - pdstr - 1;
+            n = p - pdstr - 1;
             if (nBufferAvailable < n)
             {
                 n = nBufferAvailable;
@@ -1243,7 +1240,7 @@ void mux_exec( char *buff, char **bufc, dbref executor, dbref caller,
             }
             *p2 = '\0';
 
-            int ntbuf = p2 - mux_scratch;
+            size_t ntbuf = p2 - mux_scratch;
             fp = (FUN *)hashfindLEN(mux_scratch, ntbuf, &mudstate.func_htab);
 
             // If not a builtin func, check for global func.
@@ -1356,26 +1353,22 @@ void mux_exec( char *buff, char **bufc, dbref executor, dbref caller,
                         }
                         TempPtr = tstr;
 
-                        char **preserve = NULL;
-                        int *preserve_len = NULL;
+                        reg_ref **preserve = NULL;
 
                         if (ufp->flags & FN_PRES)
                         {
-                            preserve = PushPointers(MAX_GLOBAL_REGS);
-                            preserve_len = PushIntegers(MAX_GLOBAL_REGS);
-                            save_global_regs("eval_save", preserve, preserve_len);
+                            preserve = PushRegisters(MAX_GLOBAL_REGS);
+                            save_global_regs(preserve);
                         }
 
-                        mux_exec(buff, &oldp, i, executor, enactor, feval,
-                                 &TempPtr, fargs, nfargs);
+                        mux_exec(buff, &oldp, i, executor, enactor,
+                            AttrTrace(aflags, feval), &TempPtr, fargs, nfargs);
 
                         if (ufp->flags & FN_PRES)
                         {
-                            restore_global_regs("eval_restore", preserve, preserve_len);
-                            PopIntegers(preserve_len, MAX_GLOBAL_REGS);
-                            PopPointers(preserve, MAX_GLOBAL_REGS);
+                            restore_global_regs(preserve);
+                            PopRegisters(preserve, MAX_GLOBAL_REGS);
                             preserve = NULL;
-                            preserve_len = NULL;
                         }
                         free_lbuf(tstr);
                     }
@@ -1389,29 +1382,29 @@ void mux_exec( char *buff, char **bufc, dbref executor, dbref caller,
                            && !MuxAlarm.bAlarmed)
                         {
                             fp->fun(buff, &oldp, executor, caller, enactor,
-                                    fargs, nfargs, cargs, ncargs);
+                                    feval & EV_TRACE, fargs, nfargs, cargs, ncargs);
                         }
                         else
                         {
                             if (fp->minArgs == fp->maxArgs)
                             {
-                                sprintf(mux_scratch,
+                                mux_sprintf(mux_scratch, sizeof(mux_scratch),
                                     "#-1 FUNCTION (%s) EXPECTS %d ARGUMENTS",
                                     fp->name, fp->minArgs);
                             }
                             else if (fp->minArgs + 1 == fp->maxArgs)
                             {
-                                sprintf(mux_scratch,
+                                mux_sprintf(mux_scratch, sizeof(mux_scratch),
                                     "#-1 FUNCTION (%s) EXPECTS %d OR %d ARGUMENTS",
                                     fp->name, fp->minArgs, fp->maxArgs);
                             }
                             else if (MuxAlarm.bAlarmed)
                             {
-                                sprintf(mux_scratch, "#-1 CPU LIMITED");
+                                mux_sprintf(mux_scratch, sizeof(mux_scratch), "#-1 CPU LIMITED");
                             }
                             else
                             {
-                                sprintf(mux_scratch,
+                                mux_sprintf(mux_scratch, sizeof(mux_scratch),
                                     "#-1 FUNCTION (%s) EXPECTS BETWEEN %d AND %d ARGUMENTS",
                                     fp->name, fp->minArgs, fp->maxArgs);
                             }
@@ -1487,11 +1480,11 @@ void mux_exec( char *buff, char **bufc, dbref executor, dbref caller,
                     if (  0 <= i
                        && i < MAX_GLOBAL_REGS)
                     {
-                        if (  mudstate.glob_reg_len[i] > 0
-                           && mudstate.global_regs[i])
+                        if (  mudstate.global_regs[i]
+                           && mudstate.global_regs[i]->reg_len > 0)
                         {
-                            safe_copy_buf(mudstate.global_regs[i],
-                                mudstate.glob_reg_len[i], buff, bufc);
+                            safe_copy_buf(mudstate.global_regs[i]->reg_ptr,
+                                mudstate.global_regs[i]->reg_len, buff, bufc);
                             nBufferAvailable = LBUF_SIZE - (*bufc - buff) - 1;
                         }
                     }
@@ -1510,8 +1503,8 @@ void mux_exec( char *buff, char **bufc, dbref executor, dbref caller,
                         // Enactor DB number.
                         //
                         mux_scratch[0] = '#';
-                        i = mux_ltoa(enactor, mux_scratch+1);
-                        safe_copy_buf(mux_scratch, i+1, buff, bufc);
+                        n = mux_ltoa(enactor, mux_scratch+1);
+                        safe_copy_buf(mux_scratch, n+1, buff, bufc);
                         nBufferAvailable = LBUF_SIZE - (*bufc - buff) - 1;
                     }
                     else if (iCode == 4)
@@ -1523,8 +1516,8 @@ void mux_exec( char *buff, char **bufc, dbref executor, dbref caller,
                         // Executor DB number.
                         //
                         mux_scratch[0] = '#';
-                        i = mux_ltoa(executor, mux_scratch+1);
-                        safe_copy_buf(mux_scratch, i+1, buff, bufc);
+                        n = mux_ltoa(executor, mux_scratch+1);
+                        safe_copy_buf(mux_scratch, n+1, buff, bufc);
                         nBufferAvailable = LBUF_SIZE - (*bufc - buff) - 1;
                     }
                     else
@@ -1615,8 +1608,8 @@ void mux_exec( char *buff, char **bufc, dbref executor, dbref caller,
                         if (!(eval & EV_NO_LOCATION))
                         {
                             mux_scratch[0] = '#';
-                            i = mux_ltoa(where_is(enactor), mux_scratch+1);
-                            safe_copy_buf(mux_scratch, i+1, buff, bufc);
+                            n = mux_ltoa(where_is(enactor), mux_scratch+1);
+                            safe_copy_buf(mux_scratch, n+1, buff, bufc);
                             nBufferAvailable = LBUF_SIZE - (*bufc - buff) - 1;
                         }
                     }
@@ -1645,7 +1638,7 @@ void mux_exec( char *buff, char **bufc, dbref executor, dbref caller,
                                 nBufferAvailable -= nAttrGotten;
                             }
                         }
-                        else if (*pdstr == '\0')
+                        else if ('\0' == *pdstr)
                         {
                             pdstr--;
                         }
@@ -1743,7 +1736,7 @@ void mux_exec( char *buff, char **bufc, dbref executor, dbref caller,
                             }
                             else
                             {
-                                safe_str((char *)poss[gender], buff, bufc);
+                                safe_str(poss[gender], buff, bufc);
                                 nBufferAvailable = LBUF_SIZE - (*bufc - buff) - 1;
                             }
                         }
@@ -1770,7 +1763,7 @@ void mux_exec( char *buff, char **bufc, dbref executor, dbref caller,
                             nBufferAvailable = LBUF_SIZE - (*bufc - buff) - 1;
                         }
                     }
-                    else
+                    else if (iCode <= 18)
                     {
                         if (iCode == 17)
                         {
@@ -1801,7 +1794,7 @@ void mux_exec( char *buff, char **bufc, dbref executor, dbref caller,
                                 nBufferAvailable = LBUF_SIZE - (*bufc - buff) - 1;
                             }
                         }
-                        else if (iCode == 18)
+                        else // if (iCode == 18)
                         {
                             // 00
                             // \0
@@ -1810,7 +1803,10 @@ void mux_exec( char *buff, char **bufc, dbref executor, dbref caller,
                             //
                             pdstr--;
                         }
-                        else if (iCode == 19)
+                    }
+                    else if (iCode <= 20)
+                    {
+                        if (iCode == 19)
                         {
                             // 4D
                             // M
@@ -1825,15 +1821,62 @@ void mux_exec( char *buff, char **bufc, dbref executor, dbref caller,
                             // 40
                             // @
                             //
-                            // iCode == '@'
                             // Caller DB number.
                             //
                             mux_scratch[0] = '#';
-                            i = mux_ltoa(caller, mux_scratch+1);
-                            safe_copy_buf(mux_scratch, i+1, buff, bufc);
+                            n = mux_ltoa(caller, mux_scratch+1);
+                            safe_copy_buf(mux_scratch, n+1, buff, bufc);
                             nBufferAvailable = LBUF_SIZE - (*bufc - buff) - 1;
                         }
                     }
+                    else // if (iCode == 21)
+                    {
+                        // 3D
+                        // =
+                        //
+                        // %=<attr> like v(attr).
+                        //
+                        pdstr++;
+                        if ('<' == pdstr[0])
+                        {
+                            pdstr++;
+
+                            char *p2 = mux_scratch;
+                            while (  pdstr[0]
+                                  && '>' != pdstr[0])
+                            {
+                                safe_chr(pdstr[0], mux_scratch, &p2);
+                                pdstr++;
+                            }
+                            *p2 = '\0';
+                            if ('>' == pdstr[0])
+                            {
+                                if (mux_AttrNameInitialSet(mux_scratch[0]))
+                                {
+                                    ATTR *ap = atr_str(mux_scratch);
+                                    if (ap)
+                                    {
+                                        size_t nLen;
+                                        tbuf = atr_pget_LEN(executor, ap->number, &aowner, &aflags, &nLen);
+                                        if (See_attr(executor, executor, ap))
+                                        {
+                                            safe_copy_buf(tbuf, nLen, buff, bufc);
+                                            nBufferAvailable = LBUF_SIZE - (*bufc - buff) - 1;
+                                        }
+                                        free_lbuf(tbuf);
+                                    }
+                                }
+                            }
+                            else
+                            {
+                                pdstr--;
+                            }
+                        }
+                        else if ('\0' == pdstr[0])
+                        {
+                            pdstr--;
+                        }
+                    }
                 }
 
                 // For some escape letters, if the escape letter
@@ -1854,7 +1897,7 @@ void mux_exec( char *buff, char **bufc, dbref executor, dbref caller,
             //
             tstr = pdstr++;
             mudstate.nStackNest++;
-            tbuf = parse_to_lite(&pdstr, ']', '\0', &at_space, &at_space);
+            tbuf = parse_to_lite(&pdstr, ']', '\0', &n, &at_space);
             at_space = 0;
             if (pdstr == NULL)
             {
@@ -1907,7 +1950,7 @@ void mux_exec( char *buff, char **bufc, dbref executor, dbref caller,
             //
             tstr = pdstr++;
             mudstate.nStackNest++;
-            tbuf = parse_to_lite(&pdstr, '}', '\0', &at_space, &at_space);
+            tbuf = parse_to_lite(&pdstr, '}', '\0', &n, &at_space);
             at_space = 0;
             if (pdstr == NULL)
             {
@@ -2044,7 +2087,7 @@ void mux_exec( char *buff, char **bufc, dbref executor, dbref caller,
            && 0 < tcache_count - mudconf.trace_limit)
         {
             tbuf = alloc_mbuf("exec.trace_diag");
-            sprintf(tbuf, "%d lines of trace output discarded.", tcache_count
+            mux_sprintf(tbuf, MBUF_SIZE, "%d lines of trace output discarded.", tcache_count
                 - mudconf.trace_limit);
             notify(executor, tbuf);
             free_mbuf(tbuf);
@@ -2076,8 +2119,8 @@ void mux_exec( char *buff, char **bufc, dbref executor, dbref caller,
             buff = realbuff;
         }
 
-        int nVisualWidth;
-        int nLen = ANSI_String_Finalize(&aoc, &nVisualWidth);
+        size_t nVisualWidth;
+        size_t nLen = ANSI_String_Finalize(&aoc, &nVisualWidth);
         memcpy(buff, mux_scratch, nLen+1);
         *bufc = buff + nLen;
     }
@@ -2098,81 +2141,114 @@ void mux_exec( char *buff, char **bufc, dbref executor, dbref caller,
 
 void save_global_regs
 (
-    const char *funcname,
-    char *preserve[],
-    int preserve_len[]
+    reg_ref *preserve[]
 )
 {
-    int i;
-
-    for (i = 0; i < MAX_GLOBAL_REGS; i++)
+    for (int i = 0; i < MAX_GLOBAL_REGS; i++)
     {
         if (mudstate.global_regs[i])
         {
-            preserve[i] = alloc_lbuf(funcname);
-            int n = mudstate.glob_reg_len[i];
-            memcpy(preserve[i], mudstate.global_regs[i], n);
-            preserve[i][n] = '\0';
-            preserve_len[i] = n;
-        }
-        else
-        {
-            preserve[i] = NULL;
-            preserve_len[i] = 0;
+            RegAddRef(mudstate.global_regs[i]);
         }
+        preserve[i] = mudstate.global_regs[i];
     }
 }
 
 void save_and_clear_global_regs
 (
-    const char *funcname,
-    char *preserve[],
-    int preserve_len[]
+    reg_ref *preserve[]
 )
 {
-    UNUSED_PARAMETER(funcname);
-
-    int i;
-
-    for (i = 0; i < MAX_GLOBAL_REGS; i++)
+    for (int i = 0; i < MAX_GLOBAL_REGS; i++)
     {
         preserve[i] = mudstate.global_regs[i];
-        preserve_len[i] = mudstate.glob_reg_len[i];
-
         mudstate.global_regs[i] = NULL;
-        mudstate.glob_reg_len[i] = 0;
     }
 }
 
 void restore_global_regs
 (
-    const char *funcname,
-    char *preserve[],
-    int preserve_len[]
+    reg_ref *preserve[]
 )
 {
-    UNUSED_PARAMETER(funcname);
-
-    int i;
-
-    for (i = 0; i < MAX_GLOBAL_REGS; i++)
+    for (int i = 0; i < MAX_GLOBAL_REGS; i++)
     {
+        if (mudstate.global_regs[i])
+        {
+            RegRelease(mudstate.global_regs[i]);
+            mudstate.global_regs[i] = NULL;
+        }
+
         if (preserve[i])
         {
-            if (mudstate.global_regs[i])
-            {
-                free_lbuf(mudstate.global_regs[i]);
-            }
             mudstate.global_regs[i] = preserve[i];
-            mudstate.glob_reg_len[i] = preserve_len[i];
-        }
-        else
-        {
-            if (mudstate.global_regs[i])
-            {
-                mudstate.global_regs[i][0] = '\0';
-            }
-            mudstate.glob_reg_len[i] = 0;
+            preserve[i] = NULL;
         }
     }
 }
+
+static lbuf_ref *last_lbufref = NULL;
+static size_t    last_left    = 0;
+static char     *last_ptr     = NULL;
+
+void RegAssign(reg_ref **regref, size_t nLength, const char *ptr)
+{
+    if (  NULL == regref
+       || NULL == ptr
+       || LBUF_SIZE <= nLength)
+    {
+        return;
+    }
+
+    // Put any previous register value out of the way.
+    //
+    if (NULL != *regref)
+    {
+        RegRelease(*regref);
+        *regref = NULL;
+    }
+
+    // Let go of the last lbuf if we can't use it.
+    //
+    size_t nSize = nLength + 1;
+    if (  NULL != last_lbufref
+       && last_left < nSize)
+    {
+        BufRelease(last_lbufref);
+        last_lbufref = NULL;
+        last_left    = 0;
+        last_ptr     = NULL;
+    }
+
+    // Grab a new, fresh lbuf if we don't have one.
+    //
+    if (NULL == last_lbufref)
+    {
+        last_ptr = alloc_lbuf("RegAssign");
+        last_left = LBUF_SIZE;
+
+        // Fill in new lbufref.
+        //
+        last_lbufref = alloc_lbufref("RegAssign");
+        last_lbufref->refcount = 1;
+        last_lbufref->lbuf_ptr = last_ptr;
+    }
+
+    // Use last lbuf.
+    //
+    char *p = last_ptr;
+    memcpy(last_ptr, ptr, nSize);
+    last_ptr[nLength] = '\0';
+    last_ptr  += nSize;
+    last_left -= nSize;
+
+    // Fill in new regref.
+    //
+    *regref = alloc_regref("RegAssign");
+    (*regref)->refcount = 1;
+    (*regref)->lbuf     = last_lbufref;
+    (*regref)->reg_len  = nLength;
+    (*regref)->reg_ptr  = p;
+
+    BufAddRef(last_lbufref);
+}
