@@ -1,6 +1,6 @@
 // externs.h -- Prototypes for externs not defined elsewhere.
 //
-// $Id: externs.h,v 1.65 2006/01/31 00:17:26 sdennis Exp $
+// $Id: externs.h 3049 2007-12-30 06:35:41Z brazilofmux $
 //
 
 #ifndef EXTERNS_H
@@ -14,7 +14,7 @@
 // From bsd.cpp.
 //
 void boot_slave(dbref executor, dbref caller, dbref enactor, int key);
-void close_sockets(bool emergency, char *message);
+void close_sockets(bool emergency, const char *message);
 void CleanUpSlaveSocket(void);
 void CleanUpSlaveProcess(void);
 #ifdef QUERY_SLAVE
@@ -34,7 +34,7 @@ extern NAMETAB sigactions_nametab[];
 
 // From conf.cpp
 //
-void cf_log_notfound(dbref, char *, const char *, char *);
+void cf_log_notfound(dbref, const char *, const char *, const char *);
 int  cf_modify_bits(int *, char *, void *, UINT32, dbref, char *);
 void DCL_CDECL cf_log_syntax(dbref player, char *cmd, const char *fmt, ...);
 void ValidateConfigurationDbrefs(void);
@@ -68,32 +68,51 @@ void local_disconnect(dbref, int);
 void local_data_create(dbref);
 void local_data_clone(dbref, dbref);
 void local_data_free(dbref);
+const char **local_get_info_table(void);
 
 // From mail.cpp.
 //
 void load_mail(FILE *);
 int  dump_mail(FILE *);
 struct mail *mail_fetch(dbref, int);
+char *MakeCanonicalMailAlias
+(
+    char   *pMailAlias,
+    size_t *pnValidMailAlias,
+    bool   *pbValidMailAlias
+);
+
+char *MakeCanonicalMailAliasDesc
+(
+    char   *pMailAliasDesc,
+    size_t *pnValidMailAliasDesc,
+    bool   *pbValidMailAliasDesc,
+    size_t *pnVisualWidth
+);
+
+#if defined(FIRANMUX)
+const char *MessageFetch(int number);
+size_t MessageFetchSize(int number);
+#endif // FIRANMUX
 
 // From netcommon.cpp.
 //
-void DCL_CDECL raw_broadcast(int, char *, ...);
+void DCL_CDECL raw_broadcast(int, const char *, ...);
 void list_siteinfo(dbref);
 void logged_out0(dbref executor, dbref caller, dbref enactor, int key);
-void logged_out1(dbref executor, dbref caller, dbref enactor, int key, char *arg);
+void logged_out1(dbref executor, dbref caller, dbref enactor, int eval, int key, char *arg);
 void init_logout_cmdtab(void);
 void desc_reload(dbref);
 void make_portlist(dbref, dbref, char *, char **);
+char *MakeCanonicalDoing(char *pDoing, size_t *pnValidDoing, bool *pbValidDoing);
 
 /* From cque.cpp */
 int  nfy_que(dbref, int, int, int);
 int  halt_que(dbref, dbref);
-void wait_que(dbref executor, dbref caller, dbref enactor, bool,
-    CLinearTimeAbsolute&, dbref, int, char *, char *[],int, char *[]);
+void wait_que(dbref executor, dbref caller, dbref enactor, int, bool,
+    CLinearTimeAbsolute&, dbref, int, char *, int, char *[], reg_ref *[]);
 
-#ifdef WIN32
-#include "crypt/crypt.h"
-#else // WIN32
+#ifndef WIN32
 extern "C" char *crypt(const char *inptr, const char *inkey);
 #endif // WIN32
 extern bool break_called;
@@ -107,16 +126,71 @@ int get_gender(dbref);
 void mux_exec(char *buff, char **bufc, dbref executor, dbref caller,
               dbref enactor, int eval, char **dstr, char *cargs[],
               int ncargs);
-void save_global_regs(const char *, char *[], int []);
-void save_and_clear_global_regs(const char *, char *[], int[]);
-void restore_global_regs(const char *, char *[], int []);
+
+DCL_INLINE void BufAddRef(lbuf_ref *lbufref)
+{
+    if (NULL != lbufref)
+    {
+        lbufref->refcount++;
+    }
+}
+
+DCL_INLINE void BufRelease(lbuf_ref *lbufref)
+{
+    if (NULL != lbufref)
+    {
+        lbufref->refcount--;
+        if (0 == lbufref->refcount)
+        {
+            free_lbuf(lbufref->lbuf_ptr);
+            lbufref->lbuf_ptr = NULL;
+            free_lbufref(lbufref);
+        }
+    }
+}
+
+DCL_INLINE void RegAddRef(reg_ref *regref)
+{
+    if (NULL != regref)
+    {
+        regref->refcount++;
+    }
+}
+
+DCL_INLINE void RegRelease(reg_ref *regref)
+{
+    if (NULL != regref)
+    {
+        regref->refcount--;
+        if (0 == regref->refcount)
+        {
+            BufRelease(regref->lbuf);
+            regref->lbuf    = NULL;
+            regref->reg_ptr = NULL;
+            regref->reg_len = 0;
+            free_regref(regref);
+        }
+    }
+}
+void RegAssign(reg_ref **regref, size_t nLength, const char *ptr);
+
+void save_global_regs(reg_ref *preserve[]);
+void save_and_clear_global_regs(reg_ref *preserve[]);
+void restore_global_regs(reg_ref *preserve[]);
+
 char **PushPointers(int nNeeded);
 void PopPointers(char **p, int nNeeded);
-int *PushIntegers(int nNeeded);
-void PopIntegers(int *pi, int nNeeded);
+reg_ref **PushRegisters(int nNeeded);
+void PopRegisters(reg_ref **p, int nNeeded);
+
 extern const signed char mux_RegisterSet[256];
 extern const char *ColorTable[256];
 
+#if defined(FIRANMUX)
+char *linewrap_desc(char *);
+char *linewrap_general(char *, int, const char *, const char *);
+#endif // FIRANMUX
+
 /* From game.cpp */
 #define notify(p,m)                         notify_check(p,p,m, MSG_PUP_ALWAYS|MSG_ME_ALL|MSG_F_DOWN)
 #define notify_saypose(p,m)                 notify_check(p,p,m, MSG_PUP_ALWAYS|MSG_ME_ALL|MSG_F_DOWN|MSG_SAYPOSE)
@@ -181,6 +255,11 @@ bool html_escape(const char *src, char *dest, char **destp);
 void dump_database_internal(int);
 void fork_and_dump(int key);
 
+#define MUX_OPEN_INVALID_HANDLE_VALUE (-1)
+bool mux_fopen(FILE **pFile, const char *filename, const char *mode);
+bool mux_open(int *pfh, const char *filename, int oflag);
+const char *mux_strerror(int errnum);
+
 /* From help.cpp */
 void helpindex_clean(int);
 void helpindex_load(dbref);
@@ -210,7 +289,9 @@ void log_type_and_name(dbref);
 /* From look.cpp */
 void look_in(dbref,dbref, int);
 void show_vrml_url(dbref, dbref);
-size_t decode_attr_flags(int aflags, char *buff);
+#define NUM_ATTRIBUTE_CODES 12
+size_t decode_attr_flags(int aflags, char buff[NUM_ATTRIBUTE_CODES+1]);
+void   decode_attr_flag_names(int aflags, char *buf, char **bufc);
 
 /* From move.cpp */
 void move_object(dbref, dbref);
@@ -231,7 +312,7 @@ void  destroy_obj(dbref);
 void  empty_obj(dbref);
 
 /* From player.cpp */
-dbref create_player(char *name, char *pass, dbref executor, bool isrobot, const char **pmsg);
+dbref create_player(const char *name, const char *pass, dbref executor, bool isrobot, const char **pmsg);
 void AddToPublicChannel(dbref player);
 bool add_player_name(dbref, const char *);
 bool delete_player_name(dbref, const char *);
@@ -261,8 +342,8 @@ void add_quota(dbref, int);
 bool canpayfees(dbref, dbref, int, int);
 void giveto(dbref,int);
 bool payfor(dbref,int);
-char *MakeCanonicalObjectName(const char *pName, int *pnName, bool *pbValid);
-char *MakeCanonicalExitName(const char *pName, int *pnName, bool *pbValid);
+char *MakeCanonicalObjectName(const char *pName, size_t *pnName, bool *pbValid);
+char *MakeCanonicalExitName(const char *pName, size_t *pnName, bool *pbValid);
 bool ValidatePlayerName(const char *pName);
 bool ok_password(const char *szPassword, const char **pmsg);
 void handle_ears(dbref, bool, bool);
@@ -276,7 +357,9 @@ bool locatable(dbref, dbref, dbref);
 bool nearby(dbref, dbref);
 bool exit_visible(dbref, dbref, int);
 bool exit_displayable(dbref, dbref, int);
-void did_it(dbref, dbref, int, const char *, int, const char *, int, char *[], int);
+void did_it(dbref player, dbref thing, int what, const char *def, int owhat,
+            const char *odef, int awhat, int ctrl_flags,
+            char *args[], int nargs);
 bool bCanReadAttr(dbref executor, dbref target, ATTR *tattr, bool bParentCheck);
 bool bCanSetAttr(dbref executor, dbref target, ATTR *tattr);
 bool bCanLockAttr(dbref executor, dbref target, ATTR *tattr);
@@ -289,6 +372,21 @@ dbref match_controlled_handler(dbref player, const char *name, bool bQuiet);
 #define match_controlled(player,name)       match_controlled_handler(player, name, false)
 #define match_controlled_quiet(player,name) match_controlled_handler(player, name, true)
 void set_modified(dbref thing);
+void TranslateFlags_Clone
+(
+    FLAG     aClearFlags[3],
+    dbref    executor,
+    int      key
+);
+void TranslateFlags_Chown
+(
+    FLAG     aClearFlags[3],
+    FLAG     aSetFlags[3],
+    bool    *bClearPowers,
+    dbref    executor,
+    int      key
+);
+void SetClearFlags(dbref thing, FLAG aClearFlags[3], FLAG aSetFlags[3]);
 
 /* From boolexp.cpp */
 bool eval_boolexp(dbref, dbref, dbref, BOOLEXP *);
@@ -327,8 +425,10 @@ bool quick_wild(const char *, const char *);
 /* From command.cpp */
 bool check_access(dbref player, int mask);
 void set_prefix_cmds(void);
-char *process_command(dbref executor, dbref caller, dbref enactor, bool,
+char *process_command(dbref executor, dbref caller, dbref enactor, int, bool,
     char *, char *[], int);
+size_t LeftJustifyString(char *field, size_t nWidth, const char *value);
+size_t RightJustifyNumber(char *field, size_t nWidth, INT64 value, char chFill);
 
 #define Protect(f) (cmdp->perms & f)
 
@@ -345,7 +445,7 @@ void s_Moniker(dbref thing, const char *s);
 const char *Name(dbref thing);
 const char *PureName(dbref thing);
 const char *Moniker(dbref thing);
-int  fwdlist_load(FWDLIST *, dbref, char *);
+FWDLIST *fwdlist_load(dbref player, char *atext);
 void fwdlist_set(dbref, FWDLIST *);
 void fwdlist_clr(dbref);
 int  fwdlist_rewrite(FWDLIST *, char *);
@@ -358,7 +458,7 @@ int  init_dbfile(char *game_dir_file, char *game_pag_file, int nCachePages);
 void atr_cpy(dbref dest, dbref source, bool bInternal);
 void atr_chown(dbref);
 void atr_clr(dbref, int);
-void atr_add_raw_LEN(dbref, int, const char *, int);
+void atr_add_raw_LEN(dbref, int, const char *, size_t n);
 void atr_add_raw(dbref, int, const char *);
 void atr_add(dbref, int, char *, dbref, int);
 void atr_set_flags(dbref, int, int);
@@ -395,7 +495,7 @@ extern int anum_alc_top;
 #define CEMIT_NOHEADER  1   /* Channel emit without header */
 #define CHOWN_ONE       1   /* item = new_owner */
 #define CHOWN_ALL       2   /* old_owner = new_owner */
-//#define CHOWN_NOSTRIP   4   /* Don't strip (most) flags from object */
+#define CHOWN_NOSTRIP   4   /* Don't strip (most) flags from object */
 #define CHOWN_NOZONE    8   /* Strip zones from objects */
 #define CLIST_FULL      1   /* Full listing of channels */
 #define CLIST_HEADERS   2   /* Lists channel headers, like "[Public]" */
@@ -407,6 +507,7 @@ extern int anum_alc_top;
 #define CLONE_SET_LOC   16  /* ARG2 is location of cloned object */
 #define CLONE_SET_NAME  32  /* ARG2 is alternate name of cloned object */
 #define CLONE_FROM_PARENT 64 /* Set parent on obj instd of cloning attrs */
+#define CLONE_NOSTRIP   128 // Don't strip (most) flags from object.
 #define CBOOT_QUIET     1   // Cboot without a message
 #define COMTITLE_ON     1   // Turn Comtitles on.
 #define COMTITLE_OFF    2   // Turn Comtitles off.
@@ -614,6 +715,8 @@ extern int anum_alc_top;
 #define SWITCH_DEFAULT  0   /* Use the configured default for switch */
 #define SWITCH_ANY      1   /* Execute all cases that match */
 #define SWITCH_ONE      2   /* Execute only first case that matches */
+#define SWITCH_MASK     3   /* Includes lower 2 bits */
+#define SWITCH_NOTIFY   4   /* Send a @notify after the @switch is completed */
 #define SWEEP_ME        1   /* Check my inventory */
 #define SWEEP_HERE      2   /* Check my location */
 #define SWEEP_COMMANDS  4   /* Check for $-commands */
@@ -631,12 +734,14 @@ extern int anum_alc_top;
 #define TIMECHK_LOG     4   /* Write info to log */
 #define TOAD_NO_CHOWN   1   /* Don't change ownership */
 #define TRIG_QUIET      1   /* Don't display 'Triggered.' message. */
+#define TRIG_NOTIFY     2   /* Send a @notify after the @trigger is completed. */
 #define TWARP_QUEUE     1   /* Warp the wait and sem queues */
 #define TWARP_DUMP      2   /* Warp the dump interval */
 #define TWARP_CLEAN     4   /* Warp the cleaning interval */
 #define TWARP_IDLE      8   /* Warp the idle check interval */
 /* empty       16 */
 #define TWARP_EVENTS    32  /* Warp the events checking interval */
+#define VERB_NONAME     1   // Do not preprend name to odefault.
 #define WAIT_UNTIL      1   // Absolute UTC seconds instead of delta.
 
 /* Hush codes for movement messages */
@@ -661,6 +766,7 @@ extern int anum_alc_top;
 #define EV_NO_COMPRESS  0x00040000  /* Don't compress spaces. */
 #define EV_NO_LOCATION  0x00080000  /* Supresses %l */
 #define EV_NOFCHECK     0x00100000  /* Do not evaluate functions! */
+#define EV_TRACE        0x00200000  /* Request a trace of this call to eval */
 
 /* Message forwarding directives */
 
@@ -765,7 +871,7 @@ void do_pemit_single
     int chPoseType,
     char *message
 );
-void do_say(dbref executor, dbref caller, dbref enactor, int key,
+void do_say(dbref executor, dbref caller, dbref enactor, int eval, int key,
                    char *message);
 
 int  boot_off(dbref player, const char *message);
@@ -813,7 +919,7 @@ extern PortInfo aMainGamePorts[MAX_LISTEN_PORTS];
 extern int      nMainGamePorts;
 
 #ifdef WIN32
-extern DWORD platform;
+extern bool bUseCompletionPorts;;
 #else // WIN32
 extern int maxd;
 #endif // WIN32
@@ -862,7 +968,7 @@ void dispatch_CacheTick(void *pUnused, int iUnused);
 //    heap.
 //
 // If you ever re-implement this object using another data structure,
-// please remember to maintain the properties properties.
+// please remember to maintain this property.
 //
 typedef void FTASK(void *, int);
 
@@ -912,7 +1018,8 @@ public:
     CTaskHeap();
     ~CTaskHeap();
 
-    void Insert(PTASK_RECORD, SCHCMP *);
+    void Shrink(void);
+    bool Insert(PTASK_RECORD, SCHCMP *);
     PTASK_RECORD PeekAtTopmost(void);
     PTASK_RECORD RemoveTopmost(SCHCMP *);
     void CancelTask(FTASK *fpTask, void *arg_voidptr, int arg_Integer);
@@ -945,6 +1052,7 @@ public:
     int  RunTasks(const CLinearTimeAbsolute& tNow);
     void ReadyTasks(const CLinearTimeAbsolute& tNow);
     void CancelTask(FTASK *fpTask, void *arg_voidptr, int arg_Integer);
+    void Shrink(void);
 
     void SetMinPriority(int arg_minPriority);
     int  GetMinPriority(void) { return m_minPriority; }
@@ -1011,6 +1119,7 @@ void pcache_trim(void);
 //
 void cache_redirect(void);
 void cache_pass2(void);
+void cache_cleanup(void);
 extern CLinearTimeAbsolute cs_ltime;
 
 // From speech.cpp
@@ -1019,7 +1128,10 @@ char *modSpeech(dbref player, char *message, bool bWhich, char *command);
 
 // From funceval.cpp
 //
+#ifdef DEPRECATED
 void stack_clr(dbref obj);
-bool parse_and_get_attrib(dbref, char *[], char **, dbref *, char *, char **);
+#endif // DEPRECATED
+bool parse_and_get_attrib(dbref, char *[], char **, dbref *, dbref *, int *, char *, char **);
+void SimplifyColorLetters(char Out[8], char *pIn);
 
 #endif // EXTERNS_H
