@@ -1,6 +1,6 @@
 // file_c.cpp -- File cache management.
 //
-// $Id: file_c.cpp,v 1.6 2006/01/07 23:05:01 sdennis Exp $
+// $Id: file_c.cpp 229 2006-10-04 06:42:24Z brazilofmux $
 //
 
 #include "copyright.h"
@@ -67,10 +67,11 @@ static NAMETAB list_files[] =
     { NULL,              0,  0,          0}
 };
 
-void do_list_file(dbref executor, dbref caller, dbref enactor, int extra, char *arg)
+void do_list_file(dbref executor, dbref caller, dbref enactor, int eval, int extra, char *arg)
 {
     UNUSED_PARAMETER(caller);
     UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(eval);
     UNUSED_PARAMETER(extra);
 
     int flagvalue;
@@ -100,16 +101,12 @@ static FBLOCK *fcache_fill(FBLOCK *fp, char ch)
 
 static int fcache_read(FBLOCK **cp, char *filename)
 {
-    int n, nmax, tchars, fd;
-    char *buff;
-    FBLOCK *fp, *tfp;
-
     // Free a prior buffer chain.
     //
-    fp = *cp;
+    FBLOCK *fp = *cp;
     while (fp != NULL)
     {
-        tfp = fp->hdr.nxt;
+        FBLOCK *tfp = fp->hdr.nxt;
         free_mbuf(fp);
         fp = tfp;
     }
@@ -117,13 +114,15 @@ static int fcache_read(FBLOCK **cp, char *filename)
 
     // Read the text file into a new chain.
     //
-    if ((fd = open(filename, O_RDONLY|O_BINARY)) == -1)
+    int   fd;
+    char *buff;
+    if (!mux_open(&fd, filename, O_RDONLY|O_BINARY))
     {
         // Failure: log the event
         //
         STARTLOG(LOG_PROBLEMS, "FIL", "OPEN");
         buff = alloc_mbuf("fcache_read.LOG");
-        sprintf(buff, "Couldn't open file '%s'.", filename);
+        mux_sprintf(buff, MBUF_SIZE, "Couldn't open file '%s'.", filename);
         log_text(buff);
         free_mbuf(buff);
         ENDLOG
@@ -138,14 +137,14 @@ static int fcache_read(FBLOCK **cp, char *filename)
     fp->hdr.nxt = NULL;
     fp->hdr.nchars = 0;
     *cp = fp;
-    tchars = 0;
+    int tchars = 0;
 
     // Process the file, one lbuf at a time.
     //
-    nmax = read(fd, buff, LBUF_SIZE);
+    int nmax = mux_read(fd, buff, LBUF_SIZE);
     while (nmax > 0)
     {
-        for (n = 0; n < nmax; n++)
+        for (int n = 0; n < nmax; n++)
         {
             switch (buff[n])
             {
@@ -161,10 +160,10 @@ static int fcache_read(FBLOCK **cp, char *filename)
                 tchars++;
             }
         }
-        nmax = read(fd, buff, LBUF_SIZE);
+        nmax = mux_read(fd, buff, LBUF_SIZE);
     }
     free_lbuf(buff);
-    if (close(fd) == 0)
+    if (mux_close(fd) == 0)
     {
         DebugTotalFiles--;
     }
