@@ -1,6 +1,6 @@
 // flags.cpp -- Flag manipulation routines.
 //
-// $Id: flags.cpp,v 1.24 2006/01/07 23:01:36 sdennis Exp $
+// $Id: flags.cpp 2734 2007-10-28 23:02:55Z brazilofmux $
 //
 
 #include "copyright.h"
@@ -10,6 +10,10 @@
 
 #include "command.h"
 #include "powers.h"
+#if defined(FIRANMUX)
+#include "attrs.h"
+#endif // FIRANMUX
+#include "ansi.h"
 
 /* ---------------------------------------------------------------------------
  * fh_any: set or clear indicated bit, no security checking
@@ -121,9 +125,11 @@ static bool fh_privileged
 {
     if (!God(player))
     {
-        if (  !isPlayer(player)
+        if (  isPlayer(target)
+#if !defined(FIRANMUX)
+           || !isPlayer(player)
            || player != Owner(player)
-           || isPlayer(target)
+#endif // FIRANMUX
            || (db[player].fs.word[fflags] & flag) == 0)
         {
             return false;
@@ -354,6 +360,13 @@ static FLAGBITENT fbeMarker7        = { MARK_7,       '7',    FLAG_WORD3, 0,
 static FLAGBITENT fbeMarker8        = { MARK_8,       '8',    FLAG_WORD3, 0,                    fh_god};
 static FLAGBITENT fbeMarker9        = { MARK_9,       '9',    FLAG_WORD3, 0,                    fh_god};
 #endif // WOD_REALMS
+#if defined(FIRANMUX)
+static FLAGBITENT fbeImmobile       = { IMMOBILE,     '#',    FLAG_WORD3, 0,                    fh_wiz};
+static FLAGBITENT fbeLineWrap       = { LINEWRAP,     '>',    FLAG_WORD3, 0,                    fh_any};
+static FLAGBITENT fbeQuell          = { QUELL,        ' ',    FLAG_WORD3, 0,                    fh_inherit};
+static FLAGBITENT fbeRestricted     = { RESTRICTED,   '!',    FLAG_WORD3, CA_WIZARD,            fh_wiz};
+static FLAGBITENT fbeParent         = { PARENT,       '+',    FLAG_WORD3, 0,                    fh_any};
+#endif // FIRANMUX
 
 FLAGNAMEENT gen_flag_names[] =
 {
@@ -441,6 +454,13 @@ FLAGNAMEENT gen_flag_names[] =
     {"MEDIUM",          true, &fbeMedium         },
     {"DEAD",            true, &fbeDead           },
 #endif // WOD_REALMS
+#if defined(FIRANMUX)
+    {"IMMOBILE",        true, &fbeImmobile       },
+    {"LINEWRAP",        true, &fbeLineWrap       },
+    {"QUELL",           true, &fbeQuell          },
+    {"RESTRICTED",      true, &fbeRestricted     },
+    {"PARENT",          true, &fbeParent         },
+#endif // FIRANMUX
     {NULL,             false, NULL}
 };
 
@@ -452,7 +472,8 @@ OBJENT object_types[8] =
     {"PLAYER",  'P', CA_PUBLIC, OF_CONTENTS|OF_LOCATION|OF_EXITS|OF_HOME|OF_OWNER|OF_SIBLINGS},
     {"TYPE5",   '+', CA_GOD,    0},
     {"GARBAGE", '-', CA_PUBLIC, OF_CONTENTS|OF_LOCATION|OF_EXITS|OF_HOME|OF_SIBLINGS},
-    {"GARBAGE", '#', CA_GOD,    0}
+    {"GARBAGE", '#', CA_GOD,    0},
+    {"GARBAGE", '=', CA_GOD,    0}
 };
 
 /* ---------------------------------------------------------------------------
@@ -464,9 +485,8 @@ void init_flagtab(void)
     char *nbuf = alloc_sbuf("init_flagtab");
     for (FLAGNAMEENT *fp = gen_flag_names; fp->pOrigName; fp++)
     {
-        fp->flagname = fp->pOrigName;
-        strncpy(nbuf, fp->pOrigName, SBUF_SIZE);
-        nbuf[SBUF_SIZE-1] = '\0';
+        fp->flagname = (char *)fp->pOrigName;
+        mux_strncpy(nbuf, fp->pOrigName, SBUF_SIZE-1);
         mux_strlwr(nbuf);
         if (!hashfindLEN(nbuf, strlen(nbuf), &mudstate.flags_htab))
         {
@@ -534,7 +554,8 @@ char *MakeCanonicalFlagName
         nName++;
     }
     *p = '\0';
-    if (nName < SBUF_SIZE)
+    if (  0 < nName
+       && nName < SBUF_SIZE)
     {
         *pnName = nName;
         *pbValid = true;
@@ -548,7 +569,7 @@ char *MakeCanonicalFlagName
     }
 }
 
-static FLAGNAMEENT *find_flag(char *flagname)
+static FLAGNAMEENT *find_flag(const char *flagname)
 {
     // Convert flagname to canonical lowercase format.
     //
@@ -597,6 +618,7 @@ void flag_set(dbref target, dbref player, char *flag, int key)
         {
             nflag++;
         }
+
         if (*nflag == '\0')
         {
             bDone = true;
@@ -666,7 +688,7 @@ char *decode_flags(dbref player, FLAGSET *fs)
 
     if (!Good_obj(player))
     {
-        strcpy(buf, "#-2 ERROR");
+        mux_strncpy(buf, "#-2 ERROR", SBUF_SIZE-1);
         return buf;
     }
     int flagtype = fs->word[FLAG_WORD1] & TYPE_MASK;
@@ -734,7 +756,7 @@ char *decode_flags(dbref player, FLAGSET *fs)
  * * has_flag: does object have flag visible to player?
  */
 
-bool has_flag(dbref player, dbref it, char *flagname)
+bool has_flag(dbref player, dbref it, const char *flagname)
 {
     FLAGNAMEENT *fp = find_flag(flagname);
     if (!fp)
@@ -854,33 +876,76 @@ char *unparse_object_numonly(dbref target)
     char *buf = alloc_lbuf("unparse_object_numonly");
     if (target < 0)
     {
-        strcpy(buf, aszSpecialDBRefNames[-target]);
+        mux_strncpy(buf, aszSpecialDBRefNames[-target], LBUF_SIZE-1);
     }
     else if (!Good_obj(target))
     {
-        sprintf(buf, "*ILLEGAL*(#%d)", target);
+        mux_sprintf(buf, LBUF_SIZE, "*ILLEGAL*(#%d)", target);
     }
     else
     {
-        sprintf(buf, "%.200s(#%d)", Name(target), target);
+        // Leave 100 bytes on the end for the dbref.
+        //
+        size_t vw;
+        size_t nLen = ANSI_TruncateToField(Name(target), LBUF_SIZE-100,
+            buf, LBUF_SIZE, &vw, ANSI_ENDGOAL_NORMAL);
+        char *bp = buf + nLen;
+
+        safe_str("(#", buf, &bp);
+        safe_ltoa(target, buf, &bp);
+        safe_chr(')', buf, &bp);
+        *bp = '\0';
     }
     return buf;
 }
 
+#if defined(FIRANMUX)
+static bool AcquireColor(dbref player, dbref target, char SimplifiedCodes[8])
+{
+    int   aflags;
+    dbref aowner;
+
+    // Get the value of the object's '@color' attribute (or on a parent).
+    //
+    char *color_attr = alloc_lbuf("AcquireColor.1");
+    atr_pget_str(color_attr, target, A_COLOR, &aowner, &aflags);
+
+    if ('\0' == color_attr[0])
+    {
+        free_lbuf(color_attr);
+        return false;
+    }
+    else
+    {
+        char *AnsiCodes = alloc_lbuf("AcquireColor.2");
+        char *ac = AnsiCodes;
+        char *cp = color_attr;
+        mux_exec(AnsiCodes, &ac, player, target, target,
+                AttrTrace(aflags, EV_EVAL|EV_TOP|EV_FCHECK), &cp, NULL, 0);
+        *ac = '\0';
+        free_lbuf(color_attr);
+
+        SimplifyColorLetters(SimplifiedCodes, AnsiCodes);
+        free_lbuf(AnsiCodes);
+        return true;
+    }
+}
+#endif // FIRANMUX
+
 /*
  * ---------------------------------------------------------------------------
  * * Return an lbuf pointing to the object name and possibly the db# and flags
  */
-char *unparse_object(dbref player, dbref target, bool obey_myopic)
+char *unparse_object(dbref player, dbref target, bool obey_myopic, bool bAddColor)
 {
     char *buf = alloc_lbuf("unparse_object");
     if (NOPERM <= target && target < 0)
     {
-        strcpy(buf, aszSpecialDBRefNames[-target]);
+        mux_strncpy(buf, aszSpecialDBRefNames[-target], LBUF_SIZE-1);
     }
     else if (!Good_obj(target))
     {
-        sprintf(buf, "*ILLEGAL*(#%d)", target);
+        mux_sprintf(buf, LBUF_SIZE, "*ILLEGAL*(#%d)", target);
     }
     else
     {
@@ -893,26 +958,77 @@ char *unparse_object(dbref player, dbref target, bool obey_myopic)
         {
             exam = Examinable(player, target);
         }
+
+        // Leave and extra 100 bytes for the dbref and flags at the end and
+        // color at the beginning if necessary..
+        //
+        size_t vw;
+        size_t nLen = ANSI_TruncateToField(Moniker(target), LBUF_SIZE-100,
+            buf, LBUF_SIZE, &vw, ANSI_ENDGOAL_NORMAL);
+
+        char *bp = buf + nLen;
+
+#if defined(FIRANMUX)
+        if (  vw == nLen
+           && bAddColor)
+        {
+            // There is no color in the name, so look for @color, or highlight.
+            //
+            char *buf2 = alloc_lbuf("unparse_object.color");
+            char *bp2  = buf2;
+
+            char SimplifiedCodes[8];
+            if (AcquireColor(player, target, SimplifiedCodes))
+            {
+                for (int i = 0; SimplifiedCodes[i]; i++)
+                {
+                    const char *pColor = ColorTable[(unsigned char)SimplifiedCodes[i]];
+                    if (pColor)
+                    {
+                        safe_str(pColor, buf2, &bp2);
+                    }
+                }
+            }
+            else
+            {
+                safe_str(ANSI_HILITE, buf2, &bp2);
+            }
+
+            *bp = '\0';
+            safe_str(buf, buf2, &bp2);
+            safe_str(ANSI_NORMAL, buf2, &bp2);
+
+            // Swap buffers.
+            //
+            free_lbuf(buf);
+            buf = buf2;
+            bp  = bp2;
+        }
+#else
+        UNUSED_PARAMETER(bAddColor);
+#endif // FIRANMUX
+
         if (  exam
            || (Flags(target) & (CHOWN_OK | JUMP_OK | LINK_OK | DESTROY_OK))
            || (Flags2(target) & ABODE))
         {
-            // show everything
+            // Show everything.
             //
             char *fp = decode_flags(player, &(db[target].fs));
-            sprintf(buf, "%.200s(#%d%s)", Moniker(target), target, fp);
+
+            safe_str("(#", buf, &bp);
+            safe_ltoa(target, buf, &bp);
+            safe_str(fp, buf, &bp);
+            safe_chr(')', buf, &bp);
+
             free_sbuf(fp);
         }
-        else
-        {
-            // show only the name.
-            //
-            strcpy(buf, Moniker(target));
-        }
+        *bp = '\0';
     }
     return buf;
 }
 
+
 /* ---------------------------------------------------------------------------
  * cf_flag_access: Modify who can set a flag.
  */
@@ -929,7 +1045,10 @@ CF_HAND(cf_flag_access)
     char *fstr = mux_strtok_parse(&tts);
     char *permstr = mux_strtok_parse(&tts);
 
-    if (!fstr || !*fstr)
+    if (  NULL == fstr
+       || '\0' == fstr[0]
+       || NULL == permstr
+       || '\0' == permstr[0])
     {
         return -1;
     }
@@ -1004,9 +1123,7 @@ bool convert_flags(dbref player, char *flaglist, FLAGSET *fset, FLAG *p_type)
 {
     FLAG type = NOTYPE;
     FLAGSET flagmask;
-    flagmask.word[FLAG_WORD1] = 0;
-    flagmask.word[FLAG_WORD2] = 0;
-    flagmask.word[FLAG_WORD3] = 0;
+    memset(&flagmask, 0, sizeof(flagmask));
     int i;
 
     char *s;
@@ -1200,7 +1317,7 @@ void do_flag(dbref executor, dbref caller, dbref enactor, int key, int nargs,
                    && mux_stricmp(lookup->flagname, pCheckedAlias) == 0)
                 {
                     MEMFREE(lookup->flagname);
-                    lookup->flagname = lookup->pOrigName;
+                    lookup->flagname = (char *)lookup->pOrigName;
                     hashdeleteLEN(pCheckedAlias, nAlias, &mudstate.flags_htab);
                     notify(executor, tprintf("Flag name '%s' removed from the hash table.", pCheckedAlias));
                 }
