@@ -1,6 +1,6 @@
 // flags.h -- Object flags.
 //
-// $Id: flags.h,v 1.10 2006/01/07 21:48:00 sdennis Exp $
+// $Id: flags.h 2734 2007-10-28 23:02:55Z brazilofmux $
 //
 
 #include "copyright.h"
@@ -19,6 +19,7 @@
 #define TYPE_PLAYER   0x3
 /* Empty */
 #define TYPE_GARBAGE  0x5
+/* Empty */
 #define NOTYPE        0x7
 #define TYPE_MASK     0x7
 
@@ -86,6 +87,9 @@
 // Third word of flags
 //
 #if defined(WOD_REALMS) || defined(REALITY_LVLS)
+#if defined(FIRANMUX)
+#error "FIRANMUX is incompatible with both WOD_REALMS and REALITY_LVLS"
+#endif
 #define OBF          0x00000001      // Obfuscate Flag
 #define HSS          0x00000002      // Auspex/Heightened Senses Flag
 #define UMBRA        0x00000004      // Umbra, UMBRADESC
@@ -98,6 +102,15 @@
 #define PEERING      0x00000200      // Means the a looker is seeing a
                                      // different realm than they are in.
 #endif // WOD_REALMS
+#if defined(FIRANMUX)
+#define LINEWRAP     0x00000001      // Player linewraps.
+#define IMMOBILE     0x00000002      // Player immobile.
+#define QUELL        0x00000004      // Temporarily discard wizardry.
+#define WINTELNET    0x00000008      // Add extra \n for Win/Mac Telnet.
+#define REMOTEECHO   0x00000010      // Adds remote echo!
+#define RESTRICTED   0x00000020      // Player uses restricted commandset.
+#define PARENT       0x00000040      // Object is an official parent obj.
+#endif // FIRANMUX
 
 #define SITEMON      0x00000400      // Sitemonitor Flag
 #define CMDCHECK     0x00000800      // Has @icmd set
@@ -127,7 +140,7 @@ typedef struct flag_bit_entry
 
 typedef struct flag_name_entry
 {
-    char *pOrigName;        // Original name of flag.
+    const char *pOrigName;  // Original name of flag.
     bool bPositive;         // Flag sense.
     FLAGBITENT *fbe;        // Which bit is this associated with?
     char *flagname;         // Name of the flag.
@@ -160,17 +173,17 @@ typedef struct flagset
     FLAG  word[3];
 } FLAGSET;
 
-extern void init_flagtab(void);
-extern void display_flagtab(dbref);
-extern void flag_set(dbref, dbref, char *, int);
-extern char *flag_description(dbref, dbref);
-extern char *decode_flags(dbref, FLAGSET *);
-extern bool has_flag(dbref, dbref, char *);
-extern char *unparse_object(dbref, dbref, bool);
-extern char *unparse_object_numonly(dbref);
-extern bool convert_flags(dbref, char *, FLAGSET *, FLAG *);
-extern void decompile_flags(dbref, dbref, char *);
-extern char *MakeCanonicalFlagName
+void init_flagtab(void);
+void display_flagtab(dbref);
+void flag_set(dbref, dbref, char *, int);
+char *flag_description(dbref, dbref);
+char *decode_flags(dbref, FLAGSET *);
+bool has_flag(dbref, dbref, const char *);
+char *unparse_object(dbref player, dbref target, bool obey_myopic, bool bAddColor = false);
+char *unparse_object_numonly(dbref);
+bool convert_flags(dbref, char *, FLAGSET *, FLAG *);
+void decompile_flags(dbref, dbref, char *);
+char *MakeCanonicalFlagName
 (
     const char *pName,
     int *pnName,
@@ -224,6 +237,10 @@ extern char *MakeCanonicalFlagName
 /* Marked(x)            - Check marked flag on X */
 /* See_attr(P,X.A,O,F)  - Can P see text attr A on X if attr has owner O */
 /* KeepAlive(x)         - Does the user want keepalives? */
+#if defined(FIRANMUX)
+/* Linewrap(x)          - Is X set Linewrap? */
+/* Restricted(x)        - Is X restricted to the restricted commandset? */
+#endif // FIRANMUX
 
 #define Typeof(x)           (Flags(x) & TYPE_MASK)
 #define God(x)              ((x) == GOD)
@@ -247,9 +264,31 @@ extern char *MakeCanonicalFlagName
 #define Good_owner(x)       (Good_obj(x) && OwnsOthers(x))
 
 #define Staff(x)            (Wizard(x) || Royalty(x) || ((Flags2(x) & STAFF) != 0))
-#define Royalty(x)          ((Flags(x) & ROYALTY) || \
-                            ((Flags(Owner(x)) & ROYALTY) && Inherits(x)))
+#if defined(FIRANMUX)
+#define Royalty(x)          (  (  (Flags(x) & ROYALTY) \
+                               && (Flags3(x) & QUELL) == 0 \
+                               ) \
+                            || (  (  (Flags(Owner(x)) & ROYALTY) \
+                                  && (Flags3(Owner(x)) & QUELL) == 0 \
+                                  ) \
+                               && Inherits(x) \
+                               ) \
+                            )
+#define RealRoyalty(x)      (  (Flags(x) & ROYALTY) \
+                            || (  (Flags(Owner(x)) & ROYALTY) \
+                               && Inherits(x) \
+                               ) \
+                            )
+#else // FIRANMUX
+#define Royalty(x)          (  (Flags(x) & ROYALTY) \
+                            || (  (Flags(Owner(x)) & ROYALTY) \
+                               && Inherits(x) \
+                               ) \
+                            )
+#define RealRoyalty(x)      (Royalty(x))
+#endif // FIRANMUX
 #define WizRoy(x)           (Royalty(x) || Wizard(x))
+#define RealWizRoy(x)       (RealRoyalty(x) || RealWizard(x))
 #define Head(x)             ((Flags2(x) & HEAD_FLAG) != 0)
 #define Fixed(x)            ((Flags2(x) & FIXED) != 0)
 #define Uninspected(x)      ((Flags2(x) & UNINSPECTED) != 0)
@@ -262,10 +301,31 @@ extern char *MakeCanonicalFlagName
 #define Transparent(x)      ((Flags(x) & SEETHRU) != 0)
 #define Link_ok(x)          (((Flags(x) & LINK_OK) != 0) && Has_contents(x))
 #define Open_ok(x)          (((Flags2(x) & OPEN_OK) != 0) && Has_exits(x))
-#define Wizard(x)           ((Flags(x) & WIZARD) || \
-                            ((Flags(Owner(x)) & WIZARD) && Inherits(x)))
+#if defined(FIRANMUX)
+#define Wizard(x)           (  (Flags3(x) & QUELL) == 0 \
+                            && (  (Flags(x) & WIZARD) \
+                               || (  (Flags(Owner(x)) & WIZARD) \
+                                  && (Flags3(Owner(x)) & QUELL) == 0 \
+                                  && Inherits(x) \
+                                  ) \
+                               ) \
+                            )
+#define RealWizard(x)       (  (Flags(x) & WIZARD) \
+                            || (  (Flags(Owner(x)) & WIZARD) \
+                               && Inherits(x) \
+                               ) \
+                            )
+#define Dark(x)             ((Flags(x) & DARK) != 0)
+#else // FIRANMUX
+#define Wizard(x)           (  (Flags(x) & WIZARD) \
+                            || (  (Flags(Owner(x)) & WIZARD) \
+                               && Inherits(x) \
+                               ) \
+                            )
+#define RealWizard(x)       (Wizard(x))
 #define Dark(x)             (((Flags(x) & DARK) != 0) && (Wizard(x) || \
                             !(isPlayer(x) || (Puppet(x) && Has_contents(x)))))
+#endif // FIRANMUX
 #define Jump_ok(x)          (((Flags(x) & JUMP_OK) != 0) && Has_contents(x))
 #define Sticky(x)           ((Flags(x) & STICKY) != 0)
 #define Destroy_ok(x)       ((Flags(x) & DESTROY_OK) != 0)
@@ -311,9 +371,10 @@ extern char *MakeCanonicalFlagName
 #define Hidden(x)           ((Flags(x) & DARK) != 0)
 #define Blind(x)            ((Flags2(x) & BLIND) != 0)
 
-#define H_Startup(x)        ((Flags(x) & HAS_STARTUP) != 0)
+#define H_Daily(x)          ((Flags2(x) & HAS_DAILY) != 0)
 #define H_Fwdlist(x)        ((Flags2(x) & HAS_FWDLIST) != 0)
 #define H_Listen(x)         ((Flags2(x) & HAS_LISTEN) != 0)
+#define H_Startup(x)        ((Flags(x) & HAS_STARTUP) != 0)
 
 #define s_Halted(x)         s_Flags((x), FLAG_WORD1, Flags(x) | HALT)
 #define s_Going(x)          s_Flags((x), FLAG_WORD1, Flags(x) | GOING)
@@ -378,4 +439,10 @@ extern char *MakeCanonicalFlagName
 #define Html(x)             ((Flags2(x) & HTML) != 0)
 #define s_Html(x)           s_Flags((x), FLAG_WORD2, Flags2(x) | HTML)
 
+#if defined(FIRANMUX)
+#define Linewrap(x)   ((Flags3(x) & LINEWRAP) != 0)
+#define Immobile(x)   ((Flags3(x) & IMMOBILE) != 0)
+#define Restricted(x) ((Flags3(x) & RESTRICTED) != 0)
+#endif // FIRANMUX
+
 #endif // !__FLAGS_H
