@@ -1,6 +1,6 @@
 // functions.h -- declarations for functions & function processing.
 //
-// $Id: functions.h,v 1.19 2006/01/07 21:48:00 sdennis Exp $
+// $Id: functions.h 2734 2007-10-28 23:02:55Z brazilofmux $
 //
 
 #include "copyright.h"
@@ -10,9 +10,9 @@
 
 typedef struct tagFun
 {
-    char *name;     // function name
+    const char *name;  // function name
     void (*fun)(char *buff, char **bufc, dbref executor, dbref caller,
-        dbref enactor, char *fargs[], int nfargs, char *cargs[],
+        dbref enactor, int eval, char *fargs[], int nfargs, char *cargs[],
         int ncargs);  // handler
     int maxArgsParsed;// Maximum number of arguments parsed.
     int minArgs;      // Minimum number of args needed or expected
@@ -67,6 +67,7 @@ bool delim_check
 (
     char *buff, char **bufc,
     dbref executor, dbref caller, dbref enactor,
+    int   eval,
     char *fargs[], int nfargs,
     char *cargs[], int ncargs,
     int sep_arg, SEP *sep, int dflags
@@ -75,7 +76,7 @@ bool delim_check
 void arr2list(char *arr[], int alen, char *list, char **bufc, SEP *psep);
 int list2arr(char *arr[], int maxlen, char *list, SEP *psep);
 char *trim_space_sep(char *str, SEP *psep);
-char *trim_space_sep_LEN(char *str, int nStr, SEP *psep, int *nTrim);
+char *trim_space_sep_LEN(char *str, size_t nStr, SEP *psep, size_t *nTrim);
 char *next_token(char *str, SEP *psep);
 char *split_token(char **sp, SEP *psep);
 int countwords(char *str, SEP *psep);
@@ -84,18 +85,18 @@ int countwords(char *str, SEP *psep);
 //
 #define FUNCTION(x) \
     void x(char *buff, char **bufc, dbref executor, dbref caller,     \
-        dbref enactor, char *fargs[], int nfargs, char *cargs[],      \
-        int ncargs)
+        dbref enactor, int eval, char *fargs[], int nfargs,           \
+        char *cargs[], int ncargs)
 
 // This is for functions that take an optional delimiter character.
 //
 #define OPTIONAL_DELIM(iSep, Sep, dflags)                        \
-    delim_check(buff, bufc, executor, caller, enactor,           \
+    delim_check(buff, bufc, executor, caller, enactor, eval,     \
         fargs, nfargs, cargs, ncargs, (iSep), &(Sep), (dflags))
 
 #define XFUNCTION(x) void x(char *buff, char **bufc, dbref executor,    \
- dbref caller, dbref enactor, char *fargs[], int nfargs, char *cargs[],        \
- int ncargs)
+ dbref caller, dbref enactor, int eval, char *fargs[], int nfargs,      \
+ char *cargs[], int ncargs)
 
 // Interface for adding additional hardcode functions.
 //
@@ -124,7 +125,6 @@ XFUNCTION(fun_die);
 XFUNCTION(fun_dumping);
 XFUNCTION(fun_edefault);
 XFUNCTION(fun_elements);
-XFUNCTION(fun_empty);
 XFUNCTION(fun_encrypt);
 XFUNCTION(fun_entrances);
 XFUNCTION(fun_fcount);
@@ -141,16 +141,19 @@ XFUNCTION(fun_hastype);
 XFUNCTION(fun_ifelse);
 XFUNCTION(fun_inzone);
 XFUNCTION(fun_isword);
-XFUNCTION(fun_items);
 XFUNCTION(fun_last);
 XFUNCTION(fun_lit);
 XFUNCTION(fun_localize);
 XFUNCTION(fun_lparent);
 XFUNCTION(fun_lrand);
 XFUNCTION(fun_lrooms);
-XFUNCTION(fun_lstack);
 XFUNCTION(fun_mail);
 XFUNCTION(fun_mailfrom);
+#if defined(FIRANMUX)
+XFUNCTION(fun_mailj);
+XFUNCTION(fun_mailsize);
+XFUNCTION(fun_mailsubj);
+#endif
 XFUNCTION(fun_matchall);
 XFUNCTION(fun_mix);
 XFUNCTION(fun_munge);
@@ -159,12 +162,9 @@ XFUNCTION(fun_objeval);
 XFUNCTION(fun_objmem);
 XFUNCTION(fun_orflags);
 XFUNCTION(fun_pack);
-XFUNCTION(fun_peek);
 XFUNCTION(fun_pickrand);
 XFUNCTION(fun_playmem);
-XFUNCTION(fun_pop);
 XFUNCTION(fun_ports);
-XFUNCTION(fun_push);
 XFUNCTION(fun_regmatch);
 XFUNCTION(fun_regmatchi);
 XFUNCTION(fun_regrab);
@@ -175,6 +175,7 @@ XFUNCTION(fun_scramble);
 XFUNCTION(fun_shuffle);
 XFUNCTION(fun_sortby);
 XFUNCTION(fun_squish);
+XFUNCTION(fun_step);
 XFUNCTION(fun_strcat);
 XFUNCTION(fun_stripansi);
 XFUNCTION(fun_strtrunc);
@@ -187,6 +188,16 @@ XFUNCTION(fun_visible);
 XFUNCTION(fun_zfun);
 XFUNCTION(fun_zone);
 XFUNCTION(fun_zwho);
+
+#ifdef DEPRECATED
+XFUNCTION(fun_empty);
+XFUNCTION(fun_items);
+XFUNCTION(fun_lstack);
+XFUNCTION(fun_peek);
+XFUNCTION(fun_pop);
+XFUNCTION(fun_push);
+#endif // DEPRECATED
+
 #ifdef SIDE_EFFECT_FUNCTIONS
 XFUNCTION(fun_create);
 XFUNCTION(fun_emit);
@@ -198,12 +209,19 @@ XFUNCTION(fun_cemit);
 XFUNCTION(fun_set);
 XFUNCTION(fun_tel);
 XFUNCTION(fun_textfile);
+#if defined(FIRANMUX)
+XFUNCTION(fun_setparent);
+XFUNCTION(fun_setname);
+XFUNCTION(fun_trigger);
+#endif // FIRANMUX
 #endif
+
 // In netcommon.cpp
 XFUNCTION(fun_doing);
 XFUNCTION(fun_host);
 XFUNCTION(fun_motd);
 XFUNCTION(fun_poll);
+XFUNCTION(fun_siteinfo);
 // In quota.cpp
 XFUNCTION(fun_hasquota);
 
