@@ -1,10 +1,6 @@
 // funmath.cpp -- MUX math function handlers.
 //
-// $Id: funmath.cpp,v 1.5 2006/01/07 21:19:40 sdennis Exp $
-//
-// MUX 2.4
-// Copyright (C) 1998 through 2005 Solid Vertical Domains, Ltd. All
-// rights not explicitly given are reserved.
+// $Id: funmath.cpp 1862 2007-04-07 17:17:16Z brazilofmux $
 //
 #include "copyright.h"
 #include "autoconf.h"
@@ -86,40 +82,44 @@ double MakeSpecialFloat(int iWhich)
 
 static int mux_fpclass(double result)
 {
-    UINT64 i64;
-
-    *((double *)&i64) = result;
-
-    if ((i64 & IEEE_MASK_EXPONENT) == 0)
+    union
     {
-        if (i64 & IEEE_MASK_MANTISSA)
+        UINT64 i64;
+        double d;
+    } u;
+
+    u.d = result;
+  
+    if ((u.i64 & IEEE_MASK_EXPONENT) == 0)
+    {
+        if (u.i64 & IEEE_MASK_MANTISSA)
         {
-            if (i64 & IEEE_MASK_SIGN) return MUX_FPCLASS_ND;
-            else                      return MUX_FPCLASS_PD;
+            if (u.i64 & IEEE_MASK_SIGN) return MUX_FPCLASS_ND;
+            else                        return MUX_FPCLASS_PD;
         }
         else
         {
-            if (i64 & IEEE_MASK_SIGN) return MUX_FPCLASS_NZ;
-            else                      return MUX_FPCLASS_PZ;
+            if (u.i64 & IEEE_MASK_SIGN) return MUX_FPCLASS_NZ;
+            else                        return MUX_FPCLASS_PZ;
         }
     }
-    else if ((i64 & IEEE_MASK_EXPONENT) == IEEE_MASK_EXPONENT)
+    else if ((u.i64 & IEEE_MASK_EXPONENT) == IEEE_MASK_EXPONENT)
     {
-        if (i64 & IEEE_MASK_MANTISSA)
+        if (u.i64 & IEEE_MASK_MANTISSA)
         {
-            if (i64 & IEEE_MASK_QNAN) return MUX_FPCLASS_QNAN;
-            else                      return MUX_FPCLASS_SNAN;
+            if (u.i64 & IEEE_MASK_QNAN) return MUX_FPCLASS_QNAN;
+            else                        return MUX_FPCLASS_SNAN;
         }
         else
         {
-            if (i64 & IEEE_MASK_SIGN) return MUX_FPCLASS_NINF;
-            else                      return MUX_FPCLASS_PINF;
+            if (u.i64 & IEEE_MASK_SIGN) return MUX_FPCLASS_NINF;
+            else                        return MUX_FPCLASS_PINF;
         }
     }
     else
     {
-        if (i64 & IEEE_MASK_SIGN)     return MUX_FPCLASS_NN;
-        else                          return MUX_FPCLASS_PN;
+        if (u.i64 & IEEE_MASK_SIGN)     return MUX_FPCLASS_NN;
+        else                            return MUX_FPCLASS_PN;
     }
 }
 #endif // HAVE_IEEE_FP_FORMAT
@@ -148,12 +148,13 @@ static double NearestPretty(double R)
     // R.
     //
     char *p = mux_dtoa(R, mode, 50, &decpt, &bNegative, &rve);
-    int nDigits = rve - p;
+    size_t nDigits = rve - p;
 
     // R-ulp(R)
     //
     p = mux_dtoa(R0, mode, 50, &decpt, &bNegative, &rve);
-    if (rve - p < nDigits)
+    size_t nDigitsR0 = rve - p;
+    if (nDigitsR0 < nDigits)
     {
         nDigits = rve - p;
         R  = R0;
@@ -162,7 +163,8 @@ static double NearestPretty(double R)
     // R+ulp(R)
     //
     p = mux_dtoa(R1, mode, 50, &decpt, &bNegative, &rve);
-    if (rve - p < nDigits)
+    size_t nDigitsR1 = rve - p;
+    if (nDigitsR1 < nDigits)
     {
         nDigits = rve - p;
         R = R1;
@@ -253,18 +255,26 @@ static const long nMaximums[10] =
     0, 9, 99, 999, 9999, 99999, 999999, 9999999, 99999999, 999999999
 };
 
-static double g_aDoubles[(LBUF_SIZE+1)/2];
+static double g_aDoubles[LBUF_SIZE];
+int const g_nDoubles = sizeof(g_aDoubles)/sizeof(double);
 
 FUNCTION(fun_add)
 {
     UNUSED_PARAMETER(executor);
     UNUSED_PARAMETER(caller);
     UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(eval);
     UNUSED_PARAMETER(cargs);
     UNUSED_PARAMETER(ncargs);
 
+    int nArgs = nfargs;
+    if (g_nDoubles < nArgs)
+    {
+        nArgs = g_nDoubles;
+    }
+
     int i;
-    for (i = 0; i < nfargs; i++)
+    for (i = 0; i < nArgs; i++)
     {
         int nDigits;
         long nMaxValue = 0;
@@ -274,11 +284,12 @@ FUNCTION(fun_add)
         {
             // Do it the slow way.
             //
-            for (int j = 0; j < nfargs; j++)
+            for (int j = 0; j < nArgs; j++)
             {
                 g_aDoubles[j] = mux_atof(fargs[j]);
             }
-            fval(buff, bufc, AddDoubles(nfargs, g_aDoubles));
+
+            fval(buff, bufc, AddDoubles(nArgs, g_aDoubles));
             return;
         }
     }
@@ -286,7 +297,7 @@ FUNCTION(fun_add)
     // We can do it the fast way.
     //
     long sum = 0;
-    for (i = 0; i < nfargs; i++)
+    for (i = 0; i < nArgs; i++)
     {
         sum += mux_atol(fargs[i]);
     }
@@ -298,6 +309,7 @@ FUNCTION(fun_ladd)
     UNUSED_PARAMETER(executor);
     UNUSED_PARAMETER(caller);
     UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(eval);
     UNUSED_PARAMETER(cargs);
     UNUSED_PARAMETER(ncargs);
 
@@ -311,7 +323,8 @@ FUNCTION(fun_ladd)
         }
 
         char *cp = trim_space_sep(fargs[0], &sep);
-        while (cp)
+        while (  cp
+              && n < g_nDoubles)
         {
             char *curr = split_token(&cp, &sep);
             g_aDoubles[n++] = mux_atof(curr);
@@ -331,6 +344,7 @@ FUNCTION(fun_iadd)
     UNUSED_PARAMETER(executor);
     UNUSED_PARAMETER(caller);
     UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(eval);
     UNUSED_PARAMETER(cargs);
     UNUSED_PARAMETER(ncargs);
 
@@ -347,6 +361,7 @@ FUNCTION(fun_sub)
     UNUSED_PARAMETER(executor);
     UNUSED_PARAMETER(caller);
     UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(eval);
     UNUSED_PARAMETER(nfargs);
     UNUSED_PARAMETER(cargs);
     UNUSED_PARAMETER(ncargs);
@@ -358,7 +373,9 @@ FUNCTION(fun_sub)
        && nDigits <= 9)
     {
         int iResult;
-        iResult = mux_atol(fargs[0]) - mux_atol(fargs[1]);
+        long a = mux_atol(fargs[0]);
+        long b = mux_atol(fargs[1]);
+        iResult = a - b;
         safe_ltoa(iResult, buff, bufc);
     }
     else
@@ -380,11 +397,14 @@ FUNCTION(fun_isub)
     UNUSED_PARAMETER(executor);
     UNUSED_PARAMETER(caller);
     UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(eval);
     UNUSED_PARAMETER(nfargs);
     UNUSED_PARAMETER(cargs);
     UNUSED_PARAMETER(ncargs);
 
-    INT64 diff = mux_atoi64(fargs[0]) - mux_atoi64(fargs[1]);
+    INT64 a = mux_atoi64(fargs[0]);
+    INT64 b = mux_atoi64(fargs[1]);
+    INT64 diff = a - b;
     safe_i64toa(diff, buff, bufc);
 }
 
@@ -393,6 +413,7 @@ FUNCTION(fun_mul)
     UNUSED_PARAMETER(executor);
     UNUSED_PARAMETER(caller);
     UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(eval);
     UNUSED_PARAMETER(cargs);
     UNUSED_PARAMETER(ncargs);
 
@@ -415,6 +436,7 @@ FUNCTION(fun_imul)
     UNUSED_PARAMETER(executor);
     UNUSED_PARAMETER(caller);
     UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(eval);
     UNUSED_PARAMETER(cargs);
     UNUSED_PARAMETER(ncargs);
 
@@ -431,6 +453,7 @@ FUNCTION(fun_gt)
     UNUSED_PARAMETER(executor);
     UNUSED_PARAMETER(caller);
     UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(eval);
     UNUSED_PARAMETER(nfargs);
     UNUSED_PARAMETER(cargs);
     UNUSED_PARAMETER(ncargs);
@@ -442,11 +465,15 @@ FUNCTION(fun_gt)
        && is_integer(fargs[1], &nDigits)
        && nDigits <= 9)
     {
-        bResult = (mux_atol(fargs[0]) > mux_atol(fargs[1]));
+        long a = mux_atol(fargs[0]);
+        long b = mux_atol(fargs[1]);
+        bResult = (a > b);
     }
     else
     {
-        bResult = (mux_atof(fargs[0]) > mux_atof(fargs[1]));
+        double a = mux_atof(fargs[0]);
+        double b = mux_atof(fargs[1]);
+        bResult = (a > b);
     }
     safe_bool(bResult, buff, bufc);
 }
@@ -456,6 +483,7 @@ FUNCTION(fun_gte)
     UNUSED_PARAMETER(executor);
     UNUSED_PARAMETER(caller);
     UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(eval);
     UNUSED_PARAMETER(nfargs);
     UNUSED_PARAMETER(cargs);
     UNUSED_PARAMETER(ncargs);
@@ -467,11 +495,15 @@ FUNCTION(fun_gte)
        && is_integer(fargs[1], &nDigits)
        && nDigits <= 9)
     {
-        bResult = (mux_atol(fargs[0]) >= mux_atol(fargs[1]));
+        long a = mux_atol(fargs[0]);
+        long b = mux_atol(fargs[1]);
+        bResult = (a >= b);
     }
     else
     {
-        bResult = (mux_atof(fargs[0]) >= mux_atof(fargs[1]));
+        double a = mux_atof(fargs[0]);
+        double b = mux_atof(fargs[1]);
+        bResult = (a >= b);
     }
     safe_bool(bResult, buff, bufc);
 }
@@ -481,6 +513,7 @@ FUNCTION(fun_lt)
     UNUSED_PARAMETER(executor);
     UNUSED_PARAMETER(caller);
     UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(eval);
     UNUSED_PARAMETER(nfargs);
     UNUSED_PARAMETER(cargs);
     UNUSED_PARAMETER(ncargs);
@@ -492,11 +525,15 @@ FUNCTION(fun_lt)
        && is_integer(fargs[1], &nDigits)
        && nDigits <= 9)
     {
-        bResult = (mux_atol(fargs[0]) < mux_atol(fargs[1]));
+        long a = mux_atol(fargs[0]);
+        long b = mux_atol(fargs[1]);
+        bResult = (a < b);
     }
     else
     {
-        bResult = (mux_atof(fargs[0]) < mux_atof(fargs[1]));
+        double a = mux_atof(fargs[0]);
+        double b = mux_atof(fargs[1]);
+        bResult = (a < b);
     }
     safe_bool(bResult, buff, bufc);
 }
@@ -506,6 +543,7 @@ FUNCTION(fun_lte)
     UNUSED_PARAMETER(executor);
     UNUSED_PARAMETER(caller);
     UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(eval);
     UNUSED_PARAMETER(nfargs);
     UNUSED_PARAMETER(cargs);
     UNUSED_PARAMETER(ncargs);
@@ -517,11 +555,15 @@ FUNCTION(fun_lte)
        && is_integer(fargs[1], &nDigits)
        && nDigits <= 9)
     {
-        bResult = (mux_atol(fargs[0]) <= mux_atol(fargs[1]));
+        long a = mux_atol(fargs[0]);
+        long b = mux_atol(fargs[1]);
+        bResult = (a <= b);
     }
     else
     {
-        bResult = (mux_atof(fargs[0]) <= mux_atof(fargs[1]));
+        double a = mux_atof(fargs[0]);
+        double b = mux_atof(fargs[1]);
+        bResult = (a <= b);
     }
     safe_bool(bResult, buff, bufc);
 }
@@ -531,23 +573,30 @@ FUNCTION(fun_eq)
     UNUSED_PARAMETER(executor);
     UNUSED_PARAMETER(caller);
     UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(eval);
     UNUSED_PARAMETER(nfargs);
     UNUSED_PARAMETER(cargs);
     UNUSED_PARAMETER(ncargs);
 
-    bool bResult = false;
+    bool bResult = true;
     int nDigits;
     if (  is_integer(fargs[0], &nDigits)
        && nDigits <= 9
        && is_integer(fargs[1], &nDigits)
        && nDigits <= 9)
     {
-        bResult = (mux_atol(fargs[0]) == mux_atol(fargs[1]));
+        long a = mux_atol(fargs[0]);
+        long b = mux_atol(fargs[1]);
+        bResult = (a == b);
     }
     else
     {
-        bResult = (  strcmp(fargs[0], fargs[1]) == 0
-                  || mux_atof(fargs[0]) == mux_atof(fargs[1]));
+        if (strcmp(fargs[0], fargs[1]) != 0)
+        {
+            double a = mux_atof(fargs[0]);
+            double b = mux_atof(fargs[1]);
+            bResult = (a == b);
+        }
     }
     safe_bool(bResult, buff, bufc);
 }
@@ -557,6 +606,7 @@ FUNCTION(fun_neq)
     UNUSED_PARAMETER(executor);
     UNUSED_PARAMETER(caller);
     UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(eval);
     UNUSED_PARAMETER(nfargs);
     UNUSED_PARAMETER(cargs);
     UNUSED_PARAMETER(ncargs);
@@ -568,12 +618,18 @@ FUNCTION(fun_neq)
        && is_integer(fargs[1], &nDigits)
        && nDigits <= 9)
     {
-        bResult = (mux_atol(fargs[0]) != mux_atol(fargs[1]));
+        long a = mux_atol(fargs[0]);
+        long b = mux_atol(fargs[1]);
+        bResult = (a != b);
     }
     else
     {
-        bResult = (  strcmp(fargs[0], fargs[1]) != 0
-                  && mux_atof(fargs[0]) != mux_atof(fargs[1]));
+        if (strcmp(fargs[0], fargs[1]) != 0)
+        {
+            double a = mux_atof(fargs[0]);
+            double b = mux_atof(fargs[1]);
+            bResult = (a != b);
+        }
     }
     safe_bool(bResult, buff, bufc);
 }
@@ -588,6 +644,7 @@ FUNCTION(fun_max)
     UNUSED_PARAMETER(executor);
     UNUSED_PARAMETER(caller);
     UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(eval);
     UNUSED_PARAMETER(cargs);
     UNUSED_PARAMETER(ncargs);
 
@@ -609,6 +666,7 @@ FUNCTION(fun_min)
     UNUSED_PARAMETER(executor);
     UNUSED_PARAMETER(caller);
     UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(eval);
     UNUSED_PARAMETER(cargs);
     UNUSED_PARAMETER(ncargs);
 
@@ -634,6 +692,7 @@ FUNCTION(fun_sign)
     UNUSED_PARAMETER(executor);
     UNUSED_PARAMETER(caller);
     UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(eval);
     UNUSED_PARAMETER(nfargs);
     UNUSED_PARAMETER(cargs);
     UNUSED_PARAMETER(ncargs);
@@ -656,6 +715,7 @@ FUNCTION(fun_isign)
     UNUSED_PARAMETER(executor);
     UNUSED_PARAMETER(caller);
     UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(eval);
     UNUSED_PARAMETER(nfargs);
     UNUSED_PARAMETER(cargs);
     UNUSED_PARAMETER(ncargs);
@@ -679,6 +739,7 @@ FUNCTION(fun_shl)
     UNUSED_PARAMETER(executor);
     UNUSED_PARAMETER(caller);
     UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(eval);
     UNUSED_PARAMETER(nfargs);
     UNUSED_PARAMETER(cargs);
     UNUSED_PARAMETER(ncargs);
@@ -701,6 +762,7 @@ FUNCTION(fun_shr)
     UNUSED_PARAMETER(executor);
     UNUSED_PARAMETER(caller);
     UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(eval);
     UNUSED_PARAMETER(nfargs);
     UNUSED_PARAMETER(cargs);
     UNUSED_PARAMETER(ncargs);
@@ -723,6 +785,7 @@ FUNCTION(fun_inc)
     UNUSED_PARAMETER(executor);
     UNUSED_PARAMETER(caller);
     UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(eval);
     UNUSED_PARAMETER(cargs);
     UNUSED_PARAMETER(ncargs);
 
@@ -741,6 +804,7 @@ FUNCTION(fun_dec)
     UNUSED_PARAMETER(executor);
     UNUSED_PARAMETER(caller);
     UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(eval);
     UNUSED_PARAMETER(cargs);
     UNUSED_PARAMETER(ncargs);
 
@@ -759,6 +823,7 @@ FUNCTION(fun_trunc)
     UNUSED_PARAMETER(executor);
     UNUSED_PARAMETER(caller);
     UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(eval);
     UNUSED_PARAMETER(nfargs);
     UNUSED_PARAMETER(cargs);
     UNUSED_PARAMETER(ncargs);
@@ -790,6 +855,7 @@ FUNCTION(fun_fdiv)
     UNUSED_PARAMETER(executor);
     UNUSED_PARAMETER(caller);
     UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(eval);
     UNUSED_PARAMETER(nfargs);
     UNUSED_PARAMETER(cargs);
     UNUSED_PARAMETER(ncargs);
@@ -826,6 +892,7 @@ FUNCTION(fun_idiv)
     UNUSED_PARAMETER(executor);
     UNUSED_PARAMETER(caller);
     UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(eval);
     UNUSED_PARAMETER(nfargs);
     UNUSED_PARAMETER(cargs);
     UNUSED_PARAMETER(ncargs);
@@ -850,6 +917,7 @@ FUNCTION(fun_floordiv)
     UNUSED_PARAMETER(executor);
     UNUSED_PARAMETER(caller);
     UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(eval);
     UNUSED_PARAMETER(nfargs);
     UNUSED_PARAMETER(cargs);
     UNUSED_PARAMETER(ncargs);
@@ -874,6 +942,7 @@ FUNCTION(fun_mod)
     UNUSED_PARAMETER(executor);
     UNUSED_PARAMETER(caller);
     UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(eval);
     UNUSED_PARAMETER(nfargs);
     UNUSED_PARAMETER(cargs);
     UNUSED_PARAMETER(ncargs);
@@ -895,6 +964,7 @@ FUNCTION(fun_remainder)
     UNUSED_PARAMETER(executor);
     UNUSED_PARAMETER(caller);
     UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(eval);
     UNUSED_PARAMETER(nfargs);
     UNUSED_PARAMETER(cargs);
     UNUSED_PARAMETER(ncargs);
@@ -920,12 +990,13 @@ FUNCTION(fun_abs)
     UNUSED_PARAMETER(executor);
     UNUSED_PARAMETER(caller);
     UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(eval);
     UNUSED_PARAMETER(nfargs);
     UNUSED_PARAMETER(cargs);
     UNUSED_PARAMETER(ncargs);
 
     double num = mux_atof(fargs[0]);
-    if (num == 0.0)
+    if (0.0 == num)
     {
         safe_chr('0', buff, bufc);
     }
@@ -946,6 +1017,7 @@ FUNCTION(fun_iabs)
     UNUSED_PARAMETER(executor);
     UNUSED_PARAMETER(caller);
     UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(eval);
     UNUSED_PARAMETER(nfargs);
     UNUSED_PARAMETER(cargs);
     UNUSED_PARAMETER(ncargs);
@@ -971,16 +1043,21 @@ FUNCTION(fun_dist2d)
     UNUSED_PARAMETER(executor);
     UNUSED_PARAMETER(caller);
     UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(eval);
     UNUSED_PARAMETER(nfargs);
     UNUSED_PARAMETER(cargs);
     UNUSED_PARAMETER(ncargs);
 
-    double d;
+    double a, b, d;
     double sum;
 
-    d = mux_atof(fargs[0]) - mux_atof(fargs[2]);
+    a = mux_atof(fargs[0]);
+    b = mux_atof(fargs[2]);
+    d = a - b;
     sum  = d * d;
-    d = mux_atof(fargs[1]) - mux_atof(fargs[3]);
+    a = mux_atof(fargs[1]);
+    b = mux_atof(fargs[3]);
+    d = a - b;
     sum += d * d;
 
     mux_FPRestore();
@@ -995,18 +1072,25 @@ FUNCTION(fun_dist3d)
     UNUSED_PARAMETER(executor);
     UNUSED_PARAMETER(caller);
     UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(eval);
     UNUSED_PARAMETER(nfargs);
     UNUSED_PARAMETER(cargs);
     UNUSED_PARAMETER(ncargs);
 
-    double d;
+    double a, b, d;
     double sum;
 
-    d = mux_atof(fargs[0]) - mux_atof(fargs[3]);
+    a = mux_atof(fargs[0]);
+    b = mux_atof(fargs[3]);
+    d = a - b;
     sum  = d * d;
-    d = mux_atof(fargs[1]) - mux_atof(fargs[4]);
+    a = mux_atof(fargs[1]);
+    b = mux_atof(fargs[4]);
+    d = a - b;
     sum += d * d;
-    d = mux_atof(fargs[2]) - mux_atof(fargs[5]);
+    a = mux_atof(fargs[2]);
+    b = mux_atof(fargs[5]);
+    d = a - b;
     sum += d * d;
 
     mux_FPRestore();
@@ -1032,18 +1116,22 @@ static void handle_vectors
     SEP *posep, int flag
 )
 {
-    char *v1[(LBUF_SIZE+1)/2], *v2[(LBUF_SIZE+1)/2];
-    double scalar;
-    int n, m, i;
-
-    // Split the list up, or return if the list is empty.
+    // Return if the list is empty.
     //
     if (!vecarg1 || !*vecarg1 || !vecarg2 || !*vecarg2)
     {
         return;
     }
-    n = list2arr(v1, (LBUF_SIZE+1)/2, vecarg1, psep);
-    m = list2arr(v2, (LBUF_SIZE+1)/2, vecarg2, psep);
+
+    char **v1 = new char *[(LBUF_SIZE+1)/2];
+    ISOUTOFMEMORY(v1);
+    char **v2 = new char *[(LBUF_SIZE+1)/2];
+    ISOUTOFMEMORY(v2);
+
+    // Split the list up, or return if the list is empty.
+    //
+    int n = list2arr(v1, (LBUF_SIZE+1)/2, vecarg1, psep);
+    int m = list2arr(v2, (LBUF_SIZE+1)/2, vecarg2, psep);
 
     // vmul() and vadd() accepts a scalar in the first or second arg,
     // but everything else has to be same-dimensional.
@@ -1056,9 +1144,14 @@ static void handle_vectors
               || m == 1)))
     {
         safe_str("#-1 VECTORS MUST BE SAME DIMENSIONS", buff, bufc);
+        delete [] v1;
+        delete [] v2;
         return;
     }
 
+    double scalar;
+    int i;
+
     switch (flag)
     {
     case VADD_F:
@@ -1251,6 +1344,8 @@ static void handle_vectors
         //
         safe_str("#-1 UNIMPLEMENTED", buff, bufc);
     }
+    delete [] v1;
+    delete [] v2;
 }
 
 FUNCTION(fun_vadd)
@@ -1345,7 +1440,6 @@ FUNCTION(fun_vmag)
         return;
     }
 
-    char *v1[LBUF_SIZE];
     int n, i;
     double tmp, res = 0.0;
 
@@ -1355,6 +1449,10 @@ FUNCTION(fun_vmag)
     {
         return;
     }
+
+    char **v1 = new char *[LBUF_SIZE];
+    ISOUTOFMEMORY(v1);
+
     n = list2arr(v1, LBUF_SIZE, fargs[0], &sep);
 
     // Calculate the magnitude.
@@ -1377,6 +1475,7 @@ FUNCTION(fun_vmag)
     {
         safe_chr('0', buff, bufc);
     }
+    delete [] v1;
 }
 
 FUNCTION(fun_vunit)
@@ -1387,7 +1486,6 @@ FUNCTION(fun_vunit)
         return;
     }
 
-    char *v1[LBUF_SIZE];
     int n, i;
     double tmp, res = 0.0;
 
@@ -1397,6 +1495,10 @@ FUNCTION(fun_vunit)
     {
         return;
     }
+
+    char **v1 = new char *[LBUF_SIZE];
+    ISOUTOFMEMORY(v1);
+
     n = list2arr(v1, LBUF_SIZE, fargs[0], &sep);
 
     // Calculate the magnitude.
@@ -1411,6 +1513,7 @@ FUNCTION(fun_vunit)
     {
         safe_str("#-1 CAN'T MAKE UNIT VECTOR FROM ZERO-LENGTH VECTOR",
             buff, bufc);
+        delete [] v1;
         return;
     }
     for (i = 0; i < n; i++)
@@ -1426,6 +1529,7 @@ FUNCTION(fun_vunit)
 
         fval(buff, bufc, mux_atof(v1[i]) / result);
     }
+    delete [] v1;
 }
 
 FUNCTION(fun_floor)
@@ -1433,6 +1537,7 @@ FUNCTION(fun_floor)
     UNUSED_PARAMETER(executor);
     UNUSED_PARAMETER(caller);
     UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(eval);
     UNUSED_PARAMETER(nfargs);
     UNUSED_PARAMETER(cargs);
     UNUSED_PARAMETER(ncargs);
@@ -1461,6 +1566,7 @@ FUNCTION(fun_ceil)
     UNUSED_PARAMETER(executor);
     UNUSED_PARAMETER(caller);
     UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(eval);
     UNUSED_PARAMETER(nfargs);
     UNUSED_PARAMETER(cargs);
     UNUSED_PARAMETER(ncargs);
@@ -1489,6 +1595,7 @@ FUNCTION(fun_round)
     UNUSED_PARAMETER(executor);
     UNUSED_PARAMETER(caller);
     UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(eval);
     UNUSED_PARAMETER(nfargs);
     UNUSED_PARAMETER(cargs);
     UNUSED_PARAMETER(ncargs);
@@ -1520,6 +1627,7 @@ FUNCTION(fun_pi)
     UNUSED_PARAMETER(executor);
     UNUSED_PARAMETER(caller);
     UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(eval);
     UNUSED_PARAMETER(fargs);
     UNUSED_PARAMETER(nfargs);
     UNUSED_PARAMETER(cargs);
@@ -1533,6 +1641,7 @@ FUNCTION(fun_e)
     UNUSED_PARAMETER(executor);
     UNUSED_PARAMETER(caller);
     UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(eval);
     UNUSED_PARAMETER(fargs);
     UNUSED_PARAMETER(nfargs);
     UNUSED_PARAMETER(cargs);
@@ -1584,6 +1693,7 @@ FUNCTION(fun_ctu)
     UNUSED_PARAMETER(executor);
     UNUSED_PARAMETER(caller);
     UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(eval);
     UNUSED_PARAMETER(nfargs);
     UNUSED_PARAMETER(cargs);
     UNUSED_PARAMETER(ncargs);
@@ -1599,6 +1709,7 @@ FUNCTION(fun_sin)
     UNUSED_PARAMETER(executor);
     UNUSED_PARAMETER(caller);
     UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(eval);
     UNUSED_PARAMETER(cargs);
     UNUSED_PARAMETER(ncargs);
 
@@ -1620,6 +1731,7 @@ FUNCTION(fun_cos)
     UNUSED_PARAMETER(executor);
     UNUSED_PARAMETER(caller);
     UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(eval);
     UNUSED_PARAMETER(cargs);
     UNUSED_PARAMETER(ncargs);
 
@@ -1641,6 +1753,7 @@ FUNCTION(fun_tan)
     UNUSED_PARAMETER(executor);
     UNUSED_PARAMETER(caller);
     UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(eval);
     UNUSED_PARAMETER(cargs);
     UNUSED_PARAMETER(ncargs);
 
@@ -1662,6 +1775,7 @@ FUNCTION(fun_asin)
     UNUSED_PARAMETER(executor);
     UNUSED_PARAMETER(caller);
     UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(eval);
     UNUSED_PARAMETER(cargs);
     UNUSED_PARAMETER(ncargs);
 
@@ -1689,6 +1803,7 @@ FUNCTION(fun_acos)
     UNUSED_PARAMETER(executor);
     UNUSED_PARAMETER(caller);
     UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(eval);
     UNUSED_PARAMETER(cargs);
     UNUSED_PARAMETER(ncargs);
 
@@ -1716,6 +1831,7 @@ FUNCTION(fun_atan)
     UNUSED_PARAMETER(executor);
     UNUSED_PARAMETER(caller);
     UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(eval);
     UNUSED_PARAMETER(cargs);
     UNUSED_PARAMETER(ncargs);
 
@@ -1737,6 +1853,7 @@ FUNCTION(fun_exp)
     UNUSED_PARAMETER(executor);
     UNUSED_PARAMETER(caller);
     UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(eval);
     UNUSED_PARAMETER(nfargs);
     UNUSED_PARAMETER(cargs);
     UNUSED_PARAMETER(ncargs);
@@ -1755,6 +1872,7 @@ FUNCTION(fun_power)
     UNUSED_PARAMETER(executor);
     UNUSED_PARAMETER(caller);
     UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(eval);
     UNUSED_PARAMETER(nfargs);
     UNUSED_PARAMETER(cargs);
     UNUSED_PARAMETER(ncargs);
@@ -1773,13 +1891,14 @@ FUNCTION(fun_power)
         mux_FPRestore();
         val = pow(val1, val2);
         mux_FPSet();
+        fval(buff, bufc, val);
     }
 #else
     mux_FPRestore();
     val = pow(val1, val2);
     mux_FPSet();
-#endif
     fval(buff, bufc, val);
+#endif
 }
 
 FUNCTION(fun_fmod)
@@ -1787,6 +1906,7 @@ FUNCTION(fun_fmod)
     UNUSED_PARAMETER(executor);
     UNUSED_PARAMETER(caller);
     UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(eval);
     UNUSED_PARAMETER(nfargs);
     UNUSED_PARAMETER(cargs);
     UNUSED_PARAMETER(ncargs);
@@ -1805,13 +1925,14 @@ FUNCTION(fun_fmod)
         mux_FPRestore();
         val = fmod(val1, val2);
         mux_FPSet();
+        fval(buff, bufc, val);
     }
 #else
     mux_FPRestore();
     val = fmod(val1, val2);
     mux_FPSet();
-#endif
     fval(buff, bufc, val);
+#endif
 }
 
 FUNCTION(fun_ln)
@@ -1819,6 +1940,7 @@ FUNCTION(fun_ln)
     UNUSED_PARAMETER(executor);
     UNUSED_PARAMETER(caller);
     UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(eval);
     UNUSED_PARAMETER(nfargs);
     UNUSED_PARAMETER(cargs);
     UNUSED_PARAMETER(ncargs);
@@ -1854,6 +1976,7 @@ FUNCTION(fun_log)
     UNUSED_PARAMETER(executor);
     UNUSED_PARAMETER(caller);
     UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(eval);
     UNUSED_PARAMETER(nfargs);
     UNUSED_PARAMETER(cargs);
     UNUSED_PARAMETER(ncargs);
@@ -1889,6 +2012,7 @@ FUNCTION(fun_sqrt)
     UNUSED_PARAMETER(executor);
     UNUSED_PARAMETER(caller);
     UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(eval);
     UNUSED_PARAMETER(nfargs);
     UNUSED_PARAMETER(cargs);
     UNUSED_PARAMETER(ncargs);
@@ -1928,6 +2052,7 @@ FUNCTION(fun_isnum)
     UNUSED_PARAMETER(executor);
     UNUSED_PARAMETER(caller);
     UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(eval);
     UNUSED_PARAMETER(nfargs);
     UNUSED_PARAMETER(cargs);
     UNUSED_PARAMETER(ncargs);
@@ -1944,6 +2069,7 @@ FUNCTION(fun_israt)
     UNUSED_PARAMETER(executor);
     UNUSED_PARAMETER(caller);
     UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(eval);
     UNUSED_PARAMETER(nfargs);
     UNUSED_PARAMETER(cargs);
     UNUSED_PARAMETER(ncargs);
@@ -1960,6 +2086,7 @@ FUNCTION(fun_isint)
     UNUSED_PARAMETER(executor);
     UNUSED_PARAMETER(caller);
     UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(eval);
     UNUSED_PARAMETER(nfargs);
     UNUSED_PARAMETER(cargs);
     UNUSED_PARAMETER(ncargs);
@@ -1972,6 +2099,7 @@ FUNCTION(fun_and)
     UNUSED_PARAMETER(executor);
     UNUSED_PARAMETER(caller);
     UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(eval);
     UNUSED_PARAMETER(cargs);
     UNUSED_PARAMETER(ncargs);
 
@@ -1988,6 +2116,7 @@ FUNCTION(fun_or)
     UNUSED_PARAMETER(executor);
     UNUSED_PARAMETER(caller);
     UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(eval);
     UNUSED_PARAMETER(cargs);
     UNUSED_PARAMETER(ncargs);
 
@@ -2004,6 +2133,7 @@ FUNCTION(fun_andbool)
     UNUSED_PARAMETER(executor);
     UNUSED_PARAMETER(caller);
     UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(eval);
     UNUSED_PARAMETER(cargs);
     UNUSED_PARAMETER(ncargs);
 
@@ -2020,6 +2150,7 @@ FUNCTION(fun_orbool)
     UNUSED_PARAMETER(executor);
     UNUSED_PARAMETER(caller);
     UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(eval);
     UNUSED_PARAMETER(cargs);
     UNUSED_PARAMETER(ncargs);
 
@@ -2040,7 +2171,7 @@ FUNCTION(fun_cand)
         char *bp = temp;
         char *str = fargs[i];
         mux_exec(temp, &bp, executor, caller, enactor,
-            EV_STRIP_CURLY | EV_FCHECK | EV_EVAL, &str, cargs, ncargs);
+            eval|EV_STRIP_CURLY|EV_FCHECK|EV_EVAL, &str, cargs, ncargs);
         *bp = '\0';
         val = isTRUE(mux_atol(temp));
     }
@@ -2057,7 +2188,7 @@ FUNCTION(fun_cor)
         char *bp = temp;
         char *str = fargs[i];
         mux_exec(temp, &bp, executor, caller, enactor,
-            EV_STRIP_CURLY | EV_FCHECK | EV_EVAL, &str, cargs, ncargs);
+            eval|EV_STRIP_CURLY|EV_FCHECK|EV_EVAL, &str, cargs, ncargs);
         *bp = '\0';
         val = isTRUE(mux_atol(temp));
     }
@@ -2074,7 +2205,7 @@ FUNCTION(fun_candbool)
         char *bp = temp;
         char *str = fargs[i];
         mux_exec(temp, &bp, executor, caller, enactor,
-            EV_STRIP_CURLY | EV_FCHECK | EV_EVAL, &str, cargs, ncargs);
+            eval|EV_STRIP_CURLY|EV_FCHECK|EV_EVAL, &str, cargs, ncargs);
         *bp = '\0';
         val = xlate(temp);
     }
@@ -2091,7 +2222,7 @@ FUNCTION(fun_corbool)
         char *bp = temp;
         char *str = fargs[i];
         mux_exec(temp, &bp, executor, caller, enactor,
-            EV_STRIP_CURLY | EV_FCHECK | EV_EVAL, &str, cargs, ncargs);
+            eval|EV_STRIP_CURLY|EV_FCHECK|EV_EVAL, &str, cargs, ncargs);
         *bp = '\0';
         val = xlate(temp);
     }
@@ -2104,6 +2235,7 @@ FUNCTION(fun_xor)
     UNUSED_PARAMETER(executor);
     UNUSED_PARAMETER(caller);
     UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(eval);
     UNUSED_PARAMETER(cargs);
     UNUSED_PARAMETER(ncargs);
 
@@ -2121,6 +2253,7 @@ FUNCTION(fun_not)
     UNUSED_PARAMETER(executor);
     UNUSED_PARAMETER(caller);
     UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(eval);
     UNUSED_PARAMETER(nfargs);
     UNUSED_PARAMETER(cargs);
     UNUSED_PARAMETER(ncargs);
@@ -2133,6 +2266,7 @@ FUNCTION(fun_t)
     UNUSED_PARAMETER(executor);
     UNUSED_PARAMETER(caller);
     UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(eval);
     UNUSED_PARAMETER(cargs);
     UNUSED_PARAMETER(ncargs);
 
@@ -2211,9 +2345,9 @@ public:
 
 private:
     void TwoDigits(const char *p);
-    void ThreeDigits(const char *p, int iBigOne);
-    void ManyDigits(int n, const char *p, bool bHundreds);
-    void FractionalDigits(int n, const char *p);
+    void ThreeDigits(const char *p, size_t iBigOne);
+    void ManyDigits(size_t n, const char *p, bool bHundreds);
+    void FractionalDigits(size_t n, const char *p);
 
     void StartWord(void);
     void AddWord(const char *p);
@@ -2275,7 +2409,7 @@ void CSpellNum::TwoDigits(const char *p)
 
 // Handle three-character sequences.
 //
-void CSpellNum::ThreeDigits(const char *p, int iBigOne)
+void CSpellNum::ThreeDigits(const char *p, size_t iBigOne)
 {
     if (  p[0] == '0'
        && p[1] == '0'
@@ -2303,7 +2437,7 @@ void CSpellNum::ThreeDigits(const char *p, int iBigOne)
 
 // Handle a series of patterns of three.
 //
-void CSpellNum::ManyDigits(int n, const char *p, bool bHundreds)
+void CSpellNum::ManyDigits(size_t n, const char *p, bool bHundreds)
 {
     // Handle special Hundreds cases.
     //
@@ -2320,15 +2454,15 @@ void CSpellNum::ManyDigits(int n, const char *p, bool bHundreds)
 
     // Handle normal cases.
     //
-    int ndiv = ((n + 2) / 3) - 1;
-    int nrem = n % 3;
+    size_t ndiv = ((n + 2) / 3) - 1;
+    size_t nrem = n % 3;
     char buf[3];
     if (nrem == 0)
     {
         nrem = 3;
     }
 
-    int j = nrem;
+    size_t j = nrem;
     for (int i = 2; 0 <= i; i--)
     {
         if (j)
@@ -2352,14 +2486,14 @@ void CSpellNum::ManyDigits(int n, const char *p, bool bHundreds)
 
 // Handle precision ending for part to the right of the decimal place.
 //
-void CSpellNum::FractionalDigits(int n, const char *p)
+void CSpellNum::FractionalDigits(size_t n, const char *p)
 {
     ManyDigits(n, p, false);
     if (  0 < n
        && n < 15)
     {
-        int d = n / 3;
-        int r = n % 3;
+        size_t d = n / 3;
+        size_t r = n % 3;
         StartWord();
         if (r != 0)
         {
@@ -2469,6 +2603,7 @@ FUNCTION(fun_spellnum)
     UNUSED_PARAMETER(executor);
     UNUSED_PARAMETER(caller);
     UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(eval);
     UNUSED_PARAMETER(nfargs);
     UNUSED_PARAMETER(cargs);
     UNUSED_PARAMETER(ncargs);
@@ -2482,6 +2617,7 @@ FUNCTION(fun_roman)
     UNUSED_PARAMETER(executor);
     UNUSED_PARAMETER(caller);
     UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(eval);
     UNUSED_PARAMETER(nfargs);
     UNUSED_PARAMETER(cargs);
     UNUSED_PARAMETER(ncargs);
@@ -2585,6 +2721,7 @@ FUNCTION(fun_land)
     UNUSED_PARAMETER(executor);
     UNUSED_PARAMETER(caller);
     UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(eval);
     UNUSED_PARAMETER(cargs);
     UNUSED_PARAMETER(ncargs);
 
@@ -2612,6 +2749,7 @@ FUNCTION(fun_lor)
     UNUSED_PARAMETER(executor);
     UNUSED_PARAMETER(caller);
     UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(eval);
     UNUSED_PARAMETER(cargs);
     UNUSED_PARAMETER(ncargs);
 
@@ -2639,6 +2777,7 @@ FUNCTION(fun_band)
     UNUSED_PARAMETER(executor);
     UNUSED_PARAMETER(caller);
     UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(eval);
     UNUSED_PARAMETER(cargs);
     UNUSED_PARAMETER(ncargs);
 
@@ -2663,6 +2802,7 @@ FUNCTION(fun_bor)
     UNUSED_PARAMETER(executor);
     UNUSED_PARAMETER(caller);
     UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(eval);
     UNUSED_PARAMETER(cargs);
     UNUSED_PARAMETER(ncargs);
 
@@ -2687,6 +2827,7 @@ FUNCTION(fun_bnand)
     UNUSED_PARAMETER(executor);
     UNUSED_PARAMETER(caller);
     UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(eval);
     UNUSED_PARAMETER(nfargs);
     UNUSED_PARAMETER(cargs);
     UNUSED_PARAMETER(ncargs);
@@ -2709,6 +2850,7 @@ FUNCTION(fun_bxor)
     UNUSED_PARAMETER(executor);
     UNUSED_PARAMETER(caller);
     UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(eval);
     UNUSED_PARAMETER(cargs);
     UNUSED_PARAMETER(ncargs);
 
@@ -2733,13 +2875,14 @@ FUNCTION(fun_crc32)
     UNUSED_PARAMETER(executor);
     UNUSED_PARAMETER(caller);
     UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(eval);
     UNUSED_PARAMETER(cargs);
     UNUSED_PARAMETER(ncargs);
 
     UINT32 ulCRC32 = 0;
     for (int i = 0; i < nfargs; i++)
     {
-        int n = strlen(fargs[i]);
+        size_t n = strlen(fargs[i]);
         ulCRC32 = CRC32_ProcessBuffer(ulCRC32, fargs[i], n);
     }
     safe_i64toa(ulCRC32, buff, bufc);
@@ -2750,6 +2893,7 @@ FUNCTION(fun_sha1)
     UNUSED_PARAMETER(executor);
     UNUSED_PARAMETER(caller);
     UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(eval);
     UNUSED_PARAMETER(cargs);
     UNUSED_PARAMETER(ncargs);
 
@@ -2764,7 +2908,7 @@ FUNCTION(fun_sha1)
     for (i = 0; i <= 4; i++)
     {
         char buf[9];
-        sprintf(buf, "%08X", shac.H[i]);
+        mux_sprintf(buf, sizeof(buf), "%08X", shac.H[i]);
         safe_str(buf, buff, bufc);
     }
 }
