@@ -1,6 +1,6 @@
 // game.cpp
 //
-// $Id: game.cpp,v 1.93 2006/03/12 22:45:04 sdennis Exp $
+// $Id: game.cpp 3050 2007-12-30 06:36:06Z brazilofmux $
 //
 #include "copyright.h"
 #include "autoconf.h"
@@ -22,7 +22,13 @@
 #include "help.h"
 #ifdef REALITY_LVLS
 #include "levels.h"
-#endif /* REALITY_LVLS */
+#endif // REALITY_LVLS
+
+#if defined(FIRANMUX)
+#include <mysql.h>
+
+MYSQL *mush_database = NULL;
+#endif // FIRANMUX
 
 void do_dump(dbref executor, dbref caller, dbref enactor, int key)
 {
@@ -111,7 +117,7 @@ bool regexp_match
      * Now we try to match the pattern. The relevant fields will
      * automatically be filled in by this.
      */
-    matches = pcre_exec(re, NULL, str, strlen(str), 0, 0, ovec, ovecsize);
+    matches = pcre_exec(re, NULL, str, static_cast<int>(strlen(str)), 0, 0, ovec, ovecsize);
     if (matches < 0)
     {
         delete [] ovec;
@@ -174,14 +180,36 @@ static int atr_match1
     }
 
     int match = 0;
-    if (  AMATCH_CMD == type
-       && mudstate.bfNoCommands.IsSet(parent))
-    {
-        return match;
-    }
-    else if ( AMATCH_LISTEN == type
-            && mudstate.bfNoListens.IsSet(parent))
+    if (  (  AMATCH_CMD    == type
+          && mudstate.bfNoCommands.IsSet(parent))
+       || (  AMATCH_LISTEN == type
+          && mudstate.bfNoListens.IsSet(parent)))
     {
+        // We may need to process the hash_insert to support exclusion of
+        // this parent's $-commands before we return no matches.
+        //
+        if (hash_insert)
+        {
+            // Because we know this object contains no commands, there is no
+            // need to look at the attribute values.
+            //
+            char *as;
+            atr_push();
+            for (int atr = atr_head(parent, &as); atr; atr = atr_next(&as))
+            {
+                ATTR *ap = atr_num(atr);
+
+                if (  ap
+                   && 0 == (ap->flags & AF_NOPROG)
+                   && (  !check_exclude
+                      || (  0 == (ap->flags & AF_PRIVATE)
+                         && !hashfindLEN(&(ap->number), sizeof(ap->number), &mudstate.parent_htab))))
+                {
+                    hashaddLEN(&(ap->number), sizeof(ap->number), &atr, &mudstate.parent_htab);
+                }
+            }
+            atr_pop();
+        }
         return match;
     }
 
@@ -211,14 +239,10 @@ static int atr_match1
         char buff[LBUF_SIZE];
         atr_get_str(buff, parent, atr, &aowner, &aflags);
 
-        if (aflags & AF_NOPROG)
-        {
-            continue;
-        }
-
         char *s = NULL;
-        if (  AMATCH_CMD    == buff[0]
-           || AMATCH_LISTEN == buff[0])
+        if (  0 == (aflags & AF_NOPROG)
+           &&  (  AMATCH_CMD    == buff[0]
+               || AMATCH_LISTEN == buff[0]))
         {
             s = strchr(buff+1, ':');
             if (s)
@@ -253,6 +277,11 @@ static int atr_match1
             hashaddLEN(&(ap->number), sizeof(ap->number), &atr, &mudstate.parent_htab);
         }
 
+        if (aflags & AF_NOPROG)
+        {
+            continue;
+        }
+
         // Check for the leadin character after excluding the attrib.
         // This lets non-command attribs on the child block commands
         // on the parent.
@@ -280,8 +309,11 @@ static int atr_match1
         {
             match = 1;
             CLinearTimeAbsolute lta;
-            wait_que(thing, player, player, false, lta, NOTHING, 0, s,
-                args, NUM_ENV_VARS, mudstate.global_regs);
+            wait_que(thing, player, player, AttrTrace(aflags, 0), false, lta,
+                NOTHING, 0,
+                s,
+                NUM_ENV_VARS, args,
+                mudstate.global_regs);
 
             for (int i = 0; i < NUM_ENV_VARS; i++)
             {
@@ -392,44 +424,55 @@ bool atr_match
  * optionally notify the contents, neighbors, and location also.
  */
 
-static bool check_filter(dbref object, dbref player, int filter, const char *msg)
+static bool check_filter(dbref object, dbref player, int filter, const char *msg_arg)
 {
     int aflags;
     dbref aowner;
-    char *buf, *nbuf, *cp, *dp, *str;
-
-    buf = atr_pget(object, filter, &aowner, &aflags);
+    char *buf = atr_pget(object, filter, &aowner, &aflags);
     if (!*buf)
     {
         free_lbuf(buf);
         return true;
     }
-    char **preserve = NULL;
-    int *preserve_len = NULL;
-    preserve = PushPointers(MAX_GLOBAL_REGS);
-    preserve_len = PushIntegers(MAX_GLOBAL_REGS);
-    save_global_regs("check_filter_save", preserve, preserve_len);
-    nbuf = dp = alloc_lbuf("check_filter");
-    str = buf;
+
+    // Strip ANSI from msg.
+    //
+    char *msg = alloc_lbuf("check_filter.msg");
+    char *mp = msg;
+    size_t nmsg;
+    char *msg_stripped = strip_ansi(msg_arg, &nmsg);
+    safe_copy_buf(msg_stripped, nmsg, msg, &mp);
+    *mp = '\0';
+
+    reg_ref **preserve = NULL;
+    preserve = PushRegisters(MAX_GLOBAL_REGS);
+    save_global_regs(preserve);
+
+    char *nbuf = alloc_lbuf("check_filter");
+    char *dp = nbuf;
+    char *str = buf;
     mux_exec(nbuf, &dp, object, player, player,
-             EV_FIGNORE | EV_EVAL | EV_TOP, &str, (char **)NULL, 0);
+        AttrTrace(aflags, EV_FIGNORE|EV_EVAL|EV_TOP), &str,
+        NULL, 0);
     *dp = '\0';
     dp = nbuf;
     free_lbuf(buf);
-    restore_global_regs("check_filter_restore", preserve, preserve_len);
-    PopIntegers(preserve_len, MAX_GLOBAL_REGS);
-    PopPointers(preserve, MAX_GLOBAL_REGS);
+
+    restore_global_regs(preserve);
+    PopRegisters(preserve, MAX_GLOBAL_REGS);
+    preserve = NULL;
 
     if (!(aflags & AF_REGEXP))
     {
         do
         {
-            cp = parse_to(&dp, ',', EV_STRIP_CURLY);
+            char *cp = parse_to(&dp, ',', EV_STRIP_CURLY);
             mudstate.wild_invk_ctr = 0;
             if (  MuxAlarm.bAlarmed
                || quick_wild(cp, msg))
             {
                 free_lbuf(nbuf);
+                free_lbuf(msg);
                 return false;
             }
         } while (dp != NULL);
@@ -441,19 +484,20 @@ static bool check_filter(dbref object, dbref player, int filter, const char *msg
         {
             int erroffset;
             const char *errptr;
-            cp = parse_to(&dp, ',', EV_STRIP_CURLY);
+            char *cp = parse_to(&dp, ',', EV_STRIP_CURLY);
             pcre *re;
             if (  !MuxAlarm.bAlarmed
                && (re = pcre_compile(cp, case_opt, &errptr, &erroffset, NULL)) != NULL)
             {
                 const int ovecsize = 33;
                 int ovec[ovecsize];
-                int matches = pcre_exec(re, NULL, msg, strlen(msg), 0, 0,
+                int matches = pcre_exec(re, NULL, msg, static_cast<int>(strlen(msg)), 0, 0,
                     ovec, ovecsize);
                 if (0 <= matches)
                 {
                     MEMFREE(re);
                     free_lbuf(nbuf);
+                    free_lbuf(msg);
                     return false;
                 }
                 MEMFREE(re);
@@ -461,6 +505,7 @@ static bool check_filter(dbref object, dbref player, int filter, const char *msg
         } while (dp != NULL);
     }
     free_lbuf(nbuf);
+    free_lbuf(msg);
     return true;
 }
 
@@ -479,21 +524,19 @@ static char *add_prefix(dbref object, dbref player, int prefix,
     }
     else
     {
-        char **preserve = NULL;
-        int *preserve_len = NULL;
-        preserve = PushPointers(MAX_GLOBAL_REGS);
-        preserve_len = PushIntegers(MAX_GLOBAL_REGS);
-        save_global_regs("add_prefix_save", preserve, preserve_len);
+        reg_ref **preserve = NULL;
+        preserve = PushRegisters(MAX_GLOBAL_REGS);
+        save_global_regs(preserve);
 
         nbuf = cp = alloc_lbuf("add_prefix");
         str = buf;
         mux_exec(nbuf, &cp, object, player, player,
-                 EV_FIGNORE | EV_EVAL | EV_TOP, &str, (char **)NULL, 0);
+            AttrTrace(aflags, EV_FIGNORE|EV_EVAL|EV_TOP), &str,
+            NULL, 0);
         free_lbuf(buf);
 
-        restore_global_regs("add_prefix_restore", preserve, preserve_len);
-        PopIntegers(preserve_len, MAX_GLOBAL_REGS);
-        PopPointers(preserve, MAX_GLOBAL_REGS);
+        restore_global_regs(preserve);
+        PopRegisters(preserve, MAX_GLOBAL_REGS);
 
         buf = nbuf;
     }
@@ -514,11 +557,11 @@ static char *dflt_from_msg(dbref sender, dbref sendloc)
     safe_str("From ", tbuff, &tp);
     if (Good_obj(sendloc))
     {
-        safe_str(Name(sendloc), tbuff, &tp);
+        safe_str(Moniker(sendloc), tbuff, &tp);
     }
     else
     {
-        safe_str(Name(sender), tbuff, &tp);
+        safe_str(Moniker(sender), tbuff, &tp);
     }
     safe_chr(',', tbuff, &tp);
     *tp = '\0';
@@ -647,19 +690,19 @@ void notify_check(dbref target, dbref sender, const char *msg, int key)
             //
             tbuff = alloc_sbuf("notify_check.nospoof");
             safe_chr('[', msg_ns, &mp);
-            safe_str(Name(sender), msg_ns, &mp);
-            sprintf(tbuff, "(#%d)", sender);
+            safe_str(Moniker(sender), msg_ns, &mp);
+            mux_sprintf(tbuff, SBUF_SIZE, "(#%d)", sender);
             safe_str(tbuff, msg_ns, &mp);
 
             if (sender != Owner(sender))
             {
                 safe_chr('{', msg_ns, &mp);
-                safe_str(Name(Owner(sender)), msg_ns, &mp);
+                safe_str(Moniker(Owner(sender)), msg_ns, &mp);
                 safe_chr('}', msg_ns, &mp);
             }
             if (sender != mudstate.curr_enactor)
             {
-                sprintf(tbuff, "<-(#%d)", mudstate.curr_enactor);
+                mux_sprintf(tbuff, SBUF_SIZE, "<-(#%d)", mudstate.curr_enactor);
                 safe_str(tbuff, msg_ns, &mp);
             }
             safe_str("] ", msg_ns, &mp);
@@ -736,7 +779,7 @@ void notify_check(dbref target, dbref sender, const char *msg, int key)
                  && targetloc != Owner(target))))
         {
             tp = tbuff = alloc_lbuf("notify_check.puppet");
-            safe_str(Name(target), tbuff, &tp);
+            safe_str(Moniker(target), tbuff, &tp);
             safe_str("> ", tbuff, &tp);
             safe_str(msg_ns, tbuff, &tp);
             *tp = '\0';
@@ -786,14 +829,14 @@ void notify_check(dbref target, dbref sender, const char *msg, int key)
             mudstate.nHearNest++;
             if (sender != target)
             {
-                did_it(sender, target, 0, NULL, 0, NULL, A_AHEAR, args, nargs);
+                did_it(sender, target, 0, NULL, 0, NULL, A_AHEAR, 0, args, nargs);
             }
             else
             {
-                did_it(sender, target, 0, NULL, 0, NULL, A_AMHEAR, args,
+                did_it(sender, target, 0, NULL, 0, NULL, A_AMHEAR, 0, args,
                     nargs);
             }
-            did_it(sender, target, 0, NULL, 0, NULL, A_AAHEAR, args, nargs);
+            did_it(sender, target, 0, NULL, 0, NULL, A_AAHEAR, 0, args, nargs);
             mudstate.nHearNest--;
         }
 
@@ -1173,12 +1216,14 @@ void do_shutdown
     dbref executor,
     dbref caller,
     dbref enactor,
+    int   eval,
     int   key,
     char *message
 )
 {
     UNUSED_PARAMETER(caller);
     UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(eval);
 
     if (!Can_SiteAdmin(executor))
     {
@@ -1186,7 +1231,7 @@ void do_shutdown
         return;
     }
 
-    raw_broadcast(0, "GAME: Shutdown by %s", Name(Owner(executor)));
+    raw_broadcast(0, "GAME: Shutdown by %s", Moniker(Owner(executor)));
     STARTLOG(LOG_ALWAYS, "WIZ", "SHTDN");
     log_text("Shutdown by ");
     log_name(executor);
@@ -1197,13 +1242,13 @@ void do_shutdown
     log_text(message);
     ENDLOG;
 
-    int fd = open(mudconf.status_file, O_RDWR | O_CREAT | O_TRUNC | O_BINARY, 0600);
-    if (fd != -1)
+    int fd;
+    if (mux_open(&fd, mudconf.status_file, O_RDWR|O_CREAT|O_TRUNC|O_BINARY))
     {
-        write(fd, message, strlen(message));
-        write(fd, ENDLINE, sizeof(ENDLINE)-1);
+        mux_write(fd, message, static_cast<unsigned int>(strlen(message)));
+        mux_write(fd, ENDLINE, sizeof(ENDLINE)-1);
         DebugTotalFiles++;
-        if (close(fd) == 0)
+        if (mux_close(fd) == 0)
         {
             DebugTotalFiles--;
         }
@@ -1268,10 +1313,10 @@ void do_shutdown
 typedef struct
 {
     char **ppszOutputBase;
-    char szOutputSuffix[14];
+    const char *szOutputSuffix;
     bool bUseTemporary;
     int  fType;
-    char *pszErrorMessage;
+    const char *pszErrorMessage;
 } DUMP_PROCEDURE;
 
 static DUMP_PROCEDURE DumpProcedures[NUM_DUMP_TYPES] =
@@ -1327,21 +1372,22 @@ void dump_database_internal(int dump_type)
     if (0 < dump_type)
     {
         DUMP_PROCEDURE *dp = &DumpProcedures[dump_type];
+        bool bOpen;
 
-        sprintf(outfn, "%s%s", *(dp->ppszOutputBase), dp->szOutputSuffix);
+        mux_sprintf(outfn, sizeof(outfn), "%s%s", *(dp->ppszOutputBase), dp->szOutputSuffix);
         if (dp->bUseTemporary)
         {
-            sprintf(tmpfile, "%s.#%d#", outfn, mudstate.epoch);
+            mux_sprintf(tmpfile, sizeof(tmpfile), "%s.#%d#", outfn, mudstate.epoch);
             RemoveFile(tmpfile);
-            f = fopen(tmpfile, "wb");
+            bOpen = mux_fopen(&f, tmpfile, "wb");
         }
         else
         {
             RemoveFile(outfn);
-            f = fopen(outfn, "wb");
+            bOpen = mux_fopen(&f, outfn, "wb");
         }
 
-        if (f)
+        if (bOpen)
         {
             DebugTotalFiles++;
             setvbuf(f, NULL, _IOFBF, 16384);
@@ -1365,8 +1411,7 @@ void dump_database_internal(int dump_type)
         {
             if (mudconf.have_mailer)
             {
-                f = fopen(mudconf.mail_db, "wb");
-                if (f)
+                if (mux_fopen(&f, mudconf.mail_db, "wb"))
                 {
                     DebugTotalFiles++;
                     dump_mail(f);
@@ -1388,12 +1433,11 @@ void dump_database_internal(int dump_type)
     //
     if (mudconf.compress_db)
     {
-        sprintf(prevfile, "%s.prev.gz", mudconf.outdb);
-        sprintf(tmpfile, "%s.#%d#.gz", mudconf.outdb, mudstate.epoch - 1);
+        mux_sprintf(prevfile, sizeof(prevfile), "%s.prev.gz", mudconf.outdb);
+        mux_sprintf(tmpfile, sizeof(tmpfile), "%s.#%d#.gz", mudconf.outdb, mudstate.epoch - 1);
         RemoveFile(tmpfile);
-        sprintf(tmpfile, "%s.#%d#.gz", mudconf.outdb, mudstate.epoch);
-        strcpy(outfn, mudconf.outdb);
-        strcat(outfn, ".gz");
+        mux_sprintf(tmpfile, sizeof(tmpfile), "%s.#%d#.gz", mudconf.outdb, mudstate.epoch);
+        mux_sprintf(outfn, sizeof(outfn), "%s.gz", mudconf.outdb);
 
         f = popen(tprintf("%s > %s", mudconf.compress, tmpfile), POPEN_WRITE_OP);
         if (f)
@@ -1418,13 +1462,12 @@ void dump_database_internal(int dump_type)
     }
     else
     {
-        sprintf(prevfile, "%s.prev", mudconf.outdb);
-        sprintf(tmpfile, "%s.#%d#", mudconf.outdb, mudstate.epoch - 1);
+        mux_sprintf(prevfile, sizeof(prevfile), "%s.prev", mudconf.outdb);
+        mux_sprintf(tmpfile, sizeof(tmpfile), "%s.#%d#", mudconf.outdb, mudstate.epoch - 1);
         RemoveFile(tmpfile);
-        sprintf(tmpfile, "%s.#%d#", mudconf.outdb, mudstate.epoch);
+        mux_sprintf(tmpfile, sizeof(tmpfile), "%s.#%d#", mudconf.outdb, mudstate.epoch);
 
-        f = fopen(tmpfile, "wb");
-        if (f)
+        if (mux_fopen(&f, tmpfile, "wb"))
         {
             DebugTotalFiles++;
             setvbuf(f, NULL, _IOFBF, 16384);
@@ -1447,8 +1490,7 @@ void dump_database_internal(int dump_type)
 
     if (mudconf.have_mailer)
     {
-        f = fopen(mudconf.mail_db, "wb");
-        if (f)
+        if (mux_fopen(&f, mudconf.mail_db, "wb"))
         {
             DebugTotalFiles++;
             dump_mail(f);
@@ -1490,7 +1532,7 @@ static void dump_database(void)
     mudstate.dumped  = 0;
 #endif
     buff = alloc_mbuf("dump_database");
-    sprintf(buff, "%s.#%d#", mudconf.outdb, mudstate.epoch);
+    mux_sprintf(buff, MBUF_SIZE, "%s.#%d#", mudconf.outdb, mudstate.epoch);
 
     STARTLOG(LOG_DBSAVES, "DMP", "DUMP");
     log_text("Dumping: ");
@@ -1568,7 +1610,7 @@ void fork_and_dump(int key)
         if (key & DUMP_STRUCT)
         {
             mudstate.epoch++;
-            sprintf(buff, "%s.#%d#", mudconf.outdb, mudstate.epoch);
+            mux_sprintf(buff, LBUF_SIZE, "%s.#%d#", mudconf.outdb, mudstate.epoch);
             log_text("Checkpointing: ");
             log_text(buff);
         }
@@ -1578,7 +1620,7 @@ void fork_and_dump(int key)
     {
         STARTLOG(LOG_DBSAVES, "DMP", "FLAT");
         log_text("Creating flatfile: ");
-        sprintf(buff, "%s.FLAT", mudconf.outdb);
+        mux_sprintf(buff, LBUF_SIZE, "%s.FLAT", mudconf.outdb);
         log_text(buff);
         ENDLOG;
     }
@@ -1713,8 +1755,7 @@ static int load_game(int ccPageFile)
 
     if (mudconf.compress_db)
     {
-        strcpy(infile, mudconf.indb);
-        strcat(infile, ".gz");
+        mux_sprintf(infile, sizeof(infile), "%s.gz", mudconf.indb);
         if (stat(infile, &statbuf) == 0)
         {
             f = popen(tprintf(" %s < %s", mudconf.uncompress, infile), POPEN_READ_OP);
@@ -1728,7 +1769,7 @@ static int load_game(int ccPageFile)
 
     if (!compressed)
     {
-        strcpy(infile, mudconf.indb);
+        mux_strncpy(infile, mudconf.indb, sizeof(infile)-1);
         if (stat(infile, &statbuf) != 0)
         {
             // Indicate that we couldn't load because the input db didn't
@@ -1736,7 +1777,8 @@ static int load_game(int ccPageFile)
             //
             return LOAD_GAME_NO_INPUT_DB;
         }
-        if ((f = fopen(infile, "rb")) == NULL)
+
+        if (!mux_fopen(&f, infile, "rb"))
         {
             return LOAD_GAME_CANNOT_OPEN;
         }
@@ -1827,8 +1869,7 @@ static int load_game(int ccPageFile)
 
     if (mudconf.have_mailer)
     {
-        f = fopen(mudconf.mail_db, "rb");
-        if (f)
+        if (mux_fopen(&f, mudconf.mail_db, "rb"))
         {
             DebugTotalFiles++;
             setvbuf(f, NULL, _IOFBF, 16384);
@@ -1881,7 +1922,7 @@ bool list_check
 #else
         if (  thing != player
            && !No_Command(thing))
-#endif /* REALITY_LVLS */
+#endif // REALITY_LVLS
         {
             bMatch |= atr_match(thing, player, type, str, raw_str, check_parent);
         }
@@ -2006,9 +2047,7 @@ static void process_preload(void)
     dbref thing, parent, aowner;
     int aflags, lev;
     char *tstr;
-    FWDLIST *fp;
 
-    fp = (FWDLIST *) alloc_lbuf("process_preload.fwdlist");
     tstr = alloc_lbuf("process_preload.string");
     DO_WHOLE_DB(thing)
     {
@@ -2027,10 +2066,10 @@ static void process_preload(void)
         {
             ITER_PARENTS(thing, parent, lev)
             {
-                if (Flags(thing) & HAS_STARTUP)
+                if (H_Startup(thing))
                 {
                     did_it(Owner(thing), thing, 0, NULL, 0, NULL, A_STARTUP,
-                        (char **)NULL, 0);
+                        0, NULL, 0);
 
                     // Process queue entries as we add them.
                     //
@@ -2047,15 +2086,21 @@ static void process_preload(void)
             atr_get_str(tstr, thing, A_FORWARDLIST, &aowner, &aflags);
             if (*tstr)
             {
-                fwdlist_load(fp, GOD, tstr);
-                if (fp->count > 0)
+                FWDLIST *fp = fwdlist_load(GOD, tstr);
+                if (NULL != fp)
                 {
                     fwdlist_set(thing, fp);
+
+                    if (fp->data)
+                    {
+                        delete [] fp->data;
+                    }
+                    delete fp;
+                    fp = NULL;
                 }
             }
         }
     }
-    free_lbuf(fp);
     free_lbuf(tstr);
 }
 
@@ -2078,11 +2123,24 @@ static void info(int fmt, int flags, int ver)
     {
         cp = "*unknown*";
     }
+
     Log.tinyprintf("%s version %d:", cp, ver);
-    if ((flags & MANDFLAGS) != MANDFLAGS)
+    if (  ver < MIN_SUPPORTED_VERSION
+       || MAX_SUPPORTED_VERSION < ver)
+    {
+        Log.WriteString(" Unsupported version");
+        exit(1);
+    }
+    else if (  (  (  1 == ver
+                  || 2 == ver)
+               && (flags & MANDFLAGS_V2) != MANDFLAGS_V2)
+            || (  3 == ver
+               && (flags & MANDFLAGS_V3) != MANDFLAGS_V3))
     {
         Log.WriteString(" Unsupported flags");
+        exit(1);
     }
+
     if (flags & V_DATABASE)
         Log.WriteString(" Database");
     if (flags & V_ATRNAME)
@@ -2091,12 +2149,12 @@ static void info(int fmt, int flags, int ver)
         Log.WriteString(" AtrKey");
     if (flags & V_ATRMONEY)
         Log.WriteString(" AtrMoney");
-    Log.WriteString("\n");
+    Log.WriteString(ENDLINE);
 }
 
-static char *standalone_infile = NULL;
-static char *standalone_outfile = NULL;
-static char *standalone_basename = NULL;
+static const char *standalone_infile = NULL;
+static const char *standalone_outfile = NULL;
+static const char *standalone_basename = NULL;
 static bool standalone_check = false;
 static bool standalone_load = false;
 static bool standalone_unload = false;
@@ -2147,11 +2205,16 @@ static void dbconvert(void)
     init_attrtab();
 
     char dirfile[SIZEOF_PATHNAME];
+    char *dirfile_c = dirfile;
+    safe_copy_str(standalone_basename, dirfile, &dirfile_c, (SIZEOF_PATHNAME-1));
+    safe_copy_str(".dir", dirfile, &dirfile_c, (SIZEOF_PATHNAME-1));
+    *dirfile_c = '\0';
+
     char pagfile[SIZEOF_PATHNAME];
-    strcpy(dirfile, standalone_basename);
-    strcat(dirfile, ".dir");
-    strcpy(pagfile, standalone_basename);
-    strcat(pagfile, ".pag");
+    char *pagfile_c = pagfile;
+    safe_copy_str(standalone_basename, pagfile, &pagfile_c, (SIZEOF_PATHNAME-1));
+    safe_copy_str(".pag", pagfile, &pagfile_c, (SIZEOF_PATHNAME-1));
+    *pagfile_c = '\0';
 
     int cc = init_dbfile(dirfile, pagfile, 650);
     if (cc == HF_OPEN_STATUS_ERROR)
@@ -2178,8 +2241,8 @@ static void dbconvert(void)
         }
     }
 
-    FILE *fpIn = fopen(standalone_infile, "rb");
-    if (!fpIn)
+    FILE *fpIn;
+    if (!mux_fopen(&fpIn, standalone_infile, "rb"))
     {
         exit(1);
     }
@@ -2190,8 +2253,14 @@ static void dbconvert(void)
     {
         cache_redirect();
     }
+
     setvbuf(fpIn, NULL, _IOFBF, 16384);
-    db_read(fpIn, &db_format, &db_ver, &db_flags);
+    if (db_read(fpIn, &db_format, &db_ver, &db_flags) < 0)
+    {
+        cache_cleanup();
+        exit(1);
+    }
+
     if (do_redirect)
     {
         cache_pass2();
@@ -2207,8 +2276,8 @@ static void dbconvert(void)
 
     if (do_write)
     {
-        FILE *fpOut = fopen(standalone_outfile, "wb");
-        if (!fpOut)
+        FILE *fpOut;
+        if (!mux_fopen(&fpOut, standalone_outfile, "wb"))
         {
             exit(1);
         }
@@ -2241,8 +2310,8 @@ static void dbconvert(void)
 
 static void write_pidfile(const char *pFilename)
 {
-    FILE *fp = fopen(pFilename, "wb");
-    if (fp)
+    FILE *fp;
+    if (mux_fopen(&fp, pFilename, "wb"))
     {
         fprintf(fp, "%d" ENDLINE, game_pid);
         fclose(fp);
@@ -2255,6 +2324,51 @@ static void write_pidfile(const char *pFilename)
     }
 }
 
+#ifdef FIRANMUX
+static void init_sql(void)
+{
+    if ('\0' != mudconf.sql_server[0])
+    {
+        STARTLOG(LOG_STARTUP,"SQL","CONN");
+        log_text("Connecting: ");
+        log_text(mudconf.sql_database);
+        log_text("@");
+        log_text(mudconf.sql_server);
+        log_text(" as ");
+        log_text(mudconf.sql_user);
+        ENDLOG;
+
+        mush_database = mysql_init(NULL);
+
+        if (mush_database)
+        {
+           if (mysql_real_connect(mush_database,mudconf.sql_server,
+                      mudconf.sql_user, mudconf.sql_password,
+                      mudconf.sql_database, 0, NULL, 0))
+           {
+               STARTLOG(LOG_STARTUP,"SQL","CONN");
+               log_text("Connected to MySQL");
+               ENDLOG;
+           }
+           else
+           {
+               STARTLOG(LOG_STARTUP,"SQL","CONN");
+               log_text("Unable to connect");
+               ENDLOG;
+               mysql_close(mush_database);
+               mush_database = NULL;
+           }
+        }
+        else
+        {
+           STARTLOG(LOG_STARTUP,"SQL","CONN");
+           log_text("MySQL Library unavailable");
+           ENDLOG;
+        }
+    }
+}
+
+#endif // FIRANMUX
 long DebugTotalFiles = 3;
 long DebugTotalSockets = 0;
 #ifdef WIN32
@@ -2277,18 +2391,19 @@ long DebugTotalMemory = 0;
 #define CLI_DO_BASENAME    CLI_USER+9
 #define CLI_DO_PID_FILE    CLI_USER+10
 #define CLI_DO_ERRORPATH   CLI_USER+11
+#define CLI_DO_SELECT      CLI_USER+12
 
 static bool bMinDB = false;
 static bool bSyntaxError = false;
-static char *conffile = NULL;
+static const char *conffile = NULL;
 static bool bVersion = false;
-static char *pErrorBasename = "";
+static const char *pErrorBasename = "";
 static bool bServerOption = false;
 
 #ifdef MEMORY_BASED
-#define NUM_CLI_OPTIONS 6
+#define NUM_CLI_OPTIONS 7
 #else
-#define NUM_CLI_OPTIONS 12
+#define NUM_CLI_OPTIONS 13
 #endif
 
 static CLI_OptionEntry OptionTable[NUM_CLI_OPTIONS] =
@@ -2305,11 +2420,14 @@ static CLI_OptionEntry OptionTable[NUM_CLI_OPTIONS] =
     { "u", CLI_NONE,     CLI_DO_UNLOAD      },
     { "d", CLI_REQUIRED, CLI_DO_BASENAME    },
 #endif // MEMORY_BASED
+#ifdef WIN32
+    { "n", CLI_NONE,     CLI_DO_SELECT      },
+#endif // WIN32
     { "p", CLI_REQUIRED, CLI_DO_PID_FILE    },
     { "e", CLI_REQUIRED, CLI_DO_ERRORPATH   }
 };
 
-static void CLI_CallBack(CLI_OptionEntry *p, char *pValue)
+static void CLI_CallBack(CLI_OptionEntry *p, const char *pValue)
 {
     if (p)
     {
@@ -2340,6 +2458,12 @@ static void CLI_CallBack(CLI_OptionEntry *p, char *pValue)
             pErrorBasename = pValue;
             break;
 
+#ifdef WIN32
+        case CLI_DO_SELECT:
+            bUseCompletionPorts = false;
+            break;
+#endif // WIN32
+
 #ifndef MEMORY_BASED
         case CLI_DO_INFILE:
             mudstate.bStandAlone = true;
@@ -2384,7 +2508,7 @@ static void CLI_CallBack(CLI_OptionEntry *p, char *pValue)
     }
 }
 
-#if defined(__INTEL_COMPILER)
+#if defined(__INTEL_COMPILER) && defined(_M_IX86)
 
 #define CPU_FD_ID 0x00200000UL
 
@@ -2762,14 +2886,14 @@ static void cpu_init(void)
     }
 }
 
-#endif
+#endif // __INTEL_COMPILER
 
 #define DBCONVERT_NAME1 "dbconvert"
 #define DBCONVERT_NAME2 "dbconvert.exe"
 
 int DCL_CDECL main(int argc, char *argv[])
 {
-#if defined(__INTEL_COMPILER)
+#if defined(__INTEL_COMPILER) && defined(_M_IX86)
     cpu_init();
 #endif
 
@@ -2862,6 +2986,9 @@ int DCL_CDECL main(int argc, char *argv[])
             fprintf(stderr, "  -h  Display this help." ENDLINE);
             fprintf(stderr, "  -p  Specify process ID file." ENDLINE);
             fprintf(stderr, "  -s  Start with a minimal database." ENDLINE);
+#ifdef WIN32
+            fprintf(stderr, "  -n  Disable use of NT I/O Completion Ports." ENDLINE);
+#endif // WIN32
             fprintf(stderr, "  -v  Display version string." ENDLINE ENDLINE);
         }
         return 1;
@@ -2875,7 +3002,7 @@ int DCL_CDECL main(int argc, char *argv[])
 
     Log.SetBasename(pErrorBasename);
     Log.StartLogging();
-    game_pid = getpid();
+    game_pid = mux_getpid();
     write_pidfile(mudconf.pid_file);
 
     BuildSignalNamesTable();
@@ -2890,50 +3017,51 @@ int DCL_CDECL main(int argc, char *argv[])
 #endif
 
 #ifdef WIN32
-    // Find which version of Windows we are using - Completion ports do
-    // not work with Windows 95/98
 
-    OSVERSIONINFO VersionInformation;
-
-    VersionInformation.dwOSVersionInfoSize = sizeof (VersionInformation);
-    GetVersionEx(&VersionInformation);
-    platform = VersionInformation.dwPlatformId;
     hGameProcess = GetCurrentProcess();
-    if (platform == VER_PLATFORM_WIN32_NT)
+
+    // Get a handle to the kernel32 DLL
+    //
+    HINSTANCE hInstKernel32 = LoadLibrary("kernel32");
+    if (!hInstKernel32)
     {
-        Log.WriteString("Running under Windows NT" ENDLINE);
-
-        // Get a handle to the kernel32 DLL
-        //
-        HINSTANCE hInstKernel32 = LoadLibrary("kernel32");
-        if (!hInstKernel32)
-        {
-            Log.WriteString("LoadLibrary of kernel32 for a CancelIo entry point failed. Cannot continue." ENDLINE);
-            return 1;
-        }
-
-        // Find the entry point for CancelIO so we can use it. This is done
-        // dynamically because Windows 95/98 doesn't have a CancelIO entry
-        // point. If it were done at load time, it would always fail on
-        // Windows 95/98...even though we don't use it or depend on it in
-        // that case.
-        //
-        fpCancelIo = (FCANCELIO *)GetProcAddress(hInstKernel32, "CancelIo");
-        if (fpCancelIo == NULL)
-        {
-            Log.WriteString("GetProcAddress of _CancelIo failed. Cannot continue." ENDLINE);
-            return 1;
-        }
-        fpGetProcessTimes = (FGETPROCESSTIMES *)GetProcAddress(hInstKernel32, "GetProcessTimes");
-        if (fpGetProcessTimes == NULL)
-        {
-            Log.WriteString("GetProcAddress of GetProcessTimes failed. Cannot continue." ENDLINE);
-            return 1;
-        }
+        Log.WriteString("LoadLibrary of kernel32 failed. Cannot use completion ports." ENDLINE);
+        bUseCompletionPorts = false;
     }
     else
     {
-        Log.WriteString("Running under Windows 95/98" ENDLINE);
+        if (bUseCompletionPorts)
+        {
+            // Find the entry point for CancelIO so we can use it. This is done
+            // dynamically because Windows 95/98 doesn't have a CancelIO entry
+            // point. If it were done at load time, it would always fail on
+            // Windows 95/98...even though we don't use it or depend on it in
+            // that case.
+            //
+            fpCancelIo = (FCANCELIO *)GetProcAddress(hInstKernel32, "CancelIo");
+            if (NULL == fpCancelIo)
+            {
+                Log.WriteString("GetProcAddress of _CancelIo failed." ENDLINE);
+                bUseCompletionPorts = false;
+            }
+        }
+
+        fpGetProcessTimes = (FGETPROCESSTIMES *)GetProcAddress(hInstKernel32, "GetProcessTimes");
+        if (NULL == fpGetProcessTimes)
+        {
+            // We can work with or without GetProcessTimes().
+            //
+            Log.WriteString("GetProcAddress of GetProcessTimes failed, but that's OK." ENDLINE);
+        }
+    }
+
+    if (bUseCompletionPorts)
+    {
+        Log.WriteString("Using NT I/O Completion Ports for networking." ENDLINE);
+    }
+    else
+    {
+        Log.WriteString("Using select() for networking." ENDLINE);
     }
 
     // Initialize WinSock.
@@ -2971,6 +3099,8 @@ int DCL_CDECL main(int argc, char *argv[])
 
     pool_init(POOL_DESC, sizeof(DESC));
     pool_init(POOL_QENTRY, sizeof(BQUE));
+    pool_init(POOL_LBUFREF, sizeof(lbuf_ref));
+    pool_init(POOL_REGREF, sizeof(reg_ref));
     tcache_init();
     pcache_init();
     cf_init();
@@ -2986,8 +3116,13 @@ int DCL_CDECL main(int argc, char *argv[])
     init_version();
 
     mudconf.config_file = StringClone(conffile);
+    mudconf.log_dir = StringClone(pErrorBasename);
     cf_read();
 
+#if defined(FIRANMUX)
+    init_sql();
+#endif // FIRANMUX
+
     fcache_init();
     helpindex_init();
 
@@ -3070,13 +3205,6 @@ int DCL_CDECL main(int argc, char *argv[])
     hashreset(&mudstate.fwdlist_htab);
     hashreset(&mudstate.desc_htab);
 
-    int i;
-    for (i = 0; i < MAX_GLOBAL_REGS; i++)
-    {
-        mudstate.global_regs[i] = alloc_lbuf("main.global_reg");
-        mudstate.glob_reg_len[i] = 0;
-    }
-
     ValidateConfigurationDbrefs();
     process_preload();
 
@@ -3108,7 +3236,7 @@ int DCL_CDECL main(int argc, char *argv[])
     init_timer();
 
 #ifdef WIN32
-    if (platform == VER_PLATFORM_WIN32_NT)
+    if (bUseCompletionPorts)
     {
         process_output = process_outputNT;
         shovecharsNT(nMainGamePorts, aMainGamePorts);
@@ -3122,6 +3250,17 @@ int DCL_CDECL main(int argc, char *argv[])
     shovechars(nMainGamePorts, aMainGamePorts);
 #endif // WIN32
 
+#ifdef FIRANMUX
+     if (mush_database)
+     {
+         mysql_close(mush_database);
+         mush_database = NULL;
+         STARTLOG(LOG_STARTUP,"SQL","DISC");
+         log_text("SQL shut down");
+         ENDLOG;
+     }
+#endif // FIRANMUX
+
     close_sockets(false, "Going down - Bye");
     dump_database();
 
@@ -3143,6 +3282,7 @@ int DCL_CDECL main(int argc, char *argv[])
     // Go ahead and explicitly free the memory for these things so
     // that it's easy to spot unintentional memory leaks.
     //
+    int i;
     for (i = 0; i < mudstate.nHelpDesc; i++)
     {
         helpindex_clean(i);
@@ -3153,7 +3293,7 @@ int DCL_CDECL main(int argc, char *argv[])
 #ifdef WIN32
     // Critical section not needed any more.
     //
-    if (platform == VER_PLATFORM_WIN32_NT)
+    if (bUseCompletionPorts)
     {
         DeleteCriticalSection(&csDescriptorList);
     }
@@ -3185,3 +3325,63 @@ void init_rlimit(void)
 
 }
 #endif // HAVE_SETRLIMIT RLIMIT_NOFILE
+
+bool mux_fopen(FILE **pFile, const char *filename, const char *mode)
+{
+    if (pFile)
+    {
+        *pFile = NULL;
+        if (  NULL != filename
+           && NULL != mode)
+        {
+#if defined(WIN32) && !defined(__INTEL_COMPILER) && (_MSC_VER >= 1400)
+            // 1400 is Visual C++ 2005
+            //
+            return (fopen_s(pFile, filename, mode) == 0);
+#else
+            *pFile = fopen(filename, mode);
+            if (NULL != *pFile)
+            {
+                return true;
+            }
+#endif // WIN32
+        }
+    }
+    return false;
+}
+
+bool mux_open(int *pfh, const char *filename, int oflag)
+{
+    if (NULL != pfh)
+    {
+        *pfh = MUX_OPEN_INVALID_HANDLE_VALUE;
+        if (NULL != filename)
+        {
+#if defined(WIN32) && !defined(__INTEL_COMPILER) && (_MSC_VER >= 1400)
+            // 1400 is Visual C++ 2005
+            //
+            return (_sopen_s(pfh, filename, oflag, _SH_DENYNO, _S_IREAD|_S_IWRITE) == 0);
+#elif defined(win32)
+            *pfh = _open(filename, oflag, _S_IREAD|_S_IWRITE);
+            return (0 <= *pfh);
+#else
+            *pfh = open(filename, oflag, 0600);
+            return (0 <= *pfh);
+#endif
+        }
+    }
+    return false;
+}
+
+const char *mux_strerror(int errnum)
+{
+#if defined(WIN32) && !defined(__INTEL_COMPILER) && (_MSC_VER >= 1400)
+    // 1400 is Visual C++ 2005
+    //
+    static char buffer[80];
+    strerror_s(buffer, sizeof(buffer), errnum);
+    return buffer;
+#else
+    return strerror(errnum);
+#endif
+}
