@@ -1,6 +1,6 @@
 // htab.h - Structures and declarations needed for table hashing.
 //
-// $Id: htab.h,v 1.9 2005/10/27 06:27:53 sdennis Exp $
+// $Id: htab.h 2734 2007-10-28 23:02:55Z brazilofmux $
 //
 
 #include "copyright.h"
@@ -14,7 +14,7 @@
 typedef struct name_table NAMETAB;
 struct name_table
 {
-    char    *name;
+    const char *name;
     int minlen;
     int perm;
     int flag;
@@ -29,13 +29,14 @@ struct bque
     dbref   executor;               // executor who will do command
     dbref   caller;                 // caller.
     dbref   enactor;                // enactor causing command (for %N)
+    int     eval;
     dbref   sem;                    // blocking semaphore
     int     attr;                   // blocking attribute
     int     nargs;                  // How many args I have
     char    *text;                  // buffer for comm, env, and scr text
     char    *comm;                  // command
     char    *env[NUM_ENV_VARS];     // environment vars
-    char    *scr[MAX_GLOBAL_REGS];  // temp vars
+    reg_ref *scr[MAX_GLOBAL_REGS];  // temp vars
     bool    IsTimed;                // Is there a waittime time on this entry?
 };
 
@@ -61,7 +62,7 @@ public:
 
 void hashreset(CHashTable *);
 void *hashfindLEN(const void *pKey, size_t nKey, CHashTable *htab);
-int  hashaddLEN(const void *pKey, size_t nKey, void *pData,
+bool hashaddLEN(const void *pKey, size_t nKey, void *pData,
                 CHashTable *htab);
 void hashdeleteLEN(const void *Str, size_t nStr, CHashTable *htab);
 void hashflush(CHashTable *);
@@ -77,7 +78,7 @@ extern NAMETAB powers_nametab[];
 
 extern bool search_nametab(dbref, NAMETAB *, char *, int *);
 extern NAMETAB *find_nametab_ent(dbref, NAMETAB *, char *);
-extern void display_nametab(dbref, NAMETAB *, char *, bool);
+extern void display_nametab(dbref, NAMETAB *, const char *, bool);
 extern void interp_nametab(dbref, NAMETAB *, int, const char *, const char *, const char *);
 extern void listset_nametab(dbref, NAMETAB *, int, char *, bool);
 
