@@ -1,6 +1,6 @@
 // interface.h
 //
-// $Id: interface.h,v 1.24 2006/01/08 06:16:08 sdennis Exp $
+// $Id: interface.h 456 2006-11-26 05:08:03Z brazilofmux $
 //
 
 #include "copyright.h"
@@ -23,6 +23,8 @@
 
 /* Disconnection reason codes */
 
+#define R_MIN       0
+#define R_UNKNOWN   0   /* Unknown/unexpected */
 #define R_QUIT      1   /* User quit */
 #define R_TIMEOUT   2   /* Inactivity timeout */
 #define R_BOOT      3   /* Victim of @boot, @toad, or @destroy */
@@ -32,6 +34,7 @@
 #define R_GAMEDOWN  7   /* Not admitting users now */
 #define R_LOGOUT    8   /* Logged out w/o disconnecting */
 #define R_GAMEFULL  9   /* Too many players logged in */
+#define R_MAX       9
 
 /* Logged out command table definitions */
 
@@ -68,7 +71,7 @@ typedef struct text_block_hdr
     struct text_block *nxt;
     char    *start;
     char    *end;
-    int nchars;
+    size_t   nchars;
 }   TBLOCKHDR;
 
 typedef struct text_block
@@ -78,9 +81,10 @@ typedef struct text_block
 } TBLOCK;
 
 typedef struct prog_data PROG;
-struct prog_data {
-    dbref wait_enactor;
-    char *wait_regs[MAX_GLOBAL_REGS];
+struct prog_data
+{
+    dbref    wait_enactor;
+    reg_ref *wait_regs[MAX_GLOBAL_REGS];
 };
 
 // Input state
@@ -150,14 +154,14 @@ struct descriptor_data
   dbref player;
   char *output_prefix;
   char *output_suffix;
-  int output_size;
-  int output_tot;
-  int output_lost;
+  size_t output_size;
+  size_t output_tot;
+  size_t output_lost;
   TBLOCK *output_head;
   TBLOCK *output_tail;
-  int input_size;
-  int input_tot;
-  int input_lost;
+  size_t input_size;
+  size_t input_tot;
+  size_t input_lost;
   CBLK *input_head;
   CBLK *input_tail;
   CBLK *raw_input;
@@ -174,8 +178,6 @@ struct descriptor_data
   int width;
   int height;
   int quota;
-  int wait_for_input;       /* Used by @prog */
-  dbref wait_enactor;       /* Used by @prog */
   PROG *program_data;
   struct descriptor_data *hashnext;
   struct descriptor_data *next;
@@ -237,7 +239,7 @@ extern void update_quotas(CLinearTimeAbsolute& tLast, const CLinearTimeAbsolute&
 extern void raw_notify(dbref, const char *);
 extern void raw_notify_newline(dbref);
 extern void clearstrings(DESC *);
-extern void queue_write_LEN(DESC *, const char *, int);
+extern void queue_write_LEN(DESC *, const char *, size_t n);
 extern void queue_write(DESC *, const char *);
 extern void queue_string(DESC *, const char *);
 extern void freeqs(DESC *);
