@@ -44,7 +44,7 @@ RLEVEL RxLevel(dbref thing)
 
     int i;
     RLEVEL rx = 0;
-    for (i = 0; mux_ishex(buff[i]); i++)
+    for (i = 0; mux_isxdigit(buff[i]); i++)
     {
         rx = 16 * rx + mux_hex2dec(buff[i]);
     }
@@ -90,7 +90,7 @@ RLEVEL TxLevel(dbref thing)
 
         // Decode second field.
         //
-        for ( ; mux_ishex(buff[i]); i++)
+        for ( ; mux_isxdigit(buff[i]); i++)
         {
             tx = 16 * tx + mux_hex2dec(buff[i]);
         }
@@ -348,7 +348,7 @@ void do_rxlevel
     // Set the Rx Level.
     //
     char *buff = alloc_lbuf("do_rxlevel");
-    sprintf(buff, "%08X %08X", RxLevel(thing) & andmask | ormask, TxLevel(thing));
+    mux_sprintf(buff, LBUF_SIZE, "%08X %08X", RxLevel(thing) & andmask | ormask, TxLevel(thing));
     atr_add_raw(thing, A_RLEVEL, buff);
     free_lbuf(buff);
 }
@@ -440,7 +440,7 @@ void do_txlevel
     // Set the Tx Level.
     //
     char *buff = alloc_lbuf("do_rxlevel");
-    sprintf(buff, "%08X %08X", RxLevel(thing), TxLevel(thing) & andmask | ormask);
+    mux_sprintf(buff, LBUF_SIZE, "%08X %08X", RxLevel(thing), TxLevel(thing) & andmask | ormask);
     atr_add_raw(thing, A_RLEVEL, buff);
     free_lbuf(buff);
 }
@@ -502,13 +502,14 @@ void did_it_rlevel
 (
     dbref player,
     dbref thing,
-    int what,
+    int   what,
     const char *def,
-    int owhat,
+    int   owhat,
     const char *odef,
-    int awhat,
+    int   awhat,
+    int   ctrl_flags,
     char *args[],
-    int nargs
+    int   nargs
 )
 {
     char *d, *buff, *act, *charges, *bp, *str;
@@ -516,8 +517,7 @@ void did_it_rlevel
     int num, aflags;
     int i, *desclist, found_a_desc;
 
-    char *preserve[MAX_GLOBAL_REGS];
-    int  preserve_len[MAX_GLOBAL_REGS];
+    reg_ref **preserve = NULL;
     bool need_pres = false;
 
     // Message to player.
@@ -533,7 +533,7 @@ void did_it_rlevel
             // Ok, if it's A_DESC, we need to check against A_IDESC.
             //
             if (  A_IDESC == what
-               && desclist[i] == A_DESC)
+               && A_DESC == desclist[i])
             {
                 d = atr_pget(thing, A_IDESC, &aowner, &aflags);
             }
@@ -541,6 +541,7 @@ void did_it_rlevel
             {
                 d = atr_pget(thing, desclist[i], &aowner, &aflags);
             }
+
             if (*d)
             {
                 // No need for the 'def' message.
@@ -549,12 +550,14 @@ void did_it_rlevel
                 if (!need_pres)
                 {
                     need_pres = true;
-                    save_global_regs("did_it_save", preserve, preserve_len);
+                    preserve = PushRegisters(MAX_GLOBAL_REGS);
+                    save_global_regs(preserve);
                 }
                 buff = bp = alloc_lbuf("did_it.1");
                 str = d;
                 mux_exec(buff, &bp, thing, thing, player,
-                    EV_EVAL | EV_FIGNORE | EV_TOP, &str, args, nargs);
+                    AttrTrace(aflags, EV_EVAL|EV_FIGNORE|EV_TOP), &str,
+                    args, nargs);
                 *bp = '\0';
 
                 if (  A_HTDESC == desclist[i]
@@ -572,6 +575,7 @@ void did_it_rlevel
             }
             free_lbuf(d);
         }
+
         if (!found_a_desc)
         {
             // No desc found... try the default desc (again).
@@ -586,12 +590,14 @@ void did_it_rlevel
                 if (!need_pres)
                 {
                     need_pres = true;
-                    save_global_regs("did_it_save", preserve, preserve_len);
+                    preserve = PushRegisters(MAX_GLOBAL_REGS);
+                    save_global_regs(preserve);
                 }
                 buff = bp = alloc_lbuf("did_it.1");
                 str = d;
                 mux_exec(buff, &bp, thing, thing, player,
-                    EV_EVAL | EV_FIGNORE | EV_TOP, &str, args, nargs);
+                    AttrTrace(aflags, EV_EVAL|EV_FIGNORE|EV_TOP), &str,
+                    args, nargs);
                 *bp = '\0';
 
                 if (  A_HTDESC == what
@@ -628,11 +634,14 @@ void did_it_rlevel
           if (!need_pres)
           {
              need_pres = true;
-             save_global_regs("did_it_save", preserve, preserve_len);
+             preserve = PushRegisters(MAX_GLOBAL_REGS);
+             save_global_regs(preserve);
           }
           buff = bp = alloc_lbuf("did_it.1");
           str = d;
-          mux_exec(buff, &bp, thing, thing, player, EV_EVAL | EV_FIGNORE |EV_TOP, &str, args, nargs);
+          mux_exec(buff, &bp, thing, thing, player,
+              AttrTrace(aflags, EV_EVAL|EV_FIGNORE|EV_TOP), &str,
+              args, nargs);
           *bp = '\0';
           notify(player, buff);
           free_lbuf(buff);
@@ -640,10 +649,9 @@ void did_it_rlevel
        free_lbuf(d);
     }
 
-
     // Message to neighbors.
     //
-    if (  owhat > 0
+    if (  0 < owhat
        && Has_location(player)
        && Good_obj(loc = Location(player)))
     {
@@ -653,24 +661,41 @@ void did_it_rlevel
             if (!need_pres)
             {
                 need_pres = true;
-                save_global_regs("did_it_save", preserve, preserve_len);
+                preserve = PushRegisters(MAX_GLOBAL_REGS);
+                save_global_regs(preserve);
             }
             buff = bp = alloc_lbuf("did_it.2");
             str = d;
-            mux_exec(buff, &bp, thing, thing, player, EV_EVAL | EV_FIGNORE | EV_TOP, &str, args, nargs);
+            mux_exec(buff, &bp, thing, thing, player,
+                AttrTrace(aflags, EV_EVAL|EV_FIGNORE|EV_TOP), &str,
+                args, nargs);
             *bp = '\0';
 
             if (*buff)
             {
-                notify_except2_rlevel2(loc, player, player, thing,
-                    tprintf("%s %s", Name(player), buff));
+                if (aflags & AF_NONAME)
+                {
+                    notify_except2_rlevel2(loc, player, player, thing, buff);
+                }
+                else
+                {
+                    notify_except2_rlevel2(loc, player, player, thing,
+                        tprintf("%s %s", Name(player), buff));
+                }
             }
             free_lbuf(buff);
         }
         else if (odef)
         {
-            notify_except2_rlevel2(loc, player, player, thing,
-                tprintf("%s %s", Name(player), odef));
+            if (ctrl_flags & VERB_NONAME)
+            {
+                notify_except2_rlevel2(loc, player, player, thing, odef);
+            }
+            else
+            {
+                notify_except2_rlevel2(loc, player, player, thing,
+                    tprintf("%s %s", Name(player), odef));
+            }
         }
         free_lbuf(d);
     }
@@ -679,15 +704,23 @@ void did_it_rlevel
             && Has_location(player)
             && Good_obj(loc = Location(player)))
     {
-        notify_except2_rlevel2(loc, player, player, thing,
-            tprintf("%s %s", Name(player), odef));
+        if (ctrl_flags & VERB_NONAME)
+        {
+            notify_except2_rlevel2(loc, player, player, thing, odef);
+        }
+        else
+        {
+            notify_except2_rlevel2(loc, player, player, thing,
+                tprintf("%s %s", Name(player), odef));
+        }
     }
 
     // If we preserved the state of the global registers, restore them.
     //
     if (need_pres)
     {
-        restore_global_regs("did_it_restore", preserve, preserve_len);
+        restore_global_regs(preserve);
+        PopRegisters(preserve, MAX_GLOBAL_REGS);
     }
 
     // Do the action attribute.
@@ -729,10 +762,13 @@ void did_it_rlevel
             free_lbuf(charges);
 
             CLinearTimeAbsolute lta;
-            wait_que(thing, player, player, 0, lta, NOTHING, 0, act, args,
-                 nargs, mudstate.global_regs);
+            wait_que(thing, player, player, AttrTrace(aflags, 0), false, lta,
+                NOTHING, 0,
+                act,
+                nargs, args,
+                mudstate.global_regs);
         }
         free_lbuf(act);
     }
 }
-#endif /* REALITY_LVLS */
+#endif // REALITY_LVLS
