@@ -1,6 +1,6 @@
 // log.cpp -- Logging routines.
 //
-// $Id: log.cpp,v 1.9 2006/01/07 19:57:38 sdennis Exp $
+// $Id: log.cpp 2734 2007-10-28 23:02:55Z brazilofmux $
 //
 
 #include "copyright.h"
@@ -67,7 +67,7 @@ bool start_log(const char *primary, const char *secondary)
                 ltaNow.GetLocal();
                 FIELDEDTIME ft;
                 ltaNow.ReturnFields(&ft);
-                sprintf(buffer, "%d.%02d%02d:%02d%02d%02d ",ft.iYear,
+                mux_sprintf(buffer, sizeof(buffer), "%d.%02d%02d:%02d%02d%02d ",ft.iYear,
                     ft.iMonth, ft.iDayOfMonth, ft.iHour, ft.iMinute,
                     ft.iSecond);
             }
@@ -128,7 +128,7 @@ void log_perror(const char *primary, const char *secondary, const char *extra, c
     //
     Log.WriteString(failing_object);
     Log.WriteString(": ");
-    Log.WriteString(strerror(errno));
+    Log.WriteString(mux_strerror(errno));
 #ifndef WIN32
     Log.WriteString(ENDLINE);
 #endif // !WIN32
@@ -155,7 +155,7 @@ void DCL_CDECL log_printf(const char *fmt, ...)
     va_list ap;
     va_start(ap, fmt);
     char aTempBuffer[SIZEOF_LOG_BUFFER];
-    int nString = mux_vsnprintf(aTempBuffer, SIZEOF_LOG_BUFFER, fmt, ap);
+    size_t nString = mux_vsnprintf(aTempBuffer, SIZEOF_LOG_BUFFER, fmt, ap);
     va_end(ap);
     Log.WriteBuffer(nString, aTempBuffer);
 }
@@ -247,7 +247,7 @@ void log_type_and_name(dbref thing)
     char nbuf[16];
 
     log_text(OBJTYP(thing));
-    sprintf(nbuf, " #%d(", thing);
+    mux_sprintf(nbuf, sizeof(nbuf), " #%d(", thing);
     log_text(nbuf);
     if (Good_obj(thing))
     {
@@ -316,11 +316,11 @@ void do_log
     }
 
     char *pFullName = NULL;
-    char *pMessage = "";
+    const char *pMessage = "";
     if (bValid)
     {
         pFullName = alloc_lbuf("do_log_filename");
-        sprintf(pFullName, "logs/M-%s.log", pFilename);
+        mux_sprintf(pFullName, LBUF_SIZE, "logs/M-%s.log", pFilename);
 
         // Strip the message of all ANSI.
         //
@@ -333,29 +333,33 @@ void do_log
             bValid = false;
         }
     }
+
     if (!bValid)
     {
-        if (pFullName) free_lbuf(pFullName);
+        if (pFullName)
+        {
+            free_lbuf(pFullName);
+        }
         notify(executor, "Syntax: @log file=message");
         return;
     }
 
-    FILE *hFile = fopen(pFullName, "r");
-    if (hFile)
+    FILE *hFile;
+    if (mux_fopen(&hFile, pFullName, "r"))
     {
         fclose(hFile);
-        hFile = fopen(pFullName, "a");
+        if (mux_fopen(&hFile, pFullName, "a"))
+        {
+            // Okay, at this point, the file exists.
+            //
+            free_lbuf(pFullName);
+            fprintf(hFile, "%s" ENDLINE, pMessage);
+            fclose(hFile);
+            return;
+        }
     }
-    if (hFile == NULL)
-    {
-        notify(executor, "Not a valid log file.");
-        if (pFullName) free_lbuf(pFullName);
-        return;
-    }
-
-    // Okay, at this point, the file exists.
-    //
-    fprintf(hFile, "%s" ENDLINE, pMessage);
-    fclose(hFile);
     free_lbuf(pFullName);
+
+    notify(executor, "Not a valid log file.");
+    return;
 }
