@@ -1,10 +1,6 @@
 // look.cpp -- Commands which look at things.
 //
-// $Id: look.cpp,v 1.45 2006/01/09 16:02:17 sdennis Exp $
-//
-// MUX 2.4
-// Copyright (C) 1998 through 2004 Solid Vertical Domains, Ltd. All
-// rights not explicitly given are reserved.
+// $Id: look.cpp 2734 2007-10-28 23:02:55Z brazilofmux $
 //
 #include "copyright.h"
 #include "autoconf.h"
@@ -19,7 +15,7 @@
 
 #ifdef REALITY_LVLS
 #include "levels.h"
-#endif /* REALITY_LVLS */
+#endif // REALITY_LVLS
 
 #if defined(WOD_REALMS) || defined(REALITY_LVLS)
 #define NORMAL_REALM  0
@@ -404,7 +400,7 @@ static void look_exits(dbref player, dbref loc, const char *exit_name)
     if (Dark(loc) || !IsReal(player, loc))
 #else
     if (Dark(loc))
-#endif /* REALITY_LVLS */
+#endif // REALITY_LVLS
     {
         key |= VE_BASE_DARK;
     }
@@ -477,15 +473,17 @@ static void look_exits(dbref player, dbref loc, const char *exit_name)
         char *FormatOutput = alloc_lbuf("look_exits.FO");
         tPtr = FormatOutput;
 
-        char *preserve[MAX_GLOBAL_REGS];
-        int preserve_len[MAX_GLOBAL_REGS];
-        save_and_clear_global_regs("look_exits_save", preserve, preserve_len);
+        reg_ref **preserve = NULL;
+        preserve = PushRegisters(MAX_GLOBAL_REGS);
+        save_and_clear_global_regs(preserve);
 
         mux_exec(FormatOutput, &tPtr, loc, player, player,
-                EV_FCHECK | EV_EVAL | EV_TOP,
-                &ExitFormat, &VisibleObjectList, 1);
+            AttrTrace(aflags, EV_FCHECK|EV_EVAL|EV_TOP),
+            &ExitFormat, &VisibleObjectList, 1);
+        *tPtr = '\0';
 
-        restore_global_regs("look_exits_restore", preserve, preserve_len);
+        restore_global_regs(preserve);
+        PopRegisters(preserve, MAX_GLOBAL_REGS);
         notify(player, FormatOutput);
 
         free_lbuf(FormatOutput);
@@ -518,13 +516,13 @@ static void look_exits(dbref player, dbref loc, const char *exit_name)
             {
                 if (exit_displayable(thing, player, key))
                 {
-                    strcpy(buff, Name(thing));
+                    mux_strncpy(buff, Moniker(thing), LBUF_SIZE-1);
                     for (e = buff; *e && *e != ';'; e++)
                     {
                         ; // Nothing.
                     }
                     *e = '\0';
-                    notify(player, tprintf("%s leads to %s.", buff, Name(Location(thing))));
+                    notify(player, tprintf("%s leads to %s.", buff, Moniker(Location(thing))));
                 }
             }
         }
@@ -545,7 +543,7 @@ static void look_exits(dbref player, dbref loc, const char *exit_name)
                         safe_str("  ", buff, &e);
                     }
 
-                    for (s = Name(thing); *s && (*s != ';'); s++)
+                    for (s = Moniker(thing); *s && (*s != ';'); s++)
                     {
                         safe_chr(*s, buff1, &e1);
                     }
@@ -606,7 +604,7 @@ static void look_contents(dbref player, dbref loc, const char *contents_name, in
      bool can_see_loc = ( !Dark(loc) && IsReal(player, loc)
 #else
      bool can_see_loc = (  !Dark(loc)
-#endif /* REALITY_LVLS */
+#endif // REALITY_LVLS
                        || (mudconf.see_own_dark && Examinable(player, loc)));
 
     dbref aowner;
@@ -653,15 +651,17 @@ static void look_contents(dbref player, dbref loc, const char *contents_name, in
         char* ParameterList[] =
             { VisibleObjectList, ContentsNameScratch };
 
-        char *preserve[MAX_GLOBAL_REGS];
-        int preserve_len[MAX_GLOBAL_REGS];
-        save_and_clear_global_regs("look_contents_save", preserve, preserve_len);
+        reg_ref **preserve = NULL;
+        preserve = PushRegisters(MAX_GLOBAL_REGS);
+        save_and_clear_global_regs(preserve);
 
         mux_exec(FormatOutput, &tPtr, loc, player, player,
-                EV_FCHECK | EV_EVAL | EV_TOP,
-                &ContentsFormat, ParameterList, 2);
+            AttrTrace(aflags, EV_FCHECK|EV_EVAL|EV_TOP),
+            &ContentsFormat, ParameterList, 2);
+        *tPtr = '\0';
 
-        restore_global_regs("look_contents_restore", preserve, preserve_len);
+        restore_global_regs(preserve);
+        PopRegisters(preserve, MAX_GLOBAL_REGS);
         notify(player, FormatOutput);
 
         free_lbuf(FormatOutput);
@@ -702,7 +702,7 @@ static void look_contents(dbref player, dbref loc, const char *contents_name, in
                 if (can_see(player, thing, can_see_loc))
 #endif
                 {
-                    buff = unparse_object(player, thing, true);
+                    buff = unparse_object(player, thing, true, true);
                     html_cp = html_buff;
                     if (Html(player))
                     {
@@ -710,12 +710,13 @@ static void look_contents(dbref player, dbref loc, const char *contents_name, in
                         switch (style)
                         {
                         case CONTENTS_LOCAL:
-                            safe_str(Name(thing), html_buff, &html_cp);
+                            safe_str(Moniker(thing), html_buff, &html_cp);
                             break;
+
                         case CONTENTS_NESTED:
-                            safe_str(Name(Location(thing)), html_buff, &html_cp);
+                            safe_str(Moniker(Location(thing)), html_buff, &html_cp);
                             safe_str("'s ", html_buff, &html_cp);
-                            safe_str(Name(thing), html_buff, &html_cp);
+                            safe_str(Moniker(thing), html_buff, &html_cp);
                             break;
 
                         case CONTENTS_REMOTE:
@@ -750,38 +751,59 @@ static void look_contents(dbref player, dbref loc, const char *contents_name, in
 
 typedef struct
 {
-    int mask;
+    int  mask;
     char letter;
+    const char *name;
 } ATTR_DECODE_ENTRY, *PATTR_DECODE_ENTRY;
 
-static ATTR_DECODE_ENTRY attr_decode_table[] =
+static ATTR_DECODE_ENTRY attr_decode_table[NUM_ATTRIBUTE_CODES+1] =
 {
-    { AF_LOCK,    '+' },
-    { AF_NOPROG,  '$' },
-    { AF_CASE,    'C' },
-    { AF_HTML,    'H' },
-    { AF_PRIVATE, 'I' },
-    { AF_NOPARSE, 'P' },
-    { AF_REGEXP,  'R' },
-    { AF_VISUAL,  'V' },
-    { AF_MDARK,   'M' },
-    { AF_WIZARD,  'W' },
-    { 0, 0 }
+    { AF_LOCK,    '+', "LOCK"        },
+    { AF_NOPROG,  '$', "NO_COMMAND"  },
+    { AF_CASE,    'C', "CASE"        },
+    { AF_HTML,    'H', "HTML"        },
+    { AF_PRIVATE, 'I', "NO_INHERIT"  },
+    { AF_NONAME,  'N', "NO_NAME"     },
+    { AF_NOPARSE, 'P', "NO_PARSE"    },
+    { AF_REGEXP,  'R', "REGEXP"      },
+    { AF_TRACE,   'T', "TRACE"       },
+    { AF_VISUAL,  'V', "VISUAL"      },
+    { AF_MDARK,   'M', "DARK"        },
+    { AF_WIZARD,  'W', "WIZARD"      },
+    { 0, 0, NULL }
 };
 
-size_t decode_attr_flags(int aflags, char *buff)
+size_t decode_attr_flags(int aflags, char buff[NUM_ATTRIBUTE_CODES+1])
 {
-    char *p = buff;
+    size_t n = 0;
     PATTR_DECODE_ENTRY pEntry;
+    for (pEntry = attr_decode_table; pEntry->mask && n < NUM_ATTRIBUTE_CODES; pEntry++)
+    {
+        if (aflags & pEntry->mask)
+        {
+            buff[n++] = pEntry->letter;
+        }
+    }
+    buff[n] = '\0';
+    return n;
+}
+
+void decode_attr_flag_names(int aflags, char *buf, char **bufc)
+{
+    PATTR_DECODE_ENTRY pEntry;
+    bool bFirst = true;
     for (pEntry = attr_decode_table; pEntry->mask; pEntry++)
     {
         if (aflags & pEntry->mask)
         {
-            *p++ = pEntry->letter;
+            if (!bFirst)
+            {
+                safe_chr(' ', buf, bufc);
+            }
+            bFirst = false;
+            safe_str(pEntry->name, buf, bufc);
         }
     }
-    *p = '\0';
-    return p - buff;
 }
 
 static void view_atr
@@ -993,26 +1015,32 @@ static bool show_a_desc(dbref player, dbref loc)
 
     bool ret = false;
 
-    dbref aowner;
-    int aflags;
+    dbref aowner1;
+    int   aflags1;
     bool indent = (isRoom(loc) && mudconf.indent_desc && atr_get_raw(loc, A_DESC));
 
-    char *DescFormatBuffer = atr_pget(loc, A_DESCFORMAT, &aowner, &aflags);
+    char *DescFormatBuffer = atr_pget(loc, A_DESCFORMAT, &aowner1, &aflags1);
     char *DescFormat = DescFormatBuffer;
     if (*DescFormat)
     {
+        reg_ref **preserve = NULL;
+        preserve = PushRegisters(MAX_GLOBAL_REGS);
+        save_global_regs(preserve);
+
         char *FormatOutput = alloc_lbuf("look_description.FO");
         char *tPtr = FormatOutput;
 
         ATTR *cattr = atr_num(iDescDefault);
 
-        char *tbuf1 = atr_pget(loc, iDescDefault, &aowner, &aflags);
+        dbref aowner2;
+        int   aflags2;
+        char *tbuf1 = atr_pget(loc, iDescDefault, &aowner2, &aflags2);
         char *str = tbuf1;
         char *temp = alloc_lbuf("look_description.ET");
         char *bp = temp;
         mux_exec(temp, &bp, loc, player, player,
-               EV_FCHECK | EV_EVAL | EV_TOP,
-               &str, (char **)NULL, 0);
+            AttrTrace(aflags2, EV_FCHECK|EV_EVAL|EV_TOP),
+            &str, NULL, 0);
         *bp = '\0';
 
         char *attrname = alloc_lbuf("look_description.AN");
@@ -1024,21 +1052,27 @@ static bool show_a_desc(dbref player, dbref loc)
             { temp, attrname };
 
         mux_exec(FormatOutput, &tPtr, loc, player, player,
-                EV_FCHECK | EV_EVAL | EV_TOP,
-                &DescFormat, ParameterList, 2);
+            AttrTrace(aflags1, EV_FCHECK|EV_EVAL|EV_TOP),
+            &DescFormat, ParameterList, 2);
+        *tPtr = '\0';
 
         notify(player, FormatOutput);
 #ifdef REALITY_LVLS
-        did_it_rlevel(player, loc, 0, NULL, A_ODESC, NULL, iADescDefault, (char **) NULL, 0);
+        did_it_rlevel(player, loc, 0, NULL, A_ODESC, NULL, iADescDefault, 0,
+            NULL, 0);
 #else
-        did_it(player, loc, 0, NULL, A_ODESC, NULL, iADescDefault, (char **) NULL, 0);
-#endif /* REALITY_LVLS */
+        did_it(player, loc, 0, NULL, A_ODESC, NULL, iADescDefault, 0,
+            NULL, 0);
+#endif // REALITY_LVLS
 
         free_lbuf(tbuf1);
         free_lbuf(attrname);
         free_lbuf(FormatOutput);
         free_lbuf(temp);
 
+        restore_global_regs(preserve);
+        PopRegisters(preserve, MAX_GLOBAL_REGS);
+
         ret = true;
     }
     else
@@ -1046,20 +1080,22 @@ static bool show_a_desc(dbref player, dbref loc)
         char *got;
         if (Html(player))
         {
-            got = atr_pget(loc, A_HTDESC, &aowner, &aflags);
+            got = atr_pget(loc, A_HTDESC, &aowner1, &aflags1);
             if (*got)
             {
 #ifdef REALITY_LVLS
-                did_it_rlevel(player, loc, A_HTDESC, NULL, A_ODESC, NULL, A_ADESC, (char **) NULL, 0);
+                did_it_rlevel(player, loc, A_HTDESC, NULL, A_ODESC, NULL,
+                    A_ADESC, 0, NULL, 0);
 #else
-                did_it(player, loc, A_HTDESC, NULL, A_ODESC, NULL, A_ADESC, (char **) NULL, 0);
-#endif /* REALITY_LVLS */
+                did_it(player, loc, A_HTDESC, NULL, A_ODESC, NULL, A_ADESC,
+                    0, NULL, 0);
+#endif // REALITY_LVLS
                 ret = true;
             }
             else
             {
                 free_lbuf(got);
-                got = atr_pget(loc, iDescDefault, &aowner, &aflags);
+                got = atr_pget(loc, iDescDefault, &aowner1, &aflags1);
                 if (*got)
                 {
                     if (indent)
@@ -1067,10 +1103,12 @@ static bool show_a_desc(dbref player, dbref loc)
                         raw_notify_newline(player);
                     }
 #ifdef REALITY_LVLS
-                    did_it_rlevel(player, loc, iDescDefault, NULL, A_ODESC, NULL, iADescDefault, (char **) NULL, 0);
+                    did_it_rlevel(player, loc, iDescDefault, NULL, A_ODESC,
+                        NULL, iADescDefault, 0, NULL, 0);
 #else
-                    did_it(player, loc, iDescDefault, NULL, A_ODESC, NULL, iADescDefault, (char **) NULL, 0);
-#endif /* REALITY_LVLS */
+                    did_it(player, loc, iDescDefault, NULL, A_ODESC, NULL,
+                        iADescDefault, 0, NULL, 0);
+#endif // REALITY_LVLS
                     if (indent)
                     {
                         raw_notify_newline(player);
@@ -1079,17 +1117,19 @@ static bool show_a_desc(dbref player, dbref loc)
                 }
             }
         }
-        else if (*(got = atr_pget(loc, iDescDefault, &aowner, &aflags)))
+        else if (*(got = atr_pget(loc, iDescDefault, &aowner1, &aflags1)))
         {
             if (indent)
             {
                 raw_notify_newline(player);
             }
 #ifdef REALITY_LVLS
-            did_it_rlevel(player, loc, iDescDefault, NULL, A_ODESC, NULL, iADescDefault, (char **) NULL, 0);
+            did_it_rlevel(player, loc, iDescDefault, NULL, A_ODESC, NULL,
+                iADescDefault, 0, NULL, 0);
 #else
-            did_it(player, loc, iDescDefault, NULL, A_ODESC, NULL, iADescDefault, (char **) NULL, 0);
-#endif /* REALITY_LVLS */
+            did_it(player, loc, iDescDefault, NULL, A_ODESC, NULL,
+                iADescDefault, 0, NULL, 0);
+#endif // REALITY_LVLS
             if (indent)
             {
                 raw_notify_newline(player);
@@ -1125,7 +1165,7 @@ static void look_simple(dbref player, dbref thing, bool obey_terse)
     int can_see_thing = Examinable(player, thing);
     if (can_see_thing)
     {
-        char *buff = unparse_object(player, thing, true);
+        char *buff = unparse_object(player, thing, true, true);
         notify(player, buff);
         free_lbuf(buff);
     }
@@ -1142,11 +1182,11 @@ static void look_simple(dbref player, dbref thing, bool obey_terse)
         notify(player, "You see nothing special.");
 #ifdef REALITY_LVLS
         did_it_rlevel(player, thing, 0, NULL, A_ODESC, NULL, iADescDefault,
-            (char **)NULL, 0);
+            0, NULL, 0);
 #else
         did_it(player, thing, pattr, NULL, A_ODESC, NULL, iADescDefault,
-            (char **)NULL, 0);
-#endif /* REALITY_LVLS */
+            0, NULL, 0);
+#endif // REALITY_LVLS
     }
 
     if (  !mudconf.quiet_look
@@ -1167,10 +1207,12 @@ static void show_desc(dbref player, dbref loc, int key)
        && Terse(player))
     {
 #ifdef REALITY_LVLS
-        did_it_rlevel(player, loc, 0, NULL, A_ODESC, NULL, A_ADESC, (char **)NULL, 0);
+        did_it_rlevel(player, loc, 0, NULL, A_ODESC, NULL, A_ADESC, 0,
+            NULL, 0);
 #else
-        did_it(player, loc, 0, NULL, A_ODESC, NULL, A_ADESC, (char **)NULL, 0);
-#endif /* REALITY_LVLS */
+        did_it(player, loc, 0, NULL, A_ODESC, NULL, A_ADESC, 0,
+            NULL, 0);
+#endif // REALITY_LVLS
     }
     else if (  !isRoom(loc)
             && (key & LK_IDESC))
@@ -1178,10 +1220,12 @@ static void show_desc(dbref player, dbref loc, int key)
         if (*(got = atr_pget(loc, A_IDESC, &aowner, &aflags)))
         {
 #ifdef REALITY_LVLS
-           did_it_rlevel(player, loc, A_IDESC, NULL, A_ODESC, NULL, A_ADESC, (char **)NULL, 0);
+           did_it_rlevel(player, loc, A_IDESC, NULL, A_ODESC, NULL, A_ADESC,
+               0, NULL, 0);
 #else
-            did_it(player, loc, A_IDESC, NULL, A_ODESC, NULL, A_ADESC, (char **)NULL, 0);
-#endif /* REALITY_LVLS */
+            did_it(player, loc, A_IDESC, NULL, A_ODESC, NULL, A_ADESC, 0,
+                NULL, 0);
+#endif // REALITY_LVLS
         }
         else
         {
@@ -1224,15 +1268,17 @@ void look_in(dbref player, dbref loc, int key)
         char *FormatOutput = alloc_lbuf("look_name.FO");
         char *tPtr = FormatOutput;
 
-        char *preserve[MAX_GLOBAL_REGS];
-        int preserve_len[MAX_GLOBAL_REGS];
-        save_and_clear_global_regs("look_in_save", preserve, preserve_len);
+        reg_ref **preserve = NULL;
+        preserve = PushRegisters(MAX_GLOBAL_REGS);
+        save_and_clear_global_regs(preserve);
 
         mux_exec(FormatOutput, &tPtr, loc, player, player,
-                EV_FCHECK | EV_EVAL | EV_TOP,
-                &NameFormat, 0, 0);
+            AttrTrace(aflags, EV_FCHECK|EV_EVAL|EV_TOP),
+            &NameFormat, 0, 0);
+        *tPtr = '\0';
 
-        restore_global_regs("look_in_restore", preserve, preserve_len);
+        restore_global_regs(preserve);
+        PopRegisters(preserve, MAX_GLOBAL_REGS);
         notify(player, FormatOutput);
 
         free_lbuf(FormatOutput);
@@ -1241,7 +1287,7 @@ void look_in(dbref player, dbref loc, int key)
     {
         // Okay, no @NameFormat.  Show the normal name.
         //
-        char *buff = unparse_object(player, loc, true);
+        char *buff = unparse_object(player, loc, true, true);
         if (Html(player))
         {
             notify_html(player, "<center><h3>");
@@ -1299,7 +1345,8 @@ void look_in(dbref player, dbref loc, int key)
         {
             pattr = 0;
         }
-        did_it(player, loc, pattr, NULL, oattr, NULL, aattr, (char **)NULL, 0);
+        did_it(player, loc, pattr, NULL, oattr, NULL, aattr, 0,
+            NULL, 0);
     }
 
     // Tell him the attributes, contents and exits.
@@ -1323,10 +1370,35 @@ void look_in(dbref player, dbref loc, int key)
     }
 }
 
-void do_look(dbref executor, dbref caller, dbref enactor, int key, char *name)
+static void look_here
+(
+    dbref executor,
+    dbref thing,
+    int   key,
+    int   look_key
+)
+{
+    if (Good_obj(thing))
+    {
+        if (key & LOOK_OUTSIDE)
+        {
+            if (  isRoom(thing)
+               || Opaque(thing))
+            {
+                notify_quiet(executor, "You can't look outside.");
+                return;
+            }
+            thing = Location(thing);
+        }
+        look_in(executor, thing, look_key);
+    }
+}
+
+void do_look(dbref executor, dbref caller, dbref enactor, int eval, int key, char *name)
 {
     UNUSED_PARAMETER(caller);
     UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(eval);
 
     int look_key = LK_SHOWATTR | LK_SHOWEXIT;
     if (!mudconf.terse_look)
@@ -1336,23 +1408,10 @@ void do_look(dbref executor, dbref caller, dbref enactor, int key, char *name)
 
     dbref loc = Location(executor);
     dbref thing;
-    if (!name || !*name)
+    if (  NULL == name
+       || '\0' == name[0])
     {
-        thing = loc;
-        if (Good_obj(thing))
-        {
-            if (key & LOOK_OUTSIDE)
-            {
-                if (  isRoom(thing)
-                   || Opaque(thing))
-                {
-                    notify_quiet(executor, "You can't look outside.");
-                    return;
-                }
-                thing = Location(thing);
-            }
-            look_in(executor, thing, look_key);
-        }
+        look_here(executor, loc, key, look_key);
         return;
     }
 
@@ -1381,6 +1440,15 @@ void do_look(dbref executor, dbref caller, dbref enactor, int key, char *name)
             ((key & LOOK_OUTSIDE) ? loc : executor), name, thing, false));
     }
 
+    // If the matching picked our current location, that's handled
+    // differently.
+    //
+    if (thing == loc)
+    {
+        look_here(executor, loc, key, look_key);
+        return;
+    }
+
     // If we found something, go handle it.
     //
     if (Good_obj(thing))
@@ -1388,7 +1456,7 @@ void do_look(dbref executor, dbref caller, dbref enactor, int key, char *name)
 #ifdef REALITY_LVLS
         if (!IsReal(executor, thing))
             return;
-#endif /* REALITY_LVLS */
+#endif // REALITY_LVLS
         switch (Typeof(thing))
         {
         case TYPE_ROOM:
@@ -1464,7 +1532,7 @@ static void debug_examine(dbref player, dbref thing)
     buf = txlevel_description(player, thing);
     notify(player, tprintf("TxLevel = %s", buf));
     free_lbuf(buf);
-#endif /* REALITY_LVLS */
+#endif // REALITY_LVLS
     buf = atr_get(thing, A_LOCK, &aowner, &aflags);
     pBoolExp = parse_boolexp(player, buf, true);
     free_lbuf(buf);
@@ -1592,10 +1660,11 @@ static void exam_wildattrs
     }
 }
 
-void do_examine(dbref executor, dbref caller, dbref enactor, int key, char *name)
+void do_examine(dbref executor, dbref caller, dbref enactor, int eval, int key, char *name)
 {
     UNUSED_PARAMETER(caller);
     UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(eval);
 
     // This command is pointless if the player can't hear.
     //
@@ -1673,7 +1742,7 @@ void do_examine(dbref executor, dbref caller, dbref enactor, int key, char *name
 
     if (control)
     {
-        buf2 = unparse_object(executor, thing, false);
+        buf2 = unparse_object(executor, thing, false, true);
         notify(executor, buf2);
         free_lbuf(buf2);
         if (mudconf.ex_flags)
@@ -1691,15 +1760,15 @@ void do_examine(dbref executor, dbref caller, dbref enactor, int key, char *name
             if (mudconf.read_rem_name)
             {
                 buf2 = alloc_lbuf("do_examine.pub_name");
-                strcpy(buf2, Name(thing));
+                mux_strncpy(buf2, Moniker(thing), LBUF_SIZE-1);
                 notify(executor,
                     tprintf("%s is owned by %s",
-                    buf2, Name(Owner(thing))));
+                    buf2, Moniker(Owner(thing))));
                 free_lbuf(buf2);
             }
             else
             {
-                notify(executor, tprintf("Owned by %s", Name(Owner(thing))));
+                notify(executor, tprintf("Owned by %s", Moniker(Owner(thing))));
             }
             return;
         }
@@ -1741,7 +1810,7 @@ void do_examine(dbref executor, dbref caller, dbref enactor, int key, char *name
         pBoolExp = parse_boolexp(executor, buf2, true);
         buf = unparse_boolexp(executor, pBoolExp);
         free_boolexp(pBoolExp);
-        strcpy(buf2, Name(Owner(thing)));
+        mux_strncpy(buf2, Moniker(Owner(thing)), LBUF_SIZE-1);
         notify(executor, tprintf("Owner: %s  Key: %s %s: %d", buf2, buf, mudconf.many_coins, Pennies(thing)));
         free_lbuf(buf2);
         mudconf.many_coins[0] = savec;
@@ -1750,7 +1819,7 @@ void do_examine(dbref executor, dbref caller, dbref enactor, int key, char *name
         //
         if (mudconf.have_zones)
         {
-            buf2 = unparse_object(executor, Zone(thing), false);
+            buf2 = unparse_object(executor, Zone(thing), false, true);
             notify(executor, tprintf("Zone: %s", buf2));
             free_lbuf(buf2);
         }
@@ -1760,7 +1829,7 @@ void do_examine(dbref executor, dbref caller, dbref enactor, int key, char *name
         loc = Parent(thing);
         if (loc != NOTHING)
         {
-            buf2 = unparse_object(executor, loc, false);
+            buf2 = unparse_object(executor, loc, false, true);
             notify(executor, tprintf("Parent: %s", buf2));
             free_lbuf(buf2);
         }
@@ -1768,15 +1837,15 @@ void do_examine(dbref executor, dbref caller, dbref enactor, int key, char *name
         notify(executor, tprintf("Powers: %s", buf2));
         free_lbuf(buf2);
 #ifdef REALITY_LVLS
-        /* Show Rx and Tx levels */
-
+        // Show Rx and Tx levels.
+        //
         buf2 = rxlevel_description(executor, thing);
-        notify(executor, tprintf("TxLevel: %s", buf2));
-        free_mbuf(buf2);
-        buf2 = txlevel_description(executor, thing);
         notify(executor, tprintf("RxLevel: %s", buf2));
         free_mbuf(buf2);
-#endif /* REALITY_LVLS */
+        buf2 = txlevel_description(executor, thing);
+        notify(executor, tprintf("TxLevel: %s", buf2));
+        free_mbuf(buf2);
+#endif // REALITY_LVLS
     }
     if (!(key & EXAM_BRIEF))
     {
@@ -1794,7 +1863,7 @@ void do_examine(dbref executor, dbref caller, dbref enactor, int key, char *name
             notify(executor, "Contents:");
             DOLIST(content, Contents(thing))
             {
-                buf2 = unparse_object(executor, content, false);
+                buf2 = unparse_object(executor, content, false, true);
                 notify(executor, buf2);
                 free_lbuf(buf2);
             }
@@ -1812,7 +1881,7 @@ void do_examine(dbref executor, dbref caller, dbref enactor, int key, char *name
                 notify(executor, "Exits:");
                 DOLIST(exit, Exits(thing))
                 {
-                    buf2 = unparse_object(executor, exit, false);
+                    buf2 = unparse_object(executor, exit, false, true);
                     notify(executor, buf2);
                     free_lbuf(buf2);
                 }
@@ -1826,7 +1895,7 @@ void do_examine(dbref executor, dbref caller, dbref enactor, int key, char *name
             //
             if (Dropto(thing) != NOTHING)
             {
-                buf2 = unparse_object(executor, Dropto(thing), false);
+                buf2 = unparse_object(executor, Dropto(thing), false, true);
                 notify(executor, tprintf("Dropped objects go to: %s", buf2));
                 free_lbuf(buf2);
             }
@@ -1842,7 +1911,7 @@ void do_examine(dbref executor, dbref caller, dbref enactor, int key, char *name
                 notify(executor, "Exits:");
                 DOLIST(exit, Exits(thing))
                 {
-                    buf2 = unparse_object(executor, exit, false);
+                    buf2 = unparse_object(executor, exit, false, true);
                     notify(executor, buf2);
                     free_lbuf(buf2);
                 }
@@ -1855,7 +1924,7 @@ void do_examine(dbref executor, dbref caller, dbref enactor, int key, char *name
             // Print home
             //
             loc = Home(thing);
-            buf2 = unparse_object(executor, loc, false);
+            buf2 = unparse_object(executor, loc, false, true);
             notify(executor, tprintf("Home: %s", buf2));
             free_lbuf(buf2);
 
@@ -1867,14 +1936,14 @@ void do_examine(dbref executor, dbref caller, dbref enactor, int key, char *name
                   || Examinable(executor, thing)
                   || Linkable(executor, loc)))
             {
-                buf2 = unparse_object(executor, loc, false);
+                buf2 = unparse_object(executor, loc, false, true);
                 notify(executor, tprintf("Location: %s", buf2));
                 free_lbuf(buf2);
             }
             break;
 
         case TYPE_EXIT:
-            buf2 = unparse_object(executor, Exits(thing), false);
+            buf2 = unparse_object(executor, Exits(thing), false, true);
             notify(executor, tprintf("Source: %s", buf2));
             free_lbuf(buf2);
 
@@ -1889,7 +1958,7 @@ void do_examine(dbref executor, dbref caller, dbref enactor, int key, char *name
                 break;
 
             default:
-                buf2 = unparse_object(executor, Location(thing), false);
+                buf2 = unparse_object(executor, Location(thing), false, true);
                 notify(executor, tprintf("Destination: %s", buf2));
                 free_lbuf(buf2);
                 break;
@@ -1919,13 +1988,13 @@ void do_examine(dbref executor, dbref caller, dbref enactor, int key, char *name
         if (mudconf.read_rem_name)
         {
             buf2 = alloc_lbuf("do_examine.pub_name");
-            strcpy(buf2, Name(thing));
-            notify(executor, tprintf("%s is owned by %s", buf2, Name(Owner(thing))));
+            mux_strncpy(buf2, Moniker(thing), LBUF_SIZE-1);
+            notify(executor, tprintf("%s is owned by %s", buf2, Moniker(Owner(thing))));
             free_lbuf(buf2);
         }
         else
         {
-            notify(executor, tprintf("Owned by %s", Name(Owner(thing))));
+            notify(executor, tprintf("Owned by %s", Moniker(Owner(thing))));
         }
     }
 }
@@ -1960,7 +2029,7 @@ void do_inventory(dbref executor, dbref caller, dbref enactor, int key)
         notify(executor, "You are carrying:");
         DOLIST(thing, thing)
         {
-            buff = unparse_object(executor, thing, true);
+            buff = unparse_object(executor, thing, true, true);
             notify(executor, buff);
             free_lbuf(buff);
         }
@@ -1975,7 +2044,7 @@ void do_inventory(dbref executor, dbref caller, dbref enactor, int key)
         {
             // Chop off first exit alias to display.
             //
-            for (s = Name(thing); *s && (*s != ';'); s++)
+            for (s = Moniker(thing); *s && (*s != ';'); s++)
             {
                 safe_chr(*s, buff, &e);
             }
@@ -1988,10 +2057,11 @@ void do_inventory(dbref executor, dbref caller, dbref enactor, int key)
     do_score(executor, caller, executor, 0);
 }
 
-void do_entrances(dbref executor, dbref caller, dbref enactor, int key, char *name)
+void do_entrances(dbref executor, dbref caller, dbref enactor, int eval, int key, char *name)
 {
     UNUSED_PARAMETER(caller);
     UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(eval);
     UNUSED_PARAMETER(key);
 
     dbref thing, i, j;
@@ -2045,8 +2115,8 @@ void do_entrances(dbref executor, dbref caller, dbref enactor, int key, char *na
             case TYPE_EXIT:
                 if (Location(i) == thing)
                 {
-                    exit = unparse_object(executor, Exits(i), false);
-                    notify(executor, tprintf("%s (%s)", exit, Name(i)));
+                    exit = unparse_object(executor, Exits(i), false, true);
+                    notify(executor, tprintf("%s (%s)", exit, Moniker(i)));
                     free_lbuf(exit);
                     count++;
                 }
@@ -2054,7 +2124,7 @@ void do_entrances(dbref executor, dbref caller, dbref enactor, int key, char *na
             case TYPE_ROOM:
                 if (Dropto(i) == thing)
                 {
-                    exit = unparse_object(executor, i, false);
+                    exit = unparse_object(executor, i, false, true);
                     notify(executor, tprintf("%s [dropto]", exit));
                     free_lbuf(exit);
                     count++;
@@ -2064,7 +2134,7 @@ void do_entrances(dbref executor, dbref caller, dbref enactor, int key, char *na
             case TYPE_PLAYER:
                 if (Home(i) == thing)
                 {
-                    exit = unparse_object(executor, i, false);
+                    exit = unparse_object(executor, i, false, true);
                     notify(executor, tprintf("%s [home]", exit));
                     free_lbuf(exit);
                     count++;
@@ -2076,7 +2146,7 @@ void do_entrances(dbref executor, dbref caller, dbref enactor, int key, char *na
             //
             if (Parent(i) == thing)
             {
-                exit = unparse_object(executor, i, false);
+                exit = unparse_object(executor, i, false, true);
                 notify(executor, tprintf("%s [parent]", exit));
                 free_lbuf(exit);
                 count++;
@@ -2097,7 +2167,7 @@ void do_entrances(dbref executor, dbref caller, dbref enactor, int key, char *na
                     {
                         continue;
                     }
-                    exit = unparse_object(executor, i, false);
+                    exit = unparse_object(executor, i, false, true);
                     notify(executor, tprintf("%s [forward]", exit));
                     free_lbuf(exit);
                     count++;
@@ -2281,7 +2351,7 @@ static void sweep_check(dbref player, dbref what, int key, bool is_loc)
         if (ispuppet)
         {
             safe_str("puppet(", buf, &bp);
-            safe_str(Name(Owner(what)), buf, &bp);
+            safe_str(Moniker(Owner(what)), buf, &bp);
             safe_str(") ", buf, &bp);
         }
 
@@ -2299,12 +2369,12 @@ static void sweep_check(dbref player, dbref what, int key, bool is_loc)
         if (!isExit(what))
         {
             notify(player, tprintf("  %s is listening. [%s]",
-                Name(what), buf));
+                Moniker(what), buf));
         }
         else
         {
             char *buf2 = alloc_lbuf("sweep_check.name");
-            strcpy(buf2, Name(what));
+            mux_strncpy(buf2, Moniker(what), LBUF_SIZE-1);
             for (bp = buf2; *bp && (*bp != ';'); bp++)
             {
                 ; // Nothing.
@@ -2317,10 +2387,11 @@ static void sweep_check(dbref player, dbref what, int key, bool is_loc)
     }
 }
 
-void do_sweep(dbref executor, dbref caller, dbref enactor, int key, char *where)
+void do_sweep(dbref executor, dbref caller, dbref enactor, int eval, int key, char *where)
 {
     UNUSED_PARAMETER(caller);
     UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(eval);
 
     dbref here, sweeploc;
     int where_key, what_key;
@@ -2491,20 +2562,20 @@ void do_decomp
     //
     if (qual && *qual)
     {
-        strcpy(thingname, qual);
+        mux_strncpy(thingname, qual, LBUF_SIZE-1);
     }
     else
     {
         if (key == DECOMP_DBREF)
         {
-            strcpy(thingname, tprintf("#%d",thing));
+            mux_strncpy(thingname, tprintf("#%d",thing), LBUF_SIZE-1);
         }
         else
         {
             switch (Typeof(thing))
             {
             case TYPE_THING:
-                strcpy(thingname, Name(thing));
+                mux_strncpy(thingname, Moniker(thing), LBUF_SIZE-1);
                 val = OBJECT_DEPOSIT(Pennies(thing));
                 notify(executor,
                     tprintf("@create %s=%d", translate_string(thingname, true),
@@ -2512,13 +2583,13 @@ void do_decomp
                 break;
 
             case TYPE_ROOM:
-                strcpy(thingname, "here");
+                mux_strncpy(thingname, "here", LBUF_SIZE-1);
                 notify(executor, tprintf("@dig/teleport %s",
-                    translate_string(Name(thing), true)));
+                    translate_string(Moniker(thing), true)));
                 break;
 
             case TYPE_EXIT:
-                strcpy(thingname, Name(thing));
+                mux_strncpy(thingname, Moniker(thing), LBUF_SIZE-1);
                 notify(executor,
                     tprintf("@open %s", translate_string(thingname, true)));
                 for (got = thingname; *got; got++)
@@ -2534,11 +2605,11 @@ void do_decomp
             case TYPE_PLAYER:
                 if (executor == thing)
                 {
-                    strcpy(thingname, "me");
+                    mux_strncpy(thingname, "me", LBUF_SIZE-1);
                 }
                 else
                 {
-                    strcpy(thingname, Name(thing));
+                    mux_strncpy(thingname, Name(thing), LBUF_SIZE-1);
                 }
                 break;
             }
@@ -2598,7 +2669,7 @@ void do_decomp
             }
             else
             {
-                strcpy(buff, pattr->name);
+                mux_strncpy(buff, pattr->name, MBUF_SIZE-1);
                 notify(executor, tprintf("%c%s %s=%s", ((ca < A_USER_START) ?
                     '@' : '&'), buff, thingname, got));
                 for (np = indiv_attraccess_nametab; np->name; np++)
@@ -2628,7 +2699,7 @@ void do_decomp
         decompile_powers(executor, thing, thingname);
 #ifdef REALITY_LVLS
         decompile_rlevels(executor, thing, thingname);
-#endif /* REALITY_LVLS */
+#endif // REALITY_LVLS
     }
 
     // If the object has a parent, report it.
