@@ -1,6 +1,6 @@
 // match.cpp -- Routines for parsing arguments.
 //
-// $Id: match.cpp,v 1.8 2006/01/07 19:28:34 sdennis Exp $
+// $Id: match.cpp 492 2006-11-30 17:12:25Z brazilofmux $
 //
 
 #include "copyright.h"
@@ -12,7 +12,7 @@
 #include "powers.h"
 #ifdef REALITY_LVLS
 #include "levels.h"
-#endif /* REALITY_LVLS */
+#endif // REALITY_LVLS
 
 const char *NOMATCH_MESSAGE      = "I don't see that here.";
 const char *AMBIGUOUS_MESSAGE    = "I don't know which one you mean!";
@@ -36,11 +36,16 @@ static MSTATE md;
 static void promote_match(dbref what, int confidence)
 {
 #ifdef REALITY_LVLS
-    // Check is the object is visible
-    if(Good_obj(what) && (confidence & CON_LOCAL) &&
-      !IsReal(md.player, what) && what != Location(md.player))
+    // Check is the object is visible.
+    //
+    if(  Good_obj(what)
+      && (confidence & CON_LOCAL)
+      && !IsReal(md.player, what)
+      && what != Location(md.player))
+    {
         return;
-#endif /* REALITY_LVLS */
+    }
+#endif // REALITY_LVLS
     // Check for type and locks, if requested.
     //
     if (md.pref_type != NOTYPE)
@@ -127,7 +132,8 @@ static char *munge_space_for_match(char *name)
             while (  *p
                   && !mux_isspace(*p))
             {
-                *q++ = *p++;
+                safe_chr(*p, buffer, &q);
+                p++;
             }
 
             while (mux_isspace(*p))
@@ -137,7 +143,7 @@ static char *munge_space_for_match(char *name)
 
             if (*p)
             {
-                *q++ = ' ';
+                safe_chr(' ', buffer, &q);
             }
         }
     }
@@ -539,7 +545,11 @@ dbref match_result(void)
         return md.match;
 
     default:
+#if defined(FIRANMUX)
+        return md.match;
+#else
         return AMBIGUOUS;
+#endif // FIRANMUX
     }
 }
 
@@ -584,7 +594,7 @@ void save_match_state(MSTATE *mstate)
     mstate->match = md.match;
     mstate->player = md.player;
     mstate->string = alloc_lbuf("save_match_state");
-    strcpy(mstate->string, md.string);
+    mux_strncpy(mstate->string, md.string, LBUF_SIZE-1);
 }
 
 void restore_match_state(MSTATE *mstate)
@@ -596,7 +606,7 @@ void restore_match_state(MSTATE *mstate)
     md.absolute_form = mstate->absolute_form;
     md.match = mstate->match;
     md.player = mstate->player;
-    strcpy(md.string, mstate->string);
+    mux_strncpy(md.string, mstate->string, LBUF_SIZE-1);
     free_lbuf(mstate->string);
 }
 
