@@ -1,6 +1,6 @@
 // move.cpp -- Routines for moving about.
 //
-// $Id: move.cpp,v 1.7 2006/01/07 18:29:14 sdennis Exp $
+// $Id: move.cpp 2734 2007-10-28 23:02:55Z brazilofmux $
 //
 
 #include "copyright.h"
@@ -14,7 +14,7 @@
 #include "powers.h"
 #ifdef REALITY_LVLS
 #include "levels.h"
-#endif /* REALITY_LVLS */
+#endif // REALITY_LVLS
 
 /* ---------------------------------------------------------------------------
  * process_leave_loc: Generate messages and actions resulting from leaving a
@@ -53,7 +53,7 @@ static void process_leave_loc(dbref thing, dbref dest, dbref cause, bool canhear
     bool quiet = (  (hush & HUSH_LEAVE) || !IsReal(loc, thing)
 #else
     bool quiet = (  (hush & HUSH_LEAVE)
-#endif /* REALITY_LVLS */
+#endif // REALITY_LVLS
                  || (  !Wizard(loc)
                     && (  Dark(thing)
                        || Dark(loc))
@@ -64,14 +64,13 @@ static void process_leave_loc(dbref thing, dbref dest, dbref cause, bool canhear
     int oattr = quiet ? 0 : A_OLEAVE;
     int aattr = quiet ? 0 : A_ALEAVE;
     int pattr = (!mudconf.terse_movemsg && Terse(thing)) ? 0 : A_LEAVE;
-    did_it(thing, loc, pattr, NULL, oattr, NULL, aattr,
-           (char **)NULL, 0);
+    did_it(thing, loc, pattr, NULL, oattr, NULL, aattr, 0, NULL, 0);
 
     // Do OXENTER for receiving room
     //
     if ((dest != NOTHING) && !quiet)
     {
-        did_it(thing, dest, 0, NULL, A_OXENTER, NULL, 0, (char **)NULL, 0);
+        did_it(thing, dest, 0, NULL, A_OXENTER, NULL, 0, 0, NULL, 0);
     }
 
     // Display the 'has left' message if we meet any of the following
@@ -87,14 +86,18 @@ static void process_leave_loc(dbref thing, dbref dest, dbref cause, bool canhear
         if (  (  !Dark(thing)
               && !Dark(loc))
            || (  canhear
+#if defined(FIRANMUX)
+              && !Dark(thing)))
+#else
               && !(Wizard(thing) && Dark(thing))))
+#endif // FIRANMUX
         {
 #ifdef REALITY_LVLS
             notify_except2_rlevel(loc, thing, thing, cause,
 #else
             notify_except2(loc, thing, thing, cause,
-#endif /* REALITY_LVLS */
-                       tprintf("%s has left.", Name(thing)));
+#endif // REALITY_LVLS
+                       tprintf("%s has left.", Moniker(thing)));
         }
     }
 }
@@ -128,27 +131,34 @@ static void process_enter_loc(dbref thing, dbref src, dbref cause, bool canhear,
     bool quiet = (  (hush & HUSH_ENTER) || !IsReal(loc, thing)
 #else
     bool quiet = (  (hush & HUSH_ENTER)
-#endif /* REALITY_LVLS */
+#endif // REALITY_LVLS
                  || (  !Wizard(loc)
                     && (  Dark(thing)
                        || Dark(loc))
                     && (  !canhear
+#if defined(FIRANMUX)
+                       || Dark(thing))));
+#else
                        || (  Wizard(thing)
                           && Dark(thing)))));
+#endif // FIRANMUX
 
     int oattr = quiet ? 0 : A_OENTER;
+#if defined(FIRANMUX)
+    int aattr = (hush & HUSH_ENTER) ? 0 : A_AENTER;
+#else
     int aattr = quiet ? 0 : A_AENTER;
+#endif // FIRANMUX
     int pattr = (!mudconf.terse_movemsg && Terse(thing)) ? 0 : A_ENTER;
 
-    did_it(thing, loc, pattr, NULL, oattr, NULL, aattr,
-           (char **)NULL, 0);
+    did_it(thing, loc, pattr, NULL, oattr, NULL, aattr, 0, NULL, 0);
 
     // Do OXLEAVE for sending room.
     //
     if (  src != NOTHING
        && !quiet)
     {
-        did_it(thing, src, 0, NULL, A_OXLEAVE, NULL, 0, (char **)NULL, 0);
+        did_it(thing, src, 0, NULL, A_OXLEAVE, NULL, 0, 0, NULL, 0);
     }
 
     // Display the 'has arrived' message if we meet all of the following
@@ -161,14 +171,18 @@ static void process_enter_loc(dbref thing, dbref src, dbref cause, bool canhear,
        && canhear
        && !Blind(thing)
        && !Blind(loc)
+#if defined(FIRANMUX)
+       && !Dark(thing))
+#else
        && !(Dark(thing) && Wizard(thing)))
+#endif // FIRANMUX
     {
 #ifdef REALITY_LVLS
         notify_except2_rlevel(loc, thing, thing, cause,
 #else
         notify_except2(loc, thing, thing, cause,
-#endif /* REALITY_LVLS */
-                   tprintf("%s has arrived.", Name(thing)));
+#endif // REALITY_LVLS
+                   tprintf("%s has arrived.", Moniker(thing)));
     }
 }
 
@@ -322,8 +336,12 @@ void move_via_generic(dbref thing, dbref dest, dbref cause, int hush)
     bool canhear = Hearer(thing);
     process_leave_loc(thing, dest, cause, canhear, hush);
     move_object(thing, dest);
-    did_it(thing, thing, A_MOVE, NULL, A_OMOVE, NULL, A_AMOVE,
-           (char **)NULL, 0);
+    did_it(thing, thing, A_MOVE, NULL, A_OMOVE, NULL, A_AMOVE, 0, NULL, 0);
+
+#if defined(FIRANMUX)
+    did_it(thing, thing, A_LEAD, NULL, A_OLEAD, NULL, A_ALEAD, 0, NULL, 0);
+#endif // FIRANMUX
+
     process_enter_loc(thing, src, cause, canhear, hush);
 }
 
@@ -339,27 +357,39 @@ static void move_via_exit(dbref thing, dbref dest, dbref cause, dbref exit, int
     }
     dbref src = Location(thing);
     bool canhear = Hearer(thing);
-    bool quiet = (  (Wizard(thing) && Dark(thing)) // Dark wizards don't trigger OSUCC/ASUCC
+
+#if defined(FIRANMUX)
+    bool quiet = Dark(thing) || (hush & HUSH_EXIT);
+    int aattr = (hush & HUSH_EXIT) ? 0 : A_ASUCC;
+#else
+    // Dark wizards don't trigger OSUCC/ASUCC
+    bool quiet = (  (Wizard(thing) && Dark(thing))
                  || (hush & HUSH_EXIT));
+    int aattr = quiet ? 0 : A_ASUCC;
+#endif // FIRANMUX
 
     int oattr = quiet ? 0 : A_OSUCC;
-    int aattr = quiet ? 0 : A_ASUCC;
     int pattr = (!mudconf.terse_movemsg && Terse(thing)) ? 0 : A_SUCC;
-    did_it(thing, exit, pattr, NULL, oattr, NULL, aattr,
-           (char **)NULL, 0);
+    did_it(thing, exit, pattr, NULL, oattr, NULL, aattr, 0, NULL, 0);
     process_leave_loc(thing, dest, cause, canhear, hush);
     move_object(thing, dest);
 
-    // Dark wizards don't trigger ODROP/ADROP
-    //
-    oattr = quiet ? 0 : A_ODROP;
+#if defined(FIRANMUX)
+    aattr = (hush & HUSH_EXIT) ? 0 : A_ADROP;
+#else
     aattr = quiet ? 0 : A_ADROP;
+#endif // FIRANMUX
+
+    oattr = quiet ? 0 : A_ODROP;
     pattr = (!mudconf.terse_movemsg && Terse(thing)) ? 0 : A_DROP;
-    did_it(thing, exit, pattr, NULL, oattr, NULL, aattr,
-           (char **)NULL, 0);
+    did_it(thing, exit, pattr, NULL, oattr, NULL, aattr, 0, NULL, 0);
+
+    did_it(thing, thing, A_MOVE, NULL, A_OMOVE, NULL, A_AMOVE, 0, NULL, 0);
+
+#if defined(FIRANMUX)
+    did_it(thing, thing, A_LEAD, NULL, A_OLEAD, NULL, A_ALEAD, 0, NULL, 0);
+#endif
 
-    did_it(thing, thing, A_MOVE, NULL, A_OMOVE, NULL, A_AMOVE,
-           (char **)NULL, 0);
     process_enter_loc(thing, src, cause, canhear, hush);
     process_sticky_dropto(src, thing);
 }
@@ -373,7 +403,7 @@ bool move_via_teleport(dbref thing, dbref dest, dbref cause, int hush)
 {
     dbref curr;
     int count;
-    char *failmsg;
+    const char *failmsg;
 
     dbref src = Location(thing);
     if ((dest != HOME) && Good_obj(src))
@@ -394,7 +424,7 @@ bool move_via_teleport(dbref thing, dbref dest, dbref cause, int hush)
                 }
                 did_it(thing, src,
                        A_TOFAIL, failmsg, A_OTOFAIL, NULL,
-                       A_ATOFAIL, (char **)NULL, 0);
+                       A_ATOFAIL, 0, NULL, 0);
                 return false;
             }
             if (isRoom(curr))
@@ -418,7 +448,7 @@ bool move_via_teleport(dbref thing, dbref dest, dbref cause, int hush)
     if (!(hush & HUSH_LEAVE))
     {
         did_it(thing, thing, 0, NULL, A_OXTPORT, NULL, 0,
-            (char **)NULL, 0);
+            0, NULL, 0);
     }
     process_leave_loc(thing, dest, NOTHING, canhear, hush);
 
@@ -427,10 +457,16 @@ bool move_via_teleport(dbref thing, dbref dest, dbref cause, int hush)
     if (!(hush & HUSH_ENTER))
     {
         did_it(thing, thing, A_TPORT, NULL, A_OTPORT, NULL, A_ATPORT,
-            (char **)NULL, 0);
+            0, NULL, 0);
     }
     did_it(thing, thing, A_MOVE, NULL, A_OMOVE, NULL, A_AMOVE,
-        (char **)NULL, 0);
+        0, NULL, 0);
+
+#if defined(FIRANMUX)
+    did_it(thing, thing, A_LEAD, NULL, A_OLEAD, NULL, A_ALEAD,
+           0, NULL, 0);
+#endif // FIRANMUX
+
     process_enter_loc(thing, src, NOTHING, canhear, hush);
     divest_object(thing);
     process_sticky_dropto(src, thing);
@@ -450,9 +486,10 @@ static dbref get_exit_dest(dbref executor, dbref exit)
     char *result = alloc_lbuf("get_exit_dest");
     char *ref = result;
     char *str = atr_gotten;
-    mux_exec(result, &ref, exit, executor, executor, EV_FCHECK | EV_EVAL,
-                &str, (char **)NULL, 0);
+    mux_exec(result, &ref, exit, executor, executor,
+        AttrTrace(aflags, EV_TOP|EV_FCHECK|EV_EVAL), &str, NULL, 0);
     free_lbuf(atr_gotten);
+    *ref = '\0';
 
     dbref dest = NOTHING;
     if (*result == NUMBER_TOKEN)
@@ -527,11 +564,19 @@ void move_exit(dbref player, dbref exit, bool divest, const char *failmsg, int h
         }
     }
 #else
+#if defined(FIRANMUX)
+    if (Immobile(player))
+    {
+        notify(player, mudconf.immobile_msg);
+        return;
+    }
+#endif // FIRANMUX
     if (Good_obj(loc) && could_doit(player, exit, A_LOCK))
     {
         bDoit = true;
     }
 #endif
+
     if (bDoit)
     {
         switch (Typeof(loc))
@@ -561,14 +606,18 @@ void move_exit(dbref player, dbref exit, bool divest, const char *failmsg, int h
         if ((Wizard(player) && Dark(player)) || (hush & HUSH_EXIT))
         {
             oattr = 0;
+#if defined(FIRANMUX)
+            aattr = (hush & HUSH_EXIT) ? 0 : A_AFAIL;
+#else
             aattr = 0;
+#endif // FIRANMUX
         }
         else
         {
             oattr = A_OFAIL;
             aattr = A_AFAIL;
         }
-        did_it(player, exit, A_FAIL, failmsg, oattr, NULL, aattr, (char **)NULL, 0);
+        did_it(player, exit, A_FAIL, failmsg, oattr, NULL, aattr, 0, NULL, 0);
     }
 }
 
@@ -576,10 +625,11 @@ void move_exit(dbref player, dbref exit, bool divest, const char *failmsg, int h
  * do_move: Move from one place to another via exits or 'home'.
  */
 
-void do_move(dbref executor, dbref caller, dbref enactor, int key, char *direction)
+void do_move(dbref executor, dbref caller, dbref enactor, int eval, int key, char *direction)
 {
     UNUSED_PARAMETER(caller);
     UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(eval);
 
     dbref exit, loc;
     int i, quiet;
@@ -596,13 +646,21 @@ void do_move(dbref executor, dbref caller, dbref enactor, int key, char *directi
             return;
         }
 
+#if defined(FIRANMUX)
+        if (Immobile(executor))
+        {
+            notify(executor, mudconf.immobile_msg);
+            return;
+        }
+#endif // FIRANMUX
+
         if (  (loc = Location(executor)) != NOTHING
            && !Dark(executor)
            && !Dark(loc))
         {
             // Tell all
             //
-            notify_except(loc, executor, executor, tprintf("%s goes home.", Name(executor)), 0);
+            notify_except(loc, executor, executor, tprintf("%s goes home.", Moniker(executor)), 0);
         }
         // Give the player the messages
         //
@@ -613,10 +671,17 @@ void do_move(dbref executor, dbref caller, dbref enactor, int key, char *directi
         process_sticky_dropto(loc, executor);
         return;
     }
+
     // Find the exit.
     //
     init_match_check_keys(executor, direction, TYPE_EXIT);
+
+#if defined(FIRANMUX)
+    match_exit_with_parents();
+#else
     match_exit();
+#endif // FIRANMUX
+
     exit = match_result();
     switch (exit)
     {
@@ -638,10 +703,11 @@ void do_move(dbref executor, dbref caller, dbref enactor, int key, char *directi
  * do_get: Get an object.
  */
 
-void do_get(dbref executor, dbref caller, dbref enactor, int key, char *what)
+void do_get(dbref executor, dbref caller, dbref enactor, int eval, int key, char *what)
 {
     UNUSED_PARAMETER(caller);
     UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(eval);
 
     dbref playerloc;
     if (  !Has_location(executor)
@@ -728,14 +794,14 @@ void do_get(dbref executor, dbref caller, dbref enactor, int key, char *what)
             if (thingloc != playerloc)
             {
                 notify(thingloc, tprintf("%s was taken from you.",
-                    Name(thing)));
+                    Moniker(thing)));
             }
             move_via_generic(thing, executor, executor, 0);
             notify(thing, "Taken.");
             oattr = quiet ? 0 : A_OSUCC;
             aattr = quiet ? 0 : A_ASUCC;
             did_it(executor, thing, A_SUCC, "Taken.", oattr, NULL,
-                   aattr, (char **)NULL, 0);
+                   aattr, 0, NULL, 0);
         }
         else
         {
@@ -750,7 +816,7 @@ void do_get(dbref executor, dbref caller, dbref enactor, int key, char *what)
                 failmsg = "You can't pick that up.";
             }
             did_it(executor, thing, A_FAIL, failmsg, oattr, NULL, aattr,
-                (char **)NULL, 0);
+                0, NULL, 0);
         }
         break;
 
@@ -796,10 +862,11 @@ void do_get(dbref executor, dbref caller, dbref enactor, int key, char *what)
  * do_drop: Drop an object.
  */
 
-void do_drop(dbref executor, dbref caller, dbref enactor, int key, char *name)
+void do_drop(dbref executor, dbref caller, dbref enactor, int eval, int key, char *name)
 {
     UNUSED_PARAMETER(caller);
     UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(eval);
 
     dbref loc = Location(executor);
     if (!Good_obj(loc))
@@ -836,7 +903,7 @@ void do_drop(dbref executor, dbref caller, dbref enactor, int key, char *name)
            || !could_doit(executor, thing, A_LDROP))
         {
             did_it(executor, thing, A_DFAIL, "You can't drop that.",
-                   A_ODFAIL, NULL, A_ADFAIL, (char **)NULL, 0);
+                   A_ODFAIL, NULL, A_ADFAIL, 0, NULL, 0);
             return;
         }
 
@@ -848,11 +915,11 @@ void do_drop(dbref executor, dbref caller, dbref enactor, int key, char *name)
         if ((key & DROP_QUIET) && Controls(executor, thing))
             quiet = true;
         bp = buf = alloc_lbuf("do_drop.did_it");
-        safe_tprintf_str(buf, &bp, "dropped %s.", Name(thing));
+        safe_tprintf_str(buf, &bp, "dropped %s.", Moniker(thing));
         oattr = quiet ? 0 : A_ODROP;
         aattr = quiet ? 0 : A_ADROP;
         did_it(executor, thing, A_DROP, "Dropped.", oattr, buf,
-               aattr, (char **)NULL, 0);
+               aattr, 0, NULL, 0);
         free_lbuf(buf);
 
         // Process droptos
@@ -903,7 +970,7 @@ void do_enter_internal(dbref player, dbref thing, bool quiet)
         oattr = quiet ? 0 : A_OEFAIL;
         aattr = quiet ? 0 : A_AEFAIL;
         did_it(player, thing, A_EFAIL, NOPERM_MESSAGE,
-               oattr, NULL, aattr, (char **)NULL, 0);
+               oattr, NULL, aattr, 0, NULL, 0);
     }
     else if (player == thing)
     {
@@ -922,14 +989,15 @@ void do_enter_internal(dbref player, dbref thing, bool quiet)
         oattr = quiet ? 0 : A_OEFAIL;
         aattr = quiet ? 0 : A_AEFAIL;
         did_it(player, thing, A_EFAIL, "You can't enter that.",
-               oattr, NULL, aattr, (char **)NULL, 0);
+               oattr, NULL, aattr, 0, NULL, 0);
     }
 }
 
-void do_enter(dbref executor, dbref caller, dbref enactor, int key, char *what)
+void do_enter(dbref executor, dbref caller, dbref enactor, int eval, int key, char *what)
 {
     UNUSED_PARAMETER(caller);
     UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(eval);
 
     init_match(executor, what, TYPE_THING);
     match_neighbor();
@@ -988,6 +1056,6 @@ void do_leave(dbref executor, dbref caller, dbref enactor, int key)
         int oattr = quiet ? 0 : A_OLFAIL;
         int aattr = quiet ? 0 : A_ALFAIL;
         did_it(executor, loc, A_LFAIL, "You can't leave.",
-               oattr, NULL, aattr, (char **)NULL, 0);
+               oattr, NULL, aattr, 0, NULL, 0);
     }
 }
