@@ -1,10 +1,6 @@
 // muxcli.h
 //
-// $Id: muxcli.h,v 1.4 2004/06/10 15:23:32 sdennis Exp $
-//
-// MUX 2.4
-// Copyright (C) 1998 through 2004 Solid Vertical Domains, Ltd. All
-// rights not explicitly given are reserved.
+// $Id: muxcli.h 2475 2007-09-15 16:18:35Z brazilofmux $
 //
 #define CLI_USER 0
 
@@ -16,12 +12,12 @@
 
 typedef struct
 {
-    char *m_Flag;
-    int  m_ArgControl;
-    int  m_Unique;
+    const char *m_Flag;
+    int         m_ArgControl;
+    int         m_Unique;
 } CLI_OptionEntry;
 
-typedef void CLI_CALLBACKFUNC(CLI_OptionEntry *pOption, char *Value);
+typedef void CLI_CALLBACKFUNC(CLI_OptionEntry *pOption, const char *Value);
 
 void CLI_Process
 (
