@@ -1,6 +1,6 @@
 // object.cpp -- Low-level object manipulation routines.
 //
-// $Id: object.cpp,v 1.17 2006/01/07 17:36:11 sdennis Exp $
+// $Id: object.cpp 3211 2008-01-22 19:17:24Z brazilofmux $
 //
 
 #include "copyright.h"
@@ -182,7 +182,8 @@ dbref clone_home(dbref player, dbref thing)
 dbref create_obj(dbref player, int objtype, const char *name, int cost)
 {
     dbref obj, owner;
-    int quota = 0, value = 0, nValidName;
+    int quota = 0, value = 0;
+    size_t nValidName;
     FLAGSET f;
     char *buff;
     const char *pValidName;
@@ -406,9 +407,11 @@ static void destroy_bad_obj(dbref obj)
     if (!mudstate.bStandAlone)
     {
         halt_que(NOTHING, obj);
-        nfy_que(obj, A_SEMAPHORE, NFY_DRAIN, 0);
+        nfy_que(obj, 0, NFY_DRAIN, 0);
         fwdlist_clr(obj);
+#ifdef DEPRECATED
         stack_clr(obj);
+#endif // DEPRECATED
         ReleaseAllResources(obj);
     }
     atr_free(obj);
@@ -457,12 +460,14 @@ void destroy_obj(dbref obj)
         {
             notify(owner, "Halted.");
         }
-        nfy_que(obj, A_SEMAPHORE, NFY_DRAIN, 0);
+        nfy_que(obj, 0, NFY_DRAIN, 0);
 
         // Remove forwardlists and stacks.
         //
         fwdlist_clr(obj);
+#ifdef DEPRECATED
         stack_clr(obj);
+#endif // DEPRECATED
     }
 
     // Compensate the owner for the object.
@@ -516,7 +521,7 @@ void destroy_obj(dbref obj)
             {
                 notify(owner, tprintf(
                        "You get back your %d %s deposit for %s(#%d).",
-                        val, mudconf.one_coin, Name(obj), obj));
+                        val, mudconf.one_coin, Moniker(obj), obj));
             }
         }
         if (  mudconf.quotas
@@ -888,7 +893,7 @@ static void check_dead_refs(void)
                        && !Quiet(owner))
                     {
                         notify(owner, tprintf("Parent cleared on %s(#%d)",
-                            Name(i), i));
+                            Moniker(i), i));
                     }
                 }
                 else
@@ -920,7 +925,7 @@ static void check_dead_refs(void)
                        && !Quiet(owner))
                     {
                         notify(owner, tprintf("Zone cleared on %s(#%d)",
-                            Name(i), i));
+                            Moniker(i), i));
                     }
                 }
                 else
@@ -973,7 +978,7 @@ static void check_dead_refs(void)
         {
             // Check for wizards
             //
-            if (Wizard(i))
+            if (RealWizard(i))
             {
                 if (isPlayer(i))
                 {
@@ -1043,7 +1048,7 @@ static void check_dead_refs(void)
                        && !Quiet(owner))
                     {
                         notify(owner, tprintf("Home reset on %s(#%d)",
-                            Name(i), i));
+                            Moniker(i), i));
                     }
                     else
                     {
@@ -1098,7 +1103,7 @@ static void check_dead_refs(void)
                            && !Quiet(owner))
                         {
                             notify(owner, tprintf("Dropto removed from %s(#%d)",
-                                Name(i), i));
+                                Moniker(i), i));
                         }
                     }
                     else
@@ -1571,25 +1576,26 @@ static void check_loc_contents(dbref loc)
             //
             if (check_type & DBCK_FULL)
             {
-                // Check for wizard command-handlers inside nonwiz. Just warn
-                // if we find one.
+                // Check for wizard command-handlers inside non-wizard. Just
+                // warn if we find one.
                 //
-                if (Wizard(obj) && !Wizard(loc))
+                if (  RealWizard(obj)
+                   && !Wizard(loc))
                 {
                     if (Commer(obj))
                     {
                         Log_simple_err(obj, loc,
-                            "Wizard command handling object inside nonwizard.");
+                            "Wizard command handling object inside non-wizard.");
                     }
                 }
 
-                // Check for nonwizard objects inside wizard objects.
+                // Check for non-wizard objects inside wizard objects.
                 //
-                if (  Wizard(loc)
+                if (  RealWizard(loc)
                    && !Wizard(obj)
                    && !Wizard(Owner(obj)))
                 {
-                    Log_simple_err(obj, loc, "Nonwizard object inside wizard.");
+                    Log_simple_err(obj, loc, "Non-wizard object inside wizard.");
                 }
             }
             Mark(obj);
@@ -1634,15 +1640,10 @@ static void check_contents_chains(void)
 
 static void mark_place(dbref loc)
 {
-    dbref exit;
-
     // If already marked, exit.  Otherwise set marked.
     //
-    if (!Good_obj(loc))
-    {
-        return;
-    }
-    if (Marked(loc))
+    if (  !Good_obj(loc)
+       || Marked(loc))
     {
         return;
     }
@@ -1650,6 +1651,7 @@ static void mark_place(dbref loc)
 
     // Visit all places you can get to via exits from here.
     //
+    dbref exit;
     for (exit = Exits(loc); exit != NOTHING; exit = Next(exit))
     {
         if (Good_obj(Location(exit)))
@@ -1661,29 +1663,40 @@ static void mark_place(dbref loc)
 
 static void check_floating(void)
 {
-    dbref owner, i;
+    dbref i;
 
     // Mark everyplace you can get to via exits from the starting room.
     //
     Unmark_all(i);
     mark_place(mudconf.start_room);
 
-    // Look for rooms not marked and not set FLOATING.
+    // Mark every room you can get to from a floating room.
+    //
+    DO_WHOLE_DB(i)
+    {
+        if (  isRoom(i)
+           && Floating(i)
+           && !Going(i))
+        {
+            mark_place(i);
+        }
+    }
+
+    // Look for unvisited rooms.
     //
     DO_WHOLE_DB(i)
     {
         if (  isRoom(i)
-           && !Floating(i)
            && !Going(i)
            && !Marked(i))
         {
-            owner = Owner(i);
             if (!mudstate.bStandAlone)
             {
+                dbref owner = Owner(i);
                 if (Good_owner(owner))
                 {
                     notify(owner, tprintf( "You own a floating room: %s(#%d)",
-                        Name(i), i));
+                        Moniker(i), i));
                 }
             }
             else
@@ -1716,6 +1729,7 @@ void do_dbck(dbref executor, dbref caller, dbref enactor, int key)
     }
     purge_going();
     make_freelist();
+    scheduler.Shrink();
 
     // Allow the local extensions to do data checks.
     //
