@@ -1,6 +1,6 @@
 // player.cpp
 //
-// $Id: player.cpp,v 1.36 2006/01/07 09:14:26 sdennis Exp $
+// $Id: player.cpp 2734 2007-10-28 23:02:55Z brazilofmux $
 //
 
 #include "copyright.h"
@@ -101,7 +101,8 @@ static void encrypt_logindata(char *atrbuf, LDATA *info)
             info->bad[i].dtm = &nullc;
     }
     char *bp = alloc_lbuf("encrypt_logindata");
-    sprintf(bp, "#%d;%s;%s;%s;%s;%s;%s;%s;%s;%d;%d;%s;%s;%s;%s;%s;%s;",
+    mux_sprintf(bp, LBUF_SIZE,
+        "#%d;%s;%s;%s;%s;%s;%s;%s;%s;%d;%d;%s;%s;%s;%s;%s;%s;",
         info->tot_good,
         info->good[0].host, info->good[0].dtm,
         info->good[1].host, info->good[1].dtm,
@@ -111,7 +112,7 @@ static void encrypt_logindata(char *atrbuf, LDATA *info)
         info->bad[0].host, info->bad[0].dtm,
         info->bad[1].host, info->bad[1].dtm,
         info->bad[2].host, info->bad[2].dtm);
-    strcpy(atrbuf, bp);
+    mux_strncpy(atrbuf, bp, LBUF_SIZE-1);
     free_lbuf(bp);
 }
 
@@ -274,7 +275,7 @@ static const char *GenerateSalt(void)
     szSaltRaw[SALT_LENGTH] = '\0';
 
     static char szSaltEncoded[SHA1_PREFIX_LENGTH + ENCODED_SALT_LENGTH+1];
-    strcpy(szSaltEncoded, szSHA1Prefix);
+    mux_strncpy(szSaltEncoded, szSHA1Prefix, SHA1_PREFIX_LENGTH);
     EncodeBase64(SALT_LENGTH, szSaltRaw, szSaltEncoded + SHA1_PREFIX_LENGTH);
     return szSaltEncoded;
 }
@@ -397,7 +398,7 @@ const char *mux_crypt(const char *szPassword, const char *szSetting, int *piType
     case CRYPT_OTHER:
     case CRYPT_DES_EXT:
 #ifdef WIN32
-        // The WIN32 release only supports SHA1, DES, and clear-text.
+        // The WIN32 release only supports SHA1 and clear-text.
         //
         return szFail;
 #endif // WIN32
@@ -440,7 +441,7 @@ const char *mux_crypt(const char *szPassword, const char *szSetting, int *piType
     // $SHA1$ssssssssssss$hhhhhhhhhhhhhhhhhhhhhhhhhhhh
     //
     static char buf[SHA1_PREFIX_LENGTH + ENCODED_SALT_LENGTH + 1 + ENCODED_HASH_LENGTH + 1 + 16];
-    strcpy(buf, szSHA1Prefix);
+    mux_strncpy(buf, szSHA1Prefix, SHA1_PREFIX_LENGTH);
     memcpy(buf + SHA1_PREFIX_LENGTH, pSaltField, nSaltField);
     buf[SHA1_PREFIX_LENGTH + nSaltField] = '$';
     EncodeBase64(20, szHashRaw, buf + SHA1_PREFIX_LENGTH + nSaltField + 1);
@@ -543,8 +544,8 @@ void AddToPublicChannel(dbref player)
 
 dbref create_player
 (
-    char *name,
-    char *password,
+    const char *name,
+    const char *password,
     dbref creator,
     bool isrobot,
     const char **pmsg
@@ -642,10 +643,11 @@ static void disp_from_on(dbref player, char *dtm_str, char *host_str)
     }
 }
 
-void do_last(dbref executor, dbref caller, dbref enactor, int key, char *who)
+void do_last(dbref executor, dbref caller, dbref enactor, int eval, int key, char *who)
 {
     UNUSED_PARAMETER(caller);
     UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(eval);
     UNUSED_PARAMETER(key);
 
     dbref target, aowner;
@@ -700,7 +702,7 @@ void do_last(dbref executor, dbref caller, dbref enactor, int key, char *who)
 
 bool add_player_name(dbref player, const char *name)
 {
-    bool stat;
+    bool stat = false;
     char *temp, *tp;
 
     // Convert to all lowercase.
@@ -732,21 +734,50 @@ bool add_player_name(dbref player, const char *name)
 
         // It's an alias (or an incorrect entry). Clobber it.
         //
-        MEMFREE(p);
-        p = (dbref *)MEMALLOC(sizeof(int));
-        ISOUTOFMEMORY(p);
+        delete p;
 
-        *p = player;
-        stat = hashreplLEN(temp, strlen(temp), p, &mudstate.player_htab);
+        p = NULL;
+        try
+        {
+            p = new dbref;
+        }
+        catch (...)
+        {
+            ; // Nothing.
+        }
+
+        if (NULL != p)
+        {
+            *p = player;
+            stat = hashreplLEN(temp, strlen(temp), p, &mudstate.player_htab);
+        }
+        else
+        {
+            ISOUTOFMEMORY(p);
+        }
         free_lbuf(temp);
     }
     else
     {
-        p = (dbref *)MEMALLOC(sizeof(int));
-        ISOUTOFMEMORY(p);
+        p = NULL;
+        try
+        {
+            p = new dbref;
+        }
+        catch (...)
+        {
+            ; // Nothing.
+        }
 
-        *p = player;
-        stat = (hashaddLEN(temp, strlen(temp), p, &mudstate.player_htab) >= 0);
+        if (NULL != p)
+        {
+            *p = player;
+            stat = hashaddLEN(temp, strlen(temp), p, &mudstate.player_htab);
+        }
+        else
+        {
+            ISOUTOFMEMORY(p);
+        }
         free_lbuf(temp);
     }
     return stat;
@@ -770,7 +801,7 @@ bool delete_player_name(dbref player, const char *name)
         free_lbuf(temp);
         return false;
     }
-    MEMFREE(p);
+    delete p;
     p = NULL;
     hashdeleteLEN(temp, strlen(temp), &mudstate.player_htab);
     free_lbuf(temp);
@@ -877,11 +908,26 @@ void badname_add(char *bad_name)
 {
     // Make a new node and link it in at the top.
     //
-    BADNAME *bp = (BADNAME *)MEMALLOC(sizeof(BADNAME));
-    ISOUTOFMEMORY(bp);
-    bp->name = StringClone(bad_name);
-    bp->next = mudstate.badname_head;
-    mudstate.badname_head = bp;
+    BADNAME *bp = NULL;
+    try
+    {
+        bp = new BADNAME;
+    }
+    catch (...)
+    {
+        ; // Nothing.
+    }
+
+    if (NULL != bp)
+    {
+        bp->name = StringClone(bad_name);
+        bp->next = mudstate.badname_head;
+        mudstate.badname_head = bp;
+    }
+    else
+    {
+        ISOUTOFMEMORY(bp);
+    }
 }
 
 void badname_remove(char *bad_name)
@@ -904,7 +950,7 @@ void badname_remove(char *bad_name)
             }
             MEMFREE(bp->name);
             bp->name = NULL;
-            MEMFREE(bp);
+            delete bp;
             bp = NULL;
             return;
         }
