@@ -1,6 +1,6 @@
 // powers.cpp -- Power manipulation routines.
 //
-// $Id: powers.cpp,v 1.15 2006/01/07 08:59:40 sdennis Exp $
+// $Id: powers.cpp 29 2006-09-06 22:12:43Z brazilofmux $
 //
 
 #include "copyright.h"
@@ -103,7 +103,11 @@ static POWERENT gen_powers[] =
 {
     {"announce",        POW_ANNOUNCE,   0, 0,   ph_wiz},
     {"boot",            POW_BOOT,       0, 0,   ph_wiz},
+#if defined(FIRANMUX)
+    {"builder",         POW_BUILDER,    POWER_EXT,  0,  ph_any},
+#else
     {"builder",         POW_BUILDER,    POWER_EXT,  0,  ph_wiz},
+#endif // FIRANMUX
     {"chown_anything",  POW_CHOWN_ANY,  0, 0,   ph_wiz},
     {"comm_all",        POW_COMM_ALL,   0, 0,   ph_wiz},
     {"control_all",     POW_CONTROL_ALL,0, 0,   ph_god},
@@ -113,27 +117,43 @@ static POWERENT gen_powers[] =
     {"free_quota",      POW_FREE_QUOTA, 0, 0,   ph_wiz},
     {"guest",           POW_GUEST,      0, 0,   ph_god},
     {"halt",            POW_HALT,       0, 0,   ph_wiz},
+#if defined(FIRANMUX)
+    {"hide",            POW_HIDE,       0, 0,   ph_any},
+#else
     {"hide",            POW_HIDE,       0, 0,   ph_wiz},
+#endif // FIRANMUX
     {"idle",            POW_IDLE,       0, 0,   ph_wiz},
-#ifdef FIRANMUX
+#if defined(FIRANMUX)
     {"immutable",       POW_IMMUTABLE,  POWER_EXT, 0, ph_wiz},
-#endif
+#endif // FIRANMUX
     {"long_fingers",    POW_LONGFINGERS,0, 0,   ph_wiz},
+#if defined(FIRANMUX)
+    {"monitor",         POW_MONITOR,    0, 0,   ph_any},
+#else
     {"monitor",         POW_MONITOR,    0, 0,   ph_wiz},
+#endif // FIRANMUX
     {"no_destroy",      POW_NO_DESTROY, 0, 0,   ph_wiz},
     {"pass_locks",      POW_PASS_LOCKS, 0, 0,   ph_wiz},
     {"poll",            POW_POLL,       0, 0,   ph_wiz},
     {"prog",            POW_PROG,       0, 0,   ph_wiz},
     {"quota",           POW_CHG_QUOTAS, 0, 0,   ph_wiz},
     {"search",          POW_SEARCH,     0, 0,   ph_wiz},
+#if defined(FIRANMUX)
+    {"see_all",         POW_EXAM_ALL,   0, 0,   ph_any},
+#else
     {"see_all",         POW_EXAM_ALL,   0, 0,   ph_wiz},
+#endif // FIRANMUX
     {"see_hidden",      POW_SEE_HIDDEN, 0, 0,   ph_wiz},
     {"see_queue",       POW_SEE_QUEUE,  0, 0,   ph_wiz},
     {"siteadmin",       POW_SITEADMIN,  0, 0,   ph_wiz},
     {"stat_any",        POW_STAT_ANY,   0, 0,   ph_wiz},
     {"steal_money",     POW_STEAL,      0, 0,   ph_wiz},
     {"tel_anything",    POW_TEL_UNRST,  0, 0,   ph_wiz},
+#if defined(FIRANMUX)
+    {"tel_anywhere",    POW_TEL_ANYWHR, 0, 0,   ph_any},
+#else
     {"tel_anywhere",    POW_TEL_ANYWHR, 0, 0,   ph_wiz},
+#endif // FIRANMUX
     {"unkillable",      POW_UNKILLABLE, 0, 0,   ph_wiz},
     {NULL,              0,              0, 0,   0}
 };
@@ -149,8 +169,7 @@ void init_powertab(void)
 
     for (fp = gen_powers; fp->powername; fp++)
     {
-        strncpy(nbuf, fp->powername, SBUF_SIZE);
-        nbuf[SBUF_SIZE-1] = '\0';
+        mux_strncpy(nbuf, fp->powername, SBUF_SIZE-1);
         mux_strlwr(nbuf);
 
         if (!hashfindLEN(nbuf, strlen(nbuf), &mudstate.powers_htab))
@@ -197,8 +216,7 @@ static POWERENT *find_power(dbref thing, char *powername)
     // Convert powername to canonical lowercase.
     //
     char *buff = alloc_sbuf("find_power");
-    strncpy(buff, powername, SBUF_SIZE);
-    buff[SBUF_SIZE-1] = '\0';
+    mux_strncpy(buff, powername, SBUF_SIZE-1);
     mux_strlwr(buff);
     POWERENT *p = (POWERENT *)hashfindLEN(buff, strlen(buff), &mudstate.powers_htab);
     free_sbuf(buff);
@@ -233,57 +251,82 @@ bool decode_power(dbref player, char *powername, POWERSET *pset)
 
 void power_set(dbref target, dbref player, char *power, int key)
 {
-    // Trim spaces, and handle the negation character.
-    //
-    while (mux_isspace(*power))
-    {
-        power++;
-    }
+    bool bDone = false;
 
-    bool negate = false;
-    if (*power == '!')
+    do
     {
-        negate = true;
-        power++;
-    }
-    while (mux_isspace(*power))
-    {
-        power++;
-    }
-
-    // Make sure a power name was specified.
-    //
-    if (*power == '\0')
-    {
-        if (negate)
+        // Trim spaces, and handle the negation character.
+        //
+        while (mux_isspace(*power))
         {
-            notify(player, "You must specify a power to clear.");
+            power++;
+        }
+
+        bool bNegate = false;
+        if (*power == '!')
+        {
+            bNegate = true;
+            do
+            {
+                power++;
+            } while (mux_isspace(*power));
+        }
+
+        // Beginning of power name is now 'power'.
+        //
+        char *npower = power;
+        while (  *npower != '\0'
+              && !mux_isspace(*npower))
+        {
+            npower++;
+        }
+
+        if (*npower == '\0')
+        {
+            bDone = true;
         }
         else
         {
-            notify(player, "You must specify a power to set.");
+            *npower = '\0';
         }
-        return;
-    }
-    POWERENT *fp = find_power(target, power);
-    if (fp == NULL)
-    {
-        notify(player, "I don't understand that power.");
-        return;
-    }
 
-    // Invoke the power handler, and print feedback.
-    //
-    bool result = fp->handler(target, player, fp->powervalue,
-                 fp->powerpower, negate);
-    if (!result)
-    {
-        notify(player, NOPERM_MESSAGE);
-    }
-    else if (!(key & SET_QUIET) && !Quiet(player))
-    {
-        notify(player, (negate ? "Cleared." : "Set."));
-    }
+        // Make sure a power name was specified.
+        //
+        if (*power == '\0')
+        {
+            if (bNegate)
+            {
+                notify(player, "You must specify a power to clear.");
+            }
+            else
+            {
+                notify(player, "You must specify a power to set.");
+            }
+        }
+        else
+        {
+            POWERENT *fp = find_power(target, power);
+            if (fp == NULL)
+            {
+                notify(player, "I don't understand that power.");
+            }
+            else
+            {
+                // Invoke the power handler, and print feedback.
+                //
+                if (!fp->handler(target, player, fp->powervalue, fp->powerpower, bNegate))
+                {
+                    notify(player, NOPERM_MESSAGE);
+                }
+                else if (!(key & SET_QUIET) && !Quiet(player))
+                {
+                    notify(player, (bNegate ? "Cleared." : "Set."));
+                }
+            }
+        }
+        power = npower + 1;
+
+    } while (!bDone);
 }
 
 /* ---------------------------------------------------------------------------
