@@ -1,6 +1,6 @@
 // predicates.cpp
 //
-// $Id: predicates.cpp,v 1.77 2006/01/07 08:50:53 sdennis Exp $
+// $Id: predicates.cpp 2412 2007-09-05 15:24:29Z brazilofmux $
 //
 
 #include "copyright.h"
@@ -17,7 +17,7 @@
 #include "powers.h"
 #ifdef REALITY_LVLS
 #include "levels.h"
-#endif /* REALITY_LVLS */
+#endif // REALITY_LVLS
 
 char * DCL_CDECL tprintf(const char *fmt,...)
 {
@@ -164,7 +164,7 @@ bool can_see(dbref player, dbref thing, bool can_see_loc)
        return ((!Dark(thing) && IsReal(player, thing)) ||
 #else
         return (!Dark(thing) ||
-#endif /* REALITY_LVLS */
+#endif // REALITY_LVLS
             (mudconf.see_own_dark && MyopicExam(player, thing)));
     }
     else
@@ -173,7 +173,7 @@ bool can_see(dbref player, dbref thing, bool can_see_loc)
         return ((Light(thing) && !Dark(thing) && IsReal(player, thing)) ||
 #else
         return ((Light(thing) && !Dark(thing)) ||
-#endif /* REALITY_LVLS */
+#endif // REALITY_LVLS
             (mudconf.see_own_dark && MyopicExam(player, thing)));
     }
 }
@@ -302,7 +302,7 @@ void giveto(dbref who, int pennies)
 // used for things and rooms, but not for players or exits) and generates
 // a canonical form of that name (with optimized ANSI).
 //
-char *MakeCanonicalObjectName(const char *pName, int *pnName, bool *pbValid)
+char *MakeCanonicalObjectName(const char *pName, size_t *pnName, bool *pbValid)
 {
     static char Buf[MBUF_SIZE];
 
@@ -317,8 +317,8 @@ char *MakeCanonicalObjectName(const char *pName, int *pnName, bool *pbValid)
     // Build up what the real name would be. If we pass all the
     // checks, this is what we will return as a result.
     //
-    int nVisualWidth;
-    int nBuf = ANSI_TruncateToField(pName, sizeof(Buf), Buf, MBUF_SIZE,
+    size_t nVisualWidth;
+    size_t nBuf = ANSI_TruncateToField(pName, sizeof(Buf), Buf, MBUF_SIZE,
         &nVisualWidth, ANSI_ENDGOAL_NORMAL);
 
     // Disallow pure ANSI names. There must be at least -something-
@@ -371,7 +371,7 @@ char *MakeCanonicalObjectName(const char *pName, int *pnName, bool *pbValid)
 
 // The following function validates exit names.
 //
-char *MakeCanonicalExitName(const char *pName, int *pnName, bool *pbValid)
+char *MakeCanonicalExitName(const char *pName, size_t *pnName, bool *pbValid)
 {
     static char Buf[MBUF_SIZE];
     static char Out[MBUF_SIZE];
@@ -428,11 +428,11 @@ char *MakeCanonicalExitName(const char *pName, int *pnName, bool *pbValid)
             // any segment but the first, so we can pull them directly from
             // the stripped buffer.
             //
-            int  nN;
-            bool bN;
-            char *pN = MakeCanonicalObjectName(q, &nN, &bN);
+            size_t nN;
+            bool   bN;
+            char  *pN = MakeCanonicalObjectName(q, &nN, &bN);
             if (  bN
-               && nN < MBUF_SIZE - (pOut - Out) - 1)
+               && nN < static_cast<size_t>(MBUF_SIZE - (pOut - Out) - 1))
             {
                 safe_mb_chr(';', Out, &pOut);
                 safe_mb_str(pN, Out, &pOut);
@@ -446,17 +446,17 @@ char *MakeCanonicalExitName(const char *pName, int *pnName, bool *pbValid)
             // characters leading up to the semicolon, but not including the
             // semi-colon.
             //
-            int vw;
+            size_t vw;
             ANSI_TruncateToField(pName, sizeof(Out), Out, n, &vw,
                 ANSI_ENDGOAL_NORMAL);
 
             // vw should always be equal to n, but we'll just make sure.
             //
-            if ((size_t)vw == n)
+            if (vw == n)
             {
-                int  nN;
-                bool bN;
-                char *pN = MakeCanonicalObjectName(Out, &nN, &bN);
+                size_t nN;
+                bool   bN;
+                char  *pN = MakeCanonicalObjectName(Out, &nN, &bN);
                 if (  bN
                    && nN <= MBUF_SIZE - 1)
                 {
@@ -615,7 +615,7 @@ void handle_ears(dbref thing, bool could_hear, bool can_hear)
     if (could_hear != can_hear)
     {
         buff = alloc_lbuf("handle_ears");
-        strcpy(buff, Name(thing));
+        mux_strncpy(buff, Moniker(thing), LBUF_SIZE-1);
         if (isExit(thing))
         {
             for (bp = buff; *bp && *bp != ';'; bp++)
@@ -649,9 +649,10 @@ void handle_ears(dbref thing, bool could_hear, bool can_hear)
 //
 void do_switch
 (
-    dbref player,
+    dbref executor,
     dbref caller,
     dbref enactor,
+    int   eval,
     int   key,
     char *expr,
     char *args[],
@@ -666,58 +667,83 @@ void do_switch
         return;
     }
 
-    if (key == SWITCH_DEFAULT)
+    bool bMatchOne;
+    switch (key & SWITCH_MASK)
     {
+    case SWITCH_DEFAULT:
         if (mudconf.switch_df_all)
         {
-            key = SWITCH_ANY;
+            bMatchOne = false;
         }
         else
         {
-            key = SWITCH_ONE;
+            bMatchOne = true;
         }
+        break;
+
+    case SWITCH_ANY:
+        bMatchOne = false;
+        break;
+
+    case SWITCH_ONE:
+    default:
+        bMatchOne = true;
+        break;
     }
 
     // Now try a wild card match of buff with stuff in coms.
     //
-    bool any = false;
+    bool bAny = false;
     int a;
     char *buff, *bp, *str;
     buff = bp = alloc_lbuf("do_switch");
     CLinearTimeAbsolute lta;
     for (  a = 0;
-              a < nargs - 1
+              (  !bMatchOne
+              || !bAny)
+           && a < nargs - 1
            && args[a]
            && args[a + 1];
            a += 2)
     {
         bp = buff;
         str = args[a];
-        mux_exec(buff, &bp, player, caller, enactor, EV_FCHECK | EV_EVAL | EV_TOP,
+        mux_exec(buff, &bp, executor, caller, enactor, eval|EV_FCHECK|EV_EVAL|EV_TOP,
             &str, cargs, ncargs);
         *bp = '\0';
         if (wild_match(buff, expr))
         {
             char *tbuf = replace_tokens(args[a+1], NULL, NULL, expr);
-            wait_que(player, caller, enactor, false, lta, NOTHING, 0,
-                tbuf, cargs, ncargs, mudstate.global_regs);
+            wait_que(executor, caller, enactor, eval, false, lta, NOTHING, 0,
+                tbuf,
+                ncargs, cargs,
+                mudstate.global_regs);
             free_lbuf(tbuf);
-            if (key == SWITCH_ONE)
-            {
-                free_lbuf(buff);
-                return;
-            }
-            any = true;
+            bAny = true;
         }
     }
+
     free_lbuf(buff);
     if (  a < nargs
-       && !any
+       && !bAny
        && args[a])
     {
         char *tbuf = replace_tokens(args[a], NULL, NULL, expr);
-        wait_que(player, caller, enactor, false, lta, NOTHING, 0, tbuf,
-            cargs, ncargs, mudstate.global_regs);
+        wait_que(executor, caller, enactor, eval, false, lta, NOTHING, 0,
+            tbuf,
+            ncargs, cargs,
+            mudstate.global_regs);
+        free_lbuf(tbuf);
+    }
+
+    if (key & SWITCH_NOTIFY)
+    {
+        char *tbuf = alloc_lbuf("switch.notify_cmd");
+        mux_strncpy(tbuf, "@notify/quiet me", LBUF_SIZE-1);
+        wait_que(executor, caller, enactor, eval, false, lta, NOTHING, A_SEMAPHORE,
+            tbuf,
+            ncargs, cargs,
+            mudstate.global_regs);
         free_lbuf(tbuf);
     }
 }
@@ -730,6 +756,7 @@ void do_if
     dbref player,
     dbref caller,
     dbref enactor,
+    int   eval,
     int   key,
     char *expr,
     char *args[],
@@ -750,15 +777,19 @@ void do_if
     CLinearTimeAbsolute lta;
     buff = bp = alloc_lbuf("do_if");
 
-    mux_exec(buff, &bp, player, caller, enactor, EV_FCHECK | EV_EVAL | EV_TOP,
+    mux_exec(buff, &bp, player, caller, enactor, eval|EV_FCHECK|EV_EVAL|EV_TOP,
         &expr, cargs, ncargs);
+    *bp = '\0';
+
     int a = !xlate(buff);
     free_lbuf(buff);
 
     if (a < nargs)
     {
-        wait_que(player, caller, enactor, false, lta, NOTHING, 0, args[a],
-            cargs, ncargs, mudstate.global_regs);
+        wait_que(player, caller, enactor, eval, false, lta, NOTHING, 0,
+            args[a],
+            ncargs, cargs,
+            mudstate.global_regs);
     }
 }
 
@@ -898,11 +929,12 @@ void do_addcommand
     notify(player, tprintf("Command %s added.", pName));
 }
 
-void do_listcommands(dbref player, dbref caller, dbref enactor, int key,
-                     char *name)
+void do_listcommands(dbref player, dbref caller, dbref enactor, int eval,
+                     int key, char *name)
 {
     UNUSED_PARAMETER(caller);
     UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(eval);
     UNUSED_PARAMETER(key);
 
     CMDENT *old;
@@ -1179,41 +1211,48 @@ void handle_prog(DESC *d, char *message)
     int aflags, i;
     char *cmd = atr_get(d->player, A_PROGCMD, &aowner, &aflags);
     CLinearTimeAbsolute lta;
-    wait_que(d->program_data->wait_enactor, d->player, d->player, false, lta,
-        NOTHING, 0, cmd, (char **)&message, 1,
-        (char **)d->program_data->wait_regs);
+    wait_que(d->program_data->wait_enactor, d->player, d->player,
+        AttrTrace(aflags, 0), false, lta, NOTHING, 0,
+        cmd,
+        1, &message,
+        d->program_data->wait_regs);
 
     // First, set 'all' to a descriptor we find for this player.
     //
     DESC *all = (DESC *)hashfindLEN(&(d->player), sizeof(d->player), &mudstate.desc_htab) ;
 
-    if (all && all->program_data)
+    if (  all
+       && all->program_data)
     {
+        PROG *program = all->program_data;
         for (i = 0; i < MAX_GLOBAL_REGS; i++)
         {
-            if (all->program_data->wait_regs[i])
+            if (program->wait_regs[i])
             {
-                free_lbuf(all->program_data->wait_regs[i]);
-                all->program_data->wait_regs[i] = NULL;
+                RegRelease(program->wait_regs[i]);
+                program->wait_regs[i] = NULL;
             }
         }
 
-        MEMFREE(all->program_data);
-        all->program_data = NULL;
-
         // Set info for all player descriptors to NULL
         //
         DESC_ITER_PLAYER(d->player, all)
+        {
+            mux_assert(program == all->program_data);
             all->program_data = NULL;
+        }
+
+        MEMFREE(program);
     }
     atr_clr(d->player, A_PROGCMD);
     free_lbuf(cmd);
 }
 
-void do_quitprog(dbref player, dbref caller, dbref enactor, int key, char *name)
+void do_quitprog(dbref player, dbref caller, dbref enactor, int eval, int key, char *name)
 {
     UNUSED_PARAMETER(caller);
     UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(eval);
     UNUSED_PARAMETER(key);
 
     dbref doer;
@@ -1249,7 +1288,7 @@ void do_quitprog(dbref player, dbref caller, dbref enactor, int key, char *name)
     bool isprog = false;
     DESC_ITER_PLAYER(doer, d)
     {
-        if (d->program_data != NULL)
+        if (NULL != d->program_data)
         {
             isprog = true;
         }
@@ -1264,25 +1303,28 @@ void do_quitprog(dbref player, dbref caller, dbref enactor, int key, char *name)
     d = (DESC *)hashfindLEN(&doer, sizeof(doer), &mudstate.desc_htab);
     int i;
 
-    if (d && d->program_data)
+    if (  d
+       && d->program_data)
     {
+        PROG *program = d->program_data;
         for (i = 0; i < MAX_GLOBAL_REGS; i++)
         {
-            if (d->program_data->wait_regs[i])
+            if (program->wait_regs[i])
             {
-                free_lbuf(d->program_data->wait_regs[i]);
-                d->program_data->wait_regs[i] = NULL;
+                RegRelease(program->wait_regs[i]);
+                program->wait_regs[i] = NULL;
             }
         }
-        MEMFREE(d->program_data);
-        d->program_data = NULL;
 
         // Set info for all player descriptors to NULL.
         //
         DESC_ITER_PLAYER(doer, d)
         {
+            mux_assert(program == d->program_data);
             d->program_data = NULL;
         }
+
+        MEMFREE(program);
     }
 
     atr_clr(doer, A_PROGCMD);
@@ -1410,8 +1452,11 @@ void do_prog
     program->wait_enactor = player;
     for (int i = 0; i < MAX_GLOBAL_REGS; i++)
     {
-        program->wait_regs[i] = alloc_lbuf("prog_regs");
-        memcpy(program->wait_regs[i], mudstate.global_regs[i], mudstate.glob_reg_len[i]+1);
+        program->wait_regs[i] = mudstate.global_regs[i];
+        if (mudstate.global_regs[i])
+        {
+            RegAddRef(mudstate.global_regs[i]);
+        }
     }
 
     // Now, start waiting.
@@ -1478,7 +1523,7 @@ void do_restart(dbref executor, dbref caller, dbref enactor, int key)
         return;
     }
 
-    raw_broadcast(0, "GAME: Restart by %s, please wait.", Name(Owner(executor)));
+    raw_broadcast(0, "GAME: Restart by %s, please wait.", Moniker(Owner(executor)));
     STARTLOG(LOG_ALWAYS, "WIZ", "RSTRT");
     log_text("Restart by ");
     log_name(executor);
@@ -1509,9 +1554,9 @@ void do_restart(dbref executor, dbref caller, dbref enactor, int key)
     Log.StopLogging();
 
 #ifdef GAME_DOOFERMUX
-    execl("bin/netmux", mudconf.mud_name, "-c", mudconf.config_file, "-p", mudconf.pid_file, NULL);
+    execl("bin/netmux", mudconf.mud_name, "-c", mudconf.config_file, "-p", mudconf.pid_file, "-e", mudconf.log_dir, NULL);
 #else
-    execl("bin/netmux", "netmux", "-c", mudconf.config_file, "-p", mudconf.pid_file, NULL);
+    execl("bin/netmux", "netmux", "-c", mudconf.config_file, "-p", mudconf.pid_file, "-e", mudconf.log_dir, NULL);
 #endif // GAME_DOOFERMUX
 #endif // !WIN32
 }
@@ -1824,7 +1869,7 @@ bool get_obj_and_lock(dbref player, char *what, dbref *it, ATTR **attr, char *er
     int anum;
 
     tbuf = alloc_lbuf("get_obj_and_lock");
-    strcpy(tbuf, what);
+    mux_strncpy(tbuf, what, LBUF_SIZE-1);
     if (parse_thing_slash(player, tbuf, &str, it))
     {
         // <obj>/<lock> syntax, use the named lock.
@@ -2186,7 +2231,7 @@ bool exit_visible(dbref exit, dbref player, int key)
     if (!IsReal(player, exit))
        return 0;
     }
-#endif /* REALITY_LVLS */
+#endif // REALITY_LVLS
 
     // Exam exit's location
     //
@@ -2214,6 +2259,10 @@ bool exit_visible(dbref exit, dbref player, int key)
 //
 bool exit_displayable(dbref exit, dbref player, int key)
 {
+#ifndef WOD_REALMS
+    UNUSED_PARAMETER(player);
+#endif // WOD_REALMS
+
     // Dark exit
     //
     if (Dark(exit))
@@ -2257,7 +2306,8 @@ bool exit_displayable(dbref exit, dbref player, int key)
  */
 
 void did_it(dbref player, dbref thing, int what, const char *def, int owhat,
-            const char *odef, int awhat, char *args[], int nargs)
+            const char *odef, int awhat, int ctrl_flags,
+            char *args[], int nargs)
 {
     if (MuxAlarm.bAlarmed)
     {
@@ -2279,8 +2329,7 @@ void did_it(dbref player, dbref thing, int what, const char *def, int owhat,
     //
 
     bool need_pres = false;
-    char **preserve = NULL;
-    int *preserve_len = NULL;
+    reg_ref **preserve = NULL;
 
     // message to player.
     //
@@ -2290,13 +2339,14 @@ void did_it(dbref player, dbref thing, int what, const char *def, int owhat,
         if (*d)
         {
             need_pres = true;
-            preserve = PushPointers(MAX_GLOBAL_REGS);
-            preserve_len = PushIntegers(MAX_GLOBAL_REGS);
-            save_global_regs("did_it_save", preserve, preserve_len);
+            preserve = PushRegisters(MAX_GLOBAL_REGS);
+            save_global_regs(preserve);
+
             buff = bp = alloc_lbuf("did_it.1");
             str = d;
             mux_exec(buff, &bp, thing, player, player,
-                EV_EVAL | EV_FIGNORE | EV_FCHECK | EV_TOP, &str, args, nargs);
+                AttrTrace(aflags, EV_EVAL|EV_FIGNORE|EV_FCHECK|EV_TOP),
+                &str, args, nargs);
             *bp = '\0';
             if (  (aflags & AF_HTML)
                && Html(player))
@@ -2305,6 +2355,16 @@ void did_it(dbref player, dbref thing, int what, const char *def, int owhat,
                 *bp = '\0';
                 notify_html(player, buff);
             }
+#if defined(FIRANMUX)
+            else if (  A_DESC == what
+                    && Linewrap(player)
+                    && isPlayer(player)
+                    && (  !Linewrap(thing)
+                       || isPlayer(thing)))
+            {
+                notify(player, linewrap_desc(buff));
+            }
+#endif // FIRANMUX
             else
             {
                 notify(player, buff);
@@ -2324,7 +2384,7 @@ void did_it(dbref player, dbref thing, int what, const char *def, int owhat,
 
     // message to neighbors.
     //
-    if (  owhat > 0
+    if (  0 < owhat
        && Has_location(player)
        && Good_obj(loc = Location(player)))
     {
@@ -2334,67 +2394,114 @@ void did_it(dbref player, dbref thing, int what, const char *def, int owhat,
             if (!need_pres)
             {
                 need_pres = true;
-                preserve = PushPointers(MAX_GLOBAL_REGS);
-                preserve_len = PushIntegers(MAX_GLOBAL_REGS);
-                save_global_regs("did_it_save", preserve, preserve_len);
+                preserve = PushRegisters(MAX_GLOBAL_REGS);
+                save_global_regs(preserve);
             }
             buff = bp = alloc_lbuf("did_it.2");
             str = d;
             mux_exec(buff, &bp, thing, player, player,
-                     EV_EVAL | EV_FIGNORE | EV_FCHECK | EV_TOP, &str, args, nargs);
+                 AttrTrace(aflags, EV_EVAL|EV_FIGNORE|EV_FCHECK|EV_TOP),
+                 &str, args, nargs);
             *bp = '\0';
+#if !defined(FIRANMUX)
             if (*buff)
+#endif // FIRANMUX
             {
 #ifdef REALITY_LVLS
-                notify_except2_rlevel(loc, player, player, thing, tprintf("%s %s", Name(player), buff));
+                if (aflags & AF_NONAME)
+                {
+                    notify_except2_rlevel(loc, player, player, thing, buff);
+                }
+                else
+                {
+                    notify_except2_rlevel(loc, player, player, thing,
+                        tprintf("%s %s", Name(player), buff));
+                }
 #else
-                notify_except2(loc, player, player, thing, tprintf("%s %s", Name(player), buff));
-#endif /* REALITY_LVLS */
+                if (aflags & AF_NONAME)
+                {
+                    notify_except2(loc, player, player, thing, buff);
+                }
+                else
+                {
+                    notify_except2(loc, player, player, thing,
+                        tprintf("%s %s", Name(player), buff));
+                }
+#endif // REALITY_LVLS
             }
             free_lbuf(buff);
         }
         else if (odef)
         {
 #ifdef REALITY_LVLS
-            notify_except2_rlevel(loc, player, player, thing, tprintf("%s %s", Name(player), odef));
+            if (ctrl_flags & VERB_NONAME)
+            {
+                notify_except2_rlevel(loc, player, player, thing, odef);
+            }
+            else
+            {
+                notify_except2_rlevel(loc, player, player, thing, tprintf("%s %s", Name(player), odef));
+            }
 #else
-            notify_except2(loc, player, player, thing, tprintf("%s %s", Name(player), odef));
-#endif /* REALITY_LVLS */
+            if (ctrl_flags & VERB_NONAME)
+            {
+                notify_except2(loc, player, player, thing, odef);
+            }
+            else
+            {
+                notify_except2(loc, player, player, thing, tprintf("%s %s", Name(player), odef));
+            }
+#endif // REALITY_LVLS
         }
         free_lbuf(d);
-    }
-    if (  owhat < 0
-       && odef
-       && Has_location(player)
-       && Good_obj(loc = Location(player)))
+    } else if (  owhat < 0
+              && odef
+              && Has_location(player)
+              && Good_obj(loc = Location(player)))
     {
 #ifdef REALITY_LVLS
-        notify_except2_rlevel(loc, player, player, thing, tprintf("%s %s", Name(player), odef));
+        if (ctrl_flags & VERB_NONAME)
+        {
+            notify_except2_rlevel(loc, player, player, thing, odef);
+        }
+        else
+        {
+            notify_except2_rlevel(loc, player, player, thing, tprintf("%s %s", Name(player), odef));
+        }
 #else
-        notify_except2(loc, player, player, thing, tprintf("%s %s", Name(player), odef));
-#endif /* REALITY_LVLS */
+        if (ctrl_flags & VERB_NONAME)
+        {
+            notify_except2(loc, player, player, thing, odef);
+        }
+        else
+        {
+            notify_except2(loc, player, player, thing, tprintf("%s %s", Name(player), odef));
+        }
+#endif // REALITY_LVLS
     }
 
     // If we preserved the state of the global registers, restore them.
     //
     if (need_pres)
     {
-        restore_global_regs("did_it_restore", preserve, preserve_len);
-        PopIntegers(preserve_len, MAX_GLOBAL_REGS);
-        PopPointers(preserve, MAX_GLOBAL_REGS);
+        restore_global_regs(preserve);
+        PopRegisters(preserve, MAX_GLOBAL_REGS);
     }
 
-    // do the action attribute.
+    // Do the action attribute.
     //
 #ifdef REALITY_LVLS
-    if (awhat > 0 && IsReal(thing, player))
+    if (  0 < awhat
+       && IsReal(thing, player))
 #else
-    if (awhat > 0)
-#endif /* REALITY_LVLS */
+    if (0 < awhat)
+#endif // REALITY_LVLS
     {
         if (*(act = atr_pget(thing, awhat, &aowner, &aflags)))
         {
-            charges = atr_pget(thing, A_CHARGES, &aowner, &aflags);
+            dbref aowner2;
+            int   aflags2;
+            charges = atr_pget(thing, A_CHARGES, &aowner2, &aflags2);
             if (*charges)
             {
                 num = mux_atol(charges);
@@ -2405,7 +2512,7 @@ void did_it(dbref player, dbref thing, int what, const char *def, int owhat,
                     atr_add_raw(thing, A_CHARGES, buff);
                     free_sbuf(buff);
                 }
-                else if (*(buff = atr_pget(thing, A_RUNOUT, &aowner, &aflags)))
+                else if (*(buff = atr_pget(thing, A_RUNOUT, &aowner2, &aflags2)))
                 {
                     free_lbuf(act);
                     act = buff;
@@ -2420,8 +2527,11 @@ void did_it(dbref player, dbref thing, int what, const char *def, int owhat,
             }
             free_lbuf(charges);
             CLinearTimeAbsolute lta;
-            wait_que(thing, player, player, false, lta, NOTHING, 0, act,
-                args, nargs, mudstate.global_regs);
+            wait_que(thing, player, player, AttrTrace(aflags, 0), false, lta,
+                NOTHING, 0,
+                act,
+                nargs, args,
+                mudstate.global_regs);
         }
         free_lbuf(act);
     }
@@ -2431,9 +2541,10 @@ void did_it(dbref player, dbref thing, int what, const char *def, int owhat,
  * do_verb: Command interface to did_it.
  */
 
-void do_verb(dbref executor, dbref caller, dbref enactor, int key,
+void do_verb(dbref executor, dbref caller, dbref enactor, int eval, int key,
              char *victim_str, char *args[], int nargs)
 {
+    UNUSED_PARAMETER(eval);
     UNUSED_PARAMETER(caller);
     UNUSED_PARAMETER(key);
 
@@ -2507,7 +2618,7 @@ void do_verb(dbref executor, dbref caller, dbref enactor, int key,
         // Get arguments.
         //
         parse_arglist(victim, actor, actor, args[6], '\0',
-            EV_STRIP_LS | EV_STRIP_TS, xargs, 10, (char **)NULL, 0, &nxargs);
+            EV_STRIP_LS | EV_STRIP_TS, xargs, 10, NULL, 0, &nxargs);
 
     case 6:
         // Get action attribute.
@@ -2594,7 +2705,8 @@ void do_verb(dbref executor, dbref caller, dbref enactor, int key,
 
     // Go do it.
     //
-    did_it(actor, victim, what, whatd, owhat, owhatd, awhat, xargs, nxargs);
+    did_it(actor, victim, what, whatd, owhat, owhatd, awhat,
+        key & VERB_NONAME, xargs, nxargs);
 
     // Free user args.
     //
