@@ -1,6 +1,6 @@
 // set.cpp -- Commands which set parameters.
 //
-// $Id: set.cpp,v 1.30 2006/01/07 08:23:38 sdennis Exp $
+// $Id: set.cpp 571 2006-12-27 17:56:12Z brazilofmux $
 //
 #include "copyright.h"
 #include "autoconf.h"
@@ -126,7 +126,7 @@ void do_chzone
        && isRoom(zone)
        && !isRoom(thing))
     {
-        notify(executor, "Only rooms may have parent rooms.");
+        notify(executor, "Only rooms may be zoned to other rooms.");
         return;
     }
 
@@ -139,11 +139,12 @@ void do_chzone
         // inconvenient -- although this may pose a bit of a security risk. Be
         // careful when @chzone'ing wizard or royal players.
         //
-        Flags(thing) &= ~(WIZARD | ROYALTY | INHERIT);
+        SetClearFlags(thing, mudconf.stripped_flags.word, NULL);
 
         // Wipe out all powers.
         //
         Powers(thing) = 0;
+        Powers2(thing) = 0;
     }
     notify(executor, "Zone changed.");
 }
@@ -224,7 +225,7 @@ void do_name
     }
     else
     {
-        int nValidName;
+        size_t nValidName;
         bool bValid;
         char *pValidName;
 
@@ -527,10 +528,11 @@ void do_lock
  * * Remove a lock from an object of attribute.
  */
 
-void do_unlock(dbref executor, dbref caller, dbref enactor, int key, char *name)
+void do_unlock(dbref executor, dbref caller, dbref enactor, int eval, int key, char *name)
 {
     UNUSED_PARAMETER(caller);
     UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(eval);
 
     dbref thing;
     ATTR *ap;
@@ -588,11 +590,12 @@ void do_unlock(dbref executor, dbref caller, dbref enactor, int key, char *name)
  * * do_unlink: Unlink an exit from its destination or remove a dropto.
  */
 
-void do_unlink(dbref executor, dbref caller, dbref enactor, int key, char *name)
+void do_unlink(dbref executor, dbref caller, dbref enactor, int eval, int key, char *name)
 {
     UNUSED_PARAMETER(caller);
     UNUSED_PARAMETER(enactor);
     UNUSED_PARAMETER(key);
+    UNUSED_PARAMETER(eval);
 
     dbref exit;
 
@@ -650,6 +653,117 @@ void do_unlink(dbref executor, dbref caller, dbref enactor, int key, char *name)
     }
 }
 
+void TranslateFlags_Clone
+(
+    FLAG     aClearFlags[3],
+    dbref    executor,
+    int      key
+)
+{
+    if (key & CLONE_NOSTRIP)
+    {
+        if (God(executor))
+        {
+            // #1 using /nostrip causes nothing to be stripped.
+            //
+            aClearFlags[FLAG_WORD1] = 0;
+        }
+        else
+        {
+            // With #1 powers, /nostrip still strips the WIZARD bit.
+            //
+            aClearFlags[FLAG_WORD1] = WIZARD;
+        }
+        aClearFlags[FLAG_WORD2] = 0;
+        aClearFlags[FLAG_WORD3] = 0;
+    }
+    else
+    {
+        if (  (key & CLONE_INHERIT)
+           && Inherits(executor))
+        {
+            // The /inherit switch specifically allows INHERIT powers through.
+            //
+            aClearFlags[FLAG_WORD1] = (WIZARD | mudconf.stripped_flags.word[FLAG_WORD1]) & ~(INHERIT);
+        }
+        else
+        {
+            // Normally WIZARD and the other stripped_flags are removed.
+            //
+            aClearFlags[FLAG_WORD1] = WIZARD | mudconf.stripped_flags.word[FLAG_WORD1];
+        }
+        aClearFlags[FLAG_WORD2] = mudconf.stripped_flags.word[FLAG_WORD2];
+        aClearFlags[FLAG_WORD3] = mudconf.stripped_flags.word[FLAG_WORD3];
+    }
+}
+
+void TranslateFlags_Chown
+(
+    FLAG     aClearFlags[3],
+    FLAG     aSetFlags[3],
+    bool    *bClearPowers,
+    dbref    executor,
+    int      key
+)
+{
+    if (key & CHOWN_NOSTRIP)
+    {
+        if (God(executor))
+        {
+            // #1 using /nostrip only clears CHOWN_OK and sets HALT.
+            //
+            aClearFlags[FLAG_WORD1] = CHOWN_OK;
+            *bClearPowers = false;
+        }
+        else
+        {
+            // With /nostrip, CHOWN_OK and WIZARD are cleared, HALT is
+            // set, and powers are cleared.
+            //
+            aClearFlags[FLAG_WORD1] = CHOWN_OK | WIZARD;
+            *bClearPowers = true;
+        }
+        aClearFlags[FLAG_WORD2] = 0;
+        aClearFlags[FLAG_WORD3] = 0;
+    }
+    else
+    {
+        // Without /nostrip, CHOWN_OK, WIZARD, and stripped_flags are
+        // cleared, HALT is set, powers are cleared.
+        //
+        aClearFlags[FLAG_WORD1] = CHOWN_OK | WIZARD
+                                | mudconf.stripped_flags.word[FLAG_WORD1];
+        aClearFlags[FLAG_WORD2] = mudconf.stripped_flags.word[FLAG_WORD2];
+        aClearFlags[FLAG_WORD3] = mudconf.stripped_flags.word[FLAG_WORD3];
+        *bClearPowers = true;
+    }
+    aSetFlags[FLAG_WORD1] = HALT;
+    aSetFlags[FLAG_WORD2] = 0;
+    aSetFlags[FLAG_WORD3] = 0;
+}
+
+void SetClearFlags
+(
+    dbref thing,
+    FLAG aClearFlags[3],
+    FLAG aSetFlags[3]
+)
+{
+    int j;
+    for (j = FLAG_WORD1; j <= FLAG_WORD3; j++)
+    {
+        if (NULL != aClearFlags)
+        {
+            db[thing].fs.word[j] &= ~aClearFlags[j];
+        }
+
+        if (NULL != aSetFlags)
+        {
+            db[thing].fs.word[j] |= aSetFlags[j];
+        }
+    }
+}
+
 /*
  * ---------------------------------------------------------------------------
  * * do_chown: Change ownership of an object or attribute.
@@ -668,7 +782,6 @@ void do_chown
 {
     UNUSED_PARAMETER(caller);
     UNUSED_PARAMETER(enactor);
-    UNUSED_PARAMETER(key);
     UNUSED_PARAMETER(nargs);
 
     dbref nOwnerOrig, nOwnerNew, thing;
@@ -873,28 +986,61 @@ void do_chown
         {
             add_quota(nOwnerOrig, quota);
         }
+
         if (!God(executor))
         {
             nOwnerNew = Owner(nOwnerNew);
         }
+
         s_Owner(thing, nOwnerNew);
         atr_chown(thing);
-        db[thing].fs.word[FLAG_WORD1] &= ~(CHOWN_OK | INHERIT);
-        db[thing].fs.word[FLAG_WORD1] |= HALT;
-        s_Powers(thing, 0);
-        s_Powers2(thing, 0);
+
+        FLAGSET clearflags;
+        FLAGSET setflags;
+        bool    bClearPowers;
+
+        TranslateFlags_Chown(clearflags.word, setflags.word, &bClearPowers, executor, key);
+
+        SetClearFlags(thing, clearflags.word, setflags.word);
+        if (bClearPowers)
+        {
+            s_Powers(thing, 0);
+            s_Powers2(thing, 0);
+        }
+
+        // Always halt the queue.
+        //
         halt_que(NOTHING, thing);
+
+        // Warn if ROYALTY is left set.
+        //
+        if (ROYALTY & db[thing].fs.word[FLAG_WORD1])
+        {
+            notify_quiet(executor,
+                tprintf("Warning: @chown/nostrip on %s(#%d) leaves ROYALTY priviledge intact.",
+                Moniker(thing), thing));
+        }
+
+        // Warn if INHERIT is left set.
+        //
+        if (INHERIT & db[thing].fs.word[FLAG_WORD1])
+        {
+            notify_quiet(executor,
+                tprintf("Warning: @chown/nostrip on %s(#%d) leaves INHERIT priviledge intact.",
+                Moniker(thing), thing));
+        }
+
         if (!Quiet(executor))
         {
             char *buff = alloc_lbuf("do_chown.notify");
             char *bp = buff;
 
             char *p;
-            p = tprintf("Owner of %s(#%d) changed from ", Name(thing), thing);
+            p = tprintf("Owner of %s(#%d) changed from ", Moniker(thing), thing);
             safe_str(p, buff, &bp);
-            p = tprintf("%s(#%d) to ", Name(nOwnerOrig), nOwnerOrig);
+            p = tprintf("%s(#%d) to ", Moniker(nOwnerOrig), nOwnerOrig);
             safe_str(p, buff, &bp);
-            p = tprintf("%s(#%d).", Name(nOwnerNew), nOwnerNew);
+            p = tprintf("%s(#%d).", Moniker(nOwnerNew), nOwnerNew);
             safe_str(p, buff, &bp);
             *bp = '\0';
             notify_quiet(executor, buff);
@@ -1083,7 +1229,7 @@ void do_set
             ATTR *pattr2;
             dbref thing2;
 
-            strcpy(buff, p + 1);
+            mux_strncpy(buff, p + 1, LBUF_SIZE-1);
             if (!( parse_attrib(executor, p + 1, &thing2, &pattr2)
                 && pattr2))
             {
@@ -1172,9 +1318,10 @@ void do_setattr
     set_attr_internal(executor, thing, attrnum, attrtext, 0);
 }
 
-void do_cpattr(dbref executor, dbref caller, dbref enactor, int key,
+void do_cpattr(dbref executor, dbref caller, dbref enactor, int eval, int key,
                char *oldpair, char *newpair[], int nargs)
 {
+    UNUSED_PARAMETER(eval);
     UNUSED_PARAMETER(key);
 
     int i;
@@ -1227,9 +1374,10 @@ void do_cpattr(dbref executor, dbref caller, dbref enactor, int key,
 }
 
 
-void do_mvattr(dbref executor, dbref caller, dbref enactor, int key,
+void do_mvattr(dbref executor, dbref caller, dbref enactor, int eval, int key,
                char *what, char *args[], int nargs)
 {
+    UNUSED_PARAMETER(eval);
     UNUSED_PARAMETER(caller);
     UNUSED_PARAMETER(enactor);
     UNUSED_PARAMETER(key);
@@ -1379,7 +1527,7 @@ bool parse_attrib(dbref player, char *str, dbref *thing, ATTR **attr)
     // Break apart string into obj and attr.
     //
     char *buff = alloc_lbuf("parse_attrib");
-    strcpy(buff, str);
+    mux_strncpy(buff, str, LBUF_SIZE-1);
     char *AttrName;
     bool retval = parse_thing_slash(player, buff, &AttrName, thing);
 
@@ -1468,7 +1616,7 @@ bool parse_attrib_wild(dbref player, char *str, dbref *thing,
     int lev;
     bool check_exclude, hash_insert;
     char *buff = alloc_lbuf("parse_attrib_wild");
-    strcpy(buff, str);
+    mux_strncpy(buff, str, LBUF_SIZE-1);
 
     // Separate name and attr portions at the first /.
     //
@@ -1635,9 +1783,10 @@ static void edit_string_ansi(char *src, char **dst, char **returnstr, char *from
     }
 }
 
-void do_edit(dbref executor, dbref caller, dbref enactor, int key, char *it,
-             char *args[], int nargs)
+void do_edit(dbref executor, dbref caller, dbref enactor, int eval, int key,
+             char *it, char *args[], int nargs)
 {
+    UNUSED_PARAMETER(eval);
     UNUSED_PARAMETER(caller);
     UNUSED_PARAMETER(enactor);
     UNUSED_PARAMETER(key);
@@ -1724,10 +1873,11 @@ void do_edit(dbref executor, dbref caller, dbref enactor, int key, char *it,
     }
 }
 
-void do_wipe(dbref executor, dbref caller, dbref enactor, int key, char *it)
+void do_wipe(dbref executor, dbref caller, dbref enactor, int eval, int key, char *it)
 {
     UNUSED_PARAMETER(caller);
     UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(eval);
     UNUSED_PARAMETER(key);
 
     dbref thing;
@@ -1787,12 +1937,9 @@ void do_wipe(dbref executor, dbref caller, dbref enactor, int key, char *it)
     }
 }
 
-void do_trigger(dbref executor, dbref caller, dbref enactor, int key,
+void do_trigger(dbref executor, dbref caller, dbref enactor, int eval, int key,
                 char *object, char *argv[], int nargs)
 {
-    UNUSED_PARAMETER(caller);
-    UNUSED_PARAMETER(enactor);
-
     dbref thing;
     ATTR *pattr;
 
@@ -1802,15 +1949,26 @@ void do_trigger(dbref executor, dbref caller, dbref enactor, int key,
         notify_quiet(executor, "No match.");
         return;
     }
+
     if (!Controls(executor, thing))
     {
         notify_quiet(executor, NOPERM_MESSAGE);
         return;
     }
-    did_it(executor, thing, 0, NULL, 0, NULL, pattr->number, argv, nargs);
+    did_it(executor, thing, 0, NULL, 0, NULL, pattr->number, 0, argv, nargs);
+
+    if (key & TRIG_NOTIFY)
+    {
+        char *tbuf = alloc_lbuf("trigger.notify_cmd");
+        mux_strncpy(tbuf, "@notify/quiet me", LBUF_SIZE-1);
+        CLinearTimeAbsolute lta;
+        wait_que(executor, caller, enactor, eval, false, lta, NOTHING, A_SEMAPHORE,
+            tbuf,
+            0, NULL,
+            mudstate.global_regs);
+        free_lbuf(tbuf);
+    }
 
-    // TODO: Be more descriptive as to what was triggered?
-    //
     if (  !(key & TRIG_QUIET)
        && !Quiet(executor))
     {
@@ -1818,10 +1976,11 @@ void do_trigger(dbref executor, dbref caller, dbref enactor, int key,
     }
 }
 
-void do_use(dbref executor, dbref caller, dbref enactor, int key, char *object)
+void do_use(dbref executor, dbref caller, dbref enactor, int eval, int key, char *object)
 {
     UNUSED_PARAMETER(caller);
     UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(eval);
     UNUSED_PARAMETER(key);
 
     char *df_use, *df_ouse, *temp;
@@ -1848,7 +2007,7 @@ void do_use(dbref executor, dbref caller, dbref enactor, int key, char *object)
     {
         did_it(executor, thing, A_UFAIL,
                "You can't figure out how to use that.",
-               A_OUFAIL, NULL, A_AUFAIL, (char **)NULL, 0);
+               A_OUFAIL, NULL, A_AUFAIL, 0, NULL, 0);
         return;
     }
     temp = alloc_lbuf("do_use");
@@ -1871,10 +2030,10 @@ void do_use(dbref executor, dbref caller, dbref enactor, int key, char *object)
     {
         df_use = alloc_lbuf("do_use.use");
         df_ouse = alloc_lbuf("do_use.ouse");
-        sprintf(df_use, "You use %s", Name(thing));
-        sprintf(df_ouse, "uses %s", Name(thing));
-        did_it(executor, thing, A_USE, df_use, A_OUSE, df_ouse, A_AUSE,
-               (char **)NULL, 0);
+        mux_sprintf(df_use, LBUF_SIZE, "You use %s", Moniker(thing));
+        mux_sprintf(df_ouse, LBUF_SIZE, "uses %s", Moniker(thing));
+        did_it(executor, thing, A_USE, df_use, A_OUSE, df_ouse, A_AUSE, 0,
+            NULL, 0);
         free_lbuf(df_use);
         free_lbuf(df_ouse);
     }
