@@ -1,10 +1,6 @@
 // svdhash.h -- CHashPage, CHashFile, CHashTable modules.
 //
-// $Id: svdhash.h,v 1.18 2006/01/07 07:49:53 sdennis Exp $
-//
-// MUX 2.4
-// Copyright (C) 1998 through 2004 Solid Vertical Domains, Ltd. All
-// rights not explicitly given are reserved.
+// $Id: svdhash.h 2475 2007-09-15 16:18:35Z brazilofmux $
 //
 #ifndef SVDHASH_H
 #define SVDHASH_H
@@ -35,7 +31,7 @@ extern UINT32 CRC32_ProcessBuffer
 (
     UINT32         ulCrc,
     const void    *pBuffer,
-    unsigned int   nBuffer
+    size_t         nBuffer
 );
 
 extern UINT32 CRC32_ProcessInteger(UINT32 nInteger);
@@ -59,7 +55,7 @@ typedef UINT32 UINT_OFFSET;
 #else
 typedef UINT16 UINT_OFFSET;
 #define UINT_OFFSET_MAX_VALUE UINT16_MAX_VALUE
-#define EXPAND_TO_BOUNDARY(x) (((x) + 1) & (~1))
+#define EXPAND_TO_BOUNDARY(x) (static_cast<HP_HEAPLENGTH>(((x) + 1) & (~1)))
 #endif
 
 typedef UINT_OFFSET HP_HEAPOFFSET, *HP_PHEAPOFFSET;
@@ -112,8 +108,10 @@ typedef struct tagHPHeapNode
 
 #define HP_MIN_HEAP_ALLOC HP_SIZEOF_HEAPNODE
 
+#if !defined(MEMORY_BASED)
 typedef unsigned long HF_FILEOFFSET, *HF_PFILEOFFSET;
 #define HF_SIZEOF_FILEOFFSET sizeof(HF_FILEOFFSET)
+#endif // MEMORY_BASED
 
 class CHashPage
 {
@@ -135,16 +133,16 @@ private:
     bool ValidateFreeBlock(HP_HEAPOFFSET oBlock);
     bool ValidateFreeList(void);
 #endif // HP_PROTECTION
-    bool HeapAlloc(HP_DIRINDEX iDir, HP_HEAPLENGTH nRecord, UINT32 nHash, void *pRecord);
+    bool HeapAlloc(UINT32 iDir, HP_HEAPLENGTH nRecord, UINT32 nHash, void *pRecord);
     void SetVariablePointers(void);
     void SetFixedPointers(void);
-    void GetStats(HP_HEAPLENGTH nExtra, int *pnRecords, HP_HEAPLENGTH *pnAllocatedSize, int *pnGoodDirSize);
+    void GetStats(HP_HEAPLENGTH nExtra, int *pnRecords, HP_HEAPLENGTH *pnAllocatedSize, UINT32 *pnGoodDirSize);
 
 public:
     CHashPage(void);
     bool Allocate(unsigned int nPageSize);
     ~CHashPage(void);
-    void Empty(HP_DIRINDEX arg_nDepth, UINT32 arg_nHashGroup, HP_DIRINDEX arg_nDirSize);
+    void Empty(UINT32 arg_nDepth, UINT32 arg_nHashGroup, UINT32 arg_nDirSize);
 #ifdef HP_PROTECTION
     void Protection(void);
     bool Validate(void);
@@ -156,28 +154,31 @@ public:
 #define HP_INSERT_ERROR_ILLEGAL  3
 #define IS_HP_SUCCESS(x) ((x) <= HP_INSERT_SUCCESS)
     int Insert(HP_HEAPLENGTH nRecord, UINT32 nHash, void *pRecord);
-    HP_DIRINDEX FindFirstKey(UINT32 nHash, unsigned int *numchecks);
-    HP_DIRINDEX FindNextKey(HP_DIRINDEX i, UINT32 nHash, unsigned int *numchecks);
-    HP_DIRINDEX FindFirst(HP_PHEAPLENGTH pnRecord, void *pRecord);
-    HP_DIRINDEX FindNext(HP_PHEAPLENGTH pnRecord, void *pRecord);
-    void HeapCopy(HP_DIRINDEX iDir, HP_PHEAPLENGTH pnRecord, void *pRecord);
-    void HeapFree(HP_DIRINDEX iDir);
-    void HeapUpdate(HP_DIRINDEX iDir, HP_HEAPLENGTH nRecord, void *pRecord);
+    UINT32 FindFirstKey(UINT32 nHash, unsigned int *numchecks);
+    UINT32 FindNextKey(UINT32 i, UINT32 nHash, unsigned int *numchecks);
+    UINT32 FindFirst(HP_PHEAPLENGTH pnRecord, void *pRecord);
+    UINT32 FindNext(HP_PHEAPLENGTH pnRecord, void *pRecord);
+    void HeapCopy(UINT32 iDir, HP_PHEAPLENGTH pnRecord, void *pRecord);
+    void HeapFree(UINT32 iDir);
+    void HeapUpdate(UINT32 iDir, HP_HEAPLENGTH nRecord, void *pRecord);
 
+#if !defined(MEMORY_BASED)
     bool WritePage(HANDLE hFile, HF_FILEOFFSET oWhere);
     bool ReadPage(HANDLE hFile, HF_FILEOFFSET oWhere);
+#endif // MEMORY_BASED
 
-    HP_DIRINDEX GetDepth(void);
+    UINT32 GetDepth(void);
     bool Split(CHashPage &hp0, CHashPage &hp1);
 
     bool Defrag(HP_HEAPLENGTH nExtra);
     void GetRange(UINT32 arg_nDirDepth, UINT32 &nStart, UINT32 &nEnd);
 };
 
-
 #define HF_FIND_FIRST  HP_DIR_EMPTY
 #define HF_FIND_END    HP_DIR_EMPTY
 
+#if !defined(MEMORY_BASED)
+
 #define HF_CACHE_EMPTY       0
 #define HF_CACHE_CLEAN       1
 #define HF_CACHE_UNPROTECTED 2
@@ -232,16 +233,18 @@ public:
 #define HF_OPEN_STATUS_OLD    1
     int Open(const char *szDirFile, const char *szPageFile, int nCachePages);
     bool Insert(HP_HEAPLENGTH nRecord, UINT32 nHash, void *pRecord);
-    HP_DIRINDEX FindFirstKey(UINT32 nHash);
-    HP_DIRINDEX FindNextKey(HP_DIRINDEX iDir, UINT32 nHash);
-    void Copy(HP_DIRINDEX iDir, HP_PHEAPLENGTH pnRecord, void *pRecord);
-    void Remove(HP_DIRINDEX iDir);
+    UINT32 FindFirstKey(UINT32 nHash);
+    UINT32 FindNextKey(UINT32 iDir, UINT32 nHash);
+    void Copy(UINT32 iDir, HP_PHEAPLENGTH pnRecord, void *pRecord);
+    void Remove(UINT32 iDir);
     void CloseAll(void);
     void Sync(void);
     void Tick(void);
     ~CHashFile(void);
 };
 
+#endif // MEMORY_BASED
+
 typedef CHashPage *pCHashPage;
 
 class CHashTable
@@ -271,17 +274,17 @@ public:
     void ResetStats(void);
     void GetStats( unsigned int *hashsize, int *entries, INT64 *deletes,
                    INT64 *scans, INT64 *hits, INT64 *checks, int *max_scan);
-    INT64 GetEntryCount();
+    unsigned int GetEntryCount();
 
     void Reset(void);
     bool Insert(HP_HEAPLENGTH nRecord, UINT32 nHash, void *pRecord);
-    HP_DIRINDEX FindFirstKey(UINT32 nHash);
-    HP_DIRINDEX FindNextKey(HP_DIRINDEX iDir, UINT32 nHash);
-    HP_DIRINDEX FindFirst(HP_PHEAPLENGTH pnRecord, void *pRecord);
-    HP_DIRINDEX FindNext(HP_PHEAPLENGTH pnRecord, void *pRecord);
-    void Copy(HP_DIRINDEX iDir, HP_PHEAPLENGTH pnRecord, void *pRecord);
-    void Remove(HP_DIRINDEX iDir);
-    void Update(HP_DIRINDEX iDir, HP_HEAPLENGTH nRecord, void *pRecord);
+    UINT32 FindFirstKey(UINT32 nHash);
+    UINT32 FindNextKey(UINT32 iDir, UINT32 nHash);
+    UINT32 FindFirst(HP_PHEAPLENGTH pnRecord, void *pRecord);
+    UINT32 FindNext(HP_PHEAPLENGTH pnRecord, void *pRecord);
+    void Copy(UINT32 iDir, HP_PHEAPLENGTH pnRecord, void *pRecord);
+    void Remove(UINT32 iDir);
+    void Update(UINT32 iDir, HP_HEAPLENGTH nRecord, void *pRecord);
     ~CHashTable(void);
 };
 
@@ -293,8 +296,10 @@ private:
     CLinearTimeAbsolute m_ltaStarted;
 #ifdef WIN32
     CRITICAL_SECTION csLog;
-#endif // WIN32
     HANDLE m_hFile;
+#else
+    int    m_fdFile;
+#endif // WIN32
     size_t m_nSize;
     size_t m_nBuffer;
     char m_aBuffer[SIZEOF_LOG_BUFFER];
@@ -304,7 +309,7 @@ private:
     char m_szPrefix[32];
     char m_szFilename[SIZEOF_PATHNAME];
 
-    void CreateLogFile(void);
+    bool CreateLogFile(void);
     void AppendLogFile(void);
     void CloseLogFile(void);
 public:
@@ -313,7 +318,7 @@ public:
     void WriteBuffer(size_t nString, const char *pString);
     void WriteString(const char *pString);
     void WriteInteger(int iNumber);
-    void DCL_CDECL tinyprintf(char *pFormatSpec, ...);
+    void DCL_CDECL tinyprintf(const char *pFormatSpec, ...);
     void Flush(void);
     void SetPrefix(const char *pPrefix);
     void SetBasename(const char *pBasename);
