@@ -1,10 +1,6 @@
 // svdrand.cpp -- Random Numbers.
 //
-// $Id: svdrand.cpp,v 1.7 2006/01/07 07:44:46 sdennis Exp $
-//
-// MUX 2.4
-// Copyright (C) 1998 through 2004 Solid Vertical Domains, Ltd. All
-// rights not explicitly given are reserved.
+// $Id: svdrand.cpp 289 2006-10-21 01:43:33Z brazilofmux $
 //
 // Random Numbers from Makoto Matsumoto and Takuji Nishimura.
 //
@@ -16,7 +12,6 @@
 #include "timeutil.h"
 
 #include "svdhash.h"
-#include "svdrand.h"
 
 #ifdef WIN32
 #include <wincrypt.h>
@@ -55,12 +50,11 @@ void SeedRandomNumberGenerator(void)
     // Try to seed the PRNG from /dev/urandom
     // If it doesn't work, just seed the normal way
     //
-    int fd = open("/dev/urandom", O_RDONLY);
-
-    if (fd >= 0)
+    int fd;
+    if (mux_open(&fd, "/dev/urandom", O_RDONLY))
     {
-        int len = read(fd, aRandomSystemBytes, sizeof aRandomSystemBytes);
-        close(fd);
+        int len = mux_read(fd, aRandomSystemBytes, sizeof aRandomSystemBytes);
+        mux_close(fd);
         if (len > 0)
         {
             nRandomSystemBytes = len/sizeof(UINT32);
@@ -128,7 +122,7 @@ void SeedRandomNumberGenerator(void)
     CLinearTimeAbsolute lsaNow;
     lsaNow.GetUTC();
     INT64 i64Seed = lsaNow.Return100ns();
-    int pid = getpid();
+    int pid = mux_getpid();
 
     UINT32 nSeed = CRC32_ProcessBuffer(0, &i64Seed, sizeof(INT64));
     nSeed = CRC32_ProcessBuffer(nSeed, &pid, sizeof(pid));
@@ -169,7 +163,7 @@ INT32 RandomINT32(INT32 lLow, INT32 lHigh)
     //
 
     // In order to be perfectly conservative about not introducing any
-    // further sources of statistical bias, we're going to call getrand()
+    // further sources of statistical bias, we're going to call genrand()
     // until we get a number less than the greatest representable
     // multiple of x. We'll then return n mod x.
     //
@@ -179,7 +173,7 @@ INT32 RandomINT32(INT32 lLow, INT32 lHigh)
     //      P(UINT32_MAX_VALUE - n < UINT32_MAX_VALUE % x) < 0.5, for any x.
     //
     // So even for the least desireable x, the average number of times
-    // we will call getrand() is less than 2.
+    // we will call genrand() is less than 2.
     //
     UINT32 n;
     UINT32 nLimit = UINT32_MAX_VALUE - (UINT32_MAX_VALUE%x);
