@@ -1,10 +1,6 @@
 // timer.cpp -- Mini-task scheduler for timed events.
 //
-// $Id: timer.cpp,v 1.19 2006/01/07 07:18:56 sdennis Exp $
-//
-// MUX 2.4
-// Copyright (C) 1998 through 2004 Solid Vertical Domains, Ltd. All
-// rights not explicitly given are reserved.
+// $Id: timer.cpp 2803 2007-11-27 00:30:39Z brazilofmux $
 //
 #include "copyright.h"
 #include "autoconf.h"
@@ -27,7 +23,7 @@ void dispatch_FreeListReconstruction(void *pUnused, int iUnused)
 
     if (mudconf.control_flags & CF_DBCHECK)
     {
-        char *cmdsave = mudstate.debug_cmd;
+        const char *cmdsave = mudstate.debug_cmd;
         mudstate.debug_cmd = (char *)"< dbck >";
         do_dbck(NOTHING, NOTHING, NOTHING, 0);
         Guest.CleanUp();
@@ -58,7 +54,7 @@ void dispatch_DatabaseDump(void *pUnused, int iUnused)
 
     if (mudconf.control_flags & CF_CHECKPOINT)
     {
-        char *cmdsave = mudstate.debug_cmd;
+        const char *cmdsave = mudstate.debug_cmd;
         mudstate.debug_cmd = (char *)"< dump >";
 #ifndef WIN32
         if (mudstate.dumping)
@@ -97,7 +93,7 @@ void dispatch_IdleCheck(void *pUnused, int iUnused)
 
     if (mudconf.control_flags & CF_IDLECHECK)
     {
-        char *cmdsave = mudstate.debug_cmd;
+        const char *cmdsave = mudstate.debug_cmd;
         mudstate.debug_cmd = (char *)"< idlecheck >";
         check_idle();
         mudstate.debug_cmd = cmdsave;
@@ -122,7 +118,7 @@ void dispatch_CheckEvents(void *pUnused, int iUnused)
 
     if (mudconf.control_flags & CF_EVENTCHECK)
     {
-        char *cmdsave = mudstate.debug_cmd;
+        const char *cmdsave = mudstate.debug_cmd;
         mudstate.debug_cmd = (char *)"< eventcheck >";
         check_events();
         mudstate.debug_cmd = cmdsave;
@@ -143,7 +139,7 @@ void dispatch_CacheTick(void *pUnused, int iUnused)
     UNUSED_PARAMETER(pUnused);
     UNUSED_PARAMETER(iUnused);
 
-    char *cmdsave = mudstate.debug_cmd;
+    const char *cmdsave = mudstate.debug_cmd;
     mudstate.debug_cmd = (char *)"< cachetick >";
 
     CLinearTimeDelta ltd = 0;
@@ -167,7 +163,7 @@ void dispatch_CacheTick(void *pUnused, int iUnused)
 #if 0
 void dispatch_CleanChannels(void *pUnused, int iUnused)
 {
-    char *cmdsave = mudstate.debug_cmd;
+    const char *cmdsave = mudstate.debug_cmd;
     mudstate.debug_cmd = (char *)"< cleanchannels >";
     do_cleanupchannels();
 
@@ -189,25 +185,6 @@ static void dispatch_CanRestart(void *pUnused, int iUnused)
     mudstate.bCanRestart = true;
 }
 
-#ifdef WIN32
-static void dispatch_CalibrateQueryPerformance(void *pUnused, int iUnused)
-{
-    UNUSED_PARAMETER(pUnused);
-    UNUSED_PARAMETER(iUnused);
-
-    CLinearTimeAbsolute ltaNextTime;
-    ltaNextTime.GetUTC();
-    CLinearTimeDelta ltd = time_30s;
-    ltaNextTime += ltd;
-
-    if (CalibrateQueryPerformance())
-    {
-        scheduler.DeferTask(ltaNextTime, PRIORITY_SYSTEM,
-            dispatch_CalibrateQueryPerformance, 0, 0);
-    }
-}
-#endif // WIN32
-
 void init_timer(void)
 {
     CLinearTimeAbsolute ltaNow;
@@ -262,14 +239,6 @@ void init_timer(void)
     // Setup one-shot task to enable restarting 10 seconds after startmux.
     //
     scheduler.DeferTask(ltaNow+time_15s, PRIORITY_OBJECT, dispatch_CanRestart, 0, 0);
-
-#ifdef WIN32
-    // Setup Periodic QueryPerformance Calibration.
-    //
-    scheduler.DeferTask(ltaNow+time_30s, PRIORITY_SYSTEM,
-        dispatch_CalibrateQueryPerformance, 0, 0);
-
-#endif // WIN32
 }
 
 /*
@@ -277,8 +246,10 @@ void init_timer(void)
  * * do_timewarp: Adjust various internal timers.
  */
 
-void do_timewarp(dbref executor, dbref caller, dbref enactor, int key, char *arg)
+void do_timewarp(dbref executor, dbref caller, dbref enactor, int eval, int key, char *arg)
 {
+    UNUSED_PARAMETER(eval);
+
     int secs;
 
     secs = mux_atol(arg);
@@ -287,7 +258,7 @@ void do_timewarp(dbref executor, dbref caller, dbref enactor, int key, char *arg
     //
     if ((key == 0) || (key & TWARP_QUEUE))
     {
-        do_queue(executor, caller, enactor, QUEUE_WARP, arg);
+        do_queue(executor, caller, enactor, 0, QUEUE_WARP, arg);
     }
 
     // Once these are adjusted, we need to Cancel and reschedule the task.
@@ -321,7 +292,6 @@ void do_timewarp(dbref executor, dbref caller, dbref enactor, int key, char *arg
 }
 
 #define INITIAL_TASKS 100
-#define EXTRA_TASKS    50
 
 CTaskHeap::CTaskHeap(void)
 {
@@ -352,31 +322,44 @@ CTaskHeap::~CTaskHeap(void)
     }
 }
 
-void CTaskHeap::Insert(PTASK_RECORD pTask, SCHCMP *pfCompare)
+bool CTaskHeap::Insert(PTASK_RECORD pTask, SCHCMP *pfCompare)
 {
     if (m_nCurrent == m_nAllocated)
     {
-        if (!Grow()) return;
+        if (!Grow())
+        {
+            return false;
+        }
     }
     pTask->m_iVisitedMark = m_iVisitedMark-1;
 
     m_pHeap[m_nCurrent] = pTask;
     m_nCurrent++;
     SiftUp(m_nCurrent-1, pfCompare);
+    return true;
 }
 
 bool CTaskHeap::Grow(void)
 {
     // Grow the heap.
     //
-    int n = m_nAllocated + EXTRA_TASKS;
-    PTASK_RECORD *p = new PTASK_RECORD[n];
+    int n = GrowFiftyPercent(m_nAllocated, INITIAL_TASKS, INT_MAX);
+    PTASK_RECORD *p = NULL;
+    try
+    {
+        p = new PTASK_RECORD[n];
+    }
+    catch (...)
+    {
+        ; // Nothing.
+    }
+
     if (!p)
     {
         return false;
     }
 
-    memcpy(p, m_pHeap, sizeof(PTASK_RECORD)*m_nAllocated);
+    memcpy(p, m_pHeap, sizeof(PTASK_RECORD)*m_nCurrent);
     m_nAllocated = n;
     delete [] m_pHeap;
     m_pHeap = p;
@@ -384,6 +367,39 @@ bool CTaskHeap::Grow(void)
     return true;
 }
 
+void CTaskHeap::Shrink(void)
+{
+    // Shrink the heap.
+    //
+    int n = m_nAllocated/2;
+    if (  n <= INITIAL_TASKS
+       || n <= m_nCurrent)
+    {
+        return;
+    }
+
+    PTASK_RECORD *p = NULL;
+    try
+    {
+        p = new PTASK_RECORD[n];
+    }
+    catch (...)
+    {
+        ; // Nothing.
+    }
+
+    if (!p)
+    {
+        return;
+    }
+
+    memcpy(p, m_pHeap, sizeof(PTASK_RECORD)*m_nCurrent);
+    m_nAllocated = n;
+    delete [] m_pHeap;
+    m_pHeap = p;
+}
+
+
 PTASK_RECORD CTaskHeap::PeekAtTopmost(void)
 {
     if (m_nCurrent <= 0)
@@ -436,7 +452,12 @@ static int CompareWhen(PTASK_RECORD pTaskA, PTASK_RECORD pTaskB)
     {
         return 1;
     }
-    return 0;
+    else
+    {
+        // Must subtract so that ticket rollover is handled properly.
+        //
+        return  (pTaskA->m_Ticket) - (pTaskB->m_Ticket);
+    }
 }
 
 void CScheduler::DeferTask(const CLinearTimeAbsolute& ltaWhen, int iPriority,
@@ -454,7 +475,10 @@ void CScheduler::DeferTask(const CLinearTimeAbsolute& ltaWhen, int iPriority,
 
     // Must add to the WhenHeap so that network is still serviced.
     //
-    m_WhenHeap.Insert(pTask, CompareWhen);
+    if (!m_WhenHeap.Insert(pTask, CompareWhen))
+    {
+        delete pTask;
+    }
 }
 
 void CScheduler::DeferImmediateTask(int iPriority, FTASK *fpTask, void *arg_voidptr, int arg_Integer)
@@ -471,7 +495,10 @@ void CScheduler::DeferImmediateTask(int iPriority, FTASK *fpTask, void *arg_void
 
     // Must add to the WhenHeap so that network is still serviced.
     //
-    m_WhenHeap.Insert(pTask, CompareWhen);
+    if (!m_WhenHeap.Insert(pTask, CompareWhen))
+    {
+        delete pTask;
+    }
 }
 
 void CScheduler::CancelTask(FTASK *fpTask, void *arg_voidptr, int arg_Integer)
@@ -485,16 +512,14 @@ void CScheduler::ReadyTasks(const CLinearTimeAbsolute& ltaNow)
     // Move ready-to-run tasks off the WhenHeap and onto the PriorityHeap.
     //
     PTASK_RECORD pTask = m_WhenHeap.PeekAtTopmost();
-    while (pTask && (pTask->ltaWhen < ltaNow))
+    while (  pTask
+          && pTask->ltaWhen < ltaNow)
     {
         pTask = m_WhenHeap.RemoveTopmost(CompareWhen);
         if (pTask)
         {
-            if (pTask->fpTask)
-            {
-                m_PriorityHeap.Insert(pTask, ComparePriority);
-            }
-            else
+            if (  NULL == pTask->fpTask
+               || !m_PriorityHeap.Insert(pTask, ComparePriority))
             {
                 delete pTask;
             }
@@ -780,3 +805,9 @@ void CScheduler::SetMinPriority(int arg_minPriority)
 {
     m_minPriority = arg_minPriority;
 }
+
+void CScheduler::Shrink(void)
+{
+    m_WhenHeap.Shrink();
+    m_PriorityHeap.Shrink();
+}
