@@ -1,10 +1,6 @@
 // timeutil.h -- CLinearTimeAbsolute, and CLinearTimeDelta modules.
 //
-// $Id: timeutil.h,v 1.20 2006/08/03 19:05:19 sdennis Exp $
-//
-// MUX 2.4
-// Copyright (C) 1998 through 2004 Solid Vertical Domains, Ltd. All
-// rights not explicitly given are reserved.
+// $Id: timeutil.h 2734 2007-10-28 23:02:55Z brazilofmux $
 //
 // Date/Time code based on algorithms presented in "Calendrical Calculations",
 // Cambridge Press, 1998.
@@ -57,7 +53,7 @@ public:
     void GetUTC(void);
     void GetLocal(void);
 
-    void  ReturnUniqueString(char *buffer);
+    void  ReturnUniqueString(char *buffer, size_t nBuffer);
     char *ReturnDateString(int nFracDigits = 0);
     bool  ReturnFields(FIELDEDTIME *arg_tStruct);
     INT64 ReturnSeconds(void);
@@ -162,11 +158,11 @@ extern CMuxAlarm MuxAlarm;
 #define TIMEUTIL_TIME_T_MIN_VALUE (-922283539200i64)
 #define TIMEUTIL_TIME_T_MAX_VALUE ( 910638979199i64)
 #else // WIN32
-#define EPOCH_OFFSET 116444736000000000ull
-#define EARLIEST_VALID_DATE (-9106391088000000000ull)
-#define LATEST_VALID_DATE   ( 9222834959999999999ull)
-#define TIMEUTIL_TIME_T_MIN_VALUE (-922283539200ull)
-#define TIMEUTIL_TIME_T_MAX_VALUE ( 910638979199ull)
+#define EPOCH_OFFSET 116444736000000000ll
+#define EARLIEST_VALID_DATE (-9106391088000000000ll)
+#define LATEST_VALID_DATE   ( 9222834959999999999ll)
+#define TIMEUTIL_TIME_T_MIN_VALUE (-922283539200ll)
+#define TIMEUTIL_TIME_T_MAX_VALUE ( 910638979199ll)
 #endif // WIN32
 extern const INT64 FACTOR_100NS_PER_SECOND;
 extern const INT64 FACTOR_100NS_PER_MINUTE;
@@ -178,6 +174,7 @@ const CLinearTimeDelta time_200us = 200*FACTOR_100NS_PER_MICROSECOND;
 const CLinearTimeDelta time_5ms   = 5*FACTOR_100NS_PER_MILLISECOND;
 const CLinearTimeDelta time_250ms = 250*FACTOR_100NS_PER_MILLISECOND;
 const CLinearTimeDelta time_1s    = FACTOR_100NS_PER_SECOND;
+const CLinearTimeDelta time_5s    = 5*FACTOR_100NS_PER_SECOND;
 const CLinearTimeDelta time_15s   = 15*FACTOR_100NS_PER_SECOND;
 const CLinearTimeDelta time_30s   = 30*FACTOR_100NS_PER_SECOND;
 const CLinearTimeDelta time_45s   = 45*FACTOR_100NS_PER_SECOND;
@@ -186,9 +183,6 @@ const CLinearTimeDelta time_30m   = 30*FACTOR_100NS_PER_MINUTE;
 const CLinearTimeDelta time_1w    = FACTOR_100NS_PER_WEEK;
 
 extern void TIME_Initialize(void);
-#ifdef WIN32
-extern bool CalibrateQueryPerformance(void);
-#endif
 
 #ifdef SMALLEST_INT_GTE_NEG_QUOTIENT
 INT64 i64Mod(INT64 x, INT64 y);
@@ -211,8 +205,8 @@ int DCL_INLINE iFloorDivisionMod(int x, int y, int *piMod) \
 
 extern bool ParseDate(CLinearTimeAbsolute &lta, char *pDateString, bool *pbZoneSpecified);
 extern bool isLeapYear(long iYear);
-extern const char *monthtab[];
-extern char *DayOfWeekString[];
+extern const char *monthtab[12];
+extern const char *DayOfWeekString[7];
 extern int iMod(int x, int y);
 
 #endif // TIMEUTIL_H
