@@ -1,10 +1,6 @@
 // vattr.cpp -- Manages the user-defined attributes.
 //
-// $Id: vattr.cpp,v 1.12 2006/01/07 02:06:03 sdennis Exp $
-//
-// MUX 2.4
-// Copyright (C) 1998 through 2004 Solid Vertical Domains, Ltd. All
-// rights not explicitly given are reserved.
+// $Id: vattr.cpp 8 2006-09-05 01:55:58Z brazilofmux $
 //
 #include "copyright.h"
 #include "autoconf.h"
@@ -35,7 +31,7 @@ ATTR *vattr_find_LEN(const char *pAttrName, size_t nAttrName)
     UINT32 nHash = HASH_ProcessBuffer(0, pAttrName, nAttrName);
 
     CHashTable *pht = &mudstate.vattr_name_htab;
-    HP_DIRINDEX iDir = pht->FindFirstKey(nHash);
+    UINT32 iDir = pht->FindFirstKey(nHash);
     while (iDir != HF_FIND_END)
     {
         HP_HEAPLENGTH nRecord;
@@ -67,24 +63,29 @@ ATTR *vattr_define_LEN(char *pName, size_t nName, int number, int flags)
     }
 
     vp = (ATTR *)MEMALLOC(sizeof(ATTR));
-    ISOUTOFMEMORY(vp);
+    if (vp)
+    {
+        // NOTE: By using store_string, the only way to release the
+        // memory associated with a user attribute name is to @restart
+        // the game.
+        //
+        vp->name = store_string(pName);
+        vp->flags = flags;
+        vp->number = number;
 
-    // NOTE: By using store_string, the only way to release the
-    // memory associated with a user attribute name is to @restart
-    // the game.
-    //
-    vp->name = store_string(pName);
-    vp->flags = flags;
-    vp->number = number;
+        // This entry cannot already be in the hash table because we've checked it
+        // above with vattr_find_LEN.
+        //
+        UINT32 nHash = HASH_ProcessBuffer(0, pName, nName);
+        mudstate.vattr_name_htab.Insert(sizeof(number), nHash, &number);
 
-    // This entry cannot already be in the hash table because we've checked it
-    // above with vattr_find_LEN.
-    //
-    UINT32 nHash = HASH_ProcessBuffer(0, pName, nName);
-    mudstate.vattr_name_htab.Insert(sizeof(number), nHash, &number);
-
-    anum_extend(vp->number);
-    anum_set(vp->number, (ATTR *) vp);
+        anum_extend(vp->number);
+        anum_set(vp->number, (ATTR *) vp);
+    }
+    else
+    {
+        ISOUTOFMEMORY(vp);
+    }
     return vp;
 }
 
@@ -194,7 +195,7 @@ static void dbclean_CheckATtoANH(dbref executor)
                 // Convert name to upper case.
                 //
                 char Buffer[SBUF_SIZE];
-                strcpy(Buffer, pa->name);
+                mux_strncpy(Buffer, pa->name, SBUF_SIZE-1);
                 mux_strupr(Buffer);
 
                 // Fetch the attribute structure pointer -- which should match the one
@@ -415,7 +416,7 @@ static int dbclean_RemoveStaleAttributeNames(void)
                 //
                 UINT32 nHash = HASH_ProcessBuffer(0, va->name, strlen(va->name));
                 CHashTable *pht = &mudstate.vattr_name_htab;
-                HP_DIRINDEX iDir = pht->FindFirstKey(nHash);
+                UINT32 iDir = pht->FindFirstKey(nHash);
                 while (iDir != HF_FIND_END)
                 {
                     HP_HEAPLENGTH nRecord;
@@ -480,7 +481,7 @@ static void dbclean_RenumberAttributes(int cVAttributes)
             //
             UINT32 nHash = HASH_ProcessBuffer(0, va->name, strlen(va->name));
             CHashTable *pht = &mudstate.vattr_name_htab;
-            HP_DIRINDEX iDir = pht->FindFirstKey(nHash);
+            UINT32 iDir = pht->FindFirstKey(nHash);
             while (iDir != HF_FIND_END)
             {
                 HP_HEAPLENGTH nRecord;
@@ -621,13 +622,13 @@ void do_dbclean(dbref executor, dbref caller, dbref enactor, int key)
     notify(executor, "@dbclean completed..");
 }
 
-void vattr_delete_LEN(char *pName, int nName)
+void vattr_delete_LEN(char *pName, size_t nName)
 {
     // Delete from hashtable.
     //
     UINT32 nHash = HASH_ProcessBuffer(0, pName, nName);
     CHashTable *pht = &mudstate.vattr_name_htab;
-    HP_DIRINDEX iDir = pht->FindFirstKey(nHash);
+    UINT32 iDir = pht->FindFirstKey(nHash);
     while (iDir != HF_FIND_END)
     {
         HP_HEAPLENGTH nRecord;
@@ -645,13 +646,13 @@ void vattr_delete_LEN(char *pName, int nName)
     }
 }
 
-ATTR *vattr_rename_LEN(char *pOldName, int nOldName, char *pNewName, int nNewName)
+ATTR *vattr_rename_LEN(char *pOldName, size_t nOldName, char *pNewName, size_t nNewName)
 {
     // Find and Delete old name from hashtable.
     //
     UINT32 nHash = HASH_ProcessBuffer(0, pOldName, nOldName);
     CHashTable *pht = &mudstate.vattr_name_htab;
-    HP_DIRINDEX iDir = pht->FindFirstKey(nHash);
+    UINT32 iDir = pht->FindFirstKey(nHash);
     while (iDir != HF_FIND_END)
     {
         HP_HEAPLENGTH nRecord;
@@ -679,7 +680,7 @@ ATTR *vattr_first(void)
 {
     HP_HEAPLENGTH nRecord;
     int anum;
-    HP_DIRINDEX iDir = mudstate.vattr_name_htab.FindFirst(&nRecord, &anum);
+    UINT32 iDir = mudstate.vattr_name_htab.FindFirst(&nRecord, &anum);
     if (iDir != HF_FIND_END)
     {
         return (ATTR *)anum_table[anum];
@@ -695,7 +696,7 @@ ATTR *vattr_next(ATTR *vp)
 
     HP_HEAPLENGTH nRecord;
     int anum;
-    HP_DIRINDEX iDir = mudstate.vattr_name_htab.FindNext(&nRecord, &anum);
+    UINT32 iDir = mudstate.vattr_name_htab.FindNext(&nRecord, &anum);
     if (iDir != HF_FIND_END)
     {
         return (ATTR *)anum_table[anum];
