@@ -1,6 +1,6 @@
 // walkdb.cpp -- Support for commands that walk the entire db.
 //
-// $Id: walkdb.cpp,v 1.13 2006/05/18 17:57:48 sdennis Exp $
+// $Id: walkdb.cpp 2734 2007-10-28 23:02:55Z brazilofmux $
 //
 
 #include "copyright.h"
@@ -16,13 +16,15 @@
 // Cmds run in low-prio Q after a 1 sec delay for the first one.
 //
 static void bind_and_queue(dbref executor, dbref caller, dbref enactor,
-                           char *action, char *argstr, char *cargs[],
+                           int eval, char *action, char *argstr, char *cargs[],
                            int ncargs, int number)
 {
     char *command = replace_tokens(action, argstr, mux_ltoa_t(number), NULL);
     CLinearTimeAbsolute lta;
-    wait_que(executor, caller, enactor, false, lta, NOTHING, 0, command,
-        cargs, ncargs, mudstate.global_regs);
+    wait_que(executor, caller, enactor, eval, false, lta, NOTHING, 0,
+        command,
+        ncargs, cargs,
+        mudstate.global_regs);
     free_lbuf(command);
 }
 
@@ -32,7 +34,7 @@ static void bind_and_queue(dbref executor, dbref caller, dbref enactor,
 // New switches added 12/92, /space (default) delimits list using spaces,
 // and /delimit allows specification of a delimiter.
 //
-void do_dolist(dbref executor, dbref caller, dbref enactor, int key,
+void do_dolist(dbref executor, dbref caller, dbref enactor, int eval, int key,
                char *list, char *command, char *cargs[], int ncargs)
 {
     if (!list || *list == '\0')
@@ -64,27 +66,32 @@ void do_dolist(dbref executor, dbref caller, dbref enactor, int key,
         {
             number++;
             objstring = parse_to(&curr, delimiter, EV_STRIP_CURLY);
-            bind_and_queue(executor, caller, enactor, command, objstring,
+            bind_and_queue(executor, caller, enactor, eval, command, objstring,
                 cargs, ncargs, number);
         }
     }
+
     if (key & DOLIST_NOTIFY)
     {
         char *tbuf = alloc_lbuf("dolist.notify_cmd");
-        strcpy(tbuf, "@notify/quiet me");
+        mux_strncpy(tbuf, "@notify/quiet me", LBUF_SIZE-1);
         CLinearTimeAbsolute lta;
-        wait_que(executor, caller, enactor, false, lta, NOTHING, A_SEMAPHORE,
-            tbuf, cargs, ncargs, mudstate.global_regs);
+        wait_que(executor, caller, enactor, eval, false, lta, NOTHING,
+            A_SEMAPHORE,
+            tbuf,
+            ncargs, cargs,
+            mudstate.global_regs);
         free_lbuf(tbuf);
     }
 }
 
 // Regular @find command
 //
-void do_find(dbref executor, dbref caller, dbref enactor, int key, char *name)
+void do_find(dbref executor, dbref caller, dbref enactor, int eval, int key, char *name)
 {
     UNUSED_PARAMETER(caller);
     UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(eval);
     UNUSED_PARAMETER(key);
 
     char *buff;
@@ -183,10 +190,11 @@ bool get_stats(dbref player, dbref who, STATS *info)
 
 // Reworked by R'nice
 //
-void do_stats(dbref executor, dbref caller, dbref enactor, int key, char *name)
+void do_stats(dbref executor, dbref caller, dbref enactor, int eval, int key, char *name)
 {
     UNUSED_PARAMETER(caller);
     UNUSED_PARAMETER(enactor);
+    UNUSED_PARAMETER(eval);
 
     dbref owner;
 
@@ -247,21 +255,30 @@ int chown_all(dbref from_player, dbref to_player, dbref acting_player, int key)
     {
         from_player = Owner(from_player);
     }
+
     if (!isPlayer(to_player))
     {
         to_player = Owner(to_player);
     }
-    int count     = 0;
+
+    int count = 0;
     if (  God(from_player)
        && !God(acting_player))
     {
-        notify(acting_player, "Permission denied.");
+        notify(acting_player, NOPERM_MESSAGE);
     }
     else
     {
         int i;
         int quota_out = 0;
         int quota_in  = 0;
+
+        FLAGSET clearflags;
+        FLAGSET setflags;
+        bool    bClearPowers;
+
+        TranslateFlags_Chown(clearflags.word, setflags.word, &bClearPowers, acting_player, key);
+
         DO_WHOLE_DB(i)
         {
             if (  Owner(i) == from_player
@@ -300,13 +317,23 @@ int chown_all(dbref from_player, dbref to_player, dbref acting_player, int key)
 
                     s_Owner(i, to_player);
                 }
-                s_Flags(i, FLAG_WORD1,
-                    (Flags(i) & ~(CHOWN_OK | INHERIT)) | HALT);
 
                 if (key & CHOWN_NOZONE)
                 {
                     s_Zone(i, NOTHING);
                 }
+
+                SetClearFlags(i, clearflags.word, setflags.word);
+                if (bClearPowers)
+                {
+                    s_Powers(i, 0);
+                    s_Powers2(i, 0);
+                }
+
+                // Always halt the queue.
+                //
+                halt_que(NOTHING, i);
+
                 count++;
             }
         }
@@ -369,7 +396,7 @@ void do_chownall
     }
 }
 
-#define ANY_OWNER -2
+#define ANY_OWNER (-2)
 
 static void er_mark_disabled(dbref player)
 {
@@ -393,7 +420,7 @@ bool search_setup(dbref player, char *searchfor, SEARCH *parm)
     char *pname = parse_to(&searchfor, '=', EV_STRIP_TS);
     if (!pname || !*pname)
     {
-        pname = "me";
+        pname = (char *)"me";
     }
     else
     {
@@ -411,12 +438,12 @@ bool search_setup(dbref player, char *searchfor, SEARCH *parm)
         else
         {
             searchtype = pname;
-            pname = "";
+            pname = (char *)"";
         }
     }
     else
     {
-        searchtype = "";
+        searchtype = (char *)"";
     }
 
     // If the player name is quoted, strip the quotes.
@@ -838,7 +865,7 @@ void search_perform(dbref executor, dbref caller, dbref enactor, SEARCH *parm)
             result = bp = alloc_lbuf("search_perform");
             str = buff2;
             mux_exec(result, &bp, executor, caller, enactor,
-                EV_FCHECK | EV_EVAL | EV_NOTRACE, &str, (char **)NULL, 0);
+                EV_FCHECK | EV_EVAL | EV_NOTRACE, &str, NULL, 0);
             *bp = '\0';
             free_lbuf(buff2);
             if (!*result || !xlate(result))
@@ -894,8 +921,10 @@ static void search_mark(dbref player, int key)
     return;
 }
 
-void do_search(dbref executor, dbref caller, dbref enactor, int key, char *arg)
+void do_search(dbref executor, dbref caller, dbref enactor, int eval, int key, char *arg)
 {
+    UNUSED_PARAMETER(eval);
+
     char *buff, *outbuf, *bp;
     dbref thing, from, to;
     SEARCH searchparm;
@@ -1076,7 +1105,7 @@ void do_search(dbref executor, dbref caller, dbref enactor, int key, char *arg)
     }
     else
     {
-        sprintf(outbuf,
+        mux_sprintf(outbuf, LBUF_SIZE,
             "\nFound:  Rooms...%d  Exits...%d  Objects...%d  Players...%d",
             rcount, ecount, tcount, pcount);
         notify(executor, outbuf);
@@ -1117,9 +1146,10 @@ void do_markall(dbref executor, dbref caller, dbref enactor, int key)
 // ---------------------------------------------------------------------------
 // do_apply_marked: Perform a command for each marked obj in the db.
 //
-void do_apply_marked( dbref executor, dbref caller, dbref enactor, int key,
-                      char *command, char *cargs[], int ncargs)
+void do_apply_marked( dbref executor, dbref caller, dbref enactor, int eval,
+                      int key, char *command, char *cargs[], int ncargs)
 {
+    UNUSED_PARAMETER(eval);
     UNUSED_PARAMETER(key);
 
     if (mudconf.control_flags & CF_DBCHECK)
@@ -1137,7 +1167,7 @@ void do_apply_marked( dbref executor, dbref caller, dbref enactor, int key,
             buff[0] = '#';
             mux_ltoa(i, buff+1);
             number++;
-            bind_and_queue(executor, caller, enactor, command, buff,
+            bind_and_queue(executor, caller, enactor, eval, command, buff,
                 cargs, ncargs, number);
         }
     }
@@ -1157,16 +1187,31 @@ void do_apply_marked( dbref executor, dbref caller, dbref enactor, int key,
 //
 void olist_push(void)
 {
-    OLSTK *ol = (OLSTK *)MEMALLOC(sizeof(OLSTK));
-    ISOUTOFMEMORY(ol);
-    ol->next = mudstate.olist;
-    mudstate.olist = ol;
+    OLSTK *ol = NULL;
+    try
+    {
+        ol = new OLSTK;
+    }
+    catch (...)
+    {
+        ; // Nothing.
+    }
 
-    ol->head = NULL;
-    ol->tail = NULL;
-    ol->cblock = NULL;
-    ol->count = 0;
-    ol->citm = 0;
+    if (NULL != ol)
+    {
+        ol->next = mudstate.olist;
+        mudstate.olist = ol;
+
+        ol->head = NULL;
+        ol->tail = NULL;
+        ol->cblock = NULL;
+        ol->count = 0;
+        ol->citm = 0;
+    }
+    else
+    {
+        ISOUTOFMEMORY(ol);
+    }
 }
 
 // olist_pop: Pop one entire list off the object list stack.
@@ -1180,7 +1225,7 @@ void olist_pop(void)
         onext = op->next;
         free_lbuf(op);
     }
-    MEMFREE(mudstate.olist);
+    delete mudstate.olist;
     mudstate.olist = ol;
 }
 
