@@ -1,6 +1,6 @@
 // wiz.cpp -- Wizard-only commands.
 //
-// $Id: wiz.cpp 492 2006-11-30 17:12:25Z brazilofmux $
+// $Id: wiz.cpp 3689 2008-05-31 17:36:05Z brazilofmux $
 //
 
 #include "copyright.h"
@@ -114,6 +114,7 @@ static void do_teleport_single
             return;
         }
     }
+
     if (  isGarbage(destination)
        || (  Has_location(destination)
           && isGarbage(Location(destination))))
@@ -125,16 +126,43 @@ static void do_teleport_single
     }
     else if (Has_contents(destination))
     {
-        // You must control the destination, or it must be a JUMP_OK
-        // room where you pass its TELEPORT lock.
+        // You must control the destination OR it must be a JUMP_OK room where
+        // the victim passes its TELEPORT lock (exit victims have the
+        // additional requirement that the destination must be OPEN_OK and
+        // pass the OPEN lock) OR you must be Tel_Anywhere.
         //
-        if (  !(  Controls(executor, destination)
-               || Jump_ok(destination)
-               || Tel_Anywhere(executor))
-           || !could_doit(executor, destination, A_LTPORT)
-           || (  isExit(victim)
-              && God(destination)
-              && !God(executor)))
+        // Only God may teleport exits into God.
+        //
+        if (  (  Controls(executor, destination)
+              || Tel_Anywhere(executor)
+              || (  Jump_ok(destination)
+                 && could_doit(victim, destination, A_LTPORT)
+                 && (  !isExit(victim)
+                    || (  Open_ok(destination)
+                       && could_doit(executor, destination, A_LOPEN)))))
+           && (  !isExit(victim)
+              || !God(destination)
+              || God(executor)))
+        {
+            // We're OK, do the teleport.
+            //
+            if (key & TELEPORT_QUIET)
+            {
+                hush = HUSH_ENTER | HUSH_LEAVE;
+            }
+
+            if (move_via_teleport(victim, destination, enactor, hush))
+            {
+                if (executor != victim)
+                {
+                    if (!Quiet(executor))
+                    {
+                        notify_quiet(executor, "Teleported.");
+                    }
+                }
+            }
+        }
+        else
         {
             // Nope, report failure.
             //
@@ -145,25 +173,6 @@ static void do_teleport_single
             did_it(victim, destination,
                    A_TFAIL, "You can't teleport there!",
                    A_OTFAIL, 0, A_ATFAIL, 0, NULL, 0);
-            return;
-        }
-
-        // We're OK, do the teleport.
-        //
-        if (key & TELEPORT_QUIET)
-        {
-            hush = HUSH_ENTER | HUSH_LEAVE;
-        }
-
-        if (move_via_teleport(victim, destination, enactor, hush))
-        {
-            if (executor != victim)
-            {
-                if (!Quiet(executor))
-                {
-                    notify_quiet(executor, "Teleported.");
-                }
-            }
         }
     }
     else if (isExit(destination))
